#!/usr/bin/env node
/* eslint-disable */
"use strict";

function $$SETUP_STATE(hydrateRuntimeState, basePath) {
  return hydrateRuntimeState(JSON.parse('{\
    "__info": [\
      "This file is automatically generated. Do not touch it, or risk",\
      "your modifications being lost. We also recommend you not to read",\
      "it either without using the @yarnpkg/pnp package, as the data layout",\
      "is entirely unspecified and WILL change from a version to another."\
    ],\
    "dependencyTreeRoots": [\
      {\
        "name": "ipynao",\
        "reference": "workspace:."\
      }\
    ],\
    "enableTopLevelFallback": true,\
    "ignorePatternData": "(^(?:\\\\.yarn\\\\/sdks(?:\\\\/(?!\\\\.{1,2}(?:\\\\/|$))(?:(?:(?!(?:^|\\\\/)\\\\.{1,2}(?:\\\\/|$)).)*?)|$))$)",\
    "fallbackExclusionList": [\
      ["ipynao", ["workspace:."]]\
    ],\
    "fallbackPool": [\
    ],\
    "packageRegistryData": [\
      [null, [\
        [null, {\
          "packageLocation": "./",\
          "packageDependencies": [\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/preset-env", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:7.22.5"],\
            ["@jupyter-widgets/base", "npm:6.0.4"],\
            ["@jupyter-widgets/base-manager", "npm:1.0.5"],\
            ["@jupyterlab/builder", "npm:3.6.4"],\
            ["@lumino/application", "npm:1.31.4"],\
            ["@lumino/widgets", "npm:1.37.2"],\
            ["@types/jest", "npm:26.0.24"],\
            ["@types/webpack-env", "npm:1.18.1"],\
            ["@typescript-eslint/eslint-plugin", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["@typescript-eslint/parser", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["acorn", "npm:7.4.1"],\
            ["css-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.6.0"],\
            ["eslint", "npm:7.32.0"],\
            ["eslint-config-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:6.15.0"],\
            ["eslint-plugin-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.4.1"],\
            ["fs-extra", "npm:7.0.1"],\
            ["identity-obj-proxy", "npm:3.0.0"],\
            ["jest", "npm:26.6.3"],\
            ["mkdirp", "npm:0.5.6"],\
            ["nao-socket.io", "npm:1.0.4"],\
            ["npm-run-all", "npm:4.1.5"],\
            ["prettier", "npm:2.8.8"],\
            ["rimraf", "npm:2.7.1"],\
            ["source-map-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.1.3"],\
            ["style-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.3.0"],\
            ["ts-jest", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:26.5.6"],\
            ["ts-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:8.4.0"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"],\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"]\
          ],\
          "linkType": "SOFT"\
        }]\
      ]],\
      ["@ampproject/remapping", [\
        ["npm:2.2.1", {\
          "packageLocation": "./.yarn/cache/@ampproject-remapping-npm-2.2.1-3da3d624be-03c04fd526.zip/node_modules/@ampproject/remapping/",\
          "packageDependencies": [\
            ["@ampproject/remapping", "npm:2.2.1"],\
            ["@jridgewell/gen-mapping", "npm:0.3.3"],\
            ["@jridgewell/trace-mapping", "npm:0.3.18"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/code-frame", [\
        ["npm:7.12.11", {\
          "packageLocation": "./.yarn/cache/@babel-code-frame-npm-7.12.11-1a9a1b277f-3963eff3eb.zip/node_modules/@babel/code-frame/",\
          "packageDependencies": [\
            ["@babel/code-frame", "npm:7.12.11"],\
            ["@babel/highlight", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-code-frame-npm-7.22.5-b36f88d6f9-cfe804f518.zip/node_modules/@babel/code-frame/",\
          "packageDependencies": [\
            ["@babel/code-frame", "npm:7.22.5"],\
            ["@babel/highlight", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/compat-data", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-compat-data-npm-7.22.5-282f002362-eb1a47ebf7.zip/node_modules/@babel/compat-data/",\
          "packageDependencies": [\
            ["@babel/compat-data", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/core", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-core-npm-7.22.5-d75e931080-173ae42695.zip/node_modules/@babel/core/",\
          "packageDependencies": [\
            ["@babel/core", "npm:7.22.5"],\
            ["@ampproject/remapping", "npm:2.2.1"],\
            ["@babel/code-frame", "npm:7.22.5"],\
            ["@babel/generator", "npm:7.22.5"],\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/helper-module-transforms", "npm:7.22.5"],\
            ["@babel/helpers", "npm:7.22.5"],\
            ["@babel/parser", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["convert-source-map", "npm:1.9.0"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["gensync", "npm:1.0.0-beta.2"],\
            ["json5", "npm:2.2.3"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/generator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-generator-npm-7.22.5-0e87a1a822-efa64da70c.zip/node_modules/@babel/generator/",\
          "packageDependencies": [\
            ["@babel/generator", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@jridgewell/gen-mapping", "npm:0.3.3"],\
            ["@jridgewell/trace-mapping", "npm:0.3.18"],\
            ["jsesc", "npm:2.5.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-annotate-as-pure", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-annotate-as-pure-npm-7.22.5-f38dc8aa1c-53da330f18.zip/node_modules/@babel/helper-annotate-as-pure/",\
          "packageDependencies": [\
            ["@babel/helper-annotate-as-pure", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-builder-binary-assignment-operator-visitor", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-builder-binary-assignment-operator-visitor-npm-7.22.5-edf1e207c4-d753acac62.zip/node_modules/@babel/helper-builder-binary-assignment-operator-visitor/",\
          "packageDependencies": [\
            ["@babel/helper-builder-binary-assignment-operator-visitor", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-compilation-targets", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-compilation-targets-npm-7.22.5-5e6d9af186-a479460615.zip/node_modules/@babel/helper-compilation-targets/",\
          "packageDependencies": [\
            ["@babel/helper-compilation-targets", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-helper-compilation-targets-virtual-0765db2ddf/0/cache/@babel-helper-compilation-targets-npm-7.22.5-5e6d9af186-a479460615.zip/node_modules/@babel/helper-compilation-targets/",\
          "packageDependencies": [\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/compat-data", "npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-validator-option", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["browserslist", "npm:4.21.9"],\
            ["lru-cache", "npm:5.1.1"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-create-class-features-plugin", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-create-class-features-plugin-npm-7.22.5-f032702cef-f1e91deae0.zip/node_modules/@babel/helper-create-class-features-plugin/",\
          "packageDependencies": [\
            ["@babel/helper-create-class-features-plugin", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:1f163d5394432f433356473aaacde47e38d672c0ae0f3404c834b6a32a6fbb176891c5c5f3863489a008766bcf4583f573722b959e88c8478958d7633507e0cd#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-helper-create-class-features-plugin-virtual-17b79dbfd1/0/cache/@babel-helper-create-class-features-plugin-npm-7.22.5-f032702cef-f1e91deae0.zip/node_modules/@babel/helper-create-class-features-plugin/",\
          "packageDependencies": [\
            ["@babel/helper-create-class-features-plugin", "virtual:1f163d5394432f433356473aaacde47e38d672c0ae0f3404c834b6a32a6fbb176891c5c5f3863489a008766bcf4583f573722b959e88c8478958d7633507e0cd#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-annotate-as-pure", "npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-function-name", "npm:7.22.5"],\
            ["@babel/helper-member-expression-to-functions", "npm:7.22.5"],\
            ["@babel/helper-optimise-call-expression", "npm:7.22.5"],\
            ["@babel/helper-replace-supers", "npm:7.22.5"],\
            ["@babel/helper-skip-transparent-expression-wrappers", "npm:7.22.5"],\
            ["@babel/helper-split-export-declaration", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["semver", "npm:6.3.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-create-regexp-features-plugin", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-create-regexp-features-plugin-npm-7.22.5-c8a84cb8d3-94932145be.zip/node_modules/@babel/helper-create-regexp-features-plugin/",\
          "packageDependencies": [\
            ["@babel/helper-create-regexp-features-plugin", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-helper-create-regexp-features-plugin-virtual-cc3929e237/0/cache/@babel-helper-create-regexp-features-plugin-npm-7.22.5-c8a84cb8d3-94932145be.zip/node_modules/@babel/helper-create-regexp-features-plugin/",\
          "packageDependencies": [\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-annotate-as-pure", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["regexpu-core", "npm:5.3.2"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-define-polyfill-provider", [\
        ["npm:0.4.0", {\
          "packageLocation": "./.yarn/cache/@babel-helper-define-polyfill-provider-npm-0.4.0-b101cc1d08-5dca4c5e78.zip/node_modules/@babel/helper-define-polyfill-provider/",\
          "packageDependencies": [\
            ["@babel/helper-define-polyfill-provider", "npm:0.4.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:37199b5719e67bca157608a02dec1de2f89dc59c4bc7ba7f9107eb5bf1d1f85a15b122a67078197e23664014e4511167d3b3e53ce55f6f59a1c05d9d6f083563#npm:0.4.0", {\
          "packageLocation": "./.yarn/__virtual__/@babel-helper-define-polyfill-provider-virtual-555ccc945d/0/cache/@babel-helper-define-polyfill-provider-npm-0.4.0-b101cc1d08-5dca4c5e78.zip/node_modules/@babel/helper-define-polyfill-provider/",\
          "packageDependencies": [\
            ["@babel/helper-define-polyfill-provider", "virtual:37199b5719e67bca157608a02dec1de2f89dc59c4bc7ba7f9107eb5bf1d1f85a15b122a67078197e23664014e4511167d3b3e53ce55f6f59a1c05d9d6f083563#npm:0.4.0"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["lodash.debounce", "npm:4.0.8"],\
            ["resolve", "patch:resolve@npm%3A1.22.3#~builtin<compat/resolve>::version=1.22.3&hash=c3c19d"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-environment-visitor", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-environment-visitor-npm-7.22.5-7bc52eec61-248532077d.zip/node_modules/@babel/helper-environment-visitor/",\
          "packageDependencies": [\
            ["@babel/helper-environment-visitor", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-function-name", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-function-name-npm-7.22.5-8a1a69b63d-6b1f6ce1b1.zip/node_modules/@babel/helper-function-name/",\
          "packageDependencies": [\
            ["@babel/helper-function-name", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-hoist-variables", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-hoist-variables-npm-7.22.5-6db3192347-394ca191b4.zip/node_modules/@babel/helper-hoist-variables/",\
          "packageDependencies": [\
            ["@babel/helper-hoist-variables", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-member-expression-to-functions", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-member-expression-to-functions-npm-7.22.5-04d5cbe959-4bd5791529.zip/node_modules/@babel/helper-member-expression-to-functions/",\
          "packageDependencies": [\
            ["@babel/helper-member-expression-to-functions", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-module-imports", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-module-imports-npm-7.22.5-399b6063db-9ac2b0404f.zip/node_modules/@babel/helper-module-imports/",\
          "packageDependencies": [\
            ["@babel/helper-module-imports", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-module-transforms", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-module-transforms-npm-7.22.5-c31751930e-8985dc0d97.zip/node_modules/@babel/helper-module-transforms/",\
          "packageDependencies": [\
            ["@babel/helper-module-transforms", "npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-module-imports", "npm:7.22.5"],\
            ["@babel/helper-simple-access", "npm:7.22.5"],\
            ["@babel/helper-split-export-declaration", "npm:7.22.5"],\
            ["@babel/helper-validator-identifier", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-optimise-call-expression", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-optimise-call-expression-npm-7.22.5-846964ef82-c70ef6cc6b.zip/node_modules/@babel/helper-optimise-call-expression/",\
          "packageDependencies": [\
            ["@babel/helper-optimise-call-expression", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-plugin-utils", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-plugin-utils-npm-7.22.5-192e38e1de-c0fc722707.zip/node_modules/@babel/helper-plugin-utils/",\
          "packageDependencies": [\
            ["@babel/helper-plugin-utils", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-remap-async-to-generator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-remap-async-to-generator-npm-7.22.5-dbbf1ffc2d-1e51dcff1c.zip/node_modules/@babel/helper-remap-async-to-generator/",\
          "packageDependencies": [\
            ["@babel/helper-remap-async-to-generator", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:4dc4a45695acd3398bc920a43e1730a2258dac068831cc2d5f4ffecac49439a22d6a5f6d67ce27fcfcfda1d0fc81f791dca2be2a0eaed9e3ebbb0e0432a3b529#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-helper-remap-async-to-generator-virtual-87f007b10d/0/cache/@babel-helper-remap-async-to-generator-npm-7.22.5-dbbf1ffc2d-1e51dcff1c.zip/node_modules/@babel/helper-remap-async-to-generator/",\
          "packageDependencies": [\
            ["@babel/helper-remap-async-to-generator", "virtual:4dc4a45695acd3398bc920a43e1730a2258dac068831cc2d5f4ffecac49439a22d6a5f6d67ce27fcfcfda1d0fc81f791dca2be2a0eaed9e3ebbb0e0432a3b529#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-annotate-as-pure", "npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-wrap-function", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-replace-supers", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-replace-supers-npm-7.22.5-45a4aff2bc-af29deff6c.zip/node_modules/@babel/helper-replace-supers/",\
          "packageDependencies": [\
            ["@babel/helper-replace-supers", "npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-member-expression-to-functions", "npm:7.22.5"],\
            ["@babel/helper-optimise-call-expression", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-simple-access", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-simple-access-npm-7.22.5-0a3f578780-fe9686714c.zip/node_modules/@babel/helper-simple-access/",\
          "packageDependencies": [\
            ["@babel/helper-simple-access", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-skip-transparent-expression-wrappers", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-skip-transparent-expression-wrappers-npm-7.22.5-a398428942-1012ef2295.zip/node_modules/@babel/helper-skip-transparent-expression-wrappers/",\
          "packageDependencies": [\
            ["@babel/helper-skip-transparent-expression-wrappers", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-split-export-declaration", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-split-export-declaration-npm-7.22.5-5e708abd3e-d10e05a02f.zip/node_modules/@babel/helper-split-export-declaration/",\
          "packageDependencies": [\
            ["@babel/helper-split-export-declaration", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-string-parser", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-string-parser-npm-7.22.5-448ff0e489-836851ca5e.zip/node_modules/@babel/helper-string-parser/",\
          "packageDependencies": [\
            ["@babel/helper-string-parser", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-validator-identifier", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-validator-identifier-npm-7.22.5-4536624779-7f0f301134.zip/node_modules/@babel/helper-validator-identifier/",\
          "packageDependencies": [\
            ["@babel/helper-validator-identifier", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-validator-option", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-validator-option-npm-7.22.5-eaf22b24ab-bbeca8a85e.zip/node_modules/@babel/helper-validator-option/",\
          "packageDependencies": [\
            ["@babel/helper-validator-option", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helper-wrap-function", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helper-wrap-function-npm-7.22.5-b061db16f6-a4ba2d7577.zip/node_modules/@babel/helper-wrap-function/",\
          "packageDependencies": [\
            ["@babel/helper-wrap-function", "npm:7.22.5"],\
            ["@babel/helper-function-name", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/helpers", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-helpers-npm-7.22.5-b98bfa9936-a96e785029.zip/node_modules/@babel/helpers/",\
          "packageDependencies": [\
            ["@babel/helpers", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/highlight", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-highlight-npm-7.22.5-3182ccc1fe-f61ae6de6e.zip/node_modules/@babel/highlight/",\
          "packageDependencies": [\
            ["@babel/highlight", "npm:7.22.5"],\
            ["@babel/helper-validator-identifier", "npm:7.22.5"],\
            ["chalk", "npm:2.4.2"],\
            ["js-tokens", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/parser", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-parser-npm-7.22.5-6f8647af64-470ebba516.zip/node_modules/@babel/parser/",\
          "packageDependencies": [\
            ["@babel/parser", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-bugfix-safari-id-destructuring-collision-in-function-expression-npm-7.22.5-215f43a711-1e353a060f.zip/node_modules/@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression/",\
          "packageDependencies": [\
            ["@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-bugfix-safari-id-destructuring-collision-in-function-expression-virtual-2abc82f11e/0/cache/@babel-plugin-bugfix-safari-id-destructuring-collision-in-function-expression-npm-7.22.5-215f43a711-1e353a060f.zip/node_modules/@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression/",\
          "packageDependencies": [\
            ["@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-bugfix-v8-spread-parameters-in-optional-chaining-npm-7.22.5-6dcfb282c0-16e7a5f3bf.zip/node_modules/@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-bugfix-v8-spread-parameters-in-optional-chaining-virtual-b06e24a7d6/0/cache/@babel-plugin-bugfix-v8-spread-parameters-in-optional-chaining-npm-7.22.5-6dcfb282c0-16e7a5f3bf.zip/node_modules/@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-skip-transparent-expression-wrappers", "npm:7.22.5"],\
            ["@babel/plugin-transform-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-proposal-private-property-in-object", [\
        ["npm:7.21.0-placeholder-for-preset-env.2", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-proposal-private-property-in-object-npm-7.21.0-placeholder-for-preset-env.2-eb70026c88-d97745d098.zip/node_modules/@babel/plugin-proposal-private-property-in-object/",\
          "packageDependencies": [\
            ["@babel/plugin-proposal-private-property-in-object", "npm:7.21.0-placeholder-for-preset-env.2"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.21.0-placeholder-for-preset-env.2", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-proposal-private-property-in-object-virtual-d0d777d2ff/0/cache/@babel-plugin-proposal-private-property-in-object-npm-7.21.0-placeholder-for-preset-env.2-eb70026c88-d97745d098.zip/node_modules/@babel/plugin-proposal-private-property-in-object/",\
          "packageDependencies": [\
            ["@babel/plugin-proposal-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.21.0-placeholder-for-preset-env.2"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-proposal-unicode-property-regex", [\
        ["npm:7.18.6", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-proposal-unicode-property-regex-npm-7.18.6-3a6294aa39-a8575ecb7f.zip/node_modules/@babel/plugin-proposal-unicode-property-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-proposal-unicode-property-regex", "npm:7.18.6"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:8772edcf011add7f3d61a7791fc77e2118de7d1d1c35a10c3a879bce1b85065818329f8dd0a7a9dbddf33814fb1b538b3b95b797b710ec072436863f4bed938d#npm:7.18.6", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-proposal-unicode-property-regex-virtual-89d3c596f2/0/cache/@babel-plugin-proposal-unicode-property-regex-npm-7.18.6-3a6294aa39-a8575ecb7f.zip/node_modules/@babel/plugin-proposal-unicode-property-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-proposal-unicode-property-regex", "virtual:8772edcf011add7f3d61a7791fc77e2118de7d1d1c35a10c3a879bce1b85065818329f8dd0a7a9dbddf33814fb1b538b3b95b797b710ec072436863f4bed938d#npm:7.18.6"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-async-generators", [\
        ["npm:7.8.4", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-async-generators-npm-7.8.4-d10cf993c9-7ed1c1d9b9.zip/node_modules/@babel/plugin-syntax-async-generators/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-async-generators", "npm:7.8.4"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-async-generators-virtual-2647eb172f/0/cache/@babel-plugin-syntax-async-generators-npm-7.8.4-d10cf993c9-7ed1c1d9b9.zip/node_modules/@babel/plugin-syntax-async-generators/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-async-generators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-async-generators-virtual-cbb3e63953/0/cache/@babel-plugin-syntax-async-generators-npm-7.8.4-d10cf993c9-7ed1c1d9b9.zip/node_modules/@babel/plugin-syntax-async-generators/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-async-generators", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-bigint", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-bigint-npm-7.8.3-b05d971e6c-3a10849d83.zip/node_modules/@babel/plugin-syntax-bigint/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-bigint", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-bigint-virtual-4e2aa33028/0/cache/@babel-plugin-syntax-bigint-npm-7.8.3-b05d971e6c-3a10849d83.zip/node_modules/@babel/plugin-syntax-bigint/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-bigint", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-class-properties", [\
        ["npm:7.12.13", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-class-properties-npm-7.12.13-002ee9d930-24f34b196d.zip/node_modules/@babel/plugin-syntax-class-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-class-properties", "npm:7.12.13"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.12.13", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-class-properties-virtual-67e12d8008/0/cache/@babel-plugin-syntax-class-properties-npm-7.12.13-002ee9d930-24f34b196d.zip/node_modules/@babel/plugin-syntax-class-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-class-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.12.13"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.12.13", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-class-properties-virtual-93aae934e7/0/cache/@babel-plugin-syntax-class-properties-npm-7.12.13-002ee9d930-24f34b196d.zip/node_modules/@babel/plugin-syntax-class-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-class-properties", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.12.13"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-class-static-block", [\
        ["npm:7.14.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-class-static-block-npm-7.14.5-7bdd0ff1b3-3e80814b5b.zip/node_modules/@babel/plugin-syntax-class-static-block/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-class-static-block", "npm:7.14.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-class-static-block-virtual-f2b32682d0/0/cache/@babel-plugin-syntax-class-static-block-npm-7.14.5-7bdd0ff1b3-3e80814b5b.zip/node_modules/@babel/plugin-syntax-class-static-block/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-class-static-block", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-dynamic-import", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-dynamic-import-npm-7.8.3-fb9ff5634a-ce307af83c.zip/node_modules/@babel/plugin-syntax-dynamic-import/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-dynamic-import", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-dynamic-import-virtual-ce08db6a9c/0/cache/@babel-plugin-syntax-dynamic-import-npm-7.8.3-fb9ff5634a-ce307af83c.zip/node_modules/@babel/plugin-syntax-dynamic-import/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-dynamic-import", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-export-namespace-from", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-export-namespace-from-npm-7.8.3-1747201aa9-85740478be.zip/node_modules/@babel/plugin-syntax-export-namespace-from/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-export-namespace-from", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-export-namespace-from-virtual-b52c1b7e90/0/cache/@babel-plugin-syntax-export-namespace-from-npm-7.8.3-1747201aa9-85740478be.zip/node_modules/@babel/plugin-syntax-export-namespace-from/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-export-namespace-from", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-import-assertions", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-import-assertions-npm-7.22.5-2635aad13d-2b8b5572db.zip/node_modules/@babel/plugin-syntax-import-assertions/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-assertions", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-import-assertions-virtual-20b8b5363a/0/cache/@babel-plugin-syntax-import-assertions-npm-7.22.5-2635aad13d-2b8b5572db.zip/node_modules/@babel/plugin-syntax-import-assertions/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-assertions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-import-attributes", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-import-attributes-npm-7.22.5-b93e4950ce-197b3c5ea2.zip/node_modules/@babel/plugin-syntax-import-attributes/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-attributes", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-import-attributes-virtual-c17499656d/0/cache/@babel-plugin-syntax-import-attributes-npm-7.22.5-b93e4950ce-197b3c5ea2.zip/node_modules/@babel/plugin-syntax-import-attributes/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-attributes", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-import-meta", [\
        ["npm:7.10.4", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-import-meta-npm-7.10.4-4a0a0158bc-166ac1125d.zip/node_modules/@babel/plugin-syntax-import-meta/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-meta", "npm:7.10.4"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-import-meta-virtual-e1cccc2e3d/0/cache/@babel-plugin-syntax-import-meta-npm-7.10.4-4a0a0158bc-166ac1125d.zip/node_modules/@babel/plugin-syntax-import-meta/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-meta", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-import-meta-virtual-f07913a9b0/0/cache/@babel-plugin-syntax-import-meta-npm-7.10.4-4a0a0158bc-166ac1125d.zip/node_modules/@babel/plugin-syntax-import-meta/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-import-meta", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-json-strings", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-json-strings-npm-7.8.3-6dc7848179-bf5aea1f31.zip/node_modules/@babel/plugin-syntax-json-strings/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-json-strings", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-json-strings-virtual-88a9695fb6/0/cache/@babel-plugin-syntax-json-strings-npm-7.8.3-6dc7848179-bf5aea1f31.zip/node_modules/@babel/plugin-syntax-json-strings/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-json-strings", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-json-strings-virtual-a1c4de6447/0/cache/@babel-plugin-syntax-json-strings-npm-7.8.3-6dc7848179-bf5aea1f31.zip/node_modules/@babel/plugin-syntax-json-strings/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-json-strings", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-logical-assignment-operators", [\
        ["npm:7.10.4", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-logical-assignment-operators-npm-7.10.4-72ae00fdf6-aff3357703.zip/node_modules/@babel/plugin-syntax-logical-assignment-operators/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-logical-assignment-operators", "npm:7.10.4"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-logical-assignment-operators-virtual-858d5027b6/0/cache/@babel-plugin-syntax-logical-assignment-operators-npm-7.10.4-72ae00fdf6-aff3357703.zip/node_modules/@babel/plugin-syntax-logical-assignment-operators/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-logical-assignment-operators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-logical-assignment-operators-virtual-a2e5b619ef/0/cache/@babel-plugin-syntax-logical-assignment-operators-npm-7.10.4-72ae00fdf6-aff3357703.zip/node_modules/@babel/plugin-syntax-logical-assignment-operators/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-logical-assignment-operators", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-nullish-coalescing-operator", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-nullish-coalescing-operator-npm-7.8.3-8a723173b5-87aca49189.zip/node_modules/@babel/plugin-syntax-nullish-coalescing-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-nullish-coalescing-operator", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-nullish-coalescing-operator-virtual-a6d320c70f/0/cache/@babel-plugin-syntax-nullish-coalescing-operator-npm-7.8.3-8a723173b5-87aca49189.zip/node_modules/@babel/plugin-syntax-nullish-coalescing-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-nullish-coalescing-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-nullish-coalescing-operator-virtual-c02562a7c4/0/cache/@babel-plugin-syntax-nullish-coalescing-operator-npm-7.8.3-8a723173b5-87aca49189.zip/node_modules/@babel/plugin-syntax-nullish-coalescing-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-nullish-coalescing-operator", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-numeric-separator", [\
        ["npm:7.10.4", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-numeric-separator-npm-7.10.4-81444be605-01ec5547bd.zip/node_modules/@babel/plugin-syntax-numeric-separator/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-numeric-separator", "npm:7.10.4"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-numeric-separator-virtual-18f8fe3473/0/cache/@babel-plugin-syntax-numeric-separator-npm-7.10.4-81444be605-01ec5547bd.zip/node_modules/@babel/plugin-syntax-numeric-separator/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-numeric-separator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-numeric-separator-virtual-d9526469fa/0/cache/@babel-plugin-syntax-numeric-separator-npm-7.10.4-81444be605-01ec5547bd.zip/node_modules/@babel/plugin-syntax-numeric-separator/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-numeric-separator", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-object-rest-spread", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-object-rest-spread-npm-7.8.3-60bd05b6ae-fddcf581a5.zip/node_modules/@babel/plugin-syntax-object-rest-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-object-rest-spread", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-object-rest-spread-virtual-2621e7bc98/0/cache/@babel-plugin-syntax-object-rest-spread-npm-7.8.3-60bd05b6ae-fddcf581a5.zip/node_modules/@babel/plugin-syntax-object-rest-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-object-rest-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-object-rest-spread-virtual-60dbe747e0/0/cache/@babel-plugin-syntax-object-rest-spread-npm-7.8.3-60bd05b6ae-fddcf581a5.zip/node_modules/@babel/plugin-syntax-object-rest-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-object-rest-spread", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-optional-catch-binding", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-optional-catch-binding-npm-7.8.3-ce337427d8-910d90e72b.zip/node_modules/@babel/plugin-syntax-optional-catch-binding/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-optional-catch-binding", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-optional-catch-binding-virtual-535287b2d2/0/cache/@babel-plugin-syntax-optional-catch-binding-npm-7.8.3-ce337427d8-910d90e72b.zip/node_modules/@babel/plugin-syntax-optional-catch-binding/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-optional-catch-binding", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-optional-catch-binding-virtual-9add8a6c5e/0/cache/@babel-plugin-syntax-optional-catch-binding-npm-7.8.3-ce337427d8-910d90e72b.zip/node_modules/@babel/plugin-syntax-optional-catch-binding/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-optional-catch-binding", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-optional-chaining", [\
        ["npm:7.8.3", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-optional-chaining-npm-7.8.3-f3f3c79579-eef94d53a1.zip/node_modules/@babel/plugin-syntax-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-optional-chaining", "npm:7.8.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-optional-chaining-virtual-9568794e54/0/cache/@babel-plugin-syntax-optional-chaining-npm-7.8.3-f3f3c79579-eef94d53a1.zip/node_modules/@babel/plugin-syntax-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-optional-chaining-virtual-a49181bfc9/0/cache/@babel-plugin-syntax-optional-chaining-npm-7.8.3-f3f3c79579-eef94d53a1.zip/node_modules/@babel/plugin-syntax-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-optional-chaining", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-private-property-in-object", [\
        ["npm:7.14.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-private-property-in-object-npm-7.14.5-ee837fdbb2-b317174783.zip/node_modules/@babel/plugin-syntax-private-property-in-object/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-private-property-in-object", "npm:7.14.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-private-property-in-object-virtual-21995d1a1b/0/cache/@babel-plugin-syntax-private-property-in-object-npm-7.14.5-ee837fdbb2-b317174783.zip/node_modules/@babel/plugin-syntax-private-property-in-object/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-top-level-await", [\
        ["npm:7.14.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-top-level-await-npm-7.14.5-60a0a2e83b-bbd1a56b09.zip/node_modules/@babel/plugin-syntax-top-level-await/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-top-level-await", "npm:7.14.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-top-level-await-virtual-cb7d8714fc/0/cache/@babel-plugin-syntax-top-level-await-npm-7.14.5-60a0a2e83b-bbd1a56b09.zip/node_modules/@babel/plugin-syntax-top-level-await/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-top-level-await", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.14.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-top-level-await-virtual-63388d59b6/0/cache/@babel-plugin-syntax-top-level-await-npm-7.14.5-60a0a2e83b-bbd1a56b09.zip/node_modules/@babel/plugin-syntax-top-level-await/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-top-level-await", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.14.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-syntax-unicode-sets-regex", [\
        ["npm:7.18.6", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-syntax-unicode-sets-regex-npm-7.18.6-b618a36bfd-a651d700fe.zip/node_modules/@babel/plugin-syntax-unicode-sets-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-unicode-sets-regex", "npm:7.18.6"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.18.6", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-syntax-unicode-sets-regex-virtual-0a7eefb2c8/0/cache/@babel-plugin-syntax-unicode-sets-regex-npm-7.18.6-b618a36bfd-a651d700fe.zip/node_modules/@babel/plugin-syntax-unicode-sets-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-syntax-unicode-sets-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.18.6"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-arrow-functions", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-arrow-functions-npm-7.22.5-af136ec392-35abb6c570.zip/node_modules/@babel/plugin-transform-arrow-functions/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-arrow-functions", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-arrow-functions-virtual-bce94bf89e/0/cache/@babel-plugin-transform-arrow-functions-npm-7.22.5-af136ec392-35abb6c570.zip/node_modules/@babel/plugin-transform-arrow-functions/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-arrow-functions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-async-generator-functions", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-async-generator-functions-npm-7.22.5-86dd993177-32890b69ec.zip/node_modules/@babel/plugin-transform-async-generator-functions/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-async-generator-functions", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-async-generator-functions-virtual-4dc4a45695/0/cache/@babel-plugin-transform-async-generator-functions-npm-7.22.5-86dd993177-32890b69ec.zip/node_modules/@babel/plugin-transform-async-generator-functions/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-async-generator-functions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-remap-async-to-generator", "virtual:4dc4a45695acd3398bc920a43e1730a2258dac068831cc2d5f4ffecac49439a22d6a5f6d67ce27fcfcfda1d0fc81f791dca2be2a0eaed9e3ebbb0e0432a3b529#npm:7.22.5"],\
            ["@babel/plugin-syntax-async-generators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.4"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-async-to-generator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-async-to-generator-npm-7.22.5-f69f15a70f-b95f23f99d.zip/node_modules/@babel/plugin-transform-async-to-generator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-async-to-generator", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-async-to-generator-virtual-6244d93fd0/0/cache/@babel-plugin-transform-async-to-generator-npm-7.22.5-f69f15a70f-b95f23f99d.zip/node_modules/@babel/plugin-transform-async-to-generator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-async-to-generator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-module-imports", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-remap-async-to-generator", "virtual:4dc4a45695acd3398bc920a43e1730a2258dac068831cc2d5f4ffecac49439a22d6a5f6d67ce27fcfcfda1d0fc81f791dca2be2a0eaed9e3ebbb0e0432a3b529#npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-block-scoped-functions", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-block-scoped-functions-npm-7.22.5-19b39eb7ee-416b134185.zip/node_modules/@babel/plugin-transform-block-scoped-functions/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-block-scoped-functions", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-block-scoped-functions-virtual-d861d770ca/0/cache/@babel-plugin-transform-block-scoped-functions-npm-7.22.5-19b39eb7ee-416b134185.zip/node_modules/@babel/plugin-transform-block-scoped-functions/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-block-scoped-functions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-block-scoping", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-block-scoping-npm-7.22.5-693be2c793-26987002cf.zip/node_modules/@babel/plugin-transform-block-scoping/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-block-scoping", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-block-scoping-virtual-3a7c8e554b/0/cache/@babel-plugin-transform-block-scoping-npm-7.22.5-693be2c793-26987002cf.zip/node_modules/@babel/plugin-transform-block-scoping/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-block-scoping", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-class-properties", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-class-properties-npm-7.22.5-c3b1b6b165-b830152dfc.zip/node_modules/@babel/plugin-transform-class-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-class-properties", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-class-properties-virtual-1f163d5394/0/cache/@babel-plugin-transform-class-properties-npm-7.22.5-c3b1b6b165-b830152dfc.zip/node_modules/@babel/plugin-transform-class-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-class-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-class-features-plugin", "virtual:1f163d5394432f433356473aaacde47e38d672c0ae0f3404c834b6a32a6fbb176891c5c5f3863489a008766bcf4583f573722b959e88c8478958d7633507e0cd#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-class-static-block", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-class-static-block-npm-7.22.5-8cb8058c1d-bc48b92dba.zip/node_modules/@babel/plugin-transform-class-static-block/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-class-static-block", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-class-static-block-virtual-e24224c0f0/0/cache/@babel-plugin-transform-class-static-block-npm-7.22.5-8cb8058c1d-bc48b92dba.zip/node_modules/@babel/plugin-transform-class-static-block/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-class-static-block", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-class-features-plugin", "virtual:1f163d5394432f433356473aaacde47e38d672c0ae0f3404c834b6a32a6fbb176891c5c5f3863489a008766bcf4583f573722b959e88c8478958d7633507e0cd#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-class-static-block", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-classes", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-classes-npm-7.22.5-dcc93f88df-124b1b7918.zip/node_modules/@babel/plugin-transform-classes/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-classes", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-classes-virtual-b2b75e5c08/0/cache/@babel-plugin-transform-classes-npm-7.22.5-dcc93f88df-124b1b7918.zip/node_modules/@babel/plugin-transform-classes/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-classes", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-annotate-as-pure", "npm:7.22.5"],\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-function-name", "npm:7.22.5"],\
            ["@babel/helper-optimise-call-expression", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-replace-supers", "npm:7.22.5"],\
            ["@babel/helper-split-export-declaration", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["globals", "npm:11.12.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-computed-properties", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-computed-properties-npm-7.22.5-c3ec4766ab-c2a77a0f94.zip/node_modules/@babel/plugin-transform-computed-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-computed-properties", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-computed-properties-virtual-ebd87e3e7b/0/cache/@babel-plugin-transform-computed-properties-npm-7.22.5-c3ec4766ab-c2a77a0f94.zip/node_modules/@babel/plugin-transform-computed-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-computed-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-destructuring", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-destructuring-npm-7.22.5-e9364713b6-76f6ea2aee.zip/node_modules/@babel/plugin-transform-destructuring/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-destructuring", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-destructuring-virtual-145a2ffeb9/0/cache/@babel-plugin-transform-destructuring-npm-7.22.5-e9364713b6-76f6ea2aee.zip/node_modules/@babel/plugin-transform-destructuring/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-destructuring", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-dotall-regex", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-dotall-regex-npm-7.22.5-0255db6e7c-409b658d11.zip/node_modules/@babel/plugin-transform-dotall-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-dotall-regex", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-dotall-regex-virtual-be4bff35fd/0/cache/@babel-plugin-transform-dotall-regex-npm-7.22.5-0255db6e7c-409b658d11.zip/node_modules/@babel/plugin-transform-dotall-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-dotall-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-duplicate-keys", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-duplicate-keys-npm-7.22.5-c4136fee39-bb1280fbab.zip/node_modules/@babel/plugin-transform-duplicate-keys/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-duplicate-keys", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-duplicate-keys-virtual-01b23e186c/0/cache/@babel-plugin-transform-duplicate-keys-npm-7.22.5-c4136fee39-bb1280fbab.zip/node_modules/@babel/plugin-transform-duplicate-keys/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-duplicate-keys", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-dynamic-import", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-dynamic-import-npm-7.22.5-5245ff4c0c-186a6d59f3.zip/node_modules/@babel/plugin-transform-dynamic-import/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-dynamic-import", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-dynamic-import-virtual-c7a90d5918/0/cache/@babel-plugin-transform-dynamic-import-npm-7.22.5-5245ff4c0c-186a6d59f3.zip/node_modules/@babel/plugin-transform-dynamic-import/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-dynamic-import", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-dynamic-import", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-exponentiation-operator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-exponentiation-operator-npm-7.22.5-c82f2c6e1d-f2d660c1b1.zip/node_modules/@babel/plugin-transform-exponentiation-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-exponentiation-operator", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-exponentiation-operator-virtual-d75f077a51/0/cache/@babel-plugin-transform-exponentiation-operator-npm-7.22.5-c82f2c6e1d-f2d660c1b1.zip/node_modules/@babel/plugin-transform-exponentiation-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-exponentiation-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-builder-binary-assignment-operator-visitor", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-export-namespace-from", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-export-namespace-from-npm-7.22.5-822b6dab82-3d197b7887.zip/node_modules/@babel/plugin-transform-export-namespace-from/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-export-namespace-from", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-export-namespace-from-virtual-60dc4a378c/0/cache/@babel-plugin-transform-export-namespace-from-npm-7.22.5-822b6dab82-3d197b7887.zip/node_modules/@babel/plugin-transform-export-namespace-from/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-export-namespace-from", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-export-namespace-from", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-for-of", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-for-of-npm-7.22.5-7c6623b6cb-d7b8d4db01.zip/node_modules/@babel/plugin-transform-for-of/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-for-of", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-for-of-virtual-ab70257463/0/cache/@babel-plugin-transform-for-of-npm-7.22.5-7c6623b6cb-d7b8d4db01.zip/node_modules/@babel/plugin-transform-for-of/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-for-of", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-function-name", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-function-name-npm-7.22.5-e4ae437abe-cff3b87635.zip/node_modules/@babel/plugin-transform-function-name/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-function-name", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-function-name-virtual-6fe6cbc3e8/0/cache/@babel-plugin-transform-function-name-npm-7.22.5-e4ae437abe-cff3b87635.zip/node_modules/@babel/plugin-transform-function-name/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-function-name", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/helper-function-name", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-json-strings", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-json-strings-npm-7.22.5-570f5d94c4-4e00b90248.zip/node_modules/@babel/plugin-transform-json-strings/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-json-strings", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-json-strings-virtual-f74fd0fa61/0/cache/@babel-plugin-transform-json-strings-npm-7.22.5-570f5d94c4-4e00b90248.zip/node_modules/@babel/plugin-transform-json-strings/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-json-strings", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-json-strings", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-literals", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-literals-npm-7.22.5-a6ebdb9a3f-ec37cc2ffb.zip/node_modules/@babel/plugin-transform-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-literals", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-literals-virtual-cc83105803/0/cache/@babel-plugin-transform-literals-npm-7.22.5-a6ebdb9a3f-ec37cc2ffb.zip/node_modules/@babel/plugin-transform-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-logical-assignment-operators", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-logical-assignment-operators-npm-7.22.5-e2bd9d5d66-18748e953c.zip/node_modules/@babel/plugin-transform-logical-assignment-operators/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-logical-assignment-operators", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-logical-assignment-operators-virtual-4c7042fcb8/0/cache/@babel-plugin-transform-logical-assignment-operators-npm-7.22.5-e2bd9d5d66-18748e953c.zip/node_modules/@babel/plugin-transform-logical-assignment-operators/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-logical-assignment-operators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-logical-assignment-operators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-member-expression-literals", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-member-expression-literals-npm-7.22.5-00b2f7d310-ec4b0e0791.zip/node_modules/@babel/plugin-transform-member-expression-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-member-expression-literals", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-member-expression-literals-virtual-83d6a61394/0/cache/@babel-plugin-transform-member-expression-literals-npm-7.22.5-00b2f7d310-ec4b0e0791.zip/node_modules/@babel/plugin-transform-member-expression-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-member-expression-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-modules-amd", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-modules-amd-npm-7.22.5-d4afd920af-7da4c4ebbb.zip/node_modules/@babel/plugin-transform-modules-amd/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-amd", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-modules-amd-virtual-1d1a4c8818/0/cache/@babel-plugin-transform-modules-amd-npm-7.22.5-d4afd920af-7da4c4ebbb.zip/node_modules/@babel/plugin-transform-modules-amd/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-amd", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-module-transforms", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-modules-commonjs", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-modules-commonjs-npm-7.22.5-9fb6bd76fa-2067aca8f6.zip/node_modules/@babel/plugin-transform-modules-commonjs/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-commonjs", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-modules-commonjs-virtual-a410d8f634/0/cache/@babel-plugin-transform-modules-commonjs-npm-7.22.5-9fb6bd76fa-2067aca8f6.zip/node_modules/@babel/plugin-transform-modules-commonjs/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-commonjs", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-module-transforms", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-simple-access", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-modules-systemjs", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-modules-systemjs-npm-7.22.5-f10f9d7e6d-04f4178589.zip/node_modules/@babel/plugin-transform-modules-systemjs/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-systemjs", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-modules-systemjs-virtual-5dbae32e9d/0/cache/@babel-plugin-transform-modules-systemjs-npm-7.22.5-f10f9d7e6d-04f4178589.zip/node_modules/@babel/plugin-transform-modules-systemjs/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-systemjs", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-hoist-variables", "npm:7.22.5"],\
            ["@babel/helper-module-transforms", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-validator-identifier", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-modules-umd", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-modules-umd-npm-7.22.5-332024cbaa-46622834c5.zip/node_modules/@babel/plugin-transform-modules-umd/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-umd", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-modules-umd-virtual-e912f0d51d/0/cache/@babel-plugin-transform-modules-umd-npm-7.22.5-332024cbaa-46622834c5.zip/node_modules/@babel/plugin-transform-modules-umd/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-modules-umd", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-module-transforms", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-named-capturing-groups-regex", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-named-capturing-groups-regex-npm-7.22.5-b9360fd04d-3ee564ddee.zip/node_modules/@babel/plugin-transform-named-capturing-groups-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-named-capturing-groups-regex", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-named-capturing-groups-regex-virtual-1bc7df46e6/0/cache/@babel-plugin-transform-named-capturing-groups-regex-npm-7.22.5-b9360fd04d-3ee564ddee.zip/node_modules/@babel/plugin-transform-named-capturing-groups-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-named-capturing-groups-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-new-target", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-new-target-npm-7.22.5-113516dd3d-6b72112773.zip/node_modules/@babel/plugin-transform-new-target/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-new-target", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-new-target-virtual-dccdb171a6/0/cache/@babel-plugin-transform-new-target-npm-7.22.5-113516dd3d-6b72112773.zip/node_modules/@babel/plugin-transform-new-target/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-new-target", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-nullish-coalescing-operator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-nullish-coalescing-operator-npm-7.22.5-f071b4a3c0-e6a059169d.zip/node_modules/@babel/plugin-transform-nullish-coalescing-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-nullish-coalescing-operator", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-nullish-coalescing-operator-virtual-e8b2cde304/0/cache/@babel-plugin-transform-nullish-coalescing-operator-npm-7.22.5-f071b4a3c0-e6a059169d.zip/node_modules/@babel/plugin-transform-nullish-coalescing-operator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-nullish-coalescing-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-nullish-coalescing-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-numeric-separator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-numeric-separator-npm-7.22.5-301bcf6067-9e7837d4ea.zip/node_modules/@babel/plugin-transform-numeric-separator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-numeric-separator", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-numeric-separator-virtual-392c9e3565/0/cache/@babel-plugin-transform-numeric-separator-npm-7.22.5-301bcf6067-9e7837d4ea.zip/node_modules/@babel/plugin-transform-numeric-separator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-numeric-separator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-numeric-separator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-object-rest-spread", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-object-rest-spread-npm-7.22.5-20b746e649-3b5e091f0d.zip/node_modules/@babel/plugin-transform-object-rest-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-object-rest-spread", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-object-rest-spread-virtual-e732ea3288/0/cache/@babel-plugin-transform-object-rest-spread-npm-7.22.5-20b746e649-3b5e091f0d.zip/node_modules/@babel/plugin-transform-object-rest-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-object-rest-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/compat-data", "npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-object-rest-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-transform-parameters", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-object-super", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-object-super-npm-7.22.5-6c247bd1c8-b71887877d.zip/node_modules/@babel/plugin-transform-object-super/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-object-super", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-object-super-virtual-86d49be3a1/0/cache/@babel-plugin-transform-object-super-npm-7.22.5-6c247bd1c8-b71887877d.zip/node_modules/@babel/plugin-transform-object-super/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-object-super", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-replace-supers", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-optional-catch-binding", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-optional-catch-binding-npm-7.22.5-78456ccc55-b0e8b4233f.zip/node_modules/@babel/plugin-transform-optional-catch-binding/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-optional-catch-binding", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-optional-catch-binding-virtual-dd28ac9199/0/cache/@babel-plugin-transform-optional-catch-binding-npm-7.22.5-78456ccc55-b0e8b4233f.zip/node_modules/@babel/plugin-transform-optional-catch-binding/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-optional-catch-binding", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-optional-catch-binding", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-optional-chaining", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-optional-chaining-npm-7.22.5-d95aab69dd-57b9c05fb2.zip/node_modules/@babel/plugin-transform-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-optional-chaining", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-optional-chaining-virtual-b9b70d6ca6/0/cache/@babel-plugin-transform-optional-chaining-npm-7.22.5-d95aab69dd-57b9c05fb2.zip/node_modules/@babel/plugin-transform-optional-chaining/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-skip-transparent-expression-wrappers", "npm:7.22.5"],\
            ["@babel/plugin-syntax-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-parameters", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-parameters-npm-7.22.5-0388236044-b44f89cf97.zip/node_modules/@babel/plugin-transform-parameters/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-parameters", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-parameters-virtual-9765e394f4/0/cache/@babel-plugin-transform-parameters-npm-7.22.5-0388236044-b44f89cf97.zip/node_modules/@babel/plugin-transform-parameters/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-parameters", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-private-methods", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-private-methods-npm-7.22.5-0015afb8a1-321479b4fc.zip/node_modules/@babel/plugin-transform-private-methods/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-private-methods", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-private-methods-virtual-4d7b60711d/0/cache/@babel-plugin-transform-private-methods-npm-7.22.5-0015afb8a1-321479b4fc.zip/node_modules/@babel/plugin-transform-private-methods/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-private-methods", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-class-features-plugin", "virtual:1f163d5394432f433356473aaacde47e38d672c0ae0f3404c834b6a32a6fbb176891c5c5f3863489a008766bcf4583f573722b959e88c8478958d7633507e0cd#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-private-property-in-object", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-private-property-in-object-npm-7.22.5-2ea542a7ca-9ac019fb27.zip/node_modules/@babel/plugin-transform-private-property-in-object/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-private-property-in-object", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-private-property-in-object-virtual-d5aead33a5/0/cache/@babel-plugin-transform-private-property-in-object-npm-7.22.5-2ea542a7ca-9ac019fb27.zip/node_modules/@babel/plugin-transform-private-property-in-object/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-annotate-as-pure", "npm:7.22.5"],\
            ["@babel/helper-create-class-features-plugin", "virtual:1f163d5394432f433356473aaacde47e38d672c0ae0f3404c834b6a32a6fbb176891c5c5f3863489a008766bcf4583f573722b959e88c8478958d7633507e0cd#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-syntax-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-property-literals", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-property-literals-npm-7.22.5-5b47e3b787-796176a317.zip/node_modules/@babel/plugin-transform-property-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-property-literals", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-property-literals-virtual-6546a64d5b/0/cache/@babel-plugin-transform-property-literals-npm-7.22.5-5b47e3b787-796176a317.zip/node_modules/@babel/plugin-transform-property-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-property-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-regenerator", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-regenerator-npm-7.22.5-5a4c679227-f7c5ca5151.zip/node_modules/@babel/plugin-transform-regenerator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-regenerator", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-regenerator-virtual-e96c789c8f/0/cache/@babel-plugin-transform-regenerator-npm-7.22.5-5a4c679227-f7c5ca5151.zip/node_modules/@babel/plugin-transform-regenerator/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-regenerator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["regenerator-transform", "npm:0.15.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-reserved-words", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-reserved-words-npm-7.22.5-a845b3b487-3ffd7dbc42.zip/node_modules/@babel/plugin-transform-reserved-words/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-reserved-words", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-reserved-words-virtual-09144c9012/0/cache/@babel-plugin-transform-reserved-words-npm-7.22.5-a845b3b487-3ffd7dbc42.zip/node_modules/@babel/plugin-transform-reserved-words/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-reserved-words", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-shorthand-properties", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-shorthand-properties-npm-7.22.5-362022b06f-a5ac902c56.zip/node_modules/@babel/plugin-transform-shorthand-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-shorthand-properties", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-shorthand-properties-virtual-88fdf3d502/0/cache/@babel-plugin-transform-shorthand-properties-npm-7.22.5-362022b06f-a5ac902c56.zip/node_modules/@babel/plugin-transform-shorthand-properties/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-shorthand-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-spread", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-spread-npm-7.22.5-61ed9bc888-5587f0deb6.zip/node_modules/@babel/plugin-transform-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-spread", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-spread-virtual-d7f77b9283/0/cache/@babel-plugin-transform-spread-npm-7.22.5-61ed9bc888-5587f0deb6.zip/node_modules/@babel/plugin-transform-spread/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-skip-transparent-expression-wrappers", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-sticky-regex", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-sticky-regex-npm-7.22.5-c695959c0a-63b2c575e3.zip/node_modules/@babel/plugin-transform-sticky-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-sticky-regex", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-sticky-regex-virtual-440ee88c38/0/cache/@babel-plugin-transform-sticky-regex-npm-7.22.5-c695959c0a-63b2c575e3.zip/node_modules/@babel/plugin-transform-sticky-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-sticky-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-template-literals", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-template-literals-npm-7.22.5-1a4b253e48-27e9bb0306.zip/node_modules/@babel/plugin-transform-template-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-template-literals", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-template-literals-virtual-33bfb155bc/0/cache/@babel-plugin-transform-template-literals-npm-7.22.5-1a4b253e48-27e9bb0306.zip/node_modules/@babel/plugin-transform-template-literals/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-template-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-typeof-symbol", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-typeof-symbol-npm-7.22.5-1ecab2dc9d-82a53a63ff.zip/node_modules/@babel/plugin-transform-typeof-symbol/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-typeof-symbol", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-typeof-symbol-virtual-8688a75f76/0/cache/@babel-plugin-transform-typeof-symbol-npm-7.22.5-1ecab2dc9d-82a53a63ff.zip/node_modules/@babel/plugin-transform-typeof-symbol/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-typeof-symbol", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-unicode-escapes", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-unicode-escapes-npm-7.22.5-db496ef1f1-da5e85ab3b.zip/node_modules/@babel/plugin-transform-unicode-escapes/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-escapes", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-unicode-escapes-virtual-ab7ca7d483/0/cache/@babel-plugin-transform-unicode-escapes-npm-7.22.5-db496ef1f1-da5e85ab3b.zip/node_modules/@babel/plugin-transform-unicode-escapes/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-escapes", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-unicode-property-regex", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-unicode-property-regex-npm-7.22.5-3c77f8e952-2495e5f663.zip/node_modules/@babel/plugin-transform-unicode-property-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-property-regex", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-unicode-property-regex-virtual-2b565e064c/0/cache/@babel-plugin-transform-unicode-property-regex-npm-7.22.5-3c77f8e952-2495e5f663.zip/node_modules/@babel/plugin-transform-unicode-property-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-property-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-unicode-regex", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-unicode-regex-npm-7.22.5-26e8f66a55-6b5d1404c8.zip/node_modules/@babel/plugin-transform-unicode-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-regex", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-unicode-regex-virtual-b51cbf3d1d/0/cache/@babel-plugin-transform-unicode-regex-npm-7.22.5-26e8f66a55-6b5d1404c8.zip/node_modules/@babel/plugin-transform-unicode-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/plugin-transform-unicode-sets-regex", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-plugin-transform-unicode-sets-regex-npm-7.22.5-d22925edab-c042070f98.zip/node_modules/@babel/plugin-transform-unicode-sets-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-sets-regex", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-plugin-transform-unicode-sets-regex-virtual-073b0c8cc5/0/cache/@babel-plugin-transform-unicode-sets-regex-npm-7.22.5-d22925edab-c042070f98.zip/node_modules/@babel/plugin-transform-unicode-sets-regex/",\
          "packageDependencies": [\
            ["@babel/plugin-transform-unicode-sets-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-create-regexp-features-plugin", "virtual:0a7eefb2c890c57976ce19a797e8dd1ba1e5f1f2b650b676fc5b3383e6e335e18c6a7dca5432f1a311ccc83a83c6521350ea3bc5ad48f544a5820b3adadfa825#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/preset-env", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-preset-env-npm-7.22.5-42fcccaa26-6d9d09010a.zip/node_modules/@babel/preset-env/",\
          "packageDependencies": [\
            ["@babel/preset-env", "npm:7.22.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:7.22.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-preset-env-virtual-31c9cd7546/0/cache/@babel-preset-env-npm-7.22.5-42fcccaa26-6d9d09010a.zip/node_modules/@babel/preset-env/",\
          "packageDependencies": [\
            ["@babel/preset-env", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:7.22.5"],\
            ["@babel/compat-data", "npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-compilation-targets", "virtual:d75e9310803f17bcf174ec658b2764533ffdf6562dfad24bf5d9af62c1b3a53568d73d0992aaed9662426027c436e5323b31d99958c77cde0686954baa8ae0eb#npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/helper-validator-option", "npm:7.22.5"],\
            ["@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-proposal-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.21.0-placeholder-for-preset-env.2"],\
            ["@babel/plugin-syntax-async-generators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.4"],\
            ["@babel/plugin-syntax-class-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.12.13"],\
            ["@babel/plugin-syntax-class-static-block", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@babel/plugin-syntax-dynamic-import", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-export-namespace-from", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-import-assertions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-syntax-import-attributes", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-syntax-import-meta", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@babel/plugin-syntax-json-strings", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-logical-assignment-operators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@babel/plugin-syntax-nullish-coalescing-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-numeric-separator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.10.4"],\
            ["@babel/plugin-syntax-object-rest-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-optional-catch-binding", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.8.3"],\
            ["@babel/plugin-syntax-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@babel/plugin-syntax-top-level-await", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.14.5"],\
            ["@babel/plugin-syntax-unicode-sets-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.18.6"],\
            ["@babel/plugin-transform-arrow-functions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-async-generator-functions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-async-to-generator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-block-scoped-functions", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-block-scoping", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-class-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-class-static-block", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-classes", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-computed-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-destructuring", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-dotall-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-duplicate-keys", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-dynamic-import", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-exponentiation-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-export-namespace-from", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-for-of", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-function-name", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-json-strings", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-logical-assignment-operators", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-member-expression-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-modules-amd", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-modules-commonjs", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-modules-systemjs", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-modules-umd", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-named-capturing-groups-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-new-target", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-nullish-coalescing-operator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-numeric-separator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-object-rest-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-object-super", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-optional-catch-binding", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-optional-chaining", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-parameters", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-private-methods", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-private-property-in-object", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-property-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-regenerator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-reserved-words", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-shorthand-properties", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-spread", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-sticky-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-template-literals", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-typeof-symbol", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-unicode-escapes", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-unicode-property-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-unicode-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/plugin-transform-unicode-sets-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/preset-modules", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.1.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["babel-plugin-polyfill-corejs2", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.4.3"],\
            ["babel-plugin-polyfill-corejs3", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.8.1"],\
            ["babel-plugin-polyfill-regenerator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.5.0"],\
            ["core-js-compat", "npm:3.31.0"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/preset-modules", [\
        ["npm:0.1.5", {\
          "packageLocation": "./.yarn/cache/@babel-preset-modules-npm-0.1.5-15ffcd64c2-8430e0e9e9.zip/node_modules/@babel/preset-modules/",\
          "packageDependencies": [\
            ["@babel/preset-modules", "npm:0.1.5"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.1.5", {\
          "packageLocation": "./.yarn/__virtual__/@babel-preset-modules-virtual-8772edcf01/0/cache/@babel-preset-modules-npm-0.1.5-15ffcd64c2-8430e0e9e9.zip/node_modules/@babel/preset-modules/",\
          "packageDependencies": [\
            ["@babel/preset-modules", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.1.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@babel/plugin-proposal-unicode-property-regex", "virtual:8772edcf011add7f3d61a7791fc77e2118de7d1d1c35a10c3a879bce1b85065818329f8dd0a7a9dbddf33814fb1b538b3b95b797b710ec072436863f4bed938d#npm:7.18.6"],\
            ["@babel/plugin-transform-dotall-regex", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@types/babel__core", null],\
            ["esutils", "npm:2.0.3"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/regjsgen", [\
        ["npm:0.8.0", {\
          "packageLocation": "./.yarn/cache/@babel-regjsgen-npm-0.8.0-b0fbdbf644-89c338fee7.zip/node_modules/@babel/regjsgen/",\
          "packageDependencies": [\
            ["@babel/regjsgen", "npm:0.8.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/runtime", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-runtime-npm-7.22.5-0a6711d04c-12a50b7de2.zip/node_modules/@babel/runtime/",\
          "packageDependencies": [\
            ["@babel/runtime", "npm:7.22.5"],\
            ["regenerator-runtime", "npm:0.13.11"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/template", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-template-npm-7.22.5-358c44dc9d-c574641016.zip/node_modules/@babel/template/",\
          "packageDependencies": [\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/code-frame", "npm:7.22.5"],\
            ["@babel/parser", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/traverse", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-traverse-npm-7.22.5-071d471ccd-560931422d.zip/node_modules/@babel/traverse/",\
          "packageDependencies": [\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@babel/code-frame", "npm:7.22.5"],\
            ["@babel/generator", "npm:7.22.5"],\
            ["@babel/helper-environment-visitor", "npm:7.22.5"],\
            ["@babel/helper-function-name", "npm:7.22.5"],\
            ["@babel/helper-hoist-variables", "npm:7.22.5"],\
            ["@babel/helper-split-export-declaration", "npm:7.22.5"],\
            ["@babel/parser", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["globals", "npm:11.12.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@babel/types", [\
        ["npm:7.22.5", {\
          "packageLocation": "./.yarn/cache/@babel-types-npm-7.22.5-d1e4264bef-c13a9c1dc7.zip/node_modules/@babel/types/",\
          "packageDependencies": [\
            ["@babel/types", "npm:7.22.5"],\
            ["@babel/helper-string-parser", "npm:7.22.5"],\
            ["@babel/helper-validator-identifier", "npm:7.22.5"],\
            ["to-fast-properties", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@bcoe/v8-coverage", [\
        ["npm:0.2.3", {\
          "packageLocation": "./.yarn/cache/@bcoe-v8-coverage-npm-0.2.3-9e27b3c57e-850f930553.zip/node_modules/@bcoe/v8-coverage/",\
          "packageDependencies": [\
            ["@bcoe/v8-coverage", "npm:0.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@cnakazawa/watch", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/@cnakazawa-watch-npm-1.0.4-ee43493884-88f395ca0a.zip/node_modules/@cnakazawa/watch/",\
          "packageDependencies": [\
            ["@cnakazawa/watch", "npm:1.0.4"],\
            ["exec-sh", "npm:0.3.6"],\
            ["minimist", "npm:1.2.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@discoveryjs/json-ext", [\
        ["npm:0.5.7", {\
          "packageLocation": "./.yarn/cache/@discoveryjs-json-ext-npm-0.5.7-fe04af1f31-2176d301cc.zip/node_modules/@discoveryjs/json-ext/",\
          "packageDependencies": [\
            ["@discoveryjs/json-ext", "npm:0.5.7"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@eslint/eslintrc", [\
        ["npm:0.4.3", {\
          "packageLocation": "./.yarn/cache/@eslint-eslintrc-npm-0.4.3-ee1bbcab87-03a7704150.zip/node_modules/@eslint/eslintrc/",\
          "packageDependencies": [\
            ["@eslint/eslintrc", "npm:0.4.3"],\
            ["ajv", "npm:6.12.6"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["espree", "npm:7.3.1"],\
            ["globals", "npm:13.20.0"],\
            ["ignore", "npm:4.0.6"],\
            ["import-fresh", "npm:3.3.0"],\
            ["js-yaml", "npm:3.14.1"],\
            ["minimatch", "npm:3.1.2"],\
            ["strip-json-comments", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@gar/promisify", [\
        ["npm:1.1.3", {\
          "packageLocation": "./.yarn/cache/@gar-promisify-npm-1.1.3-ac1a325862-4059f790e2.zip/node_modules/@gar/promisify/",\
          "packageDependencies": [\
            ["@gar/promisify", "npm:1.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@humanwhocodes/config-array", [\
        ["npm:0.5.0", {\
          "packageLocation": "./.yarn/cache/@humanwhocodes-config-array-npm-0.5.0-5ded120470-44ee6a9f05.zip/node_modules/@humanwhocodes/config-array/",\
          "packageDependencies": [\
            ["@humanwhocodes/config-array", "npm:0.5.0"],\
            ["@humanwhocodes/object-schema", "npm:1.2.1"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["minimatch", "npm:3.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@humanwhocodes/object-schema", [\
        ["npm:1.2.1", {\
          "packageLocation": "./.yarn/cache/@humanwhocodes-object-schema-npm-1.2.1-eb622b5d0e-a824a1ec31.zip/node_modules/@humanwhocodes/object-schema/",\
          "packageDependencies": [\
            ["@humanwhocodes/object-schema", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@isaacs/cliui", [\
        ["npm:8.0.2", {\
          "packageLocation": "./.yarn/cache/@isaacs-cliui-npm-8.0.2-f4364666d5-4a473b9b32.zip/node_modules/@isaacs/cliui/",\
          "packageDependencies": [\
            ["@isaacs/cliui", "npm:8.0.2"],\
            ["string-width", "npm:5.1.2"],\
            ["string-width-cjs", [\
              "string-width",\
              "npm:4.2.3"\
            ]],\
            ["strip-ansi", "npm:7.1.0"],\
            ["strip-ansi-cjs", [\
              "strip-ansi",\
              "npm:6.0.1"\
            ]],\
            ["wrap-ansi", "npm:8.1.0"],\
            ["wrap-ansi-cjs", [\
              "wrap-ansi",\
              "npm:7.0.0"\
            ]]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@istanbuljs/load-nyc-config", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/@istanbuljs-load-nyc-config-npm-1.1.0-42d17c9cb1-d578da5e2e.zip/node_modules/@istanbuljs/load-nyc-config/",\
          "packageDependencies": [\
            ["@istanbuljs/load-nyc-config", "npm:1.1.0"],\
            ["camelcase", "npm:5.3.1"],\
            ["find-up", "npm:4.1.0"],\
            ["get-package-type", "npm:0.1.0"],\
            ["js-yaml", "npm:3.14.1"],\
            ["resolve-from", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@istanbuljs/schema", [\
        ["npm:0.1.3", {\
          "packageLocation": "./.yarn/cache/@istanbuljs-schema-npm-0.1.3-466bd3eaaa-5282759d96.zip/node_modules/@istanbuljs/schema/",\
          "packageDependencies": [\
            ["@istanbuljs/schema", "npm:0.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/console", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-console-npm-26.6.2-bfe95527e4-69a9ca6ba3.zip/node_modules/@jest/console/",\
          "packageDependencies": [\
            ["@jest/console", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["chalk", "npm:4.1.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["slash", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/core", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/@jest-core-npm-26.6.3-9bb6c92d6b-f52b26ffe9.zip/node_modules/@jest/core/",\
          "packageDependencies": [\
            ["@jest/core", "npm:26.6.3"],\
            ["@jest/console", "npm:26.6.2"],\
            ["@jest/reporters", "npm:26.6.2"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/transform", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["ansi-escapes", "npm:4.3.2"],\
            ["chalk", "npm:4.1.2"],\
            ["exit", "npm:0.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-changed-files", "npm:26.6.2"],\
            ["jest-config", "virtual:bf7a8695861ccc96c7503a95daba2b038c9b3eca0fc65dc5ea7e5ae0e56354c6c3e3ee05f1c8d4420e3a01abf48ad9e2dea477db48ad56147605b32adf33b489#npm:26.6.3"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-regex-util", "npm:26.0.0"],\
            ["jest-resolve", "npm:26.6.2"],\
            ["jest-resolve-dependencies", "npm:26.6.3"],\
            ["jest-runner", "npm:26.6.3"],\
            ["jest-runtime", "npm:26.6.3"],\
            ["jest-snapshot", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-validate", "npm:26.6.2"],\
            ["jest-watcher", "npm:26.6.2"],\
            ["micromatch", "npm:4.0.5"],\
            ["p-each-series", "npm:2.2.0"],\
            ["rimraf", "npm:3.0.2"],\
            ["slash", "npm:3.0.0"],\
            ["strip-ansi", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/environment", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-environment-npm-26.6.2-bc869bb23e-7748081b2a.zip/node_modules/@jest/environment/",\
          "packageDependencies": [\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/fake-timers", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["jest-mock", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/fake-timers", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-fake-timers-npm-26.6.2-511850bedc-c732658fac.zip/node_modules/@jest/fake-timers/",\
          "packageDependencies": [\
            ["@jest/fake-timers", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@sinonjs/fake-timers", "npm:6.0.1"],\
            ["@types/node", "npm:20.3.1"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-mock", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/globals", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-globals-npm-26.6.2-8ad9bc1dd5-49b28d0cc7.zip/node_modules/@jest/globals/",\
          "packageDependencies": [\
            ["@jest/globals", "npm:26.6.2"],\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["expect", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/reporters", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-reporters-npm-26.6.2-d75b358f51-53c7a697c5.zip/node_modules/@jest/reporters/",\
          "packageDependencies": [\
            ["@jest/reporters", "npm:26.6.2"],\
            ["@bcoe/v8-coverage", "npm:0.2.3"],\
            ["@jest/console", "npm:26.6.2"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/transform", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["collect-v8-coverage", "npm:1.0.1"],\
            ["exit", "npm:0.1.2"],\
            ["glob", "npm:7.2.3"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["istanbul-lib-coverage", "npm:3.2.0"],\
            ["istanbul-lib-instrument", "npm:4.0.3"],\
            ["istanbul-lib-report", "npm:3.0.0"],\
            ["istanbul-lib-source-maps", "npm:4.0.1"],\
            ["istanbul-reports", "npm:3.1.5"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-resolve", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-worker", "npm:26.6.2"],\
            ["node-notifier", "npm:8.0.2"],\
            ["slash", "npm:3.0.0"],\
            ["source-map", "npm:0.6.1"],\
            ["string-length", "npm:4.0.2"],\
            ["terminal-link", "npm:2.1.1"],\
            ["v8-to-istanbul", "npm:7.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/source-map", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-source-map-npm-26.6.2-a3b9d7d3b0-b171cef442.zip/node_modules/@jest/source-map/",\
          "packageDependencies": [\
            ["@jest/source-map", "npm:26.6.2"],\
            ["callsites", "npm:3.1.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/test-result", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-test-result-npm-26.6.2-7ffd75a97e-dcb6175825.zip/node_modules/@jest/test-result/",\
          "packageDependencies": [\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/console", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/istanbul-lib-coverage", "npm:2.0.4"],\
            ["collect-v8-coverage", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/test-sequencer", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/@jest-test-sequencer-npm-26.6.3-04c59ff624-a3450b3d70.zip/node_modules/@jest/test-sequencer/",\
          "packageDependencies": [\
            ["@jest/test-sequencer", "npm:26.6.3"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-runner", "npm:26.6.3"],\
            ["jest-runtime", "npm:26.6.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/transform", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-transform-npm-26.6.2-151c44db49-31667b925a.zip/node_modules/@jest/transform/",\
          "packageDependencies": [\
            ["@jest/transform", "npm:26.6.2"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@jest/types", "npm:26.6.2"],\
            ["babel-plugin-istanbul", "npm:6.1.1"],\
            ["chalk", "npm:4.1.2"],\
            ["convert-source-map", "npm:1.9.0"],\
            ["fast-json-stable-stringify", "npm:2.1.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-regex-util", "npm:26.0.0"],\
            ["jest-util", "npm:26.6.2"],\
            ["micromatch", "npm:4.0.5"],\
            ["pirates", "npm:4.0.6"],\
            ["slash", "npm:3.0.0"],\
            ["source-map", "npm:0.6.1"],\
            ["write-file-atomic", "npm:3.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jest/types", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/@jest-types-npm-26.6.2-fca9877d98-a0bd3d2f22.zip/node_modules/@jest/types/",\
          "packageDependencies": [\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/istanbul-lib-coverage", "npm:2.0.4"],\
            ["@types/istanbul-reports", "npm:3.0.1"],\
            ["@types/node", "npm:20.3.1"],\
            ["@types/yargs", "npm:15.0.15"],\
            ["chalk", "npm:4.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jridgewell/gen-mapping", [\
        ["npm:0.3.3", {\
          "packageLocation": "./.yarn/cache/@jridgewell-gen-mapping-npm-0.3.3-1815eba94c-4a74944bd3.zip/node_modules/@jridgewell/gen-mapping/",\
          "packageDependencies": [\
            ["@jridgewell/gen-mapping", "npm:0.3.3"],\
            ["@jridgewell/set-array", "npm:1.1.2"],\
            ["@jridgewell/sourcemap-codec", "npm:1.4.15"],\
            ["@jridgewell/trace-mapping", "npm:0.3.18"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jridgewell/resolve-uri", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/@jridgewell-resolve-uri-npm-3.1.0-6ff2351e61-b5ceaaf9a1.zip/node_modules/@jridgewell/resolve-uri/",\
          "packageDependencies": [\
            ["@jridgewell/resolve-uri", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jridgewell/set-array", [\
        ["npm:1.1.2", {\
          "packageLocation": "./.yarn/cache/@jridgewell-set-array-npm-1.1.2-45b82d7fb6-69a84d5980.zip/node_modules/@jridgewell/set-array/",\
          "packageDependencies": [\
            ["@jridgewell/set-array", "npm:1.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jridgewell/source-map", [\
        ["npm:0.3.3", {\
          "packageLocation": "./.yarn/cache/@jridgewell-source-map-npm-0.3.3-eb138f3f67-ae13021463.zip/node_modules/@jridgewell/source-map/",\
          "packageDependencies": [\
            ["@jridgewell/source-map", "npm:0.3.3"],\
            ["@jridgewell/gen-mapping", "npm:0.3.3"],\
            ["@jridgewell/trace-mapping", "npm:0.3.18"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jridgewell/sourcemap-codec", [\
        ["npm:1.4.14", {\
          "packageLocation": "./.yarn/cache/@jridgewell-sourcemap-codec-npm-1.4.14-f5f0630788-61100637b6.zip/node_modules/@jridgewell/sourcemap-codec/",\
          "packageDependencies": [\
            ["@jridgewell/sourcemap-codec", "npm:1.4.14"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.4.15", {\
          "packageLocation": "./.yarn/cache/@jridgewell-sourcemap-codec-npm-1.4.15-a055fb62cf-b881c7e503.zip/node_modules/@jridgewell/sourcemap-codec/",\
          "packageDependencies": [\
            ["@jridgewell/sourcemap-codec", "npm:1.4.15"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jridgewell/trace-mapping", [\
        ["npm:0.3.18", {\
          "packageLocation": "./.yarn/cache/@jridgewell-trace-mapping-npm-0.3.18-cd96571385-0572669f85.zip/node_modules/@jridgewell/trace-mapping/",\
          "packageDependencies": [\
            ["@jridgewell/trace-mapping", "npm:0.3.18"],\
            ["@jridgewell/resolve-uri", "npm:3.1.0"],\
            ["@jridgewell/sourcemap-codec", "npm:1.4.14"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyter-widgets/base", [\
        ["npm:6.0.4", {\
          "packageLocation": "./.yarn/cache/@jupyter-widgets-base-npm-6.0.4-f43011f923-f82b4eec8f.zip/node_modules/@jupyter-widgets/base/",\
          "packageDependencies": [\
            ["@jupyter-widgets/base", "npm:6.0.4"],\
            ["@jupyterlab/services", "npm:6.6.4"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/messaging", "npm:1.10.3"],\
            ["@lumino/widgets", "npm:1.37.2"],\
            ["@types/backbone", "npm:1.4.14"],\
            ["@types/lodash", "npm:4.14.195"],\
            ["backbone", "npm:1.4.0"],\
            ["jquery", "npm:3.7.0"],\
            ["lodash", "npm:4.17.21"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyter-widgets/base-manager", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/@jupyter-widgets-base-manager-npm-1.0.5-930925dd0f-080cfec960.zip/node_modules/@jupyter-widgets/base-manager/",\
          "packageDependencies": [\
            ["@jupyter-widgets/base-manager", "npm:1.0.5"],\
            ["@jupyter-widgets/base", "npm:6.0.4"],\
            ["@jupyterlab/services", "npm:6.6.4"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["base64-js", "npm:1.5.1"],\
            ["sanitize-html", "npm:2.11.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/builder", [\
        ["npm:3.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-builder-npm-3.6.4-80d0dfd751-51a26662d0.zip/node_modules/@jupyterlab/builder/",\
          "packageDependencies": [\
            ["@jupyterlab/builder", "npm:3.6.4"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/application", "npm:1.31.4"],\
            ["@lumino/commands", "npm:1.21.1"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/domutils", "npm:1.8.2"],\
            ["@lumino/dragdrop", "npm:1.14.5"],\
            ["@lumino/messaging", "npm:1.10.3"],\
            ["@lumino/properties", "npm:1.8.2"],\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["@lumino/virtualdom", "npm:1.14.3"],\
            ["@lumino/widgets", "npm:1.37.2"],\
            ["ajv", "npm:6.12.6"],\
            ["commander", "npm:6.0.0"],\
            ["css-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.2.7"],\
            ["duplicate-package-checker-webpack-plugin", "npm:3.0.0"],\
            ["file-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0"],\
            ["fs-extra", "npm:9.1.0"],\
            ["glob", "npm:7.1.7"],\
            ["license-webpack-plugin", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:2.3.21"],\
            ["mini-css-extract-plugin", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:1.3.9"],\
            ["path-browserify", "npm:1.0.1"],\
            ["process", "npm:0.11.10"],\
            ["raw-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.0.2"],\
            ["source-map-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:1.0.2"],\
            ["style-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:2.0.0"],\
            ["supports-color", "npm:7.2.0"],\
            ["svg-url-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0"],\
            ["terser-webpack-plugin", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.2.3"],\
            ["to-string-loader", "npm:1.2.0"],\
            ["url-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.1.1"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["webpack-cli", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0"],\
            ["webpack-merge", "npm:5.9.0"],\
            ["worker-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:3.0.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/coreutils", [\
        ["npm:5.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-coreutils-npm-5.6.4-d3b61e9b36-a6cadc00c8.zip/node_modules/@jupyterlab/coreutils/",\
          "packageDependencies": [\
            ["@jupyterlab/coreutils", "npm:5.6.4"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["minimist", "npm:1.2.8"],\
            ["moment", "npm:2.29.4"],\
            ["path-browserify", "npm:1.0.1"],\
            ["url-parse", "npm:1.5.10"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/nbformat", [\
        ["npm:3.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-nbformat-npm-3.6.4-160c4eb3e0-da46160d88.zip/node_modules/@jupyterlab/nbformat/",\
          "packageDependencies": [\
            ["@jupyterlab/nbformat", "npm:3.6.4"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/observables", [\
        ["npm:4.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-observables-npm-4.6.4-f679cb681e-e6123a1f33.zip/node_modules/@jupyterlab/observables/",\
          "packageDependencies": [\
            ["@jupyterlab/observables", "npm:4.6.4"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/messaging", "npm:1.10.3"],\
            ["@lumino/signaling", "npm:1.11.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/services", [\
        ["npm:6.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-services-npm-6.6.4-dce30a2f17-f4e7fda300.zip/node_modules/@jupyterlab/services/",\
          "packageDependencies": [\
            ["@jupyterlab/services", "npm:6.6.4"],\
            ["@jupyterlab/coreutils", "npm:5.6.4"],\
            ["@jupyterlab/nbformat", "npm:3.6.4"],\
            ["@jupyterlab/observables", "npm:4.6.4"],\
            ["@jupyterlab/settingregistry", "npm:3.6.4"],\
            ["@jupyterlab/statedb", "npm:3.6.4"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/polling", "npm:1.11.4"],\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["node-fetch", "virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:2.6.11"],\
            ["ws", "virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:7.5.9"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/settingregistry", [\
        ["npm:3.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-settingregistry-npm-3.6.4-3d46a59470-b87ecaaa65.zip/node_modules/@jupyterlab/settingregistry/",\
          "packageDependencies": [\
            ["@jupyterlab/settingregistry", "npm:3.6.4"],\
            ["@jupyterlab/statedb", "npm:3.6.4"],\
            ["@lumino/commands", "npm:1.21.1"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["ajv", "npm:6.12.6"],\
            ["json5", "npm:2.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@jupyterlab/statedb", [\
        ["npm:3.6.4", {\
          "packageLocation": "./.yarn/cache/@jupyterlab-statedb-npm-3.6.4-ec68126c6e-f33899e9fc.zip/node_modules/@jupyterlab/statedb/",\
          "packageDependencies": [\
            ["@jupyterlab/statedb", "npm:3.6.4"],\
            ["@lumino/commands", "npm:1.21.1"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/properties", "npm:1.8.2"],\
            ["@lumino/signaling", "npm:1.11.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/algorithm", [\
        ["npm:1.9.2", {\
          "packageLocation": "./.yarn/cache/@lumino-algorithm-npm-1.9.2-f4ec803f49-a89e7c6350.zip/node_modules/@lumino/algorithm/",\
          "packageDependencies": [\
            ["@lumino/algorithm", "npm:1.9.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/application", [\
        ["npm:1.31.4", {\
          "packageLocation": "./.yarn/cache/@lumino-application-npm-1.31.4-c64a2827d1-0aa814bf55.zip/node_modules/@lumino/application/",\
          "packageDependencies": [\
            ["@lumino/application", "npm:1.31.4"],\
            ["@lumino/commands", "npm:1.21.1"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/widgets", "npm:1.37.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/collections", [\
        ["npm:1.9.3", {\
          "packageLocation": "./.yarn/cache/@lumino-collections-npm-1.9.3-b23270f96d-1c87a12743.zip/node_modules/@lumino/collections/",\
          "packageDependencies": [\
            ["@lumino/collections", "npm:1.9.3"],\
            ["@lumino/algorithm", "npm:1.9.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/commands", [\
        ["npm:1.21.1", {\
          "packageLocation": "./.yarn/cache/@lumino-commands-npm-1.21.1-c653d82825-1e2ee7ce14.zip/node_modules/@lumino/commands/",\
          "packageDependencies": [\
            ["@lumino/commands", "npm:1.21.1"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/domutils", "npm:1.8.2"],\
            ["@lumino/keyboard", "npm:1.8.2"],\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["@lumino/virtualdom", "npm:1.14.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/coreutils", [\
        ["npm:1.12.1", {\
          "packageLocation": "./.yarn/cache/@lumino-coreutils-npm-1.12.1-60ecc3187b-55f1b87997.zip/node_modules/@lumino/coreutils/",\
          "packageDependencies": [\
            ["@lumino/coreutils", "npm:1.12.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1", {\
          "packageLocation": "./.yarn/__virtual__/@lumino-coreutils-virtual-efae582108/0/cache/@lumino-coreutils-npm-1.12.1-60ecc3187b-55f1b87997.zip/node_modules/@lumino/coreutils/",\
          "packageDependencies": [\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@types/crypto", null],\
            ["crypto", null]\
          ],\
          "packagePeers": [\
            "@types/crypto",\
            "crypto"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/disposable", [\
        ["npm:1.10.4", {\
          "packageLocation": "./.yarn/cache/@lumino-disposable-npm-1.10.4-a7db8be386-b53e259830.zip/node_modules/@lumino/disposable/",\
          "packageDependencies": [\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/signaling", "npm:1.11.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/domutils", [\
        ["npm:1.8.2", {\
          "packageLocation": "./.yarn/cache/@lumino-domutils-npm-1.8.2-0a648b9d71-196f25316a.zip/node_modules/@lumino/domutils/",\
          "packageDependencies": [\
            ["@lumino/domutils", "npm:1.8.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/dragdrop", [\
        ["npm:1.14.5", {\
          "packageLocation": "./.yarn/cache/@lumino-dragdrop-npm-1.14.5-edee8cbe15-c10031e9aa.zip/node_modules/@lumino/dragdrop/",\
          "packageDependencies": [\
            ["@lumino/dragdrop", "npm:1.14.5"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/keyboard", [\
        ["npm:1.8.2", {\
          "packageLocation": "./.yarn/cache/@lumino-keyboard-npm-1.8.2-39cb14bc3e-30f8ced53c.zip/node_modules/@lumino/keyboard/",\
          "packageDependencies": [\
            ["@lumino/keyboard", "npm:1.8.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/messaging", [\
        ["npm:1.10.3", {\
          "packageLocation": "./.yarn/cache/@lumino-messaging-npm-1.10.3-ebbe0be380-1131e80379.zip/node_modules/@lumino/messaging/",\
          "packageDependencies": [\
            ["@lumino/messaging", "npm:1.10.3"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/collections", "npm:1.9.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/polling", [\
        ["npm:1.11.4", {\
          "packageLocation": "./.yarn/cache/@lumino-polling-npm-1.11.4-c617ba2ddd-d4625da7bf.zip/node_modules/@lumino/polling/",\
          "packageDependencies": [\
            ["@lumino/polling", "npm:1.11.4"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/signaling", "npm:1.11.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/properties", [\
        ["npm:1.8.2", {\
          "packageLocation": "./.yarn/cache/@lumino-properties-npm-1.8.2-ed0d7d5fd2-9a53709fe5.zip/node_modules/@lumino/properties/",\
          "packageDependencies": [\
            ["@lumino/properties", "npm:1.8.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/signaling", [\
        ["npm:1.11.1", {\
          "packageLocation": "./.yarn/cache/@lumino-signaling-npm-1.11.1-d66eef8a9b-3d822be705.zip/node_modules/@lumino/signaling/",\
          "packageDependencies": [\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/properties", "npm:1.8.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/virtualdom", [\
        ["npm:1.14.3", {\
          "packageLocation": "./.yarn/cache/@lumino-virtualdom-npm-1.14.3-a30f6319f6-dd6acc5402.zip/node_modules/@lumino/virtualdom/",\
          "packageDependencies": [\
            ["@lumino/virtualdom", "npm:1.14.3"],\
            ["@lumino/algorithm", "npm:1.9.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@lumino/widgets", [\
        ["npm:1.37.2", {\
          "packageLocation": "./.yarn/cache/@lumino-widgets-npm-1.37.2-89db812cb6-3193f8cca4.zip/node_modules/@lumino/widgets/",\
          "packageDependencies": [\
            ["@lumino/widgets", "npm:1.37.2"],\
            ["@lumino/algorithm", "npm:1.9.2"],\
            ["@lumino/commands", "npm:1.21.1"],\
            ["@lumino/coreutils", "virtual:d3b61e9b3607a02ca0506b51412df35c9de4078318416806e3573da973aee29df3a07161819913efbddd39334be8d7c97efa8bf4fd9d0f03726be83fa0cd6930#npm:1.12.1"],\
            ["@lumino/disposable", "npm:1.10.4"],\
            ["@lumino/domutils", "npm:1.8.2"],\
            ["@lumino/dragdrop", "npm:1.14.5"],\
            ["@lumino/keyboard", "npm:1.8.2"],\
            ["@lumino/messaging", "npm:1.10.3"],\
            ["@lumino/properties", "npm:1.8.2"],\
            ["@lumino/signaling", "npm:1.11.1"],\
            ["@lumino/virtualdom", "npm:1.14.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@npmcli/fs", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/@npmcli-fs-npm-1.1.1-17f582e0b6-f5ad92f157.zip/node_modules/@npmcli/fs/",\
          "packageDependencies": [\
            ["@npmcli/fs", "npm:1.1.1"],\
            ["@gar/promisify", "npm:1.1.3"],\
            ["semver", "npm:7.5.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/@npmcli-fs-npm-3.1.0-0844a57978-a50a6818de.zip/node_modules/@npmcli/fs/",\
          "packageDependencies": [\
            ["@npmcli/fs", "npm:3.1.0"],\
            ["semver", "npm:7.5.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@npmcli/move-file", [\
        ["npm:1.1.2", {\
          "packageLocation": "./.yarn/cache/@npmcli-move-file-npm-1.1.2-4f6c7b3354-c96381d4a3.zip/node_modules/@npmcli/move-file/",\
          "packageDependencies": [\
            ["@npmcli/move-file", "npm:1.1.2"],\
            ["mkdirp", "npm:1.0.4"],\
            ["rimraf", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@pkgjs/parseargs", [\
        ["npm:0.11.0", {\
          "packageLocation": "./.yarn/cache/@pkgjs-parseargs-npm-0.11.0-cd2a3fe948-6ad6a00fc4.zip/node_modules/@pkgjs/parseargs/",\
          "packageDependencies": [\
            ["@pkgjs/parseargs", "npm:0.11.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@sinonjs/commons", [\
        ["npm:1.8.6", {\
          "packageLocation": "./.yarn/cache/@sinonjs-commons-npm-1.8.6-e3593ed6d6-7d3f8c1e85.zip/node_modules/@sinonjs/commons/",\
          "packageDependencies": [\
            ["@sinonjs/commons", "npm:1.8.6"],\
            ["type-detect", "npm:4.0.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@sinonjs/fake-timers", [\
        ["npm:6.0.1", {\
          "packageLocation": "./.yarn/cache/@sinonjs-fake-timers-npm-6.0.1-cebf4d0bfb-8e331aa141.zip/node_modules/@sinonjs/fake-timers/",\
          "packageDependencies": [\
            ["@sinonjs/fake-timers", "npm:6.0.1"],\
            ["@sinonjs/commons", "npm:1.8.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@tootallnate/once", [\
        ["npm:1.1.2", {\
          "packageLocation": "./.yarn/cache/@tootallnate-once-npm-1.1.2-0517220057-e1fb1bbbc1.zip/node_modules/@tootallnate/once/",\
          "packageDependencies": [\
            ["@tootallnate/once", "npm:1.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/@tootallnate-once-npm-2.0.0-e36cf4f140-ad87447820.zip/node_modules/@tootallnate/once/",\
          "packageDependencies": [\
            ["@tootallnate/once", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/babel__core", [\
        ["npm:7.20.1", {\
          "packageLocation": "./.yarn/cache/@types-babel__core-npm-7.20.1-f44761b6b6-9fcd9691a3.zip/node_modules/@types/babel__core/",\
          "packageDependencies": [\
            ["@types/babel__core", "npm:7.20.1"],\
            ["@babel/parser", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@types/babel__generator", "npm:7.6.4"],\
            ["@types/babel__template", "npm:7.4.1"],\
            ["@types/babel__traverse", "npm:7.20.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/babel__generator", [\
        ["npm:7.6.4", {\
          "packageLocation": "./.yarn/cache/@types-babel__generator-npm-7.6.4-03e776f956-20effbbb5f.zip/node_modules/@types/babel__generator/",\
          "packageDependencies": [\
            ["@types/babel__generator", "npm:7.6.4"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/babel__template", [\
        ["npm:7.4.1", {\
          "packageLocation": "./.yarn/cache/@types-babel__template-npm-7.4.1-fe1db49e53-649fe8b42c.zip/node_modules/@types/babel__template/",\
          "packageDependencies": [\
            ["@types/babel__template", "npm:7.4.1"],\
            ["@babel/parser", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/babel__traverse", [\
        ["npm:7.20.1", {\
          "packageLocation": "./.yarn/cache/@types-babel__traverse-npm-7.20.1-e1e93d4b3f-58341e23c6.zip/node_modules/@types/babel__traverse/",\
          "packageDependencies": [\
            ["@types/babel__traverse", "npm:7.20.1"],\
            ["@babel/types", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/backbone", [\
        ["npm:1.4.14", {\
          "packageLocation": "./.yarn/cache/@types-backbone-npm-1.4.14-7a68fb09b4-4f44bfb71d.zip/node_modules/@types/backbone/",\
          "packageDependencies": [\
            ["@types/backbone", "npm:1.4.14"],\
            ["@types/jquery", "npm:3.5.16"],\
            ["@types/underscore", "npm:1.11.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/eslint", [\
        ["npm:8.40.2", {\
          "packageLocation": "./.yarn/cache/@types-eslint-npm-8.40.2-b5096beae4-a4780e45e6.zip/node_modules/@types/eslint/",\
          "packageDependencies": [\
            ["@types/eslint", "npm:8.40.2"],\
            ["@types/estree", "npm:1.0.1"],\
            ["@types/json-schema", "npm:7.0.12"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/eslint-scope", [\
        ["npm:3.7.4", {\
          "packageLocation": "./.yarn/cache/@types-eslint-scope-npm-3.7.4-c11d226d71-ea6a9363e9.zip/node_modules/@types/eslint-scope/",\
          "packageDependencies": [\
            ["@types/eslint-scope", "npm:3.7.4"],\
            ["@types/eslint", "npm:8.40.2"],\
            ["@types/estree", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/eslint-visitor-keys", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/@types-eslint-visitor-keys-npm-1.0.0-a300061b93-a90f0b023e.zip/node_modules/@types/eslint-visitor-keys/",\
          "packageDependencies": [\
            ["@types/eslint-visitor-keys", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/estree", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/@types-estree-npm-1.0.1-4c9469c165-e9aa175eac.zip/node_modules/@types/estree/",\
          "packageDependencies": [\
            ["@types/estree", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/graceful-fs", [\
        ["npm:4.1.6", {\
          "packageLocation": "./.yarn/cache/@types-graceful-fs-npm-4.1.6-1eadcf742d-c3070ccdc9.zip/node_modules/@types/graceful-fs/",\
          "packageDependencies": [\
            ["@types/graceful-fs", "npm:4.1.6"],\
            ["@types/node", "npm:20.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/istanbul-lib-coverage", [\
        ["npm:2.0.4", {\
          "packageLocation": "./.yarn/cache/@types-istanbul-lib-coverage-npm-2.0.4-734954bb56-a25d7589ee.zip/node_modules/@types/istanbul-lib-coverage/",\
          "packageDependencies": [\
            ["@types/istanbul-lib-coverage", "npm:2.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/istanbul-lib-report", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/@types-istanbul-lib-report-npm-3.0.0-50de3e6b3b-656398b62d.zip/node_modules/@types/istanbul-lib-report/",\
          "packageDependencies": [\
            ["@types/istanbul-lib-report", "npm:3.0.0"],\
            ["@types/istanbul-lib-coverage", "npm:2.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/istanbul-reports", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/@types-istanbul-reports-npm-3.0.1-770e825002-f1ad54bc68.zip/node_modules/@types/istanbul-reports/",\
          "packageDependencies": [\
            ["@types/istanbul-reports", "npm:3.0.1"],\
            ["@types/istanbul-lib-report", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/jest", [\
        ["npm:26.0.24", {\
          "packageLocation": "./.yarn/cache/@types-jest-npm-26.0.24-47c570294f-ae39675412.zip/node_modules/@types/jest/",\
          "packageDependencies": [\
            ["@types/jest", "npm:26.0.24"],\
            ["jest-diff", "npm:26.6.2"],\
            ["pretty-format", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/jquery", [\
        ["npm:3.5.16", {\
          "packageLocation": "./.yarn/cache/@types-jquery-npm-3.5.16-62a6b7d27d-13c995f15d.zip/node_modules/@types/jquery/",\
          "packageDependencies": [\
            ["@types/jquery", "npm:3.5.16"],\
            ["@types/sizzle", "npm:2.3.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/json-schema", [\
        ["npm:7.0.12", {\
          "packageLocation": "./.yarn/cache/@types-json-schema-npm-7.0.12-f05cfc0e99-00239e9723.zip/node_modules/@types/json-schema/",\
          "packageDependencies": [\
            ["@types/json-schema", "npm:7.0.12"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/lodash", [\
        ["npm:4.14.195", {\
          "packageLocation": "./.yarn/cache/@types-lodash-npm-4.14.195-a2a8254638-39b75ca635.zip/node_modules/@types/lodash/",\
          "packageDependencies": [\
            ["@types/lodash", "npm:4.14.195"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/node", [\
        ["npm:20.3.1", {\
          "packageLocation": "./.yarn/cache/@types-node-npm-20.3.1-86012346c0-63a393ab6d.zip/node_modules/@types/node/",\
          "packageDependencies": [\
            ["@types/node", "npm:20.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/normalize-package-data", [\
        ["npm:2.4.1", {\
          "packageLocation": "./.yarn/cache/@types-normalize-package-data-npm-2.4.1-c31c56ae6a-e87bccbf11.zip/node_modules/@types/normalize-package-data/",\
          "packageDependencies": [\
            ["@types/normalize-package-data", "npm:2.4.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/prettier", [\
        ["npm:2.7.3", {\
          "packageLocation": "./.yarn/cache/@types-prettier-npm-2.7.3-497316f37c-705384209c.zip/node_modules/@types/prettier/",\
          "packageDependencies": [\
            ["@types/prettier", "npm:2.7.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/sizzle", [\
        ["npm:2.3.3", {\
          "packageLocation": "./.yarn/cache/@types-sizzle-npm-2.3.3-9403924950-586a9fb1f6.zip/node_modules/@types/sizzle/",\
          "packageDependencies": [\
            ["@types/sizzle", "npm:2.3.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/source-list-map", [\
        ["npm:0.1.2", {\
          "packageLocation": "./.yarn/cache/@types-source-list-map-npm-0.1.2-1983e10da7-fda8f37537.zip/node_modules/@types/source-list-map/",\
          "packageDependencies": [\
            ["@types/source-list-map", "npm:0.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/stack-utils", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/@types-stack-utils-npm-2.0.1-867718ab70-205fdbe332.zip/node_modules/@types/stack-utils/",\
          "packageDependencies": [\
            ["@types/stack-utils", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/underscore", [\
        ["npm:1.11.5", {\
          "packageLocation": "./.yarn/cache/@types-underscore-npm-1.11.5-b2a391843a-6cd928c436.zip/node_modules/@types/underscore/",\
          "packageDependencies": [\
            ["@types/underscore", "npm:1.11.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/webpack-env", [\
        ["npm:1.18.1", {\
          "packageLocation": "./.yarn/cache/@types-webpack-env-npm-1.18.1-73a2ecf677-3173c06976.zip/node_modules/@types/webpack-env/",\
          "packageDependencies": [\
            ["@types/webpack-env", "npm:1.18.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/webpack-sources", [\
        ["npm:0.1.9", {\
          "packageLocation": "./.yarn/cache/@types-webpack-sources-npm-0.1.9-a1459d6944-bc09c584c7.zip/node_modules/@types/webpack-sources/",\
          "packageDependencies": [\
            ["@types/webpack-sources", "npm:0.1.9"],\
            ["@types/node", "npm:20.3.1"],\
            ["@types/source-list-map", "npm:0.1.2"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/yargs", [\
        ["npm:15.0.15", {\
          "packageLocation": "./.yarn/cache/@types-yargs-npm-15.0.15-b73a9d1c59-3420f6bcc5.zip/node_modules/@types/yargs/",\
          "packageDependencies": [\
            ["@types/yargs", "npm:15.0.15"],\
            ["@types/yargs-parser", "npm:21.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@types/yargs-parser", [\
        ["npm:21.0.0", {\
          "packageLocation": "./.yarn/cache/@types-yargs-parser-npm-21.0.0-c8a3b32c52-b2f4c8d12a.zip/node_modules/@types/yargs-parser/",\
          "packageDependencies": [\
            ["@types/yargs-parser", "npm:21.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@typescript-eslint/eslint-plugin", [\
        ["npm:3.10.1", {\
          "packageLocation": "./.yarn/cache/@typescript-eslint-eslint-plugin-npm-3.10.1-8c1dbd0dcd-f1720338b4.zip/node_modules/@typescript-eslint/eslint-plugin/",\
          "packageDependencies": [\
            ["@typescript-eslint/eslint-plugin", "npm:3.10.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1", {\
          "packageLocation": "./.yarn/__virtual__/@typescript-eslint-eslint-plugin-virtual-60dbc451ed/0/cache/@typescript-eslint-eslint-plugin-npm-3.10.1-8c1dbd0dcd-f1720338b4.zip/node_modules/@typescript-eslint/eslint-plugin/",\
          "packageDependencies": [\
            ["@typescript-eslint/eslint-plugin", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["@types/eslint", null],\
            ["@types/typescript", null],\
            ["@types/typescript-eslint__parser", null],\
            ["@typescript-eslint/experimental-utils", "virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.10.1"],\
            ["@typescript-eslint/parser", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["eslint", "npm:7.32.0"],\
            ["functional-red-black-tree", "npm:1.0.1"],\
            ["regexpp", "npm:3.2.0"],\
            ["semver", "npm:7.5.3"],\
            ["tsutils", "virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.21.0"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"]\
          ],\
          "packagePeers": [\
            "@types/eslint",\
            "@types/typescript-eslint__parser",\
            "@types/typescript",\
            "@typescript-eslint/parser",\
            "eslint",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@typescript-eslint/experimental-utils", [\
        ["npm:3.10.1", {\
          "packageLocation": "./.yarn/cache/@typescript-eslint-experimental-utils-npm-3.10.1-607d3ffe42-635cc1afe4.zip/node_modules/@typescript-eslint/experimental-utils/",\
          "packageDependencies": [\
            ["@typescript-eslint/experimental-utils", "npm:3.10.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.10.1", {\
          "packageLocation": "./.yarn/__virtual__/@typescript-eslint-experimental-utils-virtual-04a51cd333/0/cache/@typescript-eslint-experimental-utils-npm-3.10.1-607d3ffe42-635cc1afe4.zip/node_modules/@typescript-eslint/experimental-utils/",\
          "packageDependencies": [\
            ["@typescript-eslint/experimental-utils", "virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.10.1"],\
            ["@types/eslint", null],\
            ["@types/json-schema", "npm:7.0.12"],\
            ["@typescript-eslint/types", "npm:3.10.1"],\
            ["@typescript-eslint/typescript-estree", "virtual:04a51cd333a3c1495531fd04a997674b77e8890fd77a10cccb2870cdf8d6f62bd7dbc411b9ed78db4b35127fc379e1bec33f53e86ff4aa1b36f1a3c0a8a2d2ea#npm:3.10.1"],\
            ["eslint", "npm:7.32.0"],\
            ["eslint-scope", "npm:5.1.1"],\
            ["eslint-utils", "npm:2.1.0"]\
          ],\
          "packagePeers": [\
            "@types/eslint",\
            "eslint"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@typescript-eslint/parser", [\
        ["npm:3.10.1", {\
          "packageLocation": "./.yarn/cache/@typescript-eslint-parser-npm-3.10.1-8d6732da0b-4939ed1dd1.zip/node_modules/@typescript-eslint/parser/",\
          "packageDependencies": [\
            ["@typescript-eslint/parser", "npm:3.10.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1", {\
          "packageLocation": "./.yarn/__virtual__/@typescript-eslint-parser-virtual-b4b133bbb9/0/cache/@typescript-eslint-parser-npm-3.10.1-8d6732da0b-4939ed1dd1.zip/node_modules/@typescript-eslint/parser/",\
          "packageDependencies": [\
            ["@typescript-eslint/parser", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["@types/eslint", null],\
            ["@types/eslint-visitor-keys", "npm:1.0.0"],\
            ["@types/typescript", null],\
            ["@typescript-eslint/experimental-utils", "virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.10.1"],\
            ["@typescript-eslint/types", "npm:3.10.1"],\
            ["@typescript-eslint/typescript-estree", "virtual:b4b133bbb95d0ead7725483999c304fbc2e3b7d5be832409fc5ffd4d4341ffec2d4e4b018b67f06037eb3963b0a947f9eb94c683e309631c6a1f6282d7323c4c#npm:3.10.1"],\
            ["eslint", "npm:7.32.0"],\
            ["eslint-visitor-keys", "npm:1.3.0"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"]\
          ],\
          "packagePeers": [\
            "@types/eslint",\
            "@types/typescript",\
            "eslint",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@typescript-eslint/types", [\
        ["npm:3.10.1", {\
          "packageLocation": "./.yarn/cache/@typescript-eslint-types-npm-3.10.1-faddecc057-3ea820d37c.zip/node_modules/@typescript-eslint/types/",\
          "packageDependencies": [\
            ["@typescript-eslint/types", "npm:3.10.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@typescript-eslint/typescript-estree", [\
        ["npm:3.10.1", {\
          "packageLocation": "./.yarn/cache/@typescript-eslint-typescript-estree-npm-3.10.1-327d1c22a9-911680da9d.zip/node_modules/@typescript-eslint/typescript-estree/",\
          "packageDependencies": [\
            ["@typescript-eslint/typescript-estree", "npm:3.10.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:04a51cd333a3c1495531fd04a997674b77e8890fd77a10cccb2870cdf8d6f62bd7dbc411b9ed78db4b35127fc379e1bec33f53e86ff4aa1b36f1a3c0a8a2d2ea#npm:3.10.1", {\
          "packageLocation": "./.yarn/__virtual__/@typescript-eslint-typescript-estree-virtual-4e46a89bdf/0/cache/@typescript-eslint-typescript-estree-npm-3.10.1-327d1c22a9-911680da9d.zip/node_modules/@typescript-eslint/typescript-estree/",\
          "packageDependencies": [\
            ["@typescript-eslint/typescript-estree", "virtual:04a51cd333a3c1495531fd04a997674b77e8890fd77a10cccb2870cdf8d6f62bd7dbc411b9ed78db4b35127fc379e1bec33f53e86ff4aa1b36f1a3c0a8a2d2ea#npm:3.10.1"],\
            ["@types/typescript", null],\
            ["@typescript-eslint/types", "npm:3.10.1"],\
            ["@typescript-eslint/visitor-keys", "npm:3.10.1"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["glob", "npm:7.2.3"],\
            ["is-glob", "npm:4.0.3"],\
            ["lodash", "npm:4.17.21"],\
            ["semver", "npm:7.5.3"],\
            ["tsutils", "virtual:4e46a89bdf2ac87e21276dc3d81d30cea0ffd5e1ff3d5417df1e96f79ae422194dfcfa17f2a3469ab76696bae0f7f47086ff12110b9307494abd02932d6722d4#npm:3.21.0"],\
            ["typescript", null]\
          ],\
          "packagePeers": [\
            "@types/typescript",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:b4b133bbb95d0ead7725483999c304fbc2e3b7d5be832409fc5ffd4d4341ffec2d4e4b018b67f06037eb3963b0a947f9eb94c683e309631c6a1f6282d7323c4c#npm:3.10.1", {\
          "packageLocation": "./.yarn/__virtual__/@typescript-eslint-typescript-estree-virtual-8e213c90ab/0/cache/@typescript-eslint-typescript-estree-npm-3.10.1-327d1c22a9-911680da9d.zip/node_modules/@typescript-eslint/typescript-estree/",\
          "packageDependencies": [\
            ["@typescript-eslint/typescript-estree", "virtual:b4b133bbb95d0ead7725483999c304fbc2e3b7d5be832409fc5ffd4d4341ffec2d4e4b018b67f06037eb3963b0a947f9eb94c683e309631c6a1f6282d7323c4c#npm:3.10.1"],\
            ["@types/typescript", null],\
            ["@typescript-eslint/types", "npm:3.10.1"],\
            ["@typescript-eslint/visitor-keys", "npm:3.10.1"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["glob", "npm:7.2.3"],\
            ["is-glob", "npm:4.0.3"],\
            ["lodash", "npm:4.17.21"],\
            ["semver", "npm:7.5.3"],\
            ["tsutils", "virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.21.0"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"]\
          ],\
          "packagePeers": [\
            "@types/typescript",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@typescript-eslint/visitor-keys", [\
        ["npm:3.10.1", {\
          "packageLocation": "./.yarn/cache/@typescript-eslint-visitor-keys-npm-3.10.1-6a80d0d416-0c4825b982.zip/node_modules/@typescript-eslint/visitor-keys/",\
          "packageDependencies": [\
            ["@typescript-eslint/visitor-keys", "npm:3.10.1"],\
            ["eslint-visitor-keys", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/ast", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-ast-npm-1.11.6-d3fd2bb49a-38ef1b526c.zip/node_modules/@webassemblyjs/ast/",\
          "packageDependencies": [\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/helper-numbers", "npm:1.11.6"],\
            ["@webassemblyjs/helper-wasm-bytecode", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/floating-point-hex-parser", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-floating-point-hex-parser-npm-1.11.6-3a9928fc76-29b0875884.zip/node_modules/@webassemblyjs/floating-point-hex-parser/",\
          "packageDependencies": [\
            ["@webassemblyjs/floating-point-hex-parser", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/helper-api-error", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-helper-api-error-npm-1.11.6-75f6275ff4-e8563df851.zip/node_modules/@webassemblyjs/helper-api-error/",\
          "packageDependencies": [\
            ["@webassemblyjs/helper-api-error", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/helper-buffer", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-helper-buffer-npm-1.11.6-69996544b0-b14d0573bf.zip/node_modules/@webassemblyjs/helper-buffer/",\
          "packageDependencies": [\
            ["@webassemblyjs/helper-buffer", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/helper-numbers", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-helper-numbers-npm-1.11.6-819ddab1da-f4b562fa21.zip/node_modules/@webassemblyjs/helper-numbers/",\
          "packageDependencies": [\
            ["@webassemblyjs/helper-numbers", "npm:1.11.6"],\
            ["@webassemblyjs/floating-point-hex-parser", "npm:1.11.6"],\
            ["@webassemblyjs/helper-api-error", "npm:1.11.6"],\
            ["@xtuc/long", "npm:4.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/helper-wasm-bytecode", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-helper-wasm-bytecode-npm-1.11.6-3bc23747de-3535ef4f1f.zip/node_modules/@webassemblyjs/helper-wasm-bytecode/",\
          "packageDependencies": [\
            ["@webassemblyjs/helper-wasm-bytecode", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/helper-wasm-section", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-helper-wasm-section-npm-1.11.6-344f8ff2af-b2cf751bf4.zip/node_modules/@webassemblyjs/helper-wasm-section/",\
          "packageDependencies": [\
            ["@webassemblyjs/helper-wasm-section", "npm:1.11.6"],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/helper-buffer", "npm:1.11.6"],\
            ["@webassemblyjs/helper-wasm-bytecode", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-gen", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/ieee754", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-ieee754-npm-1.11.6-95c92f446a-13574b8e41.zip/node_modules/@webassemblyjs/ieee754/",\
          "packageDependencies": [\
            ["@webassemblyjs/ieee754", "npm:1.11.6"],\
            ["@xtuc/ieee754", "npm:1.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/leb128", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-leb128-npm-1.11.6-697d62da2e-7ea942dc97.zip/node_modules/@webassemblyjs/leb128/",\
          "packageDependencies": [\
            ["@webassemblyjs/leb128", "npm:1.11.6"],\
            ["@xtuc/long", "npm:4.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/utf8", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-utf8-npm-1.11.6-102c4e5d68-807fe5b5ce.zip/node_modules/@webassemblyjs/utf8/",\
          "packageDependencies": [\
            ["@webassemblyjs/utf8", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/wasm-edit", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-wasm-edit-npm-1.11.6-8d2703f828-29ce758704.zip/node_modules/@webassemblyjs/wasm-edit/",\
          "packageDependencies": [\
            ["@webassemblyjs/wasm-edit", "npm:1.11.6"],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/helper-buffer", "npm:1.11.6"],\
            ["@webassemblyjs/helper-wasm-bytecode", "npm:1.11.6"],\
            ["@webassemblyjs/helper-wasm-section", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-gen", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-opt", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-parser", "npm:1.11.6"],\
            ["@webassemblyjs/wast-printer", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/wasm-gen", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-wasm-gen-npm-1.11.6-0ca036cab0-a645a2eecb.zip/node_modules/@webassemblyjs/wasm-gen/",\
          "packageDependencies": [\
            ["@webassemblyjs/wasm-gen", "npm:1.11.6"],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/helper-wasm-bytecode", "npm:1.11.6"],\
            ["@webassemblyjs/ieee754", "npm:1.11.6"],\
            ["@webassemblyjs/leb128", "npm:1.11.6"],\
            ["@webassemblyjs/utf8", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/wasm-opt", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-wasm-opt-npm-1.11.6-8be3443975-b4557f1954.zip/node_modules/@webassemblyjs/wasm-opt/",\
          "packageDependencies": [\
            ["@webassemblyjs/wasm-opt", "npm:1.11.6"],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/helper-buffer", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-gen", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-parser", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/wasm-parser", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-wasm-parser-npm-1.11.6-88e2433c21-8200a8d77c.zip/node_modules/@webassemblyjs/wasm-parser/",\
          "packageDependencies": [\
            ["@webassemblyjs/wasm-parser", "npm:1.11.6"],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/helper-api-error", "npm:1.11.6"],\
            ["@webassemblyjs/helper-wasm-bytecode", "npm:1.11.6"],\
            ["@webassemblyjs/ieee754", "npm:1.11.6"],\
            ["@webassemblyjs/leb128", "npm:1.11.6"],\
            ["@webassemblyjs/utf8", "npm:1.11.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webassemblyjs/wast-printer", [\
        ["npm:1.11.6", {\
          "packageLocation": "./.yarn/cache/@webassemblyjs-wast-printer-npm-1.11.6-3191861e3f-d2fa6a4c42.zip/node_modules/@webassemblyjs/wast-printer/",\
          "packageDependencies": [\
            ["@webassemblyjs/wast-printer", "npm:1.11.6"],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@xtuc/long", "npm:4.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webpack-cli/configtest", [\
        ["npm:1.2.0", {\
          "packageLocation": "./.yarn/cache/@webpack-cli-configtest-npm-1.2.0-728a6bf8bd-a2726cd9ec.zip/node_modules/@webpack-cli/configtest/",\
          "packageDependencies": [\
            ["@webpack-cli/configtest", "npm:1.2.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.2.0", {\
          "packageLocation": "./.yarn/__virtual__/@webpack-cli-configtest-virtual-ea4cd208f8/0/cache/@webpack-cli-configtest-npm-1.2.0-728a6bf8bd-a2726cd9ec.zip/node_modules/@webpack-cli/configtest/",\
          "packageDependencies": [\
            ["@webpack-cli/configtest", "virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.2.0"],\
            ["@types/webpack", null],\
            ["@types/webpack-cli", null],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["webpack-cli", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "@types/webpack",\
            "webpack-cli",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.2.0", {\
          "packageLocation": "./.yarn/__virtual__/@webpack-cli-configtest-virtual-8353492141/0/cache/@webpack-cli-configtest-npm-1.2.0-728a6bf8bd-a2726cd9ec.zip/node_modules/@webpack-cli/configtest/",\
          "packageDependencies": [\
            ["@webpack-cli/configtest", "virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.2.0"],\
            ["@types/webpack", null],\
            ["@types/webpack-cli", null],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"],\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "@types/webpack",\
            "webpack-cli",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webpack-cli/info", [\
        ["npm:1.5.0", {\
          "packageLocation": "./.yarn/cache/@webpack-cli-info-npm-1.5.0-9d9627ae19-7f56fe037c.zip/node_modules/@webpack-cli/info/",\
          "packageDependencies": [\
            ["@webpack-cli/info", "npm:1.5.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.5.0", {\
          "packageLocation": "./.yarn/__virtual__/@webpack-cli-info-virtual-e5931656ec/0/cache/@webpack-cli-info-npm-1.5.0-9d9627ae19-7f56fe037c.zip/node_modules/@webpack-cli/info/",\
          "packageDependencies": [\
            ["@webpack-cli/info", "virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.5.0"],\
            ["@types/webpack-cli", null],\
            ["envinfo", "npm:7.9.0"],\
            ["webpack-cli", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "webpack-cli"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.5.0", {\
          "packageLocation": "./.yarn/__virtual__/@webpack-cli-info-virtual-8afedcf117/0/cache/@webpack-cli-info-npm-1.5.0-9d9627ae19-7f56fe037c.zip/node_modules/@webpack-cli/info/",\
          "packageDependencies": [\
            ["@webpack-cli/info", "virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.5.0"],\
            ["@types/webpack-cli", null],\
            ["envinfo", "npm:7.9.0"],\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "webpack-cli"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@webpack-cli/serve", [\
        ["npm:1.7.0", {\
          "packageLocation": "./.yarn/cache/@webpack-cli-serve-npm-1.7.0-2869abfdb4-d475e8effa.zip/node_modules/@webpack-cli/serve/",\
          "packageDependencies": [\
            ["@webpack-cli/serve", "npm:1.7.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.7.0", {\
          "packageLocation": "./.yarn/__virtual__/@webpack-cli-serve-virtual-a16c9c7f5c/0/cache/@webpack-cli-serve-npm-1.7.0-2869abfdb4-d475e8effa.zip/node_modules/@webpack-cli/serve/",\
          "packageDependencies": [\
            ["@webpack-cli/serve", "virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.7.0"],\
            ["@types/webpack-cli", null],\
            ["@types/webpack-dev-server", null],\
            ["webpack-cli", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0"],\
            ["webpack-dev-server", null]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "@types/webpack-dev-server",\
            "webpack-cli",\
            "webpack-dev-server"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.7.0", {\
          "packageLocation": "./.yarn/__virtual__/@webpack-cli-serve-virtual-c3fceb0b29/0/cache/@webpack-cli-serve-npm-1.7.0-2869abfdb4-d475e8effa.zip/node_modules/@webpack-cli/serve/",\
          "packageDependencies": [\
            ["@webpack-cli/serve", "virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.7.0"],\
            ["@types/webpack-cli", null],\
            ["@types/webpack-dev-server", null],\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"],\
            ["webpack-dev-server", null]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "@types/webpack-dev-server",\
            "webpack-cli",\
            "webpack-dev-server"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@xtuc/ieee754", [\
        ["npm:1.2.0", {\
          "packageLocation": "./.yarn/cache/@xtuc-ieee754-npm-1.2.0-ec0ce4e025-ac56d4ca6e.zip/node_modules/@xtuc/ieee754/",\
          "packageDependencies": [\
            ["@xtuc/ieee754", "npm:1.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["@xtuc/long", [\
        ["npm:4.2.2", {\
          "packageLocation": "./.yarn/cache/@xtuc-long-npm-4.2.2-37236e6d72-8ed0d477ce.zip/node_modules/@xtuc/long/",\
          "packageDependencies": [\
            ["@xtuc/long", "npm:4.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["abab", [\
        ["npm:2.0.6", {\
          "packageLocation": "./.yarn/cache/abab-npm-2.0.6-2662fba7f0-6ffc1af4ff.zip/node_modules/abab/",\
          "packageDependencies": [\
            ["abab", "npm:2.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["abbrev", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/abbrev-npm-1.1.1-3659247eab-a4a97ec07d.zip/node_modules/abbrev/",\
          "packageDependencies": [\
            ["abbrev", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["acorn", [\
        ["npm:7.4.1", {\
          "packageLocation": "./.yarn/cache/acorn-npm-7.4.1-f450b4646c-1860f23c21.zip/node_modules/acorn/",\
          "packageDependencies": [\
            ["acorn", "npm:7.4.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.9.0", {\
          "packageLocation": "./.yarn/cache/acorn-npm-8.9.0-4ebbf0f638-25dfb94952.zip/node_modules/acorn/",\
          "packageDependencies": [\
            ["acorn", "npm:8.9.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["acorn-globals", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/acorn-globals-npm-6.0.0-acbec28ad5-72d95e5b5e.zip/node_modules/acorn-globals/",\
          "packageDependencies": [\
            ["acorn-globals", "npm:6.0.0"],\
            ["acorn", "npm:7.4.1"],\
            ["acorn-walk", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["acorn-import-assertions", [\
        ["npm:1.9.0", {\
          "packageLocation": "./.yarn/cache/acorn-import-assertions-npm-1.9.0-22f56507c7-944fb2659d.zip/node_modules/acorn-import-assertions/",\
          "packageDependencies": [\
            ["acorn-import-assertions", "npm:1.9.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:1.9.0", {\
          "packageLocation": "./.yarn/__virtual__/acorn-import-assertions-virtual-4cbe79b305/0/cache/acorn-import-assertions-npm-1.9.0-22f56507c7-944fb2659d.zip/node_modules/acorn-import-assertions/",\
          "packageDependencies": [\
            ["acorn-import-assertions", "virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:1.9.0"],\
            ["@types/acorn", null],\
            ["acorn", "npm:8.9.0"]\
          ],\
          "packagePeers": [\
            "@types/acorn",\
            "acorn"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["acorn-jsx", [\
        ["npm:5.3.2", {\
          "packageLocation": "./.yarn/cache/acorn-jsx-npm-5.3.2-d7594599ea-c3d3b2a89c.zip/node_modules/acorn-jsx/",\
          "packageDependencies": [\
            ["acorn-jsx", "npm:5.3.2"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:8d8ea5d1e3376905d0290522290f47c29213c64d936d96293d758a315829a3cf4c6a5b8ffc1cfee36c3db08f700ad3aaf0711cc5d406a7218c275de6d74effa9#npm:5.3.2", {\
          "packageLocation": "./.yarn/__virtual__/acorn-jsx-virtual-6934646a20/0/cache/acorn-jsx-npm-5.3.2-d7594599ea-c3d3b2a89c.zip/node_modules/acorn-jsx/",\
          "packageDependencies": [\
            ["acorn-jsx", "virtual:8d8ea5d1e3376905d0290522290f47c29213c64d936d96293d758a315829a3cf4c6a5b8ffc1cfee36c3db08f700ad3aaf0711cc5d406a7218c275de6d74effa9#npm:5.3.2"],\
            ["@types/acorn", null],\
            ["acorn", "npm:7.4.1"]\
          ],\
          "packagePeers": [\
            "@types/acorn",\
            "acorn"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["acorn-walk", [\
        ["npm:7.2.0", {\
          "packageLocation": "./.yarn/cache/acorn-walk-npm-7.2.0-5f8b515308-9252158a79.zip/node_modules/acorn-walk/",\
          "packageDependencies": [\
            ["acorn-walk", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["active-x-obfuscator", [\
        ["npm:0.0.1", {\
          "packageLocation": "./.yarn/cache/active-x-obfuscator-npm-0.0.1-6b0d77e025-5779802ece.zip/node_modules/active-x-obfuscator/",\
          "packageDependencies": [\
            ["active-x-obfuscator", "npm:0.0.1"],\
            ["zeparser", "npm:0.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["agent-base", [\
        ["npm:6.0.2", {\
          "packageLocation": "./.yarn/cache/agent-base-npm-6.0.2-428f325a93-f52b6872cc.zip/node_modules/agent-base/",\
          "packageDependencies": [\
            ["agent-base", "npm:6.0.2"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["agentkeepalive", [\
        ["npm:4.3.0", {\
          "packageLocation": "./.yarn/cache/agentkeepalive-npm-4.3.0-ac3d8e6807-982453aa44.zip/node_modules/agentkeepalive/",\
          "packageDependencies": [\
            ["agentkeepalive", "npm:4.3.0"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["depd", "npm:2.0.0"],\
            ["humanize-ms", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["aggregate-error", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/aggregate-error-npm-3.1.0-415a406f4e-1101a33f21.zip/node_modules/aggregate-error/",\
          "packageDependencies": [\
            ["aggregate-error", "npm:3.1.0"],\
            ["clean-stack", "npm:2.2.0"],\
            ["indent-string", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ajv", [\
        ["npm:6.12.6", {\
          "packageLocation": "./.yarn/cache/ajv-npm-6.12.6-4b5105e2b2-874972efe5.zip/node_modules/ajv/",\
          "packageDependencies": [\
            ["ajv", "npm:6.12.6"],\
            ["fast-deep-equal", "npm:3.1.3"],\
            ["fast-json-stable-stringify", "npm:2.1.0"],\
            ["json-schema-traverse", "npm:0.4.1"],\
            ["uri-js", "npm:4.4.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.12.0", {\
          "packageLocation": "./.yarn/cache/ajv-npm-8.12.0-3bf6e30741-4dc13714e3.zip/node_modules/ajv/",\
          "packageDependencies": [\
            ["ajv", "npm:8.12.0"],\
            ["fast-deep-equal", "npm:3.1.3"],\
            ["json-schema-traverse", "npm:1.0.0"],\
            ["require-from-string", "npm:2.0.2"],\
            ["uri-js", "npm:4.4.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ajv-keywords", [\
        ["npm:3.5.2", {\
          "packageLocation": "./.yarn/cache/ajv-keywords-npm-3.5.2-0e391b70e2-7dc5e59316.zip/node_modules/ajv-keywords/",\
          "packageDependencies": [\
            ["ajv-keywords", "npm:3.5.2"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:f2b36937f163b579815d3163513b3330d7a31aaf0599eea66346382b8838395c613f4204e9809cc2ff6bba09c17ab0c34b37deadcb147de7e2f5e535d6ccc245#npm:3.5.2", {\
          "packageLocation": "./.yarn/__virtual__/ajv-keywords-virtual-80fc73abbe/0/cache/ajv-keywords-npm-3.5.2-0e391b70e2-7dc5e59316.zip/node_modules/ajv-keywords/",\
          "packageDependencies": [\
            ["ajv-keywords", "virtual:f2b36937f163b579815d3163513b3330d7a31aaf0599eea66346382b8838395c613f4204e9809cc2ff6bba09c17ab0c34b37deadcb147de7e2f5e535d6ccc245#npm:3.5.2"],\
            ["@types/ajv", null],\
            ["ajv", "npm:6.12.6"]\
          ],\
          "packagePeers": [\
            "@types/ajv",\
            "ajv"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ansi-colors", [\
        ["npm:4.1.3", {\
          "packageLocation": "./.yarn/cache/ansi-colors-npm-4.1.3-8ffd0ae6c7-a9c2ec8420.zip/node_modules/ansi-colors/",\
          "packageDependencies": [\
            ["ansi-colors", "npm:4.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ansi-escapes", [\
        ["npm:4.3.2", {\
          "packageLocation": "./.yarn/cache/ansi-escapes-npm-4.3.2-3ad173702f-93111c4218.zip/node_modules/ansi-escapes/",\
          "packageDependencies": [\
            ["ansi-escapes", "npm:4.3.2"],\
            ["type-fest", "npm:0.21.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ansi-regex", [\
        ["npm:5.0.1", {\
          "packageLocation": "./.yarn/cache/ansi-regex-npm-5.0.1-c963a48615-2aa4bb54ca.zip/node_modules/ansi-regex/",\
          "packageDependencies": [\
            ["ansi-regex", "npm:5.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.0.1", {\
          "packageLocation": "./.yarn/cache/ansi-regex-npm-6.0.1-8d663a607d-1ff8b7667c.zip/node_modules/ansi-regex/",\
          "packageDependencies": [\
            ["ansi-regex", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ansi-styles", [\
        ["npm:3.2.1", {\
          "packageLocation": "./.yarn/cache/ansi-styles-npm-3.2.1-8cb8107983-d85ade01c1.zip/node_modules/ansi-styles/",\
          "packageDependencies": [\
            ["ansi-styles", "npm:3.2.1"],\
            ["color-convert", "npm:1.9.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.3.0", {\
          "packageLocation": "./.yarn/cache/ansi-styles-npm-4.3.0-245c7d42c7-513b44c3b2.zip/node_modules/ansi-styles/",\
          "packageDependencies": [\
            ["ansi-styles", "npm:4.3.0"],\
            ["color-convert", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.2.1", {\
          "packageLocation": "./.yarn/cache/ansi-styles-npm-6.2.1-d43647018c-ef940f2f0c.zip/node_modules/ansi-styles/",\
          "packageDependencies": [\
            ["ansi-styles", "npm:6.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["anymatch", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/anymatch-npm-2.0.0-f2fcb92f28-f7bb192984.zip/node_modules/anymatch/",\
          "packageDependencies": [\
            ["anymatch", "npm:2.0.0"],\
            ["micromatch", "npm:3.1.10"],\
            ["normalize-path", "npm:2.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.1.3", {\
          "packageLocation": "./.yarn/cache/anymatch-npm-3.1.3-bc81d103b1-3e044fd6d1.zip/node_modules/anymatch/",\
          "packageDependencies": [\
            ["anymatch", "npm:3.1.3"],\
            ["normalize-path", "npm:3.0.0"],\
            ["picomatch", "npm:2.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["aproba", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/aproba-npm-2.0.0-8716bcfde6-5615cadcfb.zip/node_modules/aproba/",\
          "packageDependencies": [\
            ["aproba", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["are-we-there-yet", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/are-we-there-yet-npm-3.0.1-3395b1512f-52590c2486.zip/node_modules/are-we-there-yet/",\
          "packageDependencies": [\
            ["are-we-there-yet", "npm:3.0.1"],\
            ["delegates", "npm:1.0.0"],\
            ["readable-stream", "npm:3.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["argparse", [\
        ["npm:1.0.10", {\
          "packageLocation": "./.yarn/cache/argparse-npm-1.0.10-528934e59d-7ca6e45583.zip/node_modules/argparse/",\
          "packageDependencies": [\
            ["argparse", "npm:1.0.10"],\
            ["sprintf-js", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["arr-diff", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/arr-diff-npm-4.0.0-cec86ae312-ea7c883484.zip/node_modules/arr-diff/",\
          "packageDependencies": [\
            ["arr-diff", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["arr-flatten", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/arr-flatten-npm-1.1.0-0c12b693e4-963fe12564.zip/node_modules/arr-flatten/",\
          "packageDependencies": [\
            ["arr-flatten", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["arr-union", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/arr-union-npm-3.1.0-853ada9729-b5b0408c6e.zip/node_modules/arr-union/",\
          "packageDependencies": [\
            ["arr-union", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["array-buffer-byte-length", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/array-buffer-byte-length-npm-1.0.0-331671f28a-044e101ce1.zip/node_modules/array-buffer-byte-length/",\
          "packageDependencies": [\
            ["array-buffer-byte-length", "npm:1.0.0"],\
            ["call-bind", "npm:1.0.2"],\
            ["is-array-buffer", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["array-unique", [\
        ["npm:0.3.2", {\
          "packageLocation": "./.yarn/cache/array-unique-npm-0.3.2-9f62c6ac93-da344b89cf.zip/node_modules/array-unique/",\
          "packageDependencies": [\
            ["array-unique", "npm:0.3.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["assign-symbols", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/assign-symbols-npm-1.0.0-fd803ccdf1-c0eb895911.zip/node_modules/assign-symbols/",\
          "packageDependencies": [\
            ["assign-symbols", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["astral-regex", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/astral-regex-npm-2.0.0-f30d866aab-876231688c.zip/node_modules/astral-regex/",\
          "packageDependencies": [\
            ["astral-regex", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["asynckit", [\
        ["npm:0.4.0", {\
          "packageLocation": "./.yarn/cache/asynckit-npm-0.4.0-c718858525-7b78c451df.zip/node_modules/asynckit/",\
          "packageDependencies": [\
            ["asynckit", "npm:0.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["at-least-node", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/at-least-node-npm-1.0.0-2b36e661fa-463e2f8e43.zip/node_modules/at-least-node/",\
          "packageDependencies": [\
            ["at-least-node", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["atob", [\
        ["npm:2.1.2", {\
          "packageLocation": "./.yarn/cache/atob-npm-2.1.2-bcb583261e-dfeeeb7009.zip/node_modules/atob/",\
          "packageDependencies": [\
            ["atob", "npm:2.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["available-typed-arrays", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/available-typed-arrays-npm-1.0.5-88f321e4d3-20eb47b3ce.zip/node_modules/available-typed-arrays/",\
          "packageDependencies": [\
            ["available-typed-arrays", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-jest", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/babel-jest-npm-26.6.3-5630fee2b8-5917233f0d.zip/node_modules/babel-jest/",\
          "packageDependencies": [\
            ["babel-jest", "npm:26.6.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:caddf51df4928b33a437ca87b8f5ddfb6205ebd6d8231f74d4ee7223f3866e6f815b221aa1e2bd33e98915f701e95bae72a93d2288b49a34a6246bdbc2a4a132#npm:26.6.3", {\
          "packageLocation": "./.yarn/__virtual__/babel-jest-virtual-9f33f3a3f1/0/cache/babel-jest-npm-26.6.3-5630fee2b8-5917233f0d.zip/node_modules/babel-jest/",\
          "packageDependencies": [\
            ["babel-jest", "virtual:caddf51df4928b33a437ca87b8f5ddfb6205ebd6d8231f74d4ee7223f3866e6f815b221aa1e2bd33e98915f701e95bae72a93d2288b49a34a6246bdbc2a4a132#npm:26.6.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@jest/transform", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/babel__core", "npm:7.20.1"],\
            ["babel-plugin-istanbul", "npm:6.1.1"],\
            ["babel-preset-jest", "virtual:9f33f3a3f1029c851d4fc6512707159198b8a9185a8bf2a04087a9e7410eb7514881ea8169195fe0d559191ceae65b7d1a505a59d1ebb3a00c8619a55d48aa40#npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["slash", "npm:3.0.0"]\
          ],\
          "packagePeers": [\
            "@babel/core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-plugin-istanbul", [\
        ["npm:6.1.1", {\
          "packageLocation": "./.yarn/cache/babel-plugin-istanbul-npm-6.1.1-df824055e4-cb4fd95738.zip/node_modules/babel-plugin-istanbul/",\
          "packageDependencies": [\
            ["babel-plugin-istanbul", "npm:6.1.1"],\
            ["@babel/helper-plugin-utils", "npm:7.22.5"],\
            ["@istanbuljs/load-nyc-config", "npm:1.1.0"],\
            ["@istanbuljs/schema", "npm:0.1.3"],\
            ["istanbul-lib-instrument", "npm:5.2.1"],\
            ["test-exclude", "npm:6.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-plugin-jest-hoist", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/babel-plugin-jest-hoist-npm-26.6.2-1a51633e87-abe3732fdf.zip/node_modules/babel-plugin-jest-hoist/",\
          "packageDependencies": [\
            ["babel-plugin-jest-hoist", "npm:26.6.2"],\
            ["@babel/template", "npm:7.22.5"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"],\
            ["@types/babel__traverse", "npm:7.20.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-plugin-polyfill-corejs2", [\
        ["npm:0.4.3", {\
          "packageLocation": "./.yarn/cache/babel-plugin-polyfill-corejs2-npm-0.4.3-4dbaaa55cd-09ba40b9f8.zip/node_modules/babel-plugin-polyfill-corejs2/",\
          "packageDependencies": [\
            ["babel-plugin-polyfill-corejs2", "npm:0.4.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.4.3", {\
          "packageLocation": "./.yarn/__virtual__/babel-plugin-polyfill-corejs2-virtual-37199b5719/0/cache/babel-plugin-polyfill-corejs2-npm-0.4.3-4dbaaa55cd-09ba40b9f8.zip/node_modules/babel-plugin-polyfill-corejs2/",\
          "packageDependencies": [\
            ["babel-plugin-polyfill-corejs2", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.4.3"],\
            ["@babel/compat-data", "npm:7.22.5"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-define-polyfill-provider", "virtual:37199b5719e67bca157608a02dec1de2f89dc59c4bc7ba7f9107eb5bf1d1f85a15b122a67078197e23664014e4511167d3b3e53ce55f6f59a1c05d9d6f083563#npm:0.4.0"],\
            ["@types/babel__core", null],\
            ["semver", "npm:6.3.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-plugin-polyfill-corejs3", [\
        ["npm:0.8.1", {\
          "packageLocation": "./.yarn/cache/babel-plugin-polyfill-corejs3-npm-0.8.1-dfd8a5cfee-c23a581973.zip/node_modules/babel-plugin-polyfill-corejs3/",\
          "packageDependencies": [\
            ["babel-plugin-polyfill-corejs3", "npm:0.8.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.8.1", {\
          "packageLocation": "./.yarn/__virtual__/babel-plugin-polyfill-corejs3-virtual-06dbd1a64d/0/cache/babel-plugin-polyfill-corejs3-npm-0.8.1-dfd8a5cfee-c23a581973.zip/node_modules/babel-plugin-polyfill-corejs3/",\
          "packageDependencies": [\
            ["babel-plugin-polyfill-corejs3", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.8.1"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-define-polyfill-provider", "virtual:37199b5719e67bca157608a02dec1de2f89dc59c4bc7ba7f9107eb5bf1d1f85a15b122a67078197e23664014e4511167d3b3e53ce55f6f59a1c05d9d6f083563#npm:0.4.0"],\
            ["@types/babel__core", null],\
            ["core-js-compat", "npm:3.31.0"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-plugin-polyfill-regenerator", [\
        ["npm:0.5.0", {\
          "packageLocation": "./.yarn/cache/babel-plugin-polyfill-regenerator-npm-0.5.0-ec64198820-ef2bcffc7c.zip/node_modules/babel-plugin-polyfill-regenerator/",\
          "packageDependencies": [\
            ["babel-plugin-polyfill-regenerator", "npm:0.5.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.5.0", {\
          "packageLocation": "./.yarn/__virtual__/babel-plugin-polyfill-regenerator-virtual-5521ff7c9e/0/cache/babel-plugin-polyfill-regenerator-npm-0.5.0-ec64198820-ef2bcffc7c.zip/node_modules/babel-plugin-polyfill-regenerator/",\
          "packageDependencies": [\
            ["babel-plugin-polyfill-regenerator", "virtual:31c9cd7546f215e21c2ad8310f7827626a8086ff4fdf73ea77b6c72ab3be45efd527157531629bf2995f7b94104a19e2cb34fb3f1f8f21b964d8a69b227d1f9f#npm:0.5.0"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/helper-define-polyfill-provider", "virtual:37199b5719e67bca157608a02dec1de2f89dc59c4bc7ba7f9107eb5bf1d1f85a15b122a67078197e23664014e4511167d3b3e53ce55f6f59a1c05d9d6f083563#npm:0.4.0"],\
            ["@types/babel__core", null]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-preset-current-node-syntax", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/babel-preset-current-node-syntax-npm-1.0.1-849ec71e32-d118c27424.zip/node_modules/babel-preset-current-node-syntax/",\
          "packageDependencies": [\
            ["babel-preset-current-node-syntax", "npm:1.0.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:953f19a93f1fd76e03de2418470b56b6b69d974bef4780f65029f0a9afcdf2f0130bfb897370a8fe1bd125d7f8113ee186caf42ece0a92789a64117e47028619#npm:1.0.1", {\
          "packageLocation": "./.yarn/__virtual__/babel-preset-current-node-syntax-virtual-8f974720c8/0/cache/babel-preset-current-node-syntax-npm-1.0.1-849ec71e32-d118c27424.zip/node_modules/babel-preset-current-node-syntax/",\
          "packageDependencies": [\
            ["babel-preset-current-node-syntax", "virtual:953f19a93f1fd76e03de2418470b56b6b69d974bef4780f65029f0a9afcdf2f0130bfb897370a8fe1bd125d7f8113ee186caf42ece0a92789a64117e47028619#npm:1.0.1"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/plugin-syntax-async-generators", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.4"],\
            ["@babel/plugin-syntax-bigint", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/plugin-syntax-class-properties", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.12.13"],\
            ["@babel/plugin-syntax-import-meta", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4"],\
            ["@babel/plugin-syntax-json-strings", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/plugin-syntax-logical-assignment-operators", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4"],\
            ["@babel/plugin-syntax-nullish-coalescing-operator", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/plugin-syntax-numeric-separator", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.10.4"],\
            ["@babel/plugin-syntax-object-rest-spread", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/plugin-syntax-optional-catch-binding", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/plugin-syntax-optional-chaining", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.8.3"],\
            ["@babel/plugin-syntax-top-level-await", "virtual:8f974720c8a9b4f406c1db7289a59d677ba2dd045b77299345a92de141b4ac7e66b8ed54a33591ce2e62d069308b508e7fafa6082df6a96611654b7d157524cd#npm:7.14.5"],\
            ["@types/babel__core", "npm:7.20.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["babel-preset-jest", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/babel-preset-jest-npm-26.6.2-456ec36f85-1d9bef3a7a.zip/node_modules/babel-preset-jest/",\
          "packageDependencies": [\
            ["babel-preset-jest", "npm:26.6.2"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:9f33f3a3f1029c851d4fc6512707159198b8a9185a8bf2a04087a9e7410eb7514881ea8169195fe0d559191ceae65b7d1a505a59d1ebb3a00c8619a55d48aa40#npm:26.6.2", {\
          "packageLocation": "./.yarn/__virtual__/babel-preset-jest-virtual-953f19a93f/0/cache/babel-preset-jest-npm-26.6.2-456ec36f85-1d9bef3a7a.zip/node_modules/babel-preset-jest/",\
          "packageDependencies": [\
            ["babel-preset-jest", "virtual:9f33f3a3f1029c851d4fc6512707159198b8a9185a8bf2a04087a9e7410eb7514881ea8169195fe0d559191ceae65b7d1a505a59d1ebb3a00c8619a55d48aa40#npm:26.6.2"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@types/babel__core", "npm:7.20.1"],\
            ["babel-plugin-jest-hoist", "npm:26.6.2"],\
            ["babel-preset-current-node-syntax", "virtual:953f19a93f1fd76e03de2418470b56b6b69d974bef4780f65029f0a9afcdf2f0130bfb897370a8fe1bd125d7f8113ee186caf42ece0a92789a64117e47028619#npm:1.0.1"]\
          ],\
          "packagePeers": [\
            "@babel/core",\
            "@types/babel__core"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["backbone", [\
        ["npm:1.4.0", {\
          "packageLocation": "./.yarn/cache/backbone-npm-1.4.0-054c19a6ab-09abdf184c.zip/node_modules/backbone/",\
          "packageDependencies": [\
            ["backbone", "npm:1.4.0"],\
            ["underscore", "npm:1.13.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["balanced-match", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/balanced-match-npm-1.0.2-a53c126459-9706c088a2.zip/node_modules/balanced-match/",\
          "packageDependencies": [\
            ["balanced-match", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["base", [\
        ["npm:0.11.2", {\
          "packageLocation": "./.yarn/cache/base-npm-0.11.2-a9bde462d6-a4a146b912.zip/node_modules/base/",\
          "packageDependencies": [\
            ["base", "npm:0.11.2"],\
            ["cache-base", "npm:1.0.1"],\
            ["class-utils", "npm:0.3.6"],\
            ["component-emitter", "npm:1.3.0"],\
            ["define-property", "npm:1.0.0"],\
            ["isobject", "npm:3.0.1"],\
            ["mixin-deep", "npm:1.3.2"],\
            ["pascalcase", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["base64-js", [\
        ["npm:1.5.1", {\
          "packageLocation": "./.yarn/cache/base64-js-npm-1.5.1-b2f7275641-669632eb37.zip/node_modules/base64-js/",\
          "packageDependencies": [\
            ["base64-js", "npm:1.5.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["base64id", [\
        ["npm:0.1.0", {\
          "packageLocation": "./.yarn/cache/base64id-npm-0.1.0-578e261433-ef3b3dd632.zip/node_modules/base64id/",\
          "packageDependencies": [\
            ["base64id", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["big.js", [\
        ["npm:5.2.2", {\
          "packageLocation": "./.yarn/cache/big.js-npm-5.2.2-e147c30820-b89b6e8419.zip/node_modules/big.js/",\
          "packageDependencies": [\
            ["big.js", "npm:5.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["brace-expansion", [\
        ["npm:1.1.11", {\
          "packageLocation": "./.yarn/cache/brace-expansion-npm-1.1.11-fb95eb05ad-faf34a7bb0.zip/node_modules/brace-expansion/",\
          "packageDependencies": [\
            ["brace-expansion", "npm:1.1.11"],\
            ["balanced-match", "npm:1.0.2"],\
            ["concat-map", "npm:0.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/brace-expansion-npm-2.0.1-17aa2616f9-a61e7cd2e8.zip/node_modules/brace-expansion/",\
          "packageDependencies": [\
            ["brace-expansion", "npm:2.0.1"],\
            ["balanced-match", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["braces", [\
        ["npm:2.3.2", {\
          "packageLocation": "./.yarn/cache/braces-npm-2.3.2-19cadb3384-e30dcb6aaf.zip/node_modules/braces/",\
          "packageDependencies": [\
            ["braces", "npm:2.3.2"],\
            ["arr-flatten", "npm:1.1.0"],\
            ["array-unique", "npm:0.3.2"],\
            ["extend-shallow", "npm:2.0.1"],\
            ["fill-range", "npm:4.0.0"],\
            ["isobject", "npm:3.0.1"],\
            ["repeat-element", "npm:1.1.4"],\
            ["snapdragon", "npm:0.8.2"],\
            ["snapdragon-node", "npm:2.1.1"],\
            ["split-string", "npm:3.1.0"],\
            ["to-regex", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.2", {\
          "packageLocation": "./.yarn/cache/braces-npm-3.0.2-782240b28a-e2a8e769a8.zip/node_modules/braces/",\
          "packageDependencies": [\
            ["braces", "npm:3.0.2"],\
            ["fill-range", "npm:7.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["browser-process-hrtime", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/browser-process-hrtime-npm-1.0.0-db700805c2-e30f868cdb.zip/node_modules/browser-process-hrtime/",\
          "packageDependencies": [\
            ["browser-process-hrtime", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["browserslist", [\
        ["npm:4.21.9", {\
          "packageLocation": "./.yarn/cache/browserslist-npm-4.21.9-f6128308c1-80d3820584.zip/node_modules/browserslist/",\
          "packageDependencies": [\
            ["browserslist", "npm:4.21.9"],\
            ["caniuse-lite", "npm:1.0.30001507"],\
            ["electron-to-chromium", "npm:1.4.439"],\
            ["node-releases", "npm:2.0.12"],\
            ["update-browserslist-db", "virtual:f6128308c12e2b37fe427262226a88e213eb66128b3445db14a7af8a6dafb815a9c4562e232b574e216fb796ee19313d71844b195e41ad82170480289a6b575c#npm:1.0.11"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["bs-logger", [\
        ["npm:0.2.6", {\
          "packageLocation": "./.yarn/cache/bs-logger-npm-0.2.6-7670f88b66-d34bdaf68c.zip/node_modules/bs-logger/",\
          "packageDependencies": [\
            ["bs-logger", "npm:0.2.6"],\
            ["fast-json-stable-stringify", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["bser", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/bser-npm-2.1.1-cc902055ce-9ba4dc58ce.zip/node_modules/bser/",\
          "packageDependencies": [\
            ["bser", "npm:2.1.1"],\
            ["node-int64", "npm:0.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["buffer-from", [\
        ["npm:1.1.2", {\
          "packageLocation": "./.yarn/cache/buffer-from-npm-1.1.2-03d2f20d7e-0448524a56.zip/node_modules/buffer-from/",\
          "packageDependencies": [\
            ["buffer-from", "npm:1.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cacache", [\
        ["npm:15.3.0", {\
          "packageLocation": "./.yarn/cache/cacache-npm-15.3.0-a7e5239c6a-a07327c27a.zip/node_modules/cacache/",\
          "packageDependencies": [\
            ["cacache", "npm:15.3.0"],\
            ["@npmcli/fs", "npm:1.1.1"],\
            ["@npmcli/move-file", "npm:1.1.2"],\
            ["chownr", "npm:2.0.0"],\
            ["fs-minipass", "npm:2.1.0"],\
            ["glob", "npm:7.2.3"],\
            ["infer-owner", "npm:1.0.4"],\
            ["lru-cache", "npm:6.0.0"],\
            ["minipass", "npm:3.3.6"],\
            ["minipass-collect", "npm:1.0.2"],\
            ["minipass-flush", "npm:1.0.5"],\
            ["minipass-pipeline", "npm:1.2.4"],\
            ["mkdirp", "npm:1.0.4"],\
            ["p-map", "npm:4.0.0"],\
            ["promise-inflight", "virtual:a7e5239c6ae68bf6359adfd3598326db000e94dbb349bc00a3852ed53a31712a0e2e787228c6e859d3e5cf2fbb872aba1ea4abe4995cef8086a77ef619ae1be6#npm:1.0.1"],\
            ["rimraf", "npm:3.0.2"],\
            ["ssri", "npm:8.0.1"],\
            ["tar", "npm:6.1.15"],\
            ["unique-filename", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:17.1.3", {\
          "packageLocation": "./.yarn/cache/cacache-npm-17.1.3-f75f768a29-385756781e.zip/node_modules/cacache/",\
          "packageDependencies": [\
            ["cacache", "npm:17.1.3"],\
            ["@npmcli/fs", "npm:3.1.0"],\
            ["fs-minipass", "npm:3.0.2"],\
            ["glob", "npm:10.3.0"],\
            ["lru-cache", "npm:7.18.3"],\
            ["minipass", "npm:5.0.0"],\
            ["minipass-collect", "npm:1.0.2"],\
            ["minipass-flush", "npm:1.0.5"],\
            ["minipass-pipeline", "npm:1.2.4"],\
            ["p-map", "npm:4.0.0"],\
            ["ssri", "npm:10.0.4"],\
            ["tar", "npm:6.1.15"],\
            ["unique-filename", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cache-base", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/cache-base-npm-1.0.1-1538417cb9-9114b8654f.zip/node_modules/cache-base/",\
          "packageDependencies": [\
            ["cache-base", "npm:1.0.1"],\
            ["collection-visit", "npm:1.0.0"],\
            ["component-emitter", "npm:1.3.0"],\
            ["get-value", "npm:2.0.6"],\
            ["has-value", "npm:1.0.0"],\
            ["isobject", "npm:3.0.1"],\
            ["set-value", "npm:2.0.1"],\
            ["to-object-path", "npm:0.3.0"],\
            ["union-value", "npm:1.0.1"],\
            ["unset-value", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["call-bind", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/call-bind-npm-1.0.2-c957124861-f8e31de9d1.zip/node_modules/call-bind/",\
          "packageDependencies": [\
            ["call-bind", "npm:1.0.2"],\
            ["function-bind", "npm:1.1.1"],\
            ["get-intrinsic", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["callsites", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/callsites-npm-3.1.0-268f989910-072d17b6ab.zip/node_modules/callsites/",\
          "packageDependencies": [\
            ["callsites", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["camelcase", [\
        ["npm:5.3.1", {\
          "packageLocation": "./.yarn/cache/camelcase-npm-5.3.1-5db8af62c5-e6effce26b.zip/node_modules/camelcase/",\
          "packageDependencies": [\
            ["camelcase", "npm:5.3.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.3.0", {\
          "packageLocation": "./.yarn/cache/camelcase-npm-6.3.0-e5e42a0d15-8c96818a90.zip/node_modules/camelcase/",\
          "packageDependencies": [\
            ["camelcase", "npm:6.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["caniuse-lite", [\
        ["npm:1.0.30001507", {\
          "packageLocation": "./.yarn/cache/caniuse-lite-npm-1.0.30001507-b3531bb98e-7044172bdf.zip/node_modules/caniuse-lite/",\
          "packageDependencies": [\
            ["caniuse-lite", "npm:1.0.30001507"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["capture-exit", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/capture-exit-npm-2.0.0-564874b447-0b9f10daca.zip/node_modules/capture-exit/",\
          "packageDependencies": [\
            ["capture-exit", "npm:2.0.0"],\
            ["rsvp", "npm:4.8.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["chalk", [\
        ["npm:2.4.2", {\
          "packageLocation": "./.yarn/cache/chalk-npm-2.4.2-3ea16dd91e-ec3661d38f.zip/node_modules/chalk/",\
          "packageDependencies": [\
            ["chalk", "npm:2.4.2"],\
            ["ansi-styles", "npm:3.2.1"],\
            ["escape-string-regexp", "npm:1.0.5"],\
            ["supports-color", "npm:5.5.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.1.2", {\
          "packageLocation": "./.yarn/cache/chalk-npm-4.1.2-ba8b67ab80-fe75c9d5c7.zip/node_modules/chalk/",\
          "packageDependencies": [\
            ["chalk", "npm:4.1.2"],\
            ["ansi-styles", "npm:4.3.0"],\
            ["supports-color", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["char-regex", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/char-regex-npm-1.0.2-ecade5f97f-b563e4b603.zip/node_modules/char-regex/",\
          "packageDependencies": [\
            ["char-regex", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["chownr", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/chownr-npm-2.0.0-638f1c9c61-c57cf9dd07.zip/node_modules/chownr/",\
          "packageDependencies": [\
            ["chownr", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["chrome-trace-event", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/chrome-trace-event-npm-1.0.3-e0ae3dcd60-cb8b1fc7e8.zip/node_modules/chrome-trace-event/",\
          "packageDependencies": [\
            ["chrome-trace-event", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ci-info", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/ci-info-npm-2.0.0-78012236a1-3b374666a8.zip/node_modules/ci-info/",\
          "packageDependencies": [\
            ["ci-info", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cjs-module-lexer", [\
        ["npm:0.6.0", {\
          "packageLocation": "./.yarn/cache/cjs-module-lexer-npm-0.6.0-e80f3766d3-445b039607.zip/node_modules/cjs-module-lexer/",\
          "packageDependencies": [\
            ["cjs-module-lexer", "npm:0.6.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["class-utils", [\
        ["npm:0.3.6", {\
          "packageLocation": "./.yarn/cache/class-utils-npm-0.3.6-2c691ad006-be10890080.zip/node_modules/class-utils/",\
          "packageDependencies": [\
            ["class-utils", "npm:0.3.6"],\
            ["arr-union", "npm:3.1.0"],\
            ["define-property", "npm:0.2.5"],\
            ["isobject", "npm:3.0.1"],\
            ["static-extend", "npm:0.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["clean-stack", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/clean-stack-npm-2.2.0-a8ce435a5c-2ac8cd2b2f.zip/node_modules/clean-stack/",\
          "packageDependencies": [\
            ["clean-stack", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cliui", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/cliui-npm-6.0.0-488b2414c6-4fcfd26d29.zip/node_modules/cliui/",\
          "packageDependencies": [\
            ["cliui", "npm:6.0.0"],\
            ["string-width", "npm:4.2.3"],\
            ["strip-ansi", "npm:6.0.1"],\
            ["wrap-ansi", "npm:6.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["clone-deep", [\
        ["npm:4.0.1", {\
          "packageLocation": "./.yarn/cache/clone-deep-npm-4.0.1-70adab92c8-770f912fe4.zip/node_modules/clone-deep/",\
          "packageDependencies": [\
            ["clone-deep", "npm:4.0.1"],\
            ["is-plain-object", "npm:2.0.4"],\
            ["kind-of", "npm:6.0.3"],\
            ["shallow-clone", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["co", [\
        ["npm:4.6.0", {\
          "packageLocation": "./.yarn/cache/co-npm-4.6.0-03f2d1feb6-5210d92230.zip/node_modules/co/",\
          "packageDependencies": [\
            ["co", "npm:4.6.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["collect-v8-coverage", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/collect-v8-coverage-npm-1.0.1-39dec86bad-4efe0a1fcc.zip/node_modules/collect-v8-coverage/",\
          "packageDependencies": [\
            ["collect-v8-coverage", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["collection-visit", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/collection-visit-npm-1.0.0-aba2d5defc-15d9658fe6.zip/node_modules/collection-visit/",\
          "packageDependencies": [\
            ["collection-visit", "npm:1.0.0"],\
            ["map-visit", "npm:1.0.0"],\
            ["object-visit", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["color-convert", [\
        ["npm:1.9.3", {\
          "packageLocation": "./.yarn/cache/color-convert-npm-1.9.3-1fe690075e-fd7a64a17c.zip/node_modules/color-convert/",\
          "packageDependencies": [\
            ["color-convert", "npm:1.9.3"],\
            ["color-name", "npm:1.1.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/color-convert-npm-2.0.1-79730e935b-79e6bdb9fd.zip/node_modules/color-convert/",\
          "packageDependencies": [\
            ["color-convert", "npm:2.0.1"],\
            ["color-name", "npm:1.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["color-name", [\
        ["npm:1.1.3", {\
          "packageLocation": "./.yarn/cache/color-name-npm-1.1.3-728b7b5d39-09c5d3e33d.zip/node_modules/color-name/",\
          "packageDependencies": [\
            ["color-name", "npm:1.1.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.1.4", {\
          "packageLocation": "./.yarn/cache/color-name-npm-1.1.4-025792b0ea-b044585952.zip/node_modules/color-name/",\
          "packageDependencies": [\
            ["color-name", "npm:1.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["color-support", [\
        ["npm:1.1.3", {\
          "packageLocation": "./.yarn/cache/color-support-npm-1.1.3-3be5c53455-9b73568176.zip/node_modules/color-support/",\
          "packageDependencies": [\
            ["color-support", "npm:1.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["colorette", [\
        ["npm:2.0.20", {\
          "packageLocation": "./.yarn/cache/colorette-npm-2.0.20-692d428726-0c016fea2b.zip/node_modules/colorette/",\
          "packageDependencies": [\
            ["colorette", "npm:2.0.20"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["combined-stream", [\
        ["npm:1.0.8", {\
          "packageLocation": "./.yarn/cache/combined-stream-npm-1.0.8-dc14d4a63a-49fa4aeb49.zip/node_modules/combined-stream/",\
          "packageDependencies": [\
            ["combined-stream", "npm:1.0.8"],\
            ["delayed-stream", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["commander", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/commander-npm-2.1.0-59a26d686a-147dab340f.zip/node_modules/commander/",\
          "packageDependencies": [\
            ["commander", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.20.3", {\
          "packageLocation": "./.yarn/cache/commander-npm-2.20.3-d8dcbaa39b-ab8c07884e.zip/node_modules/commander/",\
          "packageDependencies": [\
            ["commander", "npm:2.20.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/commander-npm-6.0.0-9773645a16-5bbbfa9717.zip/node_modules/commander/",\
          "packageDependencies": [\
            ["commander", "npm:6.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.2.0", {\
          "packageLocation": "./.yarn/cache/commander-npm-7.2.0-19178180f8-53501cbeee.zip/node_modules/commander/",\
          "packageDependencies": [\
            ["commander", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["commondir", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/commondir-npm-1.0.1-291b790340-59715f2fc4.zip/node_modules/commondir/",\
          "packageDependencies": [\
            ["commondir", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["component-emitter", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/component-emitter-npm-1.3.0-4b848565b9-b3c46de38f.zip/node_modules/component-emitter/",\
          "packageDependencies": [\
            ["component-emitter", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["concat-map", [\
        ["npm:0.0.1", {\
          "packageLocation": "./.yarn/cache/concat-map-npm-0.0.1-85a921b7ee-902a9f5d89.zip/node_modules/concat-map/",\
          "packageDependencies": [\
            ["concat-map", "npm:0.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["console-control-strings", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/console-control-strings-npm-1.1.0-e3160e5275-8755d76787.zip/node_modules/console-control-strings/",\
          "packageDependencies": [\
            ["console-control-strings", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["convert-source-map", [\
        ["npm:1.9.0", {\
          "packageLocation": "./.yarn/cache/convert-source-map-npm-1.9.0-e294555f4b-dc55a1f28d.zip/node_modules/convert-source-map/",\
          "packageDependencies": [\
            ["convert-source-map", "npm:1.9.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["copy-descriptor", [\
        ["npm:0.1.1", {\
          "packageLocation": "./.yarn/cache/copy-descriptor-npm-0.1.1-864db4ab66-d4b7b57b14.zip/node_modules/copy-descriptor/",\
          "packageDependencies": [\
            ["copy-descriptor", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["core-js-compat", [\
        ["npm:3.31.0", {\
          "packageLocation": "./.yarn/cache/core-js-compat-npm-3.31.0-c16fee8c48-5c76ac5e4a.zip/node_modules/core-js-compat/",\
          "packageDependencies": [\
            ["core-js-compat", "npm:3.31.0"],\
            ["browserslist", "npm:4.21.9"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["core-util-is", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/core-util-is-npm-1.0.3-ca74b76c90-9de8597363.zip/node_modules/core-util-is/",\
          "packageDependencies": [\
            ["core-util-is", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cross-spawn", [\
        ["npm:6.0.5", {\
          "packageLocation": "./.yarn/cache/cross-spawn-npm-6.0.5-2deab6c280-f893bb0d96.zip/node_modules/cross-spawn/",\
          "packageDependencies": [\
            ["cross-spawn", "npm:6.0.5"],\
            ["nice-try", "npm:1.0.5"],\
            ["path-key", "npm:2.0.1"],\
            ["semver", "npm:5.7.1"],\
            ["shebang-command", "npm:1.2.0"],\
            ["which", "npm:1.3.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.0.3", {\
          "packageLocation": "./.yarn/cache/cross-spawn-npm-7.0.3-e4ff3e65b3-671cc7c728.zip/node_modules/cross-spawn/",\
          "packageDependencies": [\
            ["cross-spawn", "npm:7.0.3"],\
            ["path-key", "npm:3.1.1"],\
            ["shebang-command", "npm:2.0.0"],\
            ["which", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["css-loader", [\
        ["npm:3.6.0", {\
          "packageLocation": "./.yarn/cache/css-loader-npm-3.6.0-3394f37d07-a45d7ee810.zip/node_modules/css-loader/",\
          "packageDependencies": [\
            ["css-loader", "npm:3.6.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["npm:5.2.7", {\
          "packageLocation": "./.yarn/cache/css-loader-npm-5.2.7-e1e8b8d16f-fb0742b30a.zip/node_modules/css-loader/",\
          "packageDependencies": [\
            ["css-loader", "npm:5.2.7"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.6.0", {\
          "packageLocation": "./.yarn/__virtual__/css-loader-virtual-c64d80b076/0/cache/css-loader-npm-3.6.0-3394f37d07-a45d7ee810.zip/node_modules/css-loader/",\
          "packageDependencies": [\
            ["css-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.6.0"],\
            ["@types/webpack", null],\
            ["camelcase", "npm:5.3.1"],\
            ["cssesc", "npm:3.0.0"],\
            ["icss-utils", "npm:4.1.1"],\
            ["loader-utils", "npm:1.4.2"],\
            ["normalize-path", "npm:3.0.0"],\
            ["postcss", "npm:7.0.39"],\
            ["postcss-modules-extract-imports", "npm:2.0.0"],\
            ["postcss-modules-local-by-default", "npm:3.0.3"],\
            ["postcss-modules-scope", "npm:2.2.0"],\
            ["postcss-modules-values", "npm:3.0.0"],\
            ["postcss-value-parser", "npm:4.2.0"],\
            ["schema-utils", "npm:2.7.1"],\
            ["semver", "npm:6.3.0"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.2.7", {\
          "packageLocation": "./.yarn/__virtual__/css-loader-virtual-473f0ea469/0/cache/css-loader-npm-5.2.7-e1e8b8d16f-fb0742b30a.zip/node_modules/css-loader/",\
          "packageDependencies": [\
            ["css-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.2.7"],\
            ["@types/webpack", null],\
            ["icss-utils", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:5.1.0"],\
            ["loader-utils", "npm:2.0.4"],\
            ["postcss", "npm:8.4.24"],\
            ["postcss-modules-extract-imports", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:3.0.0"],\
            ["postcss-modules-local-by-default", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:4.0.3"],\
            ["postcss-modules-scope", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:3.0.0"],\
            ["postcss-modules-values", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:4.0.0"],\
            ["postcss-value-parser", "npm:4.2.0"],\
            ["schema-utils", "npm:3.3.0"],\
            ["semver", "npm:7.5.3"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cssesc", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/cssesc-npm-3.0.0-15ec56f86f-f8c4ababff.zip/node_modules/cssesc/",\
          "packageDependencies": [\
            ["cssesc", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cssom", [\
        ["npm:0.3.8", {\
          "packageLocation": "./.yarn/cache/cssom-npm-0.3.8-a9291d36ff-24beb3087c.zip/node_modules/cssom/",\
          "packageDependencies": [\
            ["cssom", "npm:0.3.8"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.4.4", {\
          "packageLocation": "./.yarn/cache/cssom-npm-0.4.4-818f01a6e3-e3bc1076e7.zip/node_modules/cssom/",\
          "packageDependencies": [\
            ["cssom", "npm:0.4.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["cssstyle", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/cssstyle-npm-2.3.0-b5d112c450-5f05e6fd2e.zip/node_modules/cssstyle/",\
          "packageDependencies": [\
            ["cssstyle", "npm:2.3.0"],\
            ["cssom", "npm:0.3.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["data-urls", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/data-urls-npm-2.0.0-2b80c32b82-97caf828aa.zip/node_modules/data-urls/",\
          "packageDependencies": [\
            ["data-urls", "npm:2.0.0"],\
            ["abab", "npm:2.0.6"],\
            ["whatwg-mimetype", "npm:2.3.0"],\
            ["whatwg-url", "npm:8.7.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["debug", [\
        ["npm:2.6.9", {\
          "packageLocation": "./.yarn/cache/debug-npm-2.6.9-7d4cb597dc-d2f51589ca.zip/node_modules/debug/",\
          "packageDependencies": [\
            ["debug", "npm:2.6.9"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["npm:4.3.4", {\
          "packageLocation": "./.yarn/cache/debug-npm-4.3.4-4513954577-3dbad3f94e.zip/node_modules/debug/",\
          "packageDependencies": [\
            ["debug", "npm:4.3.4"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4", {\
          "packageLocation": "./.yarn/__virtual__/debug-virtual-40bf8a96c6/0/cache/debug-npm-4.3.4-4513954577-3dbad3f94e.zip/node_modules/debug/",\
          "packageDependencies": [\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["@types/supports-color", null],\
            ["ms", "npm:2.1.2"],\
            ["supports-color", null]\
          ],\
          "packagePeers": [\
            "@types/supports-color",\
            "supports-color"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:2bcc47d217f870e3d52f0e55493dc34fd3da852877f7db32fa2940cc320151746026495daf6a34a974488cf5a949d93be1e2dc1ffba036faf081a9b15fcd2252#npm:2.6.9", {\
          "packageLocation": "./.yarn/__virtual__/debug-virtual-6cfa7ae8cd/0/cache/debug-npm-2.6.9-7d4cb597dc-d2f51589ca.zip/node_modules/debug/",\
          "packageDependencies": [\
            ["debug", "virtual:2bcc47d217f870e3d52f0e55493dc34fd3da852877f7db32fa2940cc320151746026495daf6a34a974488cf5a949d93be1e2dc1ffba036faf081a9b15fcd2252#npm:2.6.9"],\
            ["@types/supports-color", null],\
            ["ms", "npm:2.0.0"],\
            ["supports-color", null]\
          ],\
          "packagePeers": [\
            "@types/supports-color",\
            "supports-color"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["decamelize", [\
        ["npm:1.2.0", {\
          "packageLocation": "./.yarn/cache/decamelize-npm-1.2.0-c5a2fdc622-ad8c51a7e7.zip/node_modules/decamelize/",\
          "packageDependencies": [\
            ["decamelize", "npm:1.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["decimal.js", [\
        ["npm:10.4.3", {\
          "packageLocation": "./.yarn/cache/decimal.js-npm-10.4.3-e7d483387c-796404dcfa.zip/node_modules/decimal.js/",\
          "packageDependencies": [\
            ["decimal.js", "npm:10.4.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["decode-uri-component", [\
        ["npm:0.2.2", {\
          "packageLocation": "./.yarn/cache/decode-uri-component-npm-0.2.2-e22e2aa917-95476a7d28.zip/node_modules/decode-uri-component/",\
          "packageDependencies": [\
            ["decode-uri-component", "npm:0.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["deep-is", [\
        ["npm:0.1.4", {\
          "packageLocation": "./.yarn/cache/deep-is-npm-0.1.4-88938b5a67-edb65dd0d7.zip/node_modules/deep-is/",\
          "packageDependencies": [\
            ["deep-is", "npm:0.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["deepmerge", [\
        ["npm:4.3.1", {\
          "packageLocation": "./.yarn/cache/deepmerge-npm-4.3.1-4f751a0844-2024c6a980.zip/node_modules/deepmerge/",\
          "packageDependencies": [\
            ["deepmerge", "npm:4.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["define-properties", [\
        ["npm:1.2.0", {\
          "packageLocation": "./.yarn/cache/define-properties-npm-1.2.0-3547cd0fd2-e60aee6a19.zip/node_modules/define-properties/",\
          "packageDependencies": [\
            ["define-properties", "npm:1.2.0"],\
            ["has-property-descriptors", "npm:1.0.0"],\
            ["object-keys", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["define-property", [\
        ["npm:0.2.5", {\
          "packageLocation": "./.yarn/cache/define-property-npm-0.2.5-44a0da3575-85af107072.zip/node_modules/define-property/",\
          "packageDependencies": [\
            ["define-property", "npm:0.2.5"],\
            ["is-descriptor", "npm:0.1.6"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/define-property-npm-1.0.0-e2fb9f44c6-5fbed11dac.zip/node_modules/define-property/",\
          "packageDependencies": [\
            ["define-property", "npm:1.0.0"],\
            ["is-descriptor", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/define-property-npm-2.0.2-4a2067c3ba-3217ed53fc.zip/node_modules/define-property/",\
          "packageDependencies": [\
            ["define-property", "npm:2.0.2"],\
            ["is-descriptor", "npm:1.0.2"],\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["delayed-stream", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/delayed-stream-npm-1.0.0-c5a4c4cc02-46fe6e83e2.zip/node_modules/delayed-stream/",\
          "packageDependencies": [\
            ["delayed-stream", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["delegates", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/delegates-npm-1.0.0-9b1942d75f-a51744d9b5.zip/node_modules/delegates/",\
          "packageDependencies": [\
            ["delegates", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["depd", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/depd-npm-2.0.0-b6c51a4b43-abbe19c768.zip/node_modules/depd/",\
          "packageDependencies": [\
            ["depd", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["detect-newline", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/detect-newline-npm-3.1.0-6d33fa8d37-ae6cd429c4.zip/node_modules/detect-newline/",\
          "packageDependencies": [\
            ["detect-newline", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["diff-sequences", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/diff-sequences-npm-26.6.2-fbf1967570-79af871776.zip/node_modules/diff-sequences/",\
          "packageDependencies": [\
            ["diff-sequences", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["doctrine", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/doctrine-npm-3.0.0-c6f1615f04-fd7673ca77.zip/node_modules/doctrine/",\
          "packageDependencies": [\
            ["doctrine", "npm:3.0.0"],\
            ["esutils", "npm:2.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["dom-serializer", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/dom-serializer-npm-2.0.0-378ebc7200-cd1810544f.zip/node_modules/dom-serializer/",\
          "packageDependencies": [\
            ["dom-serializer", "npm:2.0.0"],\
            ["domelementtype", "npm:2.3.0"],\
            ["domhandler", "npm:5.0.3"],\
            ["entities", "npm:4.5.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["domelementtype", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/domelementtype-npm-2.3.0-02de7cbfba-ee837a318f.zip/node_modules/domelementtype/",\
          "packageDependencies": [\
            ["domelementtype", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["domexception", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/domexception-npm-2.0.1-81b20626ae-d638e9cb05.zip/node_modules/domexception/",\
          "packageDependencies": [\
            ["domexception", "npm:2.0.1"],\
            ["webidl-conversions", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["domhandler", [\
        ["npm:5.0.3", {\
          "packageLocation": "./.yarn/cache/domhandler-npm-5.0.3-3ede73dc10-0f58f4a6af.zip/node_modules/domhandler/",\
          "packageDependencies": [\
            ["domhandler", "npm:5.0.3"],\
            ["domelementtype", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["domutils", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/domutils-npm-3.1.0-66c92ef7eb-e5757456dd.zip/node_modules/domutils/",\
          "packageDependencies": [\
            ["domutils", "npm:3.1.0"],\
            ["dom-serializer", "npm:2.0.0"],\
            ["domelementtype", "npm:2.3.0"],\
            ["domhandler", "npm:5.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["duplicate-package-checker-webpack-plugin", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/duplicate-package-checker-webpack-plugin-npm-3.0.0-ac29845fdb-d77be45cb7.zip/node_modules/duplicate-package-checker-webpack-plugin/",\
          "packageDependencies": [\
            ["duplicate-package-checker-webpack-plugin", "npm:3.0.0"],\
            ["chalk", "npm:2.4.2"],\
            ["find-root", "npm:1.1.0"],\
            ["lodash", "npm:4.17.21"],\
            ["semver", "npm:5.7.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eastasianwidth", [\
        ["npm:0.2.0", {\
          "packageLocation": "./.yarn/cache/eastasianwidth-npm-0.2.0-c37eb16bd1-7d00d7cd8e.zip/node_modules/eastasianwidth/",\
          "packageDependencies": [\
            ["eastasianwidth", "npm:0.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["electron-to-chromium", [\
        ["npm:1.4.439", {\
          "packageLocation": "./.yarn/cache/electron-to-chromium-npm-1.4.439-3fb08c140e-77e2f06bd5.zip/node_modules/electron-to-chromium/",\
          "packageDependencies": [\
            ["electron-to-chromium", "npm:1.4.439"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["emittery", [\
        ["npm:0.7.2", {\
          "packageLocation": "./.yarn/cache/emittery-npm-0.7.2-4a6f20265e-908cd933d4.zip/node_modules/emittery/",\
          "packageDependencies": [\
            ["emittery", "npm:0.7.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["emoji-regex", [\
        ["npm:8.0.0", {\
          "packageLocation": "./.yarn/cache/emoji-regex-npm-8.0.0-213764015c-d4c5c39d5a.zip/node_modules/emoji-regex/",\
          "packageDependencies": [\
            ["emoji-regex", "npm:8.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:9.2.2", {\
          "packageLocation": "./.yarn/cache/emoji-regex-npm-9.2.2-e6fac8d058-8487182da7.zip/node_modules/emoji-regex/",\
          "packageDependencies": [\
            ["emoji-regex", "npm:9.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["emojis-list", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/emojis-list-npm-3.0.0-7faa48e6fd-ddaaa02542.zip/node_modules/emojis-list/",\
          "packageDependencies": [\
            ["emojis-list", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["encoding", [\
        ["npm:0.1.13", {\
          "packageLocation": "./.yarn/cache/encoding-npm-0.1.13-82a1837d30-bb98632f8f.zip/node_modules/encoding/",\
          "packageDependencies": [\
            ["encoding", "npm:0.1.13"],\
            ["iconv-lite", "npm:0.6.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["end-of-stream", [\
        ["npm:1.4.4", {\
          "packageLocation": "./.yarn/cache/end-of-stream-npm-1.4.4-497fc6dee1-530a5a5a1e.zip/node_modules/end-of-stream/",\
          "packageDependencies": [\
            ["end-of-stream", "npm:1.4.4"],\
            ["once", "npm:1.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["enhanced-resolve", [\
        ["npm:4.5.0", {\
          "packageLocation": "./.yarn/cache/enhanced-resolve-npm-4.5.0-1bcc7900d2-4d87488584.zip/node_modules/enhanced-resolve/",\
          "packageDependencies": [\
            ["enhanced-resolve", "npm:4.5.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["memory-fs", "npm:0.5.0"],\
            ["tapable", "npm:1.1.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.15.0", {\
          "packageLocation": "./.yarn/cache/enhanced-resolve-npm-5.15.0-16eb7ddef9-fbd8cdc926.zip/node_modules/enhanced-resolve/",\
          "packageDependencies": [\
            ["enhanced-resolve", "npm:5.15.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["tapable", "npm:2.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["enquirer", [\
        ["npm:2.3.6", {\
          "packageLocation": "./.yarn/cache/enquirer-npm-2.3.6-7899175762-1c0911e14a.zip/node_modules/enquirer/",\
          "packageDependencies": [\
            ["enquirer", "npm:2.3.6"],\
            ["ansi-colors", "npm:4.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["entities", [\
        ["npm:4.5.0", {\
          "packageLocation": "./.yarn/cache/entities-npm-4.5.0-7cdb83b832-853f8ebd5b.zip/node_modules/entities/",\
          "packageDependencies": [\
            ["entities", "npm:4.5.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["env-paths", [\
        ["npm:2.2.1", {\
          "packageLocation": "./.yarn/cache/env-paths-npm-2.2.1-7c7577428c-65b5df55a8.zip/node_modules/env-paths/",\
          "packageDependencies": [\
            ["env-paths", "npm:2.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["envinfo", [\
        ["npm:7.9.0", {\
          "packageLocation": "./.yarn/cache/envinfo-npm-7.9.0-f6329c04b5-133583b075.zip/node_modules/envinfo/",\
          "packageDependencies": [\
            ["envinfo", "npm:7.9.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["err-code", [\
        ["npm:2.0.3", {\
          "packageLocation": "./.yarn/cache/err-code-npm-2.0.3-082e0ff9a7-8b7b1be20d.zip/node_modules/err-code/",\
          "packageDependencies": [\
            ["err-code", "npm:2.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["errno", [\
        ["npm:0.1.8", {\
          "packageLocation": "./.yarn/cache/errno-npm-0.1.8-10ebc185bf-1271f7b9fb.zip/node_modules/errno/",\
          "packageDependencies": [\
            ["errno", "npm:0.1.8"],\
            ["prr", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["error-ex", [\
        ["npm:1.3.2", {\
          "packageLocation": "./.yarn/cache/error-ex-npm-1.3.2-5654f80c0f-c1c2b8b65f.zip/node_modules/error-ex/",\
          "packageDependencies": [\
            ["error-ex", "npm:1.3.2"],\
            ["is-arrayish", "npm:0.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["es-abstract", [\
        ["npm:1.21.2", {\
          "packageLocation": "./.yarn/cache/es-abstract-npm-1.21.2-f4ebace1ab-037f55ee5e.zip/node_modules/es-abstract/",\
          "packageDependencies": [\
            ["es-abstract", "npm:1.21.2"],\
            ["array-buffer-byte-length", "npm:1.0.0"],\
            ["available-typed-arrays", "npm:1.0.5"],\
            ["call-bind", "npm:1.0.2"],\
            ["es-set-tostringtag", "npm:2.0.1"],\
            ["es-to-primitive", "npm:1.2.1"],\
            ["function.prototype.name", "npm:1.1.5"],\
            ["get-intrinsic", "npm:1.2.1"],\
            ["get-symbol-description", "npm:1.0.0"],\
            ["globalthis", "npm:1.0.3"],\
            ["gopd", "npm:1.0.1"],\
            ["has", "npm:1.0.3"],\
            ["has-property-descriptors", "npm:1.0.0"],\
            ["has-proto", "npm:1.0.1"],\
            ["has-symbols", "npm:1.0.3"],\
            ["internal-slot", "npm:1.0.5"],\
            ["is-array-buffer", "npm:3.0.2"],\
            ["is-callable", "npm:1.2.7"],\
            ["is-negative-zero", "npm:2.0.2"],\
            ["is-regex", "npm:1.1.4"],\
            ["is-shared-array-buffer", "npm:1.0.2"],\
            ["is-string", "npm:1.0.7"],\
            ["is-typed-array", "npm:1.1.10"],\
            ["is-weakref", "npm:1.0.2"],\
            ["object-inspect", "npm:1.12.3"],\
            ["object-keys", "npm:1.1.1"],\
            ["object.assign", "npm:4.1.4"],\
            ["regexp.prototype.flags", "npm:1.5.0"],\
            ["safe-regex-test", "npm:1.0.0"],\
            ["string.prototype.trim", "npm:1.2.7"],\
            ["string.prototype.trimend", "npm:1.0.6"],\
            ["string.prototype.trimstart", "npm:1.0.6"],\
            ["typed-array-length", "npm:1.0.4"],\
            ["unbox-primitive", "npm:1.0.2"],\
            ["which-typed-array", "npm:1.1.9"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["es-module-lexer", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/es-module-lexer-npm-1.3.0-9be5e8b1c4-48fd9f504a.zip/node_modules/es-module-lexer/",\
          "packageDependencies": [\
            ["es-module-lexer", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["es-set-tostringtag", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/es-set-tostringtag-npm-2.0.1-c87b5de872-ec416a1294.zip/node_modules/es-set-tostringtag/",\
          "packageDependencies": [\
            ["es-set-tostringtag", "npm:2.0.1"],\
            ["get-intrinsic", "npm:1.2.1"],\
            ["has", "npm:1.0.3"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["es-to-primitive", [\
        ["npm:1.2.1", {\
          "packageLocation": "./.yarn/cache/es-to-primitive-npm-1.2.1-b7a7eac6c5-4ead6671a2.zip/node_modules/es-to-primitive/",\
          "packageDependencies": [\
            ["es-to-primitive", "npm:1.2.1"],\
            ["is-callable", "npm:1.2.7"],\
            ["is-date-object", "npm:1.0.5"],\
            ["is-symbol", "npm:1.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["escalade", [\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/escalade-npm-3.1.1-e02da076aa-a3e2a99f07.zip/node_modules/escalade/",\
          "packageDependencies": [\
            ["escalade", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["escape-string-regexp", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/escape-string-regexp-npm-1.0.5-3284de402f-6092fda75c.zip/node_modules/escape-string-regexp/",\
          "packageDependencies": [\
            ["escape-string-regexp", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/escape-string-regexp-npm-2.0.0-aef69d2a25-9f8a2d5743.zip/node_modules/escape-string-regexp/",\
          "packageDependencies": [\
            ["escape-string-regexp", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/escape-string-regexp-npm-4.0.0-4b531d8d59-98b48897d9.zip/node_modules/escape-string-regexp/",\
          "packageDependencies": [\
            ["escape-string-regexp", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["escodegen", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/escodegen-npm-2.0.0-6450b02925-5aa6b2966f.zip/node_modules/escodegen/",\
          "packageDependencies": [\
            ["escodegen", "npm:2.0.0"],\
            ["esprima", "npm:4.0.1"],\
            ["estraverse", "npm:5.3.0"],\
            ["esutils", "npm:2.0.3"],\
            ["optionator", "npm:0.8.3"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eslint", [\
        ["npm:7.32.0", {\
          "packageLocation": "./.yarn/cache/eslint-npm-7.32.0-e15cc6682f-cc85af9985.zip/node_modules/eslint/",\
          "packageDependencies": [\
            ["eslint", "npm:7.32.0"],\
            ["@babel/code-frame", "npm:7.12.11"],\
            ["@eslint/eslintrc", "npm:0.4.3"],\
            ["@humanwhocodes/config-array", "npm:0.5.0"],\
            ["ajv", "npm:6.12.6"],\
            ["chalk", "npm:4.1.2"],\
            ["cross-spawn", "npm:7.0.3"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["doctrine", "npm:3.0.0"],\
            ["enquirer", "npm:2.3.6"],\
            ["escape-string-regexp", "npm:4.0.0"],\
            ["eslint-scope", "npm:5.1.1"],\
            ["eslint-utils", "npm:2.1.0"],\
            ["eslint-visitor-keys", "npm:2.1.0"],\
            ["espree", "npm:7.3.1"],\
            ["esquery", "npm:1.5.0"],\
            ["esutils", "npm:2.0.3"],\
            ["fast-deep-equal", "npm:3.1.3"],\
            ["file-entry-cache", "npm:6.0.1"],\
            ["functional-red-black-tree", "npm:1.0.1"],\
            ["glob-parent", "npm:5.1.2"],\
            ["globals", "npm:13.20.0"],\
            ["ignore", "npm:4.0.6"],\
            ["import-fresh", "npm:3.3.0"],\
            ["imurmurhash", "npm:0.1.4"],\
            ["is-glob", "npm:4.0.3"],\
            ["js-yaml", "npm:3.14.1"],\
            ["json-stable-stringify-without-jsonify", "npm:1.0.1"],\
            ["levn", "npm:0.4.1"],\
            ["lodash.merge", "npm:4.6.2"],\
            ["minimatch", "npm:3.1.2"],\
            ["natural-compare", "npm:1.4.0"],\
            ["optionator", "npm:0.9.1"],\
            ["progress", "npm:2.0.3"],\
            ["regexpp", "npm:3.2.0"],\
            ["semver", "npm:7.5.3"],\
            ["strip-ansi", "npm:6.0.1"],\
            ["strip-json-comments", "npm:3.1.1"],\
            ["table", "npm:6.8.1"],\
            ["text-table", "npm:0.2.0"],\
            ["v8-compile-cache", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eslint-config-prettier", [\
        ["npm:6.15.0", {\
          "packageLocation": "./.yarn/cache/eslint-config-prettier-npm-6.15.0-d2085588b4-02f461a5d7.zip/node_modules/eslint-config-prettier/",\
          "packageDependencies": [\
            ["eslint-config-prettier", "npm:6.15.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:6.15.0", {\
          "packageLocation": "./.yarn/__virtual__/eslint-config-prettier-virtual-16433da4a5/0/cache/eslint-config-prettier-npm-6.15.0-d2085588b4-02f461a5d7.zip/node_modules/eslint-config-prettier/",\
          "packageDependencies": [\
            ["eslint-config-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:6.15.0"],\
            ["@types/eslint", null],\
            ["eslint", "npm:7.32.0"],\
            ["get-stdin", "npm:6.0.0"]\
          ],\
          "packagePeers": [\
            "@types/eslint",\
            "eslint"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eslint-plugin-prettier", [\
        ["npm:3.4.1", {\
          "packageLocation": "./.yarn/cache/eslint-plugin-prettier-npm-3.4.1-c0bf5af22f-fa6a89f0d7.zip/node_modules/eslint-plugin-prettier/",\
          "packageDependencies": [\
            ["eslint-plugin-prettier", "npm:3.4.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.4.1", {\
          "packageLocation": "./.yarn/__virtual__/eslint-plugin-prettier-virtual-4f6fd4a945/0/cache/eslint-plugin-prettier-npm-3.4.1-c0bf5af22f-fa6a89f0d7.zip/node_modules/eslint-plugin-prettier/",\
          "packageDependencies": [\
            ["eslint-plugin-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.4.1"],\
            ["@types/eslint", null],\
            ["@types/eslint-config-prettier", null],\
            ["@types/prettier", null],\
            ["eslint", "npm:7.32.0"],\
            ["eslint-config-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:6.15.0"],\
            ["prettier", "npm:2.8.8"],\
            ["prettier-linter-helpers", "npm:1.0.0"]\
          ],\
          "packagePeers": [\
            "@types/eslint-config-prettier",\
            "@types/eslint",\
            "@types/prettier",\
            "eslint-config-prettier",\
            "eslint",\
            "prettier"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eslint-scope", [\
        ["npm:5.1.1", {\
          "packageLocation": "./.yarn/cache/eslint-scope-npm-5.1.1-71fe59b18a-47e4b6a3f0.zip/node_modules/eslint-scope/",\
          "packageDependencies": [\
            ["eslint-scope", "npm:5.1.1"],\
            ["esrecurse", "npm:4.3.0"],\
            ["estraverse", "npm:4.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eslint-utils", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/eslint-utils-npm-2.1.0-a3a7ebf4fa-27500938f3.zip/node_modules/eslint-utils/",\
          "packageDependencies": [\
            ["eslint-utils", "npm:2.1.0"],\
            ["eslint-visitor-keys", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["eslint-visitor-keys", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/eslint-visitor-keys-npm-1.3.0-c07780a0fb-37a19b712f.zip/node_modules/eslint-visitor-keys/",\
          "packageDependencies": [\
            ["eslint-visitor-keys", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/eslint-visitor-keys-npm-2.1.0-c31806b6b9-e3081d7dd2.zip/node_modules/eslint-visitor-keys/",\
          "packageDependencies": [\
            ["eslint-visitor-keys", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["espree", [\
        ["npm:7.3.1", {\
          "packageLocation": "./.yarn/cache/espree-npm-7.3.1-8d8ea5d1e3-aa9b50dcce.zip/node_modules/espree/",\
          "packageDependencies": [\
            ["espree", "npm:7.3.1"],\
            ["acorn", "npm:7.4.1"],\
            ["acorn-jsx", "virtual:8d8ea5d1e3376905d0290522290f47c29213c64d936d96293d758a315829a3cf4c6a5b8ffc1cfee36c3db08f700ad3aaf0711cc5d406a7218c275de6d74effa9#npm:5.3.2"],\
            ["eslint-visitor-keys", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["esprima", [\
        ["npm:4.0.1", {\
          "packageLocation": "./.yarn/cache/esprima-npm-4.0.1-1084e98778-b45bc805a6.zip/node_modules/esprima/",\
          "packageDependencies": [\
            ["esprima", "npm:4.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["esquery", [\
        ["npm:1.5.0", {\
          "packageLocation": "./.yarn/cache/esquery-npm-1.5.0-d8f8a06879-aefb0d2596.zip/node_modules/esquery/",\
          "packageDependencies": [\
            ["esquery", "npm:1.5.0"],\
            ["estraverse", "npm:5.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["esrecurse", [\
        ["npm:4.3.0", {\
          "packageLocation": "./.yarn/cache/esrecurse-npm-4.3.0-10b86a887a-ebc17b1a33.zip/node_modules/esrecurse/",\
          "packageDependencies": [\
            ["esrecurse", "npm:4.3.0"],\
            ["estraverse", "npm:5.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["estraverse", [\
        ["npm:4.3.0", {\
          "packageLocation": "./.yarn/cache/estraverse-npm-4.3.0-920a32f3c6-a6299491f9.zip/node_modules/estraverse/",\
          "packageDependencies": [\
            ["estraverse", "npm:4.3.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.3.0", {\
          "packageLocation": "./.yarn/cache/estraverse-npm-5.3.0-03284f8f63-072780882d.zip/node_modules/estraverse/",\
          "packageDependencies": [\
            ["estraverse", "npm:5.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["esutils", [\
        ["npm:2.0.3", {\
          "packageLocation": "./.yarn/cache/esutils-npm-2.0.3-f865beafd5-22b5b08f74.zip/node_modules/esutils/",\
          "packageDependencies": [\
            ["esutils", "npm:2.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["events", [\
        ["npm:3.3.0", {\
          "packageLocation": "./.yarn/cache/events-npm-3.3.0-c280bc7e48-f6f487ad21.zip/node_modules/events/",\
          "packageDependencies": [\
            ["events", "npm:3.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["exec-sh", [\
        ["npm:0.3.6", {\
          "packageLocation": "./.yarn/cache/exec-sh-npm-0.3.6-8a29d03ae2-0be4f06929.zip/node_modules/exec-sh/",\
          "packageDependencies": [\
            ["exec-sh", "npm:0.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["execa", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/execa-npm-1.0.0-7028e37029-ddf1342c1c.zip/node_modules/execa/",\
          "packageDependencies": [\
            ["execa", "npm:1.0.0"],\
            ["cross-spawn", "npm:6.0.5"],\
            ["get-stream", "npm:4.1.0"],\
            ["is-stream", "npm:1.1.0"],\
            ["npm-run-path", "npm:2.0.2"],\
            ["p-finally", "npm:1.0.0"],\
            ["signal-exit", "npm:3.0.7"],\
            ["strip-eof", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.1.0", {\
          "packageLocation": "./.yarn/cache/execa-npm-4.1.0-cc675b4189-e30d298934.zip/node_modules/execa/",\
          "packageDependencies": [\
            ["execa", "npm:4.1.0"],\
            ["cross-spawn", "npm:7.0.3"],\
            ["get-stream", "npm:5.2.0"],\
            ["human-signals", "npm:1.1.1"],\
            ["is-stream", "npm:2.0.1"],\
            ["merge-stream", "npm:2.0.0"],\
            ["npm-run-path", "npm:4.0.1"],\
            ["onetime", "npm:5.1.2"],\
            ["signal-exit", "npm:3.0.7"],\
            ["strip-final-newline", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["exit", [\
        ["npm:0.1.2", {\
          "packageLocation": "./.yarn/cache/exit-npm-0.1.2-ef3761a67d-abc407f07a.zip/node_modules/exit/",\
          "packageDependencies": [\
            ["exit", "npm:0.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["expand-brackets", [\
        ["npm:2.1.4", {\
          "packageLocation": "./.yarn/cache/expand-brackets-npm-2.1.4-392c703c48-1781d422e7.zip/node_modules/expand-brackets/",\
          "packageDependencies": [\
            ["expand-brackets", "npm:2.1.4"],\
            ["debug", "virtual:2bcc47d217f870e3d52f0e55493dc34fd3da852877f7db32fa2940cc320151746026495daf6a34a974488cf5a949d93be1e2dc1ffba036faf081a9b15fcd2252#npm:2.6.9"],\
            ["define-property", "npm:0.2.5"],\
            ["extend-shallow", "npm:2.0.1"],\
            ["posix-character-classes", "npm:0.1.1"],\
            ["regex-not", "npm:1.0.2"],\
            ["snapdragon", "npm:0.8.2"],\
            ["to-regex", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["expect", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/expect-npm-26.6.2-256ea90344-79a9b888c5.zip/node_modules/expect/",\
          "packageDependencies": [\
            ["expect", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["ansi-styles", "npm:4.3.0"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["jest-matcher-utils", "npm:26.6.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-regex-util", "npm:26.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["exponential-backoff", [\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/exponential-backoff-npm-3.1.1-04df458b30-3d21519a4f.zip/node_modules/exponential-backoff/",\
          "packageDependencies": [\
            ["exponential-backoff", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["extend-shallow", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/extend-shallow-npm-2.0.1-e6ef52b29c-8fb58d9d7a.zip/node_modules/extend-shallow/",\
          "packageDependencies": [\
            ["extend-shallow", "npm:2.0.1"],\
            ["is-extendable", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.2", {\
          "packageLocation": "./.yarn/cache/extend-shallow-npm-3.0.2-77bbe1bbf5-a920b0cd58.zip/node_modules/extend-shallow/",\
          "packageDependencies": [\
            ["extend-shallow", "npm:3.0.2"],\
            ["assign-symbols", "npm:1.0.0"],\
            ["is-extendable", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["extglob", [\
        ["npm:2.0.4", {\
          "packageLocation": "./.yarn/cache/extglob-npm-2.0.4-0f39bc9899-a41531b893.zip/node_modules/extglob/",\
          "packageDependencies": [\
            ["extglob", "npm:2.0.4"],\
            ["array-unique", "npm:0.3.2"],\
            ["define-property", "npm:1.0.0"],\
            ["expand-brackets", "npm:2.1.4"],\
            ["extend-shallow", "npm:2.0.1"],\
            ["fragment-cache", "npm:0.2.1"],\
            ["regex-not", "npm:1.0.2"],\
            ["snapdragon", "npm:0.8.2"],\
            ["to-regex", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fast-deep-equal", [\
        ["npm:3.1.3", {\
          "packageLocation": "./.yarn/cache/fast-deep-equal-npm-3.1.3-790edcfcf5-e21a9d8d84.zip/node_modules/fast-deep-equal/",\
          "packageDependencies": [\
            ["fast-deep-equal", "npm:3.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fast-diff", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/fast-diff-npm-1.3.0-9f19e3b743-d22d371b99.zip/node_modules/fast-diff/",\
          "packageDependencies": [\
            ["fast-diff", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fast-json-stable-stringify", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/fast-json-stable-stringify-npm-2.1.0-02e8905fda-b191531e36.zip/node_modules/fast-json-stable-stringify/",\
          "packageDependencies": [\
            ["fast-json-stable-stringify", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fast-levenshtein", [\
        ["npm:2.0.6", {\
          "packageLocation": "./.yarn/cache/fast-levenshtein-npm-2.0.6-fcd74b8df5-92cfec0a8d.zip/node_modules/fast-levenshtein/",\
          "packageDependencies": [\
            ["fast-levenshtein", "npm:2.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fastest-levenshtein", [\
        ["npm:1.0.16", {\
          "packageLocation": "./.yarn/cache/fastest-levenshtein-npm-1.0.16-192d328856-a78d44285c.zip/node_modules/fastest-levenshtein/",\
          "packageDependencies": [\
            ["fastest-levenshtein", "npm:1.0.16"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fb-watchman", [\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/fb-watchman-npm-2.0.2-bcb6f8f831-b15a124cef.zip/node_modules/fb-watchman/",\
          "packageDependencies": [\
            ["fb-watchman", "npm:2.0.2"],\
            ["bser", "npm:2.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["file-entry-cache", [\
        ["npm:6.0.1", {\
          "packageLocation": "./.yarn/cache/file-entry-cache-npm-6.0.1-31965cf0af-f49701feaa.zip/node_modules/file-entry-cache/",\
          "packageDependencies": [\
            ["file-entry-cache", "npm:6.0.1"],\
            ["flat-cache", "npm:3.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["file-loader", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/file-loader-npm-6.0.0-56aa137384-5e242b2365.zip/node_modules/file-loader/",\
          "packageDependencies": [\
            ["file-loader", "npm:6.0.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0", {\
          "packageLocation": "./.yarn/__virtual__/file-loader-virtual-538480049d/0/cache/file-loader-npm-6.0.0-56aa137384-5e242b2365.zip/node_modules/file-loader/",\
          "packageDependencies": [\
            ["file-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0"],\
            ["@types/webpack", null],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:2.7.1"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fill-range", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/fill-range-npm-4.0.0-95a6e45784-dbb5102467.zip/node_modules/fill-range/",\
          "packageDependencies": [\
            ["fill-range", "npm:4.0.0"],\
            ["extend-shallow", "npm:2.0.1"],\
            ["is-number", "npm:3.0.0"],\
            ["repeat-string", "npm:1.6.1"],\
            ["to-regex-range", "npm:2.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.0.1", {\
          "packageLocation": "./.yarn/cache/fill-range-npm-7.0.1-b8b1817caa-cc283f4e65.zip/node_modules/fill-range/",\
          "packageDependencies": [\
            ["fill-range", "npm:7.0.1"],\
            ["to-regex-range", "npm:5.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["find-cache-dir", [\
        ["npm:3.3.2", {\
          "packageLocation": "./.yarn/cache/find-cache-dir-npm-3.3.2-836e68dd83-1e61c2e64f.zip/node_modules/find-cache-dir/",\
          "packageDependencies": [\
            ["find-cache-dir", "npm:3.3.2"],\
            ["commondir", "npm:1.0.1"],\
            ["make-dir", "npm:3.1.0"],\
            ["pkg-dir", "npm:4.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["find-root", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/find-root-npm-1.1.0-a16a94005f-b2a59fe4b6.zip/node_modules/find-root/",\
          "packageDependencies": [\
            ["find-root", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["find-up", [\
        ["npm:4.1.0", {\
          "packageLocation": "./.yarn/cache/find-up-npm-4.1.0-c3ccf8d855-4c172680e8.zip/node_modules/find-up/",\
          "packageDependencies": [\
            ["find-up", "npm:4.1.0"],\
            ["locate-path", "npm:5.0.0"],\
            ["path-exists", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["flat-cache", [\
        ["npm:3.0.4", {\
          "packageLocation": "./.yarn/cache/flat-cache-npm-3.0.4-ee77e5911e-4fdd10ecbc.zip/node_modules/flat-cache/",\
          "packageDependencies": [\
            ["flat-cache", "npm:3.0.4"],\
            ["flatted", "npm:3.2.7"],\
            ["rimraf", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["flatted", [\
        ["npm:3.2.7", {\
          "packageLocation": "./.yarn/cache/flatted-npm-3.2.7-0da10b7c56-427633049d.zip/node_modules/flatted/",\
          "packageDependencies": [\
            ["flatted", "npm:3.2.7"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["for-each", [\
        ["npm:0.3.3", {\
          "packageLocation": "./.yarn/cache/for-each-npm-0.3.3-0010ca8cdd-6c48ff2bc6.zip/node_modules/for-each/",\
          "packageDependencies": [\
            ["for-each", "npm:0.3.3"],\
            ["is-callable", "npm:1.2.7"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["for-in", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/for-in-npm-1.0.2-37e3d7aae5-09f4ae93ce.zip/node_modules/for-in/",\
          "packageDependencies": [\
            ["for-in", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["foreground-child", [\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/foreground-child-npm-3.1.1-77e78ed774-139d270bc8.zip/node_modules/foreground-child/",\
          "packageDependencies": [\
            ["foreground-child", "npm:3.1.1"],\
            ["cross-spawn", "npm:7.0.3"],\
            ["signal-exit", "npm:4.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["form-data", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/form-data-npm-3.0.1-d080d436e0-b019e8d35c.zip/node_modules/form-data/",\
          "packageDependencies": [\
            ["form-data", "npm:3.0.1"],\
            ["asynckit", "npm:0.4.0"],\
            ["combined-stream", "npm:1.0.8"],\
            ["mime-types", "npm:2.1.35"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fragment-cache", [\
        ["npm:0.2.1", {\
          "packageLocation": "./.yarn/cache/fragment-cache-npm-0.2.1-407fe74319-1cbbd0b011.zip/node_modules/fragment-cache/",\
          "packageDependencies": [\
            ["fragment-cache", "npm:0.2.1"],\
            ["map-cache", "npm:0.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fs-extra", [\
        ["npm:7.0.1", {\
          "packageLocation": "./.yarn/cache/fs-extra-npm-7.0.1-b33a5e53e9-141b9dccb2.zip/node_modules/fs-extra/",\
          "packageDependencies": [\
            ["fs-extra", "npm:7.0.1"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jsonfile", "npm:4.0.0"],\
            ["universalify", "npm:0.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:9.1.0", {\
          "packageLocation": "./.yarn/cache/fs-extra-npm-9.1.0-983c2ddb4c-ba71ba32e0.zip/node_modules/fs-extra/",\
          "packageDependencies": [\
            ["fs-extra", "npm:9.1.0"],\
            ["at-least-node", "npm:1.0.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jsonfile", "npm:6.1.0"],\
            ["universalify", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fs-minipass", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/fs-minipass-npm-2.1.0-501ef87306-1b8d128dae.zip/node_modules/fs-minipass/",\
          "packageDependencies": [\
            ["fs-minipass", "npm:2.1.0"],\
            ["minipass", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.2", {\
          "packageLocation": "./.yarn/cache/fs-minipass-npm-3.0.2-a27ef235f5-e9cc0e1f2d.zip/node_modules/fs-minipass/",\
          "packageDependencies": [\
            ["fs-minipass", "npm:3.0.2"],\
            ["minipass", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fs.realpath", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/fs.realpath-npm-1.0.0-c8f05d8126-99ddea01a7.zip/node_modules/fs.realpath/",\
          "packageDependencies": [\
            ["fs.realpath", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["fsevents", [\
        ["patch:fsevents@npm%3A2.3.2#~builtin<compat/fsevents>::version=2.3.2&hash=df0bf1", {\
          "packageLocation": "./.yarn/unplugged/fsevents-patch-2882183fbf/node_modules/fsevents/",\
          "packageDependencies": [\
            ["fsevents", "patch:fsevents@npm%3A2.3.2#~builtin<compat/fsevents>::version=2.3.2&hash=df0bf1"],\
            ["node-gyp", "npm:9.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["function-bind", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/function-bind-npm-1.1.1-b56b322ae9-b32fbaebb3.zip/node_modules/function-bind/",\
          "packageDependencies": [\
            ["function-bind", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["function.prototype.name", [\
        ["npm:1.1.5", {\
          "packageLocation": "./.yarn/cache/function.prototype.name-npm-1.1.5-e776a642bb-acd21d733a.zip/node_modules/function.prototype.name/",\
          "packageDependencies": [\
            ["function.prototype.name", "npm:1.1.5"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["es-abstract", "npm:1.21.2"],\
            ["functions-have-names", "npm:1.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["functional-red-black-tree", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/functional-red-black-tree-npm-1.0.1-ccfe924dcd-ca6c170f37.zip/node_modules/functional-red-black-tree/",\
          "packageDependencies": [\
            ["functional-red-black-tree", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["functions-have-names", [\
        ["npm:1.2.3", {\
          "packageLocation": "./.yarn/cache/functions-have-names-npm-1.2.3-e5cf1e2208-c3f1f5ba20.zip/node_modules/functions-have-names/",\
          "packageDependencies": [\
            ["functions-have-names", "npm:1.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["gauge", [\
        ["npm:4.0.4", {\
          "packageLocation": "./.yarn/cache/gauge-npm-4.0.4-8f878385e9-788b6bfe52.zip/node_modules/gauge/",\
          "packageDependencies": [\
            ["gauge", "npm:4.0.4"],\
            ["aproba", "npm:2.0.0"],\
            ["color-support", "npm:1.1.3"],\
            ["console-control-strings", "npm:1.1.0"],\
            ["has-unicode", "npm:2.0.1"],\
            ["signal-exit", "npm:3.0.7"],\
            ["string-width", "npm:4.2.3"],\
            ["strip-ansi", "npm:6.0.1"],\
            ["wide-align", "npm:1.1.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["gensync", [\
        ["npm:1.0.0-beta.2", {\
          "packageLocation": "./.yarn/cache/gensync-npm-1.0.0-beta.2-224666d72f-a7437e58c6.zip/node_modules/gensync/",\
          "packageDependencies": [\
            ["gensync", "npm:1.0.0-beta.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-caller-file", [\
        ["npm:2.0.5", {\
          "packageLocation": "./.yarn/cache/get-caller-file-npm-2.0.5-80e8a86305-b9769a836d.zip/node_modules/get-caller-file/",\
          "packageDependencies": [\
            ["get-caller-file", "npm:2.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-intrinsic", [\
        ["npm:1.2.1", {\
          "packageLocation": "./.yarn/cache/get-intrinsic-npm-1.2.1-ae857fd610-5b61d88552.zip/node_modules/get-intrinsic/",\
          "packageDependencies": [\
            ["get-intrinsic", "npm:1.2.1"],\
            ["function-bind", "npm:1.1.1"],\
            ["has", "npm:1.0.3"],\
            ["has-proto", "npm:1.0.1"],\
            ["has-symbols", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-package-type", [\
        ["npm:0.1.0", {\
          "packageLocation": "./.yarn/cache/get-package-type-npm-0.1.0-6c70cdc8ab-bba0811116.zip/node_modules/get-package-type/",\
          "packageDependencies": [\
            ["get-package-type", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-stdin", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/get-stdin-npm-6.0.0-22ebabe125-593f6fb4ff.zip/node_modules/get-stdin/",\
          "packageDependencies": [\
            ["get-stdin", "npm:6.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-stream", [\
        ["npm:4.1.0", {\
          "packageLocation": "./.yarn/cache/get-stream-npm-4.1.0-314d430a5d-443e191417.zip/node_modules/get-stream/",\
          "packageDependencies": [\
            ["get-stream", "npm:4.1.0"],\
            ["pump", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.2.0", {\
          "packageLocation": "./.yarn/cache/get-stream-npm-5.2.0-2cfd3b452b-8bc1a23174.zip/node_modules/get-stream/",\
          "packageDependencies": [\
            ["get-stream", "npm:5.2.0"],\
            ["pump", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-symbol-description", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/get-symbol-description-npm-1.0.0-9c95a4bc1f-9ceff8fe96.zip/node_modules/get-symbol-description/",\
          "packageDependencies": [\
            ["get-symbol-description", "npm:1.0.0"],\
            ["call-bind", "npm:1.0.2"],\
            ["get-intrinsic", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["get-value", [\
        ["npm:2.0.6", {\
          "packageLocation": "./.yarn/cache/get-value-npm-2.0.6-03cd422e0a-5c3b99cb53.zip/node_modules/get-value/",\
          "packageDependencies": [\
            ["get-value", "npm:2.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["glob", [\
        ["npm:10.3.0", {\
          "packageLocation": "./.yarn/cache/glob-npm-10.3.0-da3187b4ef-6fa4ac0a86.zip/node_modules/glob/",\
          "packageDependencies": [\
            ["glob", "npm:10.3.0"],\
            ["foreground-child", "npm:3.1.1"],\
            ["jackspeak", "npm:2.2.1"],\
            ["minimatch", "npm:9.0.1"],\
            ["minipass", "npm:6.0.2"],\
            ["path-scurry", "npm:1.9.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.1.7", {\
          "packageLocation": "./.yarn/cache/glob-npm-7.1.7-5698ad9c48-b61f48973b.zip/node_modules/glob/",\
          "packageDependencies": [\
            ["glob", "npm:7.1.7"],\
            ["fs.realpath", "npm:1.0.0"],\
            ["inflight", "npm:1.0.6"],\
            ["inherits", "npm:2.0.4"],\
            ["minimatch", "npm:3.1.2"],\
            ["once", "npm:1.4.0"],\
            ["path-is-absolute", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.2.3", {\
          "packageLocation": "./.yarn/cache/glob-npm-7.2.3-2d866d17a5-29452e97b3.zip/node_modules/glob/",\
          "packageDependencies": [\
            ["glob", "npm:7.2.3"],\
            ["fs.realpath", "npm:1.0.0"],\
            ["inflight", "npm:1.0.6"],\
            ["inherits", "npm:2.0.4"],\
            ["minimatch", "npm:3.1.2"],\
            ["once", "npm:1.4.0"],\
            ["path-is-absolute", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["glob-parent", [\
        ["npm:5.1.2", {\
          "packageLocation": "./.yarn/cache/glob-parent-npm-5.1.2-021ab32634-f4f2bfe242.zip/node_modules/glob-parent/",\
          "packageDependencies": [\
            ["glob-parent", "npm:5.1.2"],\
            ["is-glob", "npm:4.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["glob-to-regexp", [\
        ["npm:0.4.1", {\
          "packageLocation": "./.yarn/cache/glob-to-regexp-npm-0.4.1-cd697e0fc7-e795f4e8f0.zip/node_modules/glob-to-regexp/",\
          "packageDependencies": [\
            ["glob-to-regexp", "npm:0.4.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["globals", [\
        ["npm:11.12.0", {\
          "packageLocation": "./.yarn/cache/globals-npm-11.12.0-1fa7f41a6c-67051a45ec.zip/node_modules/globals/",\
          "packageDependencies": [\
            ["globals", "npm:11.12.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:13.20.0", {\
          "packageLocation": "./.yarn/cache/globals-npm-13.20.0-4565a722e7-ad1ecf914b.zip/node_modules/globals/",\
          "packageDependencies": [\
            ["globals", "npm:13.20.0"],\
            ["type-fest", "npm:0.20.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["globalthis", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/globalthis-npm-1.0.3-96cd56020d-fbd7d760dc.zip/node_modules/globalthis/",\
          "packageDependencies": [\
            ["globalthis", "npm:1.0.3"],\
            ["define-properties", "npm:1.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["gopd", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/gopd-npm-1.0.1-10c1d0b534-a5ccfb8806.zip/node_modules/gopd/",\
          "packageDependencies": [\
            ["gopd", "npm:1.0.1"],\
            ["get-intrinsic", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["graceful-fs", [\
        ["npm:4.2.11", {\
          "packageLocation": "./.yarn/cache/graceful-fs-npm-4.2.11-24bb648a68-ac85f94da9.zip/node_modules/graceful-fs/",\
          "packageDependencies": [\
            ["graceful-fs", "npm:4.2.11"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["growly", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/growly-npm-1.3.0-b3a02d4d2a-53cdecd4c1.zip/node_modules/growly/",\
          "packageDependencies": [\
            ["growly", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["harmony-reflect", [\
        ["npm:1.6.2", {\
          "packageLocation": "./.yarn/cache/harmony-reflect-npm-1.6.2-47a9f53361-2e5bae414c.zip/node_modules/harmony-reflect/",\
          "packageDependencies": [\
            ["harmony-reflect", "npm:1.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/has-npm-1.0.3-b7f00631c1-b9ad53d53b.zip/node_modules/has/",\
          "packageDependencies": [\
            ["has", "npm:1.0.3"],\
            ["function-bind", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-bigints", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/has-bigints-npm-1.0.2-52732e614d-390e31e7be.zip/node_modules/has-bigints/",\
          "packageDependencies": [\
            ["has-bigints", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-flag", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/has-flag-npm-3.0.0-16ac11fe05-4a15638b45.zip/node_modules/has-flag/",\
          "packageDependencies": [\
            ["has-flag", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/has-flag-npm-4.0.0-32af9f0536-261a135703.zip/node_modules/has-flag/",\
          "packageDependencies": [\
            ["has-flag", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-property-descriptors", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/has-property-descriptors-npm-1.0.0-56289b918d-a6d3f0a266.zip/node_modules/has-property-descriptors/",\
          "packageDependencies": [\
            ["has-property-descriptors", "npm:1.0.0"],\
            ["get-intrinsic", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-proto", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/has-proto-npm-1.0.1-631ea9d820-febc5b5b53.zip/node_modules/has-proto/",\
          "packageDependencies": [\
            ["has-proto", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-symbols", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/has-symbols-npm-1.0.3-1986bff2c4-a054c40c63.zip/node_modules/has-symbols/",\
          "packageDependencies": [\
            ["has-symbols", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-tostringtag", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/has-tostringtag-npm-1.0.0-b1fcf3ab55-cc12eb28cb.zip/node_modules/has-tostringtag/",\
          "packageDependencies": [\
            ["has-tostringtag", "npm:1.0.0"],\
            ["has-symbols", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-unicode", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/has-unicode-npm-2.0.1-893adb4747-1eab07a743.zip/node_modules/has-unicode/",\
          "packageDependencies": [\
            ["has-unicode", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-value", [\
        ["npm:0.3.1", {\
          "packageLocation": "./.yarn/cache/has-value-npm-0.3.1-4a15b6c29f-29e2a1e657.zip/node_modules/has-value/",\
          "packageDependencies": [\
            ["has-value", "npm:0.3.1"],\
            ["get-value", "npm:2.0.6"],\
            ["has-values", "npm:0.1.4"],\
            ["isobject", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/has-value-npm-1.0.0-19d82fd04b-b9421d354e.zip/node_modules/has-value/",\
          "packageDependencies": [\
            ["has-value", "npm:1.0.0"],\
            ["get-value", "npm:2.0.6"],\
            ["has-values", "npm:1.0.0"],\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["has-values", [\
        ["npm:0.1.4", {\
          "packageLocation": "./.yarn/cache/has-values-npm-0.1.4-6b4397786d-ab1c4bcaf8.zip/node_modules/has-values/",\
          "packageDependencies": [\
            ["has-values", "npm:0.1.4"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/has-values-npm-1.0.0-890c077bbd-77e6693f73.zip/node_modules/has-values/",\
          "packageDependencies": [\
            ["has-values", "npm:1.0.0"],\
            ["is-number", "npm:3.0.0"],\
            ["kind-of", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["hosted-git-info", [\
        ["npm:2.8.9", {\
          "packageLocation": "./.yarn/cache/hosted-git-info-npm-2.8.9-62c44fa93f-c955394bda.zip/node_modules/hosted-git-info/",\
          "packageDependencies": [\
            ["hosted-git-info", "npm:2.8.9"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["html-encoding-sniffer", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/html-encoding-sniffer-npm-2.0.1-381bf15a76-bf30cce461.zip/node_modules/html-encoding-sniffer/",\
          "packageDependencies": [\
            ["html-encoding-sniffer", "npm:2.0.1"],\
            ["whatwg-encoding", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["html-escaper", [\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/html-escaper-npm-2.0.2-38e51ef294-d2df2da3ad.zip/node_modules/html-escaper/",\
          "packageDependencies": [\
            ["html-escaper", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["htmlparser2", [\
        ["npm:8.0.2", {\
          "packageLocation": "./.yarn/cache/htmlparser2-npm-8.0.2-5d9f901bb6-29167a0f92.zip/node_modules/htmlparser2/",\
          "packageDependencies": [\
            ["htmlparser2", "npm:8.0.2"],\
            ["domelementtype", "npm:2.3.0"],\
            ["domhandler", "npm:5.0.3"],\
            ["domutils", "npm:3.1.0"],\
            ["entities", "npm:4.5.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["http-cache-semantics", [\
        ["npm:4.1.1", {\
          "packageLocation": "./.yarn/cache/http-cache-semantics-npm-4.1.1-1120131375-83ac0bc60b.zip/node_modules/http-cache-semantics/",\
          "packageDependencies": [\
            ["http-cache-semantics", "npm:4.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["http-proxy-agent", [\
        ["npm:4.0.1", {\
          "packageLocation": "./.yarn/cache/http-proxy-agent-npm-4.0.1-ce9ef61788-c6a5da5a19.zip/node_modules/http-proxy-agent/",\
          "packageDependencies": [\
            ["http-proxy-agent", "npm:4.0.1"],\
            ["@tootallnate/once", "npm:1.1.2"],\
            ["agent-base", "npm:6.0.2"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/http-proxy-agent-npm-5.0.0-7f1f121b83-e2ee1ff165.zip/node_modules/http-proxy-agent/",\
          "packageDependencies": [\
            ["http-proxy-agent", "npm:5.0.0"],\
            ["@tootallnate/once", "npm:2.0.0"],\
            ["agent-base", "npm:6.0.2"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["https-proxy-agent", [\
        ["npm:5.0.1", {\
          "packageLocation": "./.yarn/cache/https-proxy-agent-npm-5.0.1-42d65f358e-571fccdf38.zip/node_modules/https-proxy-agent/",\
          "packageDependencies": [\
            ["https-proxy-agent", "npm:5.0.1"],\
            ["agent-base", "npm:6.0.2"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["human-signals", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/human-signals-npm-1.1.1-616b2586c2-d587647c9e.zip/node_modules/human-signals/",\
          "packageDependencies": [\
            ["human-signals", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["humanize-ms", [\
        ["npm:1.2.1", {\
          "packageLocation": "./.yarn/cache/humanize-ms-npm-1.2.1-e942bd7329-9c7a74a282.zip/node_modules/humanize-ms/",\
          "packageDependencies": [\
            ["humanize-ms", "npm:1.2.1"],\
            ["ms", "npm:2.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["iconv-lite", [\
        ["npm:0.4.24", {\
          "packageLocation": "./.yarn/cache/iconv-lite-npm-0.4.24-c5c4ac6695-bd9f120f5a.zip/node_modules/iconv-lite/",\
          "packageDependencies": [\
            ["iconv-lite", "npm:0.4.24"],\
            ["safer-buffer", "npm:2.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.6.3", {\
          "packageLocation": "./.yarn/cache/iconv-lite-npm-0.6.3-24b8aae27e-3f60d47a5c.zip/node_modules/iconv-lite/",\
          "packageDependencies": [\
            ["iconv-lite", "npm:0.6.3"],\
            ["safer-buffer", "npm:2.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["icss-utils", [\
        ["npm:4.1.1", {\
          "packageLocation": "./.yarn/cache/icss-utils-npm-4.1.1-9d588ebc46-a4ca2c6b82.zip/node_modules/icss-utils/",\
          "packageDependencies": [\
            ["icss-utils", "npm:4.1.1"],\
            ["postcss", "npm:7.0.39"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.1.0", {\
          "packageLocation": "./.yarn/cache/icss-utils-npm-5.1.0-8d8c062d07-5c324d2835.zip/node_modules/icss-utils/",\
          "packageDependencies": [\
            ["icss-utils", "npm:5.1.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:5.1.0", {\
          "packageLocation": "./.yarn/__virtual__/icss-utils-virtual-e00196ab57/0/cache/icss-utils-npm-5.1.0-8d8c062d07-5c324d2835.zip/node_modules/icss-utils/",\
          "packageDependencies": [\
            ["icss-utils", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:5.1.0"],\
            ["@types/postcss", null],\
            ["postcss", "npm:8.4.24"]\
          ],\
          "packagePeers": [\
            "@types/postcss",\
            "postcss"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["identity-obj-proxy", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/identity-obj-proxy-npm-3.0.0-080ab9ac1e-97559f8ea2.zip/node_modules/identity-obj-proxy/",\
          "packageDependencies": [\
            ["identity-obj-proxy", "npm:3.0.0"],\
            ["harmony-reflect", "npm:1.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ignore", [\
        ["npm:4.0.6", {\
          "packageLocation": "./.yarn/cache/ignore-npm-4.0.6-66c0d6543e-248f82e50a.zip/node_modules/ignore/",\
          "packageDependencies": [\
            ["ignore", "npm:4.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["import-fresh", [\
        ["npm:3.3.0", {\
          "packageLocation": "./.yarn/cache/import-fresh-npm-3.3.0-3e34265ca9-2cacfad06e.zip/node_modules/import-fresh/",\
          "packageDependencies": [\
            ["import-fresh", "npm:3.3.0"],\
            ["parent-module", "npm:1.0.1"],\
            ["resolve-from", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["import-local", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/import-local-npm-3.1.0-8960af5e51-bfcdb63b5e.zip/node_modules/import-local/",\
          "packageDependencies": [\
            ["import-local", "npm:3.1.0"],\
            ["pkg-dir", "npm:4.2.0"],\
            ["resolve-cwd", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["imurmurhash", [\
        ["npm:0.1.4", {\
          "packageLocation": "./.yarn/cache/imurmurhash-npm-0.1.4-610c5068a0-7cae75c8cd.zip/node_modules/imurmurhash/",\
          "packageDependencies": [\
            ["imurmurhash", "npm:0.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["indent-string", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/indent-string-npm-4.0.0-7b717435b2-824cfb9929.zip/node_modules/indent-string/",\
          "packageDependencies": [\
            ["indent-string", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["infer-owner", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/infer-owner-npm-1.0.4-685ac3d2af-181e732764.zip/node_modules/infer-owner/",\
          "packageDependencies": [\
            ["infer-owner", "npm:1.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["inflight", [\
        ["npm:1.0.6", {\
          "packageLocation": "./.yarn/cache/inflight-npm-1.0.6-ccedb4b908-f4f76aa072.zip/node_modules/inflight/",\
          "packageDependencies": [\
            ["inflight", "npm:1.0.6"],\
            ["once", "npm:1.4.0"],\
            ["wrappy", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["inherits", [\
        ["npm:2.0.4", {\
          "packageLocation": "./.yarn/cache/inherits-npm-2.0.4-c66b3957a0-4a48a73384.zip/node_modules/inherits/",\
          "packageDependencies": [\
            ["inherits", "npm:2.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["internal-slot", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/internal-slot-npm-1.0.5-a2241f3e66-97e84046bf.zip/node_modules/internal-slot/",\
          "packageDependencies": [\
            ["internal-slot", "npm:1.0.5"],\
            ["get-intrinsic", "npm:1.2.1"],\
            ["has", "npm:1.0.3"],\
            ["side-channel", "npm:1.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["interpret", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/interpret-npm-2.2.0-3603a544e1-f51efef7cb.zip/node_modules/interpret/",\
          "packageDependencies": [\
            ["interpret", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ip", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/ip-npm-2.0.0-204facb3cc-cfcfac6b87.zip/node_modules/ip/",\
          "packageDependencies": [\
            ["ip", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ipynao", [\
        ["workspace:.", {\
          "packageLocation": "./",\
          "packageDependencies": [\
            ["ipynao", "workspace:."],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/preset-env", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:7.22.5"],\
            ["@jupyter-widgets/base", "npm:6.0.4"],\
            ["@jupyter-widgets/base-manager", "npm:1.0.5"],\
            ["@jupyterlab/builder", "npm:3.6.4"],\
            ["@lumino/application", "npm:1.31.4"],\
            ["@lumino/widgets", "npm:1.37.2"],\
            ["@types/jest", "npm:26.0.24"],\
            ["@types/webpack-env", "npm:1.18.1"],\
            ["@typescript-eslint/eslint-plugin", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["@typescript-eslint/parser", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.10.1"],\
            ["acorn", "npm:7.4.1"],\
            ["css-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.6.0"],\
            ["eslint", "npm:7.32.0"],\
            ["eslint-config-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:6.15.0"],\
            ["eslint-plugin-prettier", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:3.4.1"],\
            ["fs-extra", "npm:7.0.1"],\
            ["identity-obj-proxy", "npm:3.0.0"],\
            ["jest", "npm:26.6.3"],\
            ["mkdirp", "npm:0.5.6"],\
            ["nao-socket.io", "npm:1.0.4"],\
            ["npm-run-all", "npm:4.1.5"],\
            ["prettier", "npm:2.8.8"],\
            ["rimraf", "npm:2.7.1"],\
            ["source-map-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.1.3"],\
            ["style-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.3.0"],\
            ["ts-jest", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:26.5.6"],\
            ["ts-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:8.4.0"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"],\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"]\
          ],\
          "linkType": "SOFT"\
        }]\
      ]],\
      ["is-accessor-descriptor", [\
        ["npm:0.1.6", {\
          "packageLocation": "./.yarn/cache/is-accessor-descriptor-npm-0.1.6-41c495d517-3d629a086a.zip/node_modules/is-accessor-descriptor/",\
          "packageDependencies": [\
            ["is-accessor-descriptor", "npm:0.1.6"],\
            ["kind-of", "npm:3.2.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/is-accessor-descriptor-npm-1.0.0-d8ce016e98-8e475968e9.zip/node_modules/is-accessor-descriptor/",\
          "packageDependencies": [\
            ["is-accessor-descriptor", "npm:1.0.0"],\
            ["kind-of", "npm:6.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-array-buffer", [\
        ["npm:3.0.2", {\
          "packageLocation": "./.yarn/cache/is-array-buffer-npm-3.0.2-0dec897785-dcac9dda66.zip/node_modules/is-array-buffer/",\
          "packageDependencies": [\
            ["is-array-buffer", "npm:3.0.2"],\
            ["call-bind", "npm:1.0.2"],\
            ["get-intrinsic", "npm:1.2.1"],\
            ["is-typed-array", "npm:1.1.10"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-arrayish", [\
        ["npm:0.2.1", {\
          "packageLocation": "./.yarn/cache/is-arrayish-npm-0.2.1-23927dfb15-eef4417e3c.zip/node_modules/is-arrayish/",\
          "packageDependencies": [\
            ["is-arrayish", "npm:0.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-bigint", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/is-bigint-npm-1.0.4-31c2eecbc9-c56edfe09b.zip/node_modules/is-bigint/",\
          "packageDependencies": [\
            ["is-bigint", "npm:1.0.4"],\
            ["has-bigints", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-boolean-object", [\
        ["npm:1.1.2", {\
          "packageLocation": "./.yarn/cache/is-boolean-object-npm-1.1.2-ecbd575e6a-c03b23dbaa.zip/node_modules/is-boolean-object/",\
          "packageDependencies": [\
            ["is-boolean-object", "npm:1.1.2"],\
            ["call-bind", "npm:1.0.2"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-buffer", [\
        ["npm:1.1.6", {\
          "packageLocation": "./.yarn/cache/is-buffer-npm-1.1.6-08199d9ccc-4a186d995d.zip/node_modules/is-buffer/",\
          "packageDependencies": [\
            ["is-buffer", "npm:1.1.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-callable", [\
        ["npm:1.2.7", {\
          "packageLocation": "./.yarn/cache/is-callable-npm-1.2.7-808a303e61-61fd57d03b.zip/node_modules/is-callable/",\
          "packageDependencies": [\
            ["is-callable", "npm:1.2.7"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-ci", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/is-ci-npm-2.0.0-8662a0f445-77b8690575.zip/node_modules/is-ci/",\
          "packageDependencies": [\
            ["is-ci", "npm:2.0.0"],\
            ["ci-info", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-core-module", [\
        ["npm:2.12.1", {\
          "packageLocation": "./.yarn/cache/is-core-module-npm-2.12.1-ce74e89160-f04ea30533.zip/node_modules/is-core-module/",\
          "packageDependencies": [\
            ["is-core-module", "npm:2.12.1"],\
            ["has", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-data-descriptor", [\
        ["npm:0.1.4", {\
          "packageLocation": "./.yarn/cache/is-data-descriptor-npm-0.1.4-6f53f71c67-5c622e078b.zip/node_modules/is-data-descriptor/",\
          "packageDependencies": [\
            ["is-data-descriptor", "npm:0.1.4"],\
            ["kind-of", "npm:3.2.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/is-data-descriptor-npm-1.0.0-f7d2e852ca-e705e68162.zip/node_modules/is-data-descriptor/",\
          "packageDependencies": [\
            ["is-data-descriptor", "npm:1.0.0"],\
            ["kind-of", "npm:6.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-date-object", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/is-date-object-npm-1.0.5-88f3d08b5e-baa9077cdf.zip/node_modules/is-date-object/",\
          "packageDependencies": [\
            ["is-date-object", "npm:1.0.5"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-descriptor", [\
        ["npm:0.1.6", {\
          "packageLocation": "./.yarn/cache/is-descriptor-npm-0.1.6-15c7346839-0f780c1b46.zip/node_modules/is-descriptor/",\
          "packageDependencies": [\
            ["is-descriptor", "npm:0.1.6"],\
            ["is-accessor-descriptor", "npm:0.1.6"],\
            ["is-data-descriptor", "npm:0.1.4"],\
            ["kind-of", "npm:5.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/is-descriptor-npm-1.0.2-5cfc02c444-2ed623560b.zip/node_modules/is-descriptor/",\
          "packageDependencies": [\
            ["is-descriptor", "npm:1.0.2"],\
            ["is-accessor-descriptor", "npm:1.0.0"],\
            ["is-data-descriptor", "npm:1.0.0"],\
            ["kind-of", "npm:6.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-docker", [\
        ["npm:2.2.1", {\
          "packageLocation": "./.yarn/cache/is-docker-npm-2.2.1-3f18a53aff-3fef7ddbf0.zip/node_modules/is-docker/",\
          "packageDependencies": [\
            ["is-docker", "npm:2.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-extendable", [\
        ["npm:0.1.1", {\
          "packageLocation": "./.yarn/cache/is-extendable-npm-0.1.1-322b4649ec-3875571d20.zip/node_modules/is-extendable/",\
          "packageDependencies": [\
            ["is-extendable", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/is-extendable-npm-1.0.1-7095ad8b16-db07bc1e9d.zip/node_modules/is-extendable/",\
          "packageDependencies": [\
            ["is-extendable", "npm:1.0.1"],\
            ["is-plain-object", "npm:2.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-extglob", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/is-extglob-npm-2.1.1-0870ea68b5-df033653d0.zip/node_modules/is-extglob/",\
          "packageDependencies": [\
            ["is-extglob", "npm:2.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-fullwidth-code-point", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/is-fullwidth-code-point-npm-3.0.0-1ecf4ebee5-44a30c2945.zip/node_modules/is-fullwidth-code-point/",\
          "packageDependencies": [\
            ["is-fullwidth-code-point", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-generator-fn", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/is-generator-fn-npm-2.1.0-37895c2d2b-a6ad5492cf.zip/node_modules/is-generator-fn/",\
          "packageDependencies": [\
            ["is-generator-fn", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-glob", [\
        ["npm:4.0.3", {\
          "packageLocation": "./.yarn/cache/is-glob-npm-4.0.3-cb87bf1bdb-d381c1319f.zip/node_modules/is-glob/",\
          "packageDependencies": [\
            ["is-glob", "npm:4.0.3"],\
            ["is-extglob", "npm:2.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-lambda", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/is-lambda-npm-1.0.1-7ab55bc8a8-93a32f0194.zip/node_modules/is-lambda/",\
          "packageDependencies": [\
            ["is-lambda", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-negative-zero", [\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/is-negative-zero-npm-2.0.2-0adac91f15-f3232194c4.zip/node_modules/is-negative-zero/",\
          "packageDependencies": [\
            ["is-negative-zero", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-number", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/is-number-npm-3.0.0-9088035ade-0c62bf8e9d.zip/node_modules/is-number/",\
          "packageDependencies": [\
            ["is-number", "npm:3.0.0"],\
            ["kind-of", "npm:3.2.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.0.0", {\
          "packageLocation": "./.yarn/cache/is-number-npm-7.0.0-060086935c-456ac6f8e0.zip/node_modules/is-number/",\
          "packageDependencies": [\
            ["is-number", "npm:7.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-number-object", [\
        ["npm:1.0.7", {\
          "packageLocation": "./.yarn/cache/is-number-object-npm-1.0.7-539d0e274d-d1e8d01bb0.zip/node_modules/is-number-object/",\
          "packageDependencies": [\
            ["is-number-object", "npm:1.0.7"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-plain-object", [\
        ["npm:2.0.4", {\
          "packageLocation": "./.yarn/cache/is-plain-object-npm-2.0.4-da3265d804-2a401140cf.zip/node_modules/is-plain-object/",\
          "packageDependencies": [\
            ["is-plain-object", "npm:2.0.4"],\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/is-plain-object-npm-5.0.0-285b70faa3-e32d27061e.zip/node_modules/is-plain-object/",\
          "packageDependencies": [\
            ["is-plain-object", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-potential-custom-element-name", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/is-potential-custom-element-name-npm-1.0.1-f352f606f8-ced7bbbb64.zip/node_modules/is-potential-custom-element-name/",\
          "packageDependencies": [\
            ["is-potential-custom-element-name", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-regex", [\
        ["npm:1.1.4", {\
          "packageLocation": "./.yarn/cache/is-regex-npm-1.1.4-cca193ef11-362399b335.zip/node_modules/is-regex/",\
          "packageDependencies": [\
            ["is-regex", "npm:1.1.4"],\
            ["call-bind", "npm:1.0.2"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-shared-array-buffer", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/is-shared-array-buffer-npm-1.0.2-32e4181fcd-9508929cf1.zip/node_modules/is-shared-array-buffer/",\
          "packageDependencies": [\
            ["is-shared-array-buffer", "npm:1.0.2"],\
            ["call-bind", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-stream", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/is-stream-npm-1.1.0-818ecbf6bb-063c6bec9d.zip/node_modules/is-stream/",\
          "packageDependencies": [\
            ["is-stream", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/is-stream-npm-2.0.1-c802db55e7-b8e05ccdf9.zip/node_modules/is-stream/",\
          "packageDependencies": [\
            ["is-stream", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-string", [\
        ["npm:1.0.7", {\
          "packageLocation": "./.yarn/cache/is-string-npm-1.0.7-9f7066daed-323b3d0462.zip/node_modules/is-string/",\
          "packageDependencies": [\
            ["is-string", "npm:1.0.7"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-symbol", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/is-symbol-npm-1.0.4-eb9baac703-92805812ef.zip/node_modules/is-symbol/",\
          "packageDependencies": [\
            ["is-symbol", "npm:1.0.4"],\
            ["has-symbols", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-typed-array", [\
        ["npm:1.1.10", {\
          "packageLocation": "./.yarn/cache/is-typed-array-npm-1.1.10-fe4ef83cdc-aac6ecb59d.zip/node_modules/is-typed-array/",\
          "packageDependencies": [\
            ["is-typed-array", "npm:1.1.10"],\
            ["available-typed-arrays", "npm:1.0.5"],\
            ["call-bind", "npm:1.0.2"],\
            ["for-each", "npm:0.3.3"],\
            ["gopd", "npm:1.0.1"],\
            ["has-tostringtag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-typedarray", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/is-typedarray-npm-1.0.0-bbd99de5b6-3508c6cd0a.zip/node_modules/is-typedarray/",\
          "packageDependencies": [\
            ["is-typedarray", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-weakref", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/is-weakref-npm-1.0.2-ff80e8c314-95bd9a57cd.zip/node_modules/is-weakref/",\
          "packageDependencies": [\
            ["is-weakref", "npm:1.0.2"],\
            ["call-bind", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-windows", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/is-windows-npm-1.0.2-898cd6f3d7-438b7e5265.zip/node_modules/is-windows/",\
          "packageDependencies": [\
            ["is-windows", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["is-wsl", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/is-wsl-npm-2.2.0-2ba10d6393-20849846ae.zip/node_modules/is-wsl/",\
          "packageDependencies": [\
            ["is-wsl", "npm:2.2.0"],\
            ["is-docker", "npm:2.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["isarray", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/isarray-npm-1.0.0-db4f547720-f032df8e02.zip/node_modules/isarray/",\
          "packageDependencies": [\
            ["isarray", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["isexe", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/isexe-npm-2.0.0-b58870bd2e-26bf6c5480.zip/node_modules/isexe/",\
          "packageDependencies": [\
            ["isexe", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["isobject", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/isobject-npm-2.1.0-2798cf0d94-811c6f5a86.zip/node_modules/isobject/",\
          "packageDependencies": [\
            ["isobject", "npm:2.1.0"],\
            ["isarray", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/isobject-npm-3.0.1-8145901fd2-db85c4c970.zip/node_modules/isobject/",\
          "packageDependencies": [\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["istanbul-lib-coverage", [\
        ["npm:3.2.0", {\
          "packageLocation": "./.yarn/cache/istanbul-lib-coverage-npm-3.2.0-93f84b2c8c-a2a545033b.zip/node_modules/istanbul-lib-coverage/",\
          "packageDependencies": [\
            ["istanbul-lib-coverage", "npm:3.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["istanbul-lib-instrument", [\
        ["npm:4.0.3", {\
          "packageLocation": "./.yarn/cache/istanbul-lib-instrument-npm-4.0.3-4d4c2263f8-fa1171d302.zip/node_modules/istanbul-lib-instrument/",\
          "packageDependencies": [\
            ["istanbul-lib-instrument", "npm:4.0.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@istanbuljs/schema", "npm:0.1.3"],\
            ["istanbul-lib-coverage", "npm:3.2.0"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.2.1", {\
          "packageLocation": "./.yarn/cache/istanbul-lib-instrument-npm-5.2.1-1b3ad719a9-bf16f1803b.zip/node_modules/istanbul-lib-instrument/",\
          "packageDependencies": [\
            ["istanbul-lib-instrument", "npm:5.2.1"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@babel/parser", "npm:7.22.5"],\
            ["@istanbuljs/schema", "npm:0.1.3"],\
            ["istanbul-lib-coverage", "npm:3.2.0"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["istanbul-lib-report", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/istanbul-lib-report-npm-3.0.0-660f97340a-3f29eb3f53.zip/node_modules/istanbul-lib-report/",\
          "packageDependencies": [\
            ["istanbul-lib-report", "npm:3.0.0"],\
            ["istanbul-lib-coverage", "npm:3.2.0"],\
            ["make-dir", "npm:3.1.0"],\
            ["supports-color", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["istanbul-lib-source-maps", [\
        ["npm:4.0.1", {\
          "packageLocation": "./.yarn/cache/istanbul-lib-source-maps-npm-4.0.1-af0f859df7-21ad3df45d.zip/node_modules/istanbul-lib-source-maps/",\
          "packageDependencies": [\
            ["istanbul-lib-source-maps", "npm:4.0.1"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["istanbul-lib-coverage", "npm:3.2.0"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["istanbul-reports", [\
        ["npm:3.1.5", {\
          "packageLocation": "./.yarn/cache/istanbul-reports-npm-3.1.5-fb11324e3e-7867228f83.zip/node_modules/istanbul-reports/",\
          "packageDependencies": [\
            ["istanbul-reports", "npm:3.1.5"],\
            ["html-escaper", "npm:2.0.2"],\
            ["istanbul-lib-report", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jackspeak", [\
        ["npm:2.2.1", {\
          "packageLocation": "./.yarn/cache/jackspeak-npm-2.2.1-0644c98bfe-e29291c0d0.zip/node_modules/jackspeak/",\
          "packageDependencies": [\
            ["jackspeak", "npm:2.2.1"],\
            ["@isaacs/cliui", "npm:8.0.2"],\
            ["@pkgjs/parseargs", "npm:0.11.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-npm-26.6.3-dafe93d52f-3a9b4c70e9.zip/node_modules/jest/",\
          "packageDependencies": [\
            ["jest", "npm:26.6.3"],\
            ["@jest/core", "npm:26.6.3"],\
            ["import-local", "npm:3.1.0"],\
            ["jest-cli", "npm:26.6.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-changed-files", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-changed-files-npm-26.6.2-f511617ef2-8c405f5ff9.zip/node_modules/jest-changed-files/",\
          "packageDependencies": [\
            ["jest-changed-files", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["execa", "npm:4.1.0"],\
            ["throat", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-cli", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-cli-npm-26.6.3-983e1fcf7f-c8554147be.zip/node_modules/jest-cli/",\
          "packageDependencies": [\
            ["jest-cli", "npm:26.6.3"],\
            ["@jest/core", "npm:26.6.3"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["exit", "npm:0.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["import-local", "npm:3.1.0"],\
            ["is-ci", "npm:2.0.0"],\
            ["jest-config", "virtual:bf7a8695861ccc96c7503a95daba2b038c9b3eca0fc65dc5ea7e5ae0e56354c6c3e3ee05f1c8d4420e3a01abf48ad9e2dea477db48ad56147605b32adf33b489#npm:26.6.3"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-validate", "npm:26.6.2"],\
            ["prompts", "npm:2.4.2"],\
            ["yargs", "npm:15.4.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-config", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-config-npm-26.6.3-ac5d27f4ad-303c798582.zip/node_modules/jest-config/",\
          "packageDependencies": [\
            ["jest-config", "npm:26.6.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:bf7a8695861ccc96c7503a95daba2b038c9b3eca0fc65dc5ea7e5ae0e56354c6c3e3ee05f1c8d4420e3a01abf48ad9e2dea477db48ad56147605b32adf33b489#npm:26.6.3", {\
          "packageLocation": "./.yarn/__virtual__/jest-config-virtual-caddf51df4/0/cache/jest-config-npm-26.6.3-ac5d27f4ad-303c798582.zip/node_modules/jest-config/",\
          "packageDependencies": [\
            ["jest-config", "virtual:bf7a8695861ccc96c7503a95daba2b038c9b3eca0fc65dc5ea7e5ae0e56354c6c3e3ee05f1c8d4420e3a01abf48ad9e2dea477db48ad56147605b32adf33b489#npm:26.6.3"],\
            ["@babel/core", "npm:7.22.5"],\
            ["@jest/test-sequencer", "npm:26.6.3"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/ts-node", null],\
            ["babel-jest", "virtual:caddf51df4928b33a437ca87b8f5ddfb6205ebd6d8231f74d4ee7223f3866e6f815b221aa1e2bd33e98915f701e95bae72a93d2288b49a34a6246bdbc2a4a132#npm:26.6.3"],\
            ["chalk", "npm:4.1.2"],\
            ["deepmerge", "npm:4.3.1"],\
            ["glob", "npm:7.2.3"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-environment-jsdom", "npm:26.6.2"],\
            ["jest-environment-node", "npm:26.6.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["jest-jasmine2", "npm:26.6.3"],\
            ["jest-regex-util", "npm:26.0.0"],\
            ["jest-resolve", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-validate", "npm:26.6.2"],\
            ["micromatch", "npm:4.0.5"],\
            ["pretty-format", "npm:26.6.2"],\
            ["ts-node", null]\
          ],\
          "packagePeers": [\
            "@types/ts-node",\
            "ts-node"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-diff", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-diff-npm-26.6.2-06ca2a96ca-d00d297f31.zip/node_modules/jest-diff/",\
          "packageDependencies": [\
            ["jest-diff", "npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["diff-sequences", "npm:26.6.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["pretty-format", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-docblock", [\
        ["npm:26.0.0", {\
          "packageLocation": "./.yarn/cache/jest-docblock-npm-26.0.0-7d0129b0be-e03ef104ee.zip/node_modules/jest-docblock/",\
          "packageDependencies": [\
            ["jest-docblock", "npm:26.0.0"],\
            ["detect-newline", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-each", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-each-npm-26.6.2-651078df67-4e00ea4667.zip/node_modules/jest-each/",\
          "packageDependencies": [\
            ["jest-each", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["jest-util", "npm:26.6.2"],\
            ["pretty-format", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-environment-jsdom", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-environment-jsdom-npm-26.6.2-defa486869-8af9ffdf1b.zip/node_modules/jest-environment-jsdom/",\
          "packageDependencies": [\
            ["jest-environment-jsdom", "npm:26.6.2"],\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/fake-timers", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["jest-mock", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["jsdom", "virtual:defa486869c88441047200a53b3aa18d79743b272095f3ee31b5b7b80b2c93d87f722added867470dcb94104504489a1a89040ea8fd89dffb9cfb1864d4bf54e#npm:16.7.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-environment-node", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-environment-node-npm-26.6.2-9030128a2f-0b69b481e6.zip/node_modules/jest-environment-node/",\
          "packageDependencies": [\
            ["jest-environment-node", "npm:26.6.2"],\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/fake-timers", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["jest-mock", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-get-type", [\
        ["npm:26.3.0", {\
          "packageLocation": "./.yarn/cache/jest-get-type-npm-26.3.0-a481f14d96-1cc6465ae4.zip/node_modules/jest-get-type/",\
          "packageDependencies": [\
            ["jest-get-type", "npm:26.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-haste-map", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-haste-map-npm-26.6.2-16ea967617-8ad5236d56.zip/node_modules/jest-haste-map/",\
          "packageDependencies": [\
            ["jest-haste-map", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/graceful-fs", "npm:4.1.6"],\
            ["@types/node", "npm:20.3.1"],\
            ["anymatch", "npm:3.1.3"],\
            ["fb-watchman", "npm:2.0.2"],\
            ["fsevents", "patch:fsevents@npm%3A2.3.2#~builtin<compat/fsevents>::version=2.3.2&hash=df0bf1"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-regex-util", "npm:26.0.0"],\
            ["jest-serializer", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-worker", "npm:26.6.2"],\
            ["micromatch", "npm:4.0.5"],\
            ["sane", "npm:4.1.0"],\
            ["walker", "npm:1.0.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-jasmine2", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-jasmine2-npm-26.6.3-aba0c11c28-41df0b993a.zip/node_modules/jest-jasmine2/",\
          "packageDependencies": [\
            ["jest-jasmine2", "npm:26.6.3"],\
            ["@babel/traverse", "npm:7.22.5"],\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/source-map", "npm:26.6.2"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["chalk", "npm:4.1.2"],\
            ["co", "npm:4.6.0"],\
            ["expect", "npm:26.6.2"],\
            ["is-generator-fn", "npm:2.1.0"],\
            ["jest-each", "npm:26.6.2"],\
            ["jest-matcher-utils", "npm:26.6.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-runtime", "npm:26.6.3"],\
            ["jest-snapshot", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["pretty-format", "npm:26.6.2"],\
            ["throat", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-leak-detector", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-leak-detector-npm-26.6.2-3b253a4eef-364dd4d021.zip/node_modules/jest-leak-detector/",\
          "packageDependencies": [\
            ["jest-leak-detector", "npm:26.6.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["pretty-format", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-matcher-utils", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-matcher-utils-npm-26.6.2-90bcfd468b-74d2165c1a.zip/node_modules/jest-matcher-utils/",\
          "packageDependencies": [\
            ["jest-matcher-utils", "npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["jest-diff", "npm:26.6.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["pretty-format", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-message-util", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-message-util-npm-26.6.2-2dd6f71234-ffe5a71559.zip/node_modules/jest-message-util/",\
          "packageDependencies": [\
            ["jest-message-util", "npm:26.6.2"],\
            ["@babel/code-frame", "npm:7.22.5"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/stack-utils", "npm:2.0.1"],\
            ["chalk", "npm:4.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["micromatch", "npm:4.0.5"],\
            ["pretty-format", "npm:26.6.2"],\
            ["slash", "npm:3.0.0"],\
            ["stack-utils", "npm:2.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-mock", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-mock-npm-26.6.2-87f47530a8-6c0fe028ff.zip/node_modules/jest-mock/",\
          "packageDependencies": [\
            ["jest-mock", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-pnp-resolver", [\
        ["npm:1.2.3", {\
          "packageLocation": "./.yarn/cache/jest-pnp-resolver-npm-1.2.3-70e06bf27c-db1a8ab2cb.zip/node_modules/jest-pnp-resolver/",\
          "packageDependencies": [\
            ["jest-pnp-resolver", "npm:1.2.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:6ad3c87f852a744f0ca052ddcf60cfb1d20f148e44c4d3bc933aed1297626798d738b65c746a40d5eb58079971d680449b7c0894918212fabb0b9f5575e9f921#npm:1.2.3", {\
          "packageLocation": "./.yarn/__virtual__/jest-pnp-resolver-virtual-9b8bc2e9c8/0/cache/jest-pnp-resolver-npm-1.2.3-70e06bf27c-db1a8ab2cb.zip/node_modules/jest-pnp-resolver/",\
          "packageDependencies": [\
            ["jest-pnp-resolver", "virtual:6ad3c87f852a744f0ca052ddcf60cfb1d20f148e44c4d3bc933aed1297626798d738b65c746a40d5eb58079971d680449b7c0894918212fabb0b9f5575e9f921#npm:1.2.3"],\
            ["@types/jest-resolve", null],\
            ["jest-resolve", "npm:26.6.2"]\
          ],\
          "packagePeers": [\
            "@types/jest-resolve",\
            "jest-resolve"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-regex-util", [\
        ["npm:26.0.0", {\
          "packageLocation": "./.yarn/cache/jest-regex-util-npm-26.0.0-310f72dd82-930a00665e.zip/node_modules/jest-regex-util/",\
          "packageDependencies": [\
            ["jest-regex-util", "npm:26.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-resolve", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-resolve-npm-26.6.2-6ad3c87f85-d6264d3f39.zip/node_modules/jest-resolve/",\
          "packageDependencies": [\
            ["jest-resolve", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["chalk", "npm:4.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-pnp-resolver", "virtual:6ad3c87f852a744f0ca052ddcf60cfb1d20f148e44c4d3bc933aed1297626798d738b65c746a40d5eb58079971d680449b7c0894918212fabb0b9f5575e9f921#npm:1.2.3"],\
            ["jest-util", "npm:26.6.2"],\
            ["read-pkg-up", "npm:7.0.1"],\
            ["resolve", "patch:resolve@npm%3A1.22.3#~builtin<compat/resolve>::version=1.22.3&hash=c3c19d"],\
            ["slash", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-resolve-dependencies", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-resolve-dependencies-npm-26.6.3-e36bbb2abc-533ea1e271.zip/node_modules/jest-resolve-dependencies/",\
          "packageDependencies": [\
            ["jest-resolve-dependencies", "npm:26.6.3"],\
            ["@jest/types", "npm:26.6.2"],\
            ["jest-regex-util", "npm:26.0.0"],\
            ["jest-snapshot", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-runner", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-runner-npm-26.6.3-4a4a361df8-ccd69918ba.zip/node_modules/jest-runner/",\
          "packageDependencies": [\
            ["jest-runner", "npm:26.6.3"],\
            ["@jest/console", "npm:26.6.2"],\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["chalk", "npm:4.1.2"],\
            ["emittery", "npm:0.7.2"],\
            ["exit", "npm:0.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-config", "virtual:bf7a8695861ccc96c7503a95daba2b038c9b3eca0fc65dc5ea7e5ae0e56354c6c3e3ee05f1c8d4420e3a01abf48ad9e2dea477db48ad56147605b32adf33b489#npm:26.6.3"],\
            ["jest-docblock", "npm:26.0.0"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-leak-detector", "npm:26.6.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-resolve", "npm:26.6.2"],\
            ["jest-runtime", "npm:26.6.3"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-worker", "npm:26.6.2"],\
            ["source-map-support", "npm:0.5.21"],\
            ["throat", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-runtime", [\
        ["npm:26.6.3", {\
          "packageLocation": "./.yarn/cache/jest-runtime-npm-26.6.3-bf7a869586-867922b49f.zip/node_modules/jest-runtime/",\
          "packageDependencies": [\
            ["jest-runtime", "npm:26.6.3"],\
            ["@jest/console", "npm:26.6.2"],\
            ["@jest/environment", "npm:26.6.2"],\
            ["@jest/fake-timers", "npm:26.6.2"],\
            ["@jest/globals", "npm:26.6.2"],\
            ["@jest/source-map", "npm:26.6.2"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/transform", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/yargs", "npm:15.0.15"],\
            ["chalk", "npm:4.1.2"],\
            ["cjs-module-lexer", "npm:0.6.0"],\
            ["collect-v8-coverage", "npm:1.0.1"],\
            ["exit", "npm:0.1.2"],\
            ["glob", "npm:7.2.3"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-config", "virtual:bf7a8695861ccc96c7503a95daba2b038c9b3eca0fc65dc5ea7e5ae0e56354c6c3e3ee05f1c8d4420e3a01abf48ad9e2dea477db48ad56147605b32adf33b489#npm:26.6.3"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-mock", "npm:26.6.2"],\
            ["jest-regex-util", "npm:26.0.0"],\
            ["jest-resolve", "npm:26.6.2"],\
            ["jest-snapshot", "npm:26.6.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["jest-validate", "npm:26.6.2"],\
            ["slash", "npm:3.0.0"],\
            ["strip-bom", "npm:4.0.0"],\
            ["yargs", "npm:15.4.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-serializer", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-serializer-npm-26.6.2-0907990487-dbecfb0d01.zip/node_modules/jest-serializer/",\
          "packageDependencies": [\
            ["jest-serializer", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["graceful-fs", "npm:4.2.11"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-snapshot", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-snapshot-npm-26.6.2-b91f41a0f2-53f1de055b.zip/node_modules/jest-snapshot/",\
          "packageDependencies": [\
            ["jest-snapshot", "npm:26.6.2"],\
            ["@babel/types", "npm:7.22.5"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/babel__traverse", "npm:7.20.1"],\
            ["@types/prettier", "npm:2.7.3"],\
            ["chalk", "npm:4.1.2"],\
            ["expect", "npm:26.6.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["jest-diff", "npm:26.6.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["jest-haste-map", "npm:26.6.2"],\
            ["jest-matcher-utils", "npm:26.6.2"],\
            ["jest-message-util", "npm:26.6.2"],\
            ["jest-resolve", "npm:26.6.2"],\
            ["natural-compare", "npm:1.4.0"],\
            ["pretty-format", "npm:26.6.2"],\
            ["semver", "npm:7.5.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-util", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-util-npm-26.6.2-28a10c2acf-3c6a5fba05.zip/node_modules/jest-util/",\
          "packageDependencies": [\
            ["jest-util", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["chalk", "npm:4.1.2"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["is-ci", "npm:2.0.0"],\
            ["micromatch", "npm:4.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-validate", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-validate-npm-26.6.2-26ab48a006-bac11d6586.zip/node_modules/jest-validate/",\
          "packageDependencies": [\
            ["jest-validate", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["camelcase", "npm:6.3.0"],\
            ["chalk", "npm:4.1.2"],\
            ["jest-get-type", "npm:26.3.0"],\
            ["leven", "npm:3.1.0"],\
            ["pretty-format", "npm:26.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-watcher", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-watcher-npm-26.6.2-d4afa66ea5-401137f1a7.zip/node_modules/jest-watcher/",\
          "packageDependencies": [\
            ["jest-watcher", "npm:26.6.2"],\
            ["@jest/test-result", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["ansi-escapes", "npm:4.3.2"],\
            ["chalk", "npm:4.1.2"],\
            ["jest-util", "npm:26.6.2"],\
            ["string-length", "npm:4.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jest-worker", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/jest-worker-npm-26.6.2-46cbcd449f-f9afa3b88e.zip/node_modules/jest-worker/",\
          "packageDependencies": [\
            ["jest-worker", "npm:26.6.2"],\
            ["@types/node", "npm:20.3.1"],\
            ["merge-stream", "npm:2.0.0"],\
            ["supports-color", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:27.5.1", {\
          "packageLocation": "./.yarn/cache/jest-worker-npm-27.5.1-1c110b5894-98cd68b696.zip/node_modules/jest-worker/",\
          "packageDependencies": [\
            ["jest-worker", "npm:27.5.1"],\
            ["@types/node", "npm:20.3.1"],\
            ["merge-stream", "npm:2.0.0"],\
            ["supports-color", "npm:8.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jquery", [\
        ["npm:3.7.0", {\
          "packageLocation": "./.yarn/cache/jquery-npm-3.7.0-a02a382bf4-907785e133.zip/node_modules/jquery/",\
          "packageDependencies": [\
            ["jquery", "npm:3.7.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["js-tokens", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/js-tokens-npm-4.0.0-0ac852e9e2-8a95213a5a.zip/node_modules/js-tokens/",\
          "packageDependencies": [\
            ["js-tokens", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["js-yaml", [\
        ["npm:3.14.1", {\
          "packageLocation": "./.yarn/cache/js-yaml-npm-3.14.1-b968c6095e-bef146085f.zip/node_modules/js-yaml/",\
          "packageDependencies": [\
            ["js-yaml", "npm:3.14.1"],\
            ["argparse", "npm:1.0.10"],\
            ["esprima", "npm:4.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jsdom", [\
        ["npm:16.7.0", {\
          "packageLocation": "./.yarn/cache/jsdom-npm-16.7.0-216c5c4bf9-454b833718.zip/node_modules/jsdom/",\
          "packageDependencies": [\
            ["jsdom", "npm:16.7.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:defa486869c88441047200a53b3aa18d79743b272095f3ee31b5b7b80b2c93d87f722added867470dcb94104504489a1a89040ea8fd89dffb9cfb1864d4bf54e#npm:16.7.0", {\
          "packageLocation": "./.yarn/__virtual__/jsdom-virtual-d1d747a211/0/cache/jsdom-npm-16.7.0-216c5c4bf9-454b833718.zip/node_modules/jsdom/",\
          "packageDependencies": [\
            ["jsdom", "virtual:defa486869c88441047200a53b3aa18d79743b272095f3ee31b5b7b80b2c93d87f722added867470dcb94104504489a1a89040ea8fd89dffb9cfb1864d4bf54e#npm:16.7.0"],\
            ["@types/canvas", null],\
            ["abab", "npm:2.0.6"],\
            ["acorn", "npm:8.9.0"],\
            ["acorn-globals", "npm:6.0.0"],\
            ["canvas", null],\
            ["cssom", "npm:0.4.4"],\
            ["cssstyle", "npm:2.3.0"],\
            ["data-urls", "npm:2.0.0"],\
            ["decimal.js", "npm:10.4.3"],\
            ["domexception", "npm:2.0.1"],\
            ["escodegen", "npm:2.0.0"],\
            ["form-data", "npm:3.0.1"],\
            ["html-encoding-sniffer", "npm:2.0.1"],\
            ["http-proxy-agent", "npm:4.0.1"],\
            ["https-proxy-agent", "npm:5.0.1"],\
            ["is-potential-custom-element-name", "npm:1.0.1"],\
            ["nwsapi", "npm:2.2.5"],\
            ["parse5", "npm:6.0.1"],\
            ["saxes", "npm:5.0.1"],\
            ["symbol-tree", "npm:3.2.4"],\
            ["tough-cookie", "npm:4.1.3"],\
            ["w3c-hr-time", "npm:1.0.2"],\
            ["w3c-xmlserializer", "npm:2.0.0"],\
            ["webidl-conversions", "npm:6.1.0"],\
            ["whatwg-encoding", "npm:1.0.5"],\
            ["whatwg-mimetype", "npm:2.3.0"],\
            ["whatwg-url", "npm:8.7.0"],\
            ["ws", "virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:7.5.9"],\
            ["xml-name-validator", "npm:3.0.0"]\
          ],\
          "packagePeers": [\
            "@types/canvas",\
            "canvas"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jsesc", [\
        ["npm:0.5.0", {\
          "packageLocation": "./.yarn/cache/jsesc-npm-0.5.0-6827074492-b8b44cbfc9.zip/node_modules/jsesc/",\
          "packageDependencies": [\
            ["jsesc", "npm:0.5.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.5.2", {\
          "packageLocation": "./.yarn/cache/jsesc-npm-2.5.2-c5acb78804-4dc1907711.zip/node_modules/jsesc/",\
          "packageDependencies": [\
            ["jsesc", "npm:2.5.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["json-parse-better-errors", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/json-parse-better-errors-npm-1.0.2-7f37637d19-ff2b5ba2a7.zip/node_modules/json-parse-better-errors/",\
          "packageDependencies": [\
            ["json-parse-better-errors", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["json-parse-even-better-errors", [\
        ["npm:2.3.1", {\
          "packageLocation": "./.yarn/cache/json-parse-even-better-errors-npm-2.3.1-144d62256e-798ed4cf33.zip/node_modules/json-parse-even-better-errors/",\
          "packageDependencies": [\
            ["json-parse-even-better-errors", "npm:2.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["json-schema-traverse", [\
        ["npm:0.4.1", {\
          "packageLocation": "./.yarn/cache/json-schema-traverse-npm-0.4.1-4759091693-7486074d3b.zip/node_modules/json-schema-traverse/",\
          "packageDependencies": [\
            ["json-schema-traverse", "npm:0.4.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/json-schema-traverse-npm-1.0.0-fb3684f4f0-02f2f466cd.zip/node_modules/json-schema-traverse/",\
          "packageDependencies": [\
            ["json-schema-traverse", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["json-stable-stringify-without-jsonify", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/json-stable-stringify-without-jsonify-npm-1.0.1-b65772b28b-cff44156dd.zip/node_modules/json-stable-stringify-without-jsonify/",\
          "packageDependencies": [\
            ["json-stable-stringify-without-jsonify", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["json5", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/json5-npm-1.0.2-9607f93e30-866458a8c5.zip/node_modules/json5/",\
          "packageDependencies": [\
            ["json5", "npm:1.0.2"],\
            ["minimist", "npm:1.2.8"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.2.3", {\
          "packageLocation": "./.yarn/cache/json5-npm-2.2.3-9962c55073-2a7436a933.zip/node_modules/json5/",\
          "packageDependencies": [\
            ["json5", "npm:2.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["jsonfile", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/jsonfile-npm-4.0.0-10ce3aea15-6447d6224f.zip/node_modules/jsonfile/",\
          "packageDependencies": [\
            ["jsonfile", "npm:4.0.0"],\
            ["graceful-fs", "npm:4.2.11"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.1.0", {\
          "packageLocation": "./.yarn/cache/jsonfile-npm-6.1.0-20a4796cee-7af3b8e1ac.zip/node_modules/jsonfile/",\
          "packageDependencies": [\
            ["jsonfile", "npm:6.1.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["universalify", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["kind-of", [\
        ["npm:3.2.2", {\
          "packageLocation": "./.yarn/cache/kind-of-npm-3.2.2-7deaffa5f9-e898df8ca2.zip/node_modules/kind-of/",\
          "packageDependencies": [\
            ["kind-of", "npm:3.2.2"],\
            ["is-buffer", "npm:1.1.6"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/kind-of-npm-4.0.0-69fd153375-1b9e7624a8.zip/node_modules/kind-of/",\
          "packageDependencies": [\
            ["kind-of", "npm:4.0.0"],\
            ["is-buffer", "npm:1.1.6"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.1.0", {\
          "packageLocation": "./.yarn/cache/kind-of-npm-5.1.0-ce82f43eaa-f2a0102ae0.zip/node_modules/kind-of/",\
          "packageDependencies": [\
            ["kind-of", "npm:5.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.0.3", {\
          "packageLocation": "./.yarn/cache/kind-of-npm-6.0.3-ab15f36220-3ab01e7b1d.zip/node_modules/kind-of/",\
          "packageDependencies": [\
            ["kind-of", "npm:6.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["kleur", [\
        ["npm:3.0.3", {\
          "packageLocation": "./.yarn/cache/kleur-npm-3.0.3-f6f53649a4-df82cd1e17.zip/node_modules/kleur/",\
          "packageDependencies": [\
            ["kleur", "npm:3.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["leven", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/leven-npm-3.1.0-b7697736a3-638401d534.zip/node_modules/leven/",\
          "packageDependencies": [\
            ["leven", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["levn", [\
        ["npm:0.3.0", {\
          "packageLocation": "./.yarn/cache/levn-npm-0.3.0-48d774b1c2-0d084a5242.zip/node_modules/levn/",\
          "packageDependencies": [\
            ["levn", "npm:0.3.0"],\
            ["prelude-ls", "npm:1.1.2"],\
            ["type-check", "npm:0.3.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.4.1", {\
          "packageLocation": "./.yarn/cache/levn-npm-0.4.1-d183b2d7bb-12c5021c85.zip/node_modules/levn/",\
          "packageDependencies": [\
            ["levn", "npm:0.4.1"],\
            ["prelude-ls", "npm:1.2.1"],\
            ["type-check", "npm:0.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["license-webpack-plugin", [\
        ["npm:2.3.21", {\
          "packageLocation": "./.yarn/cache/license-webpack-plugin-npm-2.3.21-74008160a2-6208bd2060.zip/node_modules/license-webpack-plugin/",\
          "packageDependencies": [\
            ["license-webpack-plugin", "npm:2.3.21"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:2.3.21", {\
          "packageLocation": "./.yarn/__virtual__/license-webpack-plugin-virtual-7efb993f9d/0/cache/license-webpack-plugin-npm-2.3.21-74008160a2-6208bd2060.zip/node_modules/license-webpack-plugin/",\
          "packageDependencies": [\
            ["license-webpack-plugin", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:2.3.21"],\
            ["@types/webpack", null],\
            ["@types/webpack-sources", "npm:0.1.9"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["webpack-sources", "npm:1.4.3"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["lines-and-columns", [\
        ["npm:1.2.4", {\
          "packageLocation": "./.yarn/cache/lines-and-columns-npm-1.2.4-d6c7cc5799-0c37f9f7fa.zip/node_modules/lines-and-columns/",\
          "packageDependencies": [\
            ["lines-and-columns", "npm:1.2.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["load-json-file", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/load-json-file-npm-4.0.0-c9f09d85eb-8f5d6d93ba.zip/node_modules/load-json-file/",\
          "packageDependencies": [\
            ["load-json-file", "npm:4.0.0"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["parse-json", "npm:4.0.0"],\
            ["pify", "npm:3.0.0"],\
            ["strip-bom", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["loader-runner", [\
        ["npm:4.3.0", {\
          "packageLocation": "./.yarn/cache/loader-runner-npm-4.3.0-9ca67df372-a90e00dee9.zip/node_modules/loader-runner/",\
          "packageDependencies": [\
            ["loader-runner", "npm:4.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["loader-utils", [\
        ["npm:1.4.2", {\
          "packageLocation": "./.yarn/cache/loader-utils-npm-1.4.2-b4537b2f88-eb6fb622ef.zip/node_modules/loader-utils/",\
          "packageDependencies": [\
            ["loader-utils", "npm:1.4.2"],\
            ["big.js", "npm:5.2.2"],\
            ["emojis-list", "npm:3.0.0"],\
            ["json5", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.4", {\
          "packageLocation": "./.yarn/cache/loader-utils-npm-2.0.4-ba3800585b-a5281f5fff.zip/node_modules/loader-utils/",\
          "packageDependencies": [\
            ["loader-utils", "npm:2.0.4"],\
            ["big.js", "npm:5.2.2"],\
            ["emojis-list", "npm:3.0.0"],\
            ["json5", "npm:2.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["locate-path", [\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/locate-path-npm-5.0.0-46580c43e4-83e51725e6.zip/node_modules/locate-path/",\
          "packageDependencies": [\
            ["locate-path", "npm:5.0.0"],\
            ["p-locate", "npm:4.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["lodash", [\
        ["npm:4.17.21", {\
          "packageLocation": "./.yarn/cache/lodash-npm-4.17.21-6382451519-eb835a2e51.zip/node_modules/lodash/",\
          "packageDependencies": [\
            ["lodash", "npm:4.17.21"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["lodash.debounce", [\
        ["npm:4.0.8", {\
          "packageLocation": "./.yarn/cache/lodash.debounce-npm-4.0.8-f1d6e09799-a3f527d22c.zip/node_modules/lodash.debounce/",\
          "packageDependencies": [\
            ["lodash.debounce", "npm:4.0.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["lodash.merge", [\
        ["npm:4.6.2", {\
          "packageLocation": "./.yarn/cache/lodash.merge-npm-4.6.2-77cb4416bf-ad580b4bdb.zip/node_modules/lodash.merge/",\
          "packageDependencies": [\
            ["lodash.merge", "npm:4.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["lodash.truncate", [\
        ["npm:4.4.2", {\
          "packageLocation": "./.yarn/cache/lodash.truncate-npm-4.4.2-bc50fe1663-b463d8a382.zip/node_modules/lodash.truncate/",\
          "packageDependencies": [\
            ["lodash.truncate", "npm:4.4.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["lru-cache", [\
        ["npm:5.1.1", {\
          "packageLocation": "./.yarn/cache/lru-cache-npm-5.1.1-f475882a51-c154ae1cbb.zip/node_modules/lru-cache/",\
          "packageDependencies": [\
            ["lru-cache", "npm:5.1.1"],\
            ["yallist", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/lru-cache-npm-6.0.0-b4c8668fe1-f97f499f89.zip/node_modules/lru-cache/",\
          "packageDependencies": [\
            ["lru-cache", "npm:6.0.0"],\
            ["yallist", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.18.3", {\
          "packageLocation": "./.yarn/cache/lru-cache-npm-7.18.3-e68be5b11c-e550d77238.zip/node_modules/lru-cache/",\
          "packageDependencies": [\
            ["lru-cache", "npm:7.18.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:9.1.2", {\
          "packageLocation": "./.yarn/cache/lru-cache-npm-9.1.2-4846dc8c34-d3415634be.zip/node_modules/lru-cache/",\
          "packageDependencies": [\
            ["lru-cache", "npm:9.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["make-dir", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/make-dir-npm-3.1.0-d1d7505142-484200020a.zip/node_modules/make-dir/",\
          "packageDependencies": [\
            ["make-dir", "npm:3.1.0"],\
            ["semver", "npm:6.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["make-error", [\
        ["npm:1.3.6", {\
          "packageLocation": "./.yarn/cache/make-error-npm-1.3.6-ccb85d9458-b86e5e0e25.zip/node_modules/make-error/",\
          "packageDependencies": [\
            ["make-error", "npm:1.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["make-fetch-happen", [\
        ["npm:11.1.1", {\
          "packageLocation": "./.yarn/cache/make-fetch-happen-npm-11.1.1-f32b79aaaa-7268bf274a.zip/node_modules/make-fetch-happen/",\
          "packageDependencies": [\
            ["make-fetch-happen", "npm:11.1.1"],\
            ["agentkeepalive", "npm:4.3.0"],\
            ["cacache", "npm:17.1.3"],\
            ["http-cache-semantics", "npm:4.1.1"],\
            ["http-proxy-agent", "npm:5.0.0"],\
            ["https-proxy-agent", "npm:5.0.1"],\
            ["is-lambda", "npm:1.0.1"],\
            ["lru-cache", "npm:7.18.3"],\
            ["minipass", "npm:5.0.0"],\
            ["minipass-fetch", "npm:3.0.3"],\
            ["minipass-flush", "npm:1.0.5"],\
            ["minipass-pipeline", "npm:1.2.4"],\
            ["negotiator", "npm:0.6.3"],\
            ["promise-retry", "npm:2.0.1"],\
            ["socks-proxy-agent", "npm:7.0.0"],\
            ["ssri", "npm:10.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["makeerror", [\
        ["npm:1.0.12", {\
          "packageLocation": "./.yarn/cache/makeerror-npm-1.0.12-69abf085d7-b38a025a12.zip/node_modules/makeerror/",\
          "packageDependencies": [\
            ["makeerror", "npm:1.0.12"],\
            ["tmpl", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["map-cache", [\
        ["npm:0.2.2", {\
          "packageLocation": "./.yarn/cache/map-cache-npm-0.2.2-1620199b05-3067cea542.zip/node_modules/map-cache/",\
          "packageDependencies": [\
            ["map-cache", "npm:0.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["map-visit", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/map-visit-npm-1.0.0-33a7988a9d-c27045a502.zip/node_modules/map-visit/",\
          "packageDependencies": [\
            ["map-visit", "npm:1.0.0"],\
            ["object-visit", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["memory-fs", [\
        ["npm:0.5.0", {\
          "packageLocation": "./.yarn/cache/memory-fs-npm-0.5.0-8be5938449-a9f25b0a8e.zip/node_modules/memory-fs/",\
          "packageDependencies": [\
            ["memory-fs", "npm:0.5.0"],\
            ["errno", "npm:0.1.8"],\
            ["readable-stream", "npm:2.3.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["memorystream", [\
        ["npm:0.3.1", {\
          "packageLocation": "./.yarn/cache/memorystream-npm-0.3.1-ae973f1d16-f18b42440d.zip/node_modules/memorystream/",\
          "packageDependencies": [\
            ["memorystream", "npm:0.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["merge-stream", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/merge-stream-npm-2.0.0-2ac83efea5-6fa4dcc8d8.zip/node_modules/merge-stream/",\
          "packageDependencies": [\
            ["merge-stream", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["micromatch", [\
        ["npm:3.1.10", {\
          "packageLocation": "./.yarn/cache/micromatch-npm-3.1.10-016e80c79d-ad226cba4d.zip/node_modules/micromatch/",\
          "packageDependencies": [\
            ["micromatch", "npm:3.1.10"],\
            ["arr-diff", "npm:4.0.0"],\
            ["array-unique", "npm:0.3.2"],\
            ["braces", "npm:2.3.2"],\
            ["define-property", "npm:2.0.2"],\
            ["extend-shallow", "npm:3.0.2"],\
            ["extglob", "npm:2.0.4"],\
            ["fragment-cache", "npm:0.2.1"],\
            ["kind-of", "npm:6.0.3"],\
            ["nanomatch", "npm:1.2.13"],\
            ["object.pick", "npm:1.3.0"],\
            ["regex-not", "npm:1.0.2"],\
            ["snapdragon", "npm:0.8.2"],\
            ["to-regex", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.5", {\
          "packageLocation": "./.yarn/cache/micromatch-npm-4.0.5-cfab5d7669-02a17b671c.zip/node_modules/micromatch/",\
          "packageDependencies": [\
            ["micromatch", "npm:4.0.5"],\
            ["braces", "npm:3.0.2"],\
            ["picomatch", "npm:2.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["mime-db", [\
        ["npm:1.52.0", {\
          "packageLocation": "./.yarn/cache/mime-db-npm-1.52.0-b5371d6fd2-0d99a03585.zip/node_modules/mime-db/",\
          "packageDependencies": [\
            ["mime-db", "npm:1.52.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["mime-types", [\
        ["npm:2.1.35", {\
          "packageLocation": "./.yarn/cache/mime-types-npm-2.1.35-dd9ea9f3e2-89a5b7f1de.zip/node_modules/mime-types/",\
          "packageDependencies": [\
            ["mime-types", "npm:2.1.35"],\
            ["mime-db", "npm:1.52.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["mimic-fn", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/mimic-fn-npm-2.1.0-4fbeb3abb4-d2421a3444.zip/node_modules/mimic-fn/",\
          "packageDependencies": [\
            ["mimic-fn", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["mini-css-extract-plugin", [\
        ["npm:1.3.9", {\
          "packageLocation": "./.yarn/cache/mini-css-extract-plugin-npm-1.3.9-33c6a7bca5-19d0f07c5b.zip/node_modules/mini-css-extract-plugin/",\
          "packageDependencies": [\
            ["mini-css-extract-plugin", "npm:1.3.9"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:1.3.9", {\
          "packageLocation": "./.yarn/__virtual__/mini-css-extract-plugin-virtual-338da7cbc9/0/cache/mini-css-extract-plugin-npm-1.3.9-33c6a7bca5-19d0f07c5b.zip/node_modules/mini-css-extract-plugin/",\
          "packageDependencies": [\
            ["mini-css-extract-plugin", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:1.3.9"],\
            ["@types/webpack", null],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:3.3.0"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["webpack-sources", "npm:1.4.3"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minimatch", [\
        ["npm:3.1.2", {\
          "packageLocation": "./.yarn/cache/minimatch-npm-3.1.2-9405269906-c154e56640.zip/node_modules/minimatch/",\
          "packageDependencies": [\
            ["minimatch", "npm:3.1.2"],\
            ["brace-expansion", "npm:1.1.11"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:9.0.1", {\
          "packageLocation": "./.yarn/cache/minimatch-npm-9.0.1-277fdc6fbd-97f5f5284b.zip/node_modules/minimatch/",\
          "packageDependencies": [\
            ["minimatch", "npm:9.0.1"],\
            ["brace-expansion", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minimist", [\
        ["npm:1.2.8", {\
          "packageLocation": "./.yarn/cache/minimist-npm-1.2.8-d7af7b1dce-75a6d645fb.zip/node_modules/minimist/",\
          "packageDependencies": [\
            ["minimist", "npm:1.2.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minipass", [\
        ["npm:3.3.6", {\
          "packageLocation": "./.yarn/cache/minipass-npm-3.3.6-b8d93a945b-a30d083c80.zip/node_modules/minipass/",\
          "packageDependencies": [\
            ["minipass", "npm:3.3.6"],\
            ["yallist", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/minipass-npm-5.0.0-c64fb63c92-425dab2887.zip/node_modules/minipass/",\
          "packageDependencies": [\
            ["minipass", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.0.2", {\
          "packageLocation": "./.yarn/cache/minipass-npm-6.0.2-a7fca64b94-d140b91f4a.zip/node_modules/minipass/",\
          "packageDependencies": [\
            ["minipass", "npm:6.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minipass-collect", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/minipass-collect-npm-1.0.2-3b4676eab5-14df761028.zip/node_modules/minipass-collect/",\
          "packageDependencies": [\
            ["minipass-collect", "npm:1.0.2"],\
            ["minipass", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minipass-fetch", [\
        ["npm:3.0.3", {\
          "packageLocation": "./.yarn/cache/minipass-fetch-npm-3.0.3-2c4966d142-af5ab2552a.zip/node_modules/minipass-fetch/",\
          "packageDependencies": [\
            ["minipass-fetch", "npm:3.0.3"],\
            ["encoding", "npm:0.1.13"],\
            ["minipass", "npm:5.0.0"],\
            ["minipass-sized", "npm:1.0.3"],\
            ["minizlib", "npm:2.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minipass-flush", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/minipass-flush-npm-1.0.5-efe79d9826-56269a0b22.zip/node_modules/minipass-flush/",\
          "packageDependencies": [\
            ["minipass-flush", "npm:1.0.5"],\
            ["minipass", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minipass-pipeline", [\
        ["npm:1.2.4", {\
          "packageLocation": "./.yarn/cache/minipass-pipeline-npm-1.2.4-5924cb077f-b14240dac0.zip/node_modules/minipass-pipeline/",\
          "packageDependencies": [\
            ["minipass-pipeline", "npm:1.2.4"],\
            ["minipass", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minipass-sized", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/minipass-sized-npm-1.0.3-306d86f432-79076749fc.zip/node_modules/minipass-sized/",\
          "packageDependencies": [\
            ["minipass-sized", "npm:1.0.3"],\
            ["minipass", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["minizlib", [\
        ["npm:2.1.2", {\
          "packageLocation": "./.yarn/cache/minizlib-npm-2.1.2-ea89cd0cfb-f1fdeac0b0.zip/node_modules/minizlib/",\
          "packageDependencies": [\
            ["minizlib", "npm:2.1.2"],\
            ["minipass", "npm:3.3.6"],\
            ["yallist", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["mixin-deep", [\
        ["npm:1.3.2", {\
          "packageLocation": "./.yarn/cache/mixin-deep-npm-1.3.2-29b528e571-820d5a51fc.zip/node_modules/mixin-deep/",\
          "packageDependencies": [\
            ["mixin-deep", "npm:1.3.2"],\
            ["for-in", "npm:1.0.2"],\
            ["is-extendable", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["mkdirp", [\
        ["npm:0.5.6", {\
          "packageLocation": "./.yarn/cache/mkdirp-npm-0.5.6-dcd5a6b97b-0c91b721bb.zip/node_modules/mkdirp/",\
          "packageDependencies": [\
            ["mkdirp", "npm:0.5.6"],\
            ["minimist", "npm:1.2.8"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/mkdirp-npm-1.0.4-37f6ef56b9-a96865108c.zip/node_modules/mkdirp/",\
          "packageDependencies": [\
            ["mkdirp", "npm:1.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["moment", [\
        ["npm:2.29.4", {\
          "packageLocation": "./.yarn/cache/moment-npm-2.29.4-902943305d-0ec3f9c2bc.zip/node_modules/moment/",\
          "packageDependencies": [\
            ["moment", "npm:2.29.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ms", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/ms-npm-2.0.0-9e1101a471-0e6a22b8b7.zip/node_modules/ms/",\
          "packageDependencies": [\
            ["ms", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.1.2", {\
          "packageLocation": "./.yarn/cache/ms-npm-2.1.2-ec0c1512ff-673cdb2c31.zip/node_modules/ms/",\
          "packageDependencies": [\
            ["ms", "npm:2.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.1.3", {\
          "packageLocation": "./.yarn/cache/ms-npm-2.1.3-81ff3cfac1-aa92de6080.zip/node_modules/ms/",\
          "packageDependencies": [\
            ["ms", "npm:2.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nan", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/unplugged/nan-npm-1.0.0-d088aa8424/node_modules/nan/",\
          "packageDependencies": [\
            ["nan", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nanoid", [\
        ["npm:3.3.6", {\
          "packageLocation": "./.yarn/cache/nanoid-npm-3.3.6-e6d6ae7e71-7d0eda6570.zip/node_modules/nanoid/",\
          "packageDependencies": [\
            ["nanoid", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nanomatch", [\
        ["npm:1.2.13", {\
          "packageLocation": "./.yarn/cache/nanomatch-npm-1.2.13-bc9173dbe7-54d4166d6e.zip/node_modules/nanomatch/",\
          "packageDependencies": [\
            ["nanomatch", "npm:1.2.13"],\
            ["arr-diff", "npm:4.0.0"],\
            ["array-unique", "npm:0.3.2"],\
            ["define-property", "npm:2.0.2"],\
            ["extend-shallow", "npm:3.0.2"],\
            ["fragment-cache", "npm:0.2.1"],\
            ["is-windows", "npm:1.0.2"],\
            ["kind-of", "npm:6.0.3"],\
            ["object.pick", "npm:1.3.0"],\
            ["regex-not", "npm:1.0.2"],\
            ["snapdragon", "npm:0.8.2"],\
            ["to-regex", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nao-socket.io", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/nao-socket.io-npm-1.0.4-80a28e46d3-40b97b4680.zip/node_modules/nao-socket.io/",\
          "packageDependencies": [\
            ["nao-socket.io", "npm:1.0.4"],\
            ["base64id", "npm:0.1.0"],\
            ["policyfile", "npm:0.0.4"],\
            ["redis", "npm:0.7.3"],\
            ["socket.io-client", "npm:0.9.16"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["natural-compare", [\
        ["npm:1.4.0", {\
          "packageLocation": "./.yarn/cache/natural-compare-npm-1.4.0-97b75b362d-23ad088b08.zip/node_modules/natural-compare/",\
          "packageDependencies": [\
            ["natural-compare", "npm:1.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["negotiator", [\
        ["npm:0.6.3", {\
          "packageLocation": "./.yarn/cache/negotiator-npm-0.6.3-9d50e36171-b8ffeb1e26.zip/node_modules/negotiator/",\
          "packageDependencies": [\
            ["negotiator", "npm:0.6.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["neo-async", [\
        ["npm:2.6.2", {\
          "packageLocation": "./.yarn/cache/neo-async-npm-2.6.2-75d6902586-deac9f8d00.zip/node_modules/neo-async/",\
          "packageDependencies": [\
            ["neo-async", "npm:2.6.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nice-try", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/nice-try-npm-1.0.5-963856b16f-0b4af3b5bb.zip/node_modules/nice-try/",\
          "packageDependencies": [\
            ["nice-try", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["node-fetch", [\
        ["npm:2.6.11", {\
          "packageLocation": "./.yarn/cache/node-fetch-npm-2.6.11-160e4174c3-249d0666a9.zip/node_modules/node-fetch/",\
          "packageDependencies": [\
            ["node-fetch", "npm:2.6.11"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:2.6.11", {\
          "packageLocation": "./.yarn/__virtual__/node-fetch-virtual-4576af0716/0/cache/node-fetch-npm-2.6.11-160e4174c3-249d0666a9.zip/node_modules/node-fetch/",\
          "packageDependencies": [\
            ["node-fetch", "virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:2.6.11"],\
            ["@types/encoding", null],\
            ["encoding", null],\
            ["whatwg-url", "npm:5.0.0"]\
          ],\
          "packagePeers": [\
            "@types/encoding",\
            "encoding"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["node-gyp", [\
        ["npm:9.4.0", {\
          "packageLocation": "./.yarn/unplugged/node-gyp-npm-9.4.0-ebf5f5573e/node_modules/node-gyp/",\
          "packageDependencies": [\
            ["node-gyp", "npm:9.4.0"],\
            ["env-paths", "npm:2.2.1"],\
            ["exponential-backoff", "npm:3.1.1"],\
            ["glob", "npm:7.2.3"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["make-fetch-happen", "npm:11.1.1"],\
            ["nopt", "npm:6.0.0"],\
            ["npmlog", "npm:6.0.2"],\
            ["rimraf", "npm:3.0.2"],\
            ["semver", "npm:7.5.3"],\
            ["tar", "npm:6.1.15"],\
            ["which", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["node-int64", [\
        ["npm:0.4.0", {\
          "packageLocation": "./.yarn/cache/node-int64-npm-0.4.0-0dc04ec3b2-d0b30b1ee6.zip/node_modules/node-int64/",\
          "packageDependencies": [\
            ["node-int64", "npm:0.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["node-notifier", [\
        ["npm:8.0.2", {\
          "packageLocation": "./.yarn/unplugged/node-notifier-npm-8.0.2-e8d89b5d6b/node_modules/node-notifier/",\
          "packageDependencies": [\
            ["node-notifier", "npm:8.0.2"],\
            ["growly", "npm:1.3.0"],\
            ["is-wsl", "npm:2.2.0"],\
            ["semver", "npm:7.5.3"],\
            ["shellwords", "npm:0.1.1"],\
            ["uuid", "npm:8.3.2"],\
            ["which", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["node-releases", [\
        ["npm:2.0.12", {\
          "packageLocation": "./.yarn/cache/node-releases-npm-2.0.12-888ed1398a-b8c56db82c.zip/node_modules/node-releases/",\
          "packageDependencies": [\
            ["node-releases", "npm:2.0.12"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nopt", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/nopt-npm-6.0.0-5ea8050815-82149371f8.zip/node_modules/nopt/",\
          "packageDependencies": [\
            ["nopt", "npm:6.0.0"],\
            ["abbrev", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["normalize-package-data", [\
        ["npm:2.5.0", {\
          "packageLocation": "./.yarn/cache/normalize-package-data-npm-2.5.0-af0345deed-7999112efc.zip/node_modules/normalize-package-data/",\
          "packageDependencies": [\
            ["normalize-package-data", "npm:2.5.0"],\
            ["hosted-git-info", "npm:2.8.9"],\
            ["resolve", "patch:resolve@npm%3A1.22.3#~builtin<compat/resolve>::version=1.22.3&hash=c3c19d"],\
            ["semver", "npm:5.7.1"],\
            ["validate-npm-package-license", "npm:3.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["normalize-path", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/normalize-path-npm-2.1.1-65c4766716-7e9cbdcf7f.zip/node_modules/normalize-path/",\
          "packageDependencies": [\
            ["normalize-path", "npm:2.1.1"],\
            ["remove-trailing-separator", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/normalize-path-npm-3.0.0-658ba7d77f-88eeb4da89.zip/node_modules/normalize-path/",\
          "packageDependencies": [\
            ["normalize-path", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["npm-run-all", [\
        ["npm:4.1.5", {\
          "packageLocation": "./.yarn/cache/npm-run-all-npm-4.1.5-3281f1c563-373b72c6a3.zip/node_modules/npm-run-all/",\
          "packageDependencies": [\
            ["npm-run-all", "npm:4.1.5"],\
            ["ansi-styles", "npm:3.2.1"],\
            ["chalk", "npm:2.4.2"],\
            ["cross-spawn", "npm:6.0.5"],\
            ["memorystream", "npm:0.3.1"],\
            ["minimatch", "npm:3.1.2"],\
            ["pidtree", "npm:0.3.1"],\
            ["read-pkg", "npm:3.0.0"],\
            ["shell-quote", "npm:1.8.1"],\
            ["string.prototype.padend", "npm:3.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["npm-run-path", [\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/npm-run-path-npm-2.0.2-96c8b48857-acd5ad8164.zip/node_modules/npm-run-path/",\
          "packageDependencies": [\
            ["npm-run-path", "npm:2.0.2"],\
            ["path-key", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.1", {\
          "packageLocation": "./.yarn/cache/npm-run-path-npm-4.0.1-7aebd8bab3-5374c0cea4.zip/node_modules/npm-run-path/",\
          "packageDependencies": [\
            ["npm-run-path", "npm:4.0.1"],\
            ["path-key", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["npmlog", [\
        ["npm:6.0.2", {\
          "packageLocation": "./.yarn/cache/npmlog-npm-6.0.2-e0e69455c7-ae238cd264.zip/node_modules/npmlog/",\
          "packageDependencies": [\
            ["npmlog", "npm:6.0.2"],\
            ["are-we-there-yet", "npm:3.0.1"],\
            ["console-control-strings", "npm:1.1.0"],\
            ["gauge", "npm:4.0.4"],\
            ["set-blocking", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["nwsapi", [\
        ["npm:2.2.5", {\
          "packageLocation": "./.yarn/cache/nwsapi-npm-2.2.5-c2a1b1f329-3acfe38721.zip/node_modules/nwsapi/",\
          "packageDependencies": [\
            ["nwsapi", "npm:2.2.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["object-copy", [\
        ["npm:0.1.0", {\
          "packageLocation": "./.yarn/cache/object-copy-npm-0.1.0-e229d02f2b-a9e35f07e3.zip/node_modules/object-copy/",\
          "packageDependencies": [\
            ["object-copy", "npm:0.1.0"],\
            ["copy-descriptor", "npm:0.1.1"],\
            ["define-property", "npm:0.2.5"],\
            ["kind-of", "npm:3.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["object-inspect", [\
        ["npm:1.12.3", {\
          "packageLocation": "./.yarn/cache/object-inspect-npm-1.12.3-1e7d20f5ff-dabfd824d9.zip/node_modules/object-inspect/",\
          "packageDependencies": [\
            ["object-inspect", "npm:1.12.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["object-keys", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/object-keys-npm-1.1.1-1bf2f1be93-b363c5e764.zip/node_modules/object-keys/",\
          "packageDependencies": [\
            ["object-keys", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["object-visit", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/object-visit-npm-1.0.1-c5c9057c24-b0ee07f5bf.zip/node_modules/object-visit/",\
          "packageDependencies": [\
            ["object-visit", "npm:1.0.1"],\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["object.assign", [\
        ["npm:4.1.4", {\
          "packageLocation": "./.yarn/cache/object.assign-npm-4.1.4-fb3deb1c3a-76cab513a5.zip/node_modules/object.assign/",\
          "packageDependencies": [\
            ["object.assign", "npm:4.1.4"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["has-symbols", "npm:1.0.3"],\
            ["object-keys", "npm:1.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["object.pick", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/object.pick-npm-1.3.0-dad8eae8fb-77fb6eed57.zip/node_modules/object.pick/",\
          "packageDependencies": [\
            ["object.pick", "npm:1.3.0"],\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["once", [\
        ["npm:1.4.0", {\
          "packageLocation": "./.yarn/cache/once-npm-1.4.0-ccf03ef07a-cd0a885013.zip/node_modules/once/",\
          "packageDependencies": [\
            ["once", "npm:1.4.0"],\
            ["wrappy", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["onetime", [\
        ["npm:5.1.2", {\
          "packageLocation": "./.yarn/cache/onetime-npm-5.1.2-3ed148fa42-2478859ef8.zip/node_modules/onetime/",\
          "packageDependencies": [\
            ["onetime", "npm:5.1.2"],\
            ["mimic-fn", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["optionator", [\
        ["npm:0.8.3", {\
          "packageLocation": "./.yarn/cache/optionator-npm-0.8.3-bc555bc5b7-b8695ddf3d.zip/node_modules/optionator/",\
          "packageDependencies": [\
            ["optionator", "npm:0.8.3"],\
            ["deep-is", "npm:0.1.4"],\
            ["fast-levenshtein", "npm:2.0.6"],\
            ["levn", "npm:0.3.0"],\
            ["prelude-ls", "npm:1.1.2"],\
            ["type-check", "npm:0.3.2"],\
            ["word-wrap", "npm:1.2.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.9.1", {\
          "packageLocation": "./.yarn/cache/optionator-npm-0.9.1-577e397aae-dbc6fa0656.zip/node_modules/optionator/",\
          "packageDependencies": [\
            ["optionator", "npm:0.9.1"],\
            ["deep-is", "npm:0.1.4"],\
            ["fast-levenshtein", "npm:2.0.6"],\
            ["levn", "npm:0.4.1"],\
            ["prelude-ls", "npm:1.2.1"],\
            ["type-check", "npm:0.4.0"],\
            ["word-wrap", "npm:1.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["options", [\
        ["npm:0.0.6", {\
          "packageLocation": "./.yarn/cache/options-npm-0.0.6-1fce20f056-8601fdc0a3.zip/node_modules/options/",\
          "packageDependencies": [\
            ["options", "npm:0.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["p-each-series", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/p-each-series-npm-2.2.0-b9907a1ae6-5fbe2f1f19.zip/node_modules/p-each-series/",\
          "packageDependencies": [\
            ["p-each-series", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["p-finally", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/p-finally-npm-1.0.0-35fbaa57c6-93a654c53d.zip/node_modules/p-finally/",\
          "packageDependencies": [\
            ["p-finally", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["p-limit", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/p-limit-npm-2.3.0-94a0310039-84ff17f1a3.zip/node_modules/p-limit/",\
          "packageDependencies": [\
            ["p-limit", "npm:2.3.0"],\
            ["p-try", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/p-limit-npm-3.1.0-05d2ede37f-7c3690c4db.zip/node_modules/p-limit/",\
          "packageDependencies": [\
            ["p-limit", "npm:3.1.0"],\
            ["yocto-queue", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["p-locate", [\
        ["npm:4.1.0", {\
          "packageLocation": "./.yarn/cache/p-locate-npm-4.1.0-eec6872537-513bd14a45.zip/node_modules/p-locate/",\
          "packageDependencies": [\
            ["p-locate", "npm:4.1.0"],\
            ["p-limit", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["p-map", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/p-map-npm-4.0.0-4677ae07c7-cb0ab21ec0.zip/node_modules/p-map/",\
          "packageDependencies": [\
            ["p-map", "npm:4.0.0"],\
            ["aggregate-error", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["p-try", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/p-try-npm-2.2.0-e0390dbaf8-f8a8e9a769.zip/node_modules/p-try/",\
          "packageDependencies": [\
            ["p-try", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["parent-module", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/parent-module-npm-1.0.1-1fae11b095-6ba8b25514.zip/node_modules/parent-module/",\
          "packageDependencies": [\
            ["parent-module", "npm:1.0.1"],\
            ["callsites", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["parse-json", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/parse-json-npm-4.0.0-a6f7771010-0fe227d410.zip/node_modules/parse-json/",\
          "packageDependencies": [\
            ["parse-json", "npm:4.0.0"],\
            ["error-ex", "npm:1.3.2"],\
            ["json-parse-better-errors", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.2.0", {\
          "packageLocation": "./.yarn/cache/parse-json-npm-5.2.0-00a63b1199-62085b17d6.zip/node_modules/parse-json/",\
          "packageDependencies": [\
            ["parse-json", "npm:5.2.0"],\
            ["@babel/code-frame", "npm:7.22.5"],\
            ["error-ex", "npm:1.3.2"],\
            ["json-parse-even-better-errors", "npm:2.3.1"],\
            ["lines-and-columns", "npm:1.2.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["parse-srcset", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/parse-srcset-npm-1.0.2-8acc142245-3a0380380c.zip/node_modules/parse-srcset/",\
          "packageDependencies": [\
            ["parse-srcset", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["parse5", [\
        ["npm:6.0.1", {\
          "packageLocation": "./.yarn/cache/parse5-npm-6.0.1-70a35a494a-7d569a176c.zip/node_modules/parse5/",\
          "packageDependencies": [\
            ["parse5", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pascalcase", [\
        ["npm:0.1.1", {\
          "packageLocation": "./.yarn/cache/pascalcase-npm-0.1.1-d04964fcda-f83681c3c8.zip/node_modules/pascalcase/",\
          "packageDependencies": [\
            ["pascalcase", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-browserify", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/path-browserify-npm-1.0.1-f975d99a99-c6d7fa3764.zip/node_modules/path-browserify/",\
          "packageDependencies": [\
            ["path-browserify", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-exists", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/path-exists-npm-4.0.0-e9e4f63eb0-505807199d.zip/node_modules/path-exists/",\
          "packageDependencies": [\
            ["path-exists", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-is-absolute", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/path-is-absolute-npm-1.0.1-31bc695ffd-060840f92c.zip/node_modules/path-is-absolute/",\
          "packageDependencies": [\
            ["path-is-absolute", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-key", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/path-key-npm-2.0.1-b1a971833d-f7ab0ad42f.zip/node_modules/path-key/",\
          "packageDependencies": [\
            ["path-key", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/path-key-npm-3.1.1-0e66ea8321-55cd7a9dd4.zip/node_modules/path-key/",\
          "packageDependencies": [\
            ["path-key", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-parse", [\
        ["npm:1.0.7", {\
          "packageLocation": "./.yarn/cache/path-parse-npm-1.0.7-09564527b7-49abf3d811.zip/node_modules/path-parse/",\
          "packageDependencies": [\
            ["path-parse", "npm:1.0.7"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-scurry", [\
        ["npm:1.9.2", {\
          "packageLocation": "./.yarn/cache/path-scurry-npm-1.9.2-e4789f2bee-92888dfb68.zip/node_modules/path-scurry/",\
          "packageDependencies": [\
            ["path-scurry", "npm:1.9.2"],\
            ["lru-cache", "npm:9.1.2"],\
            ["minipass", "npm:6.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["path-type", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/path-type-npm-3.0.0-252361a0eb-735b35e256.zip/node_modules/path-type/",\
          "packageDependencies": [\
            ["path-type", "npm:3.0.0"],\
            ["pify", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["picocolors", [\
        ["npm:0.2.1", {\
          "packageLocation": "./.yarn/cache/picocolors-npm-0.2.1-fa0e648c44-3b0f441f00.zip/node_modules/picocolors/",\
          "packageDependencies": [\
            ["picocolors", "npm:0.2.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/picocolors-npm-1.0.0-d81e0b1927-a2e8092dd8.zip/node_modules/picocolors/",\
          "packageDependencies": [\
            ["picocolors", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["picomatch", [\
        ["npm:2.3.1", {\
          "packageLocation": "./.yarn/cache/picomatch-npm-2.3.1-c782cfd986-050c865ce8.zip/node_modules/picomatch/",\
          "packageDependencies": [\
            ["picomatch", "npm:2.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pidtree", [\
        ["npm:0.3.1", {\
          "packageLocation": "./.yarn/cache/pidtree-npm-0.3.1-70dda1cc59-eb49025099.zip/node_modules/pidtree/",\
          "packageDependencies": [\
            ["pidtree", "npm:0.3.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pify", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/pify-npm-3.0.0-679ee405c8-6cdcbc3567.zip/node_modules/pify/",\
          "packageDependencies": [\
            ["pify", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pirates", [\
        ["npm:4.0.6", {\
          "packageLocation": "./.yarn/cache/pirates-npm-4.0.6-a8ec571a43-46a65fefaf.zip/node_modules/pirates/",\
          "packageDependencies": [\
            ["pirates", "npm:4.0.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pkg-dir", [\
        ["npm:4.2.0", {\
          "packageLocation": "./.yarn/cache/pkg-dir-npm-4.2.0-2b5d0a8d32-9863e3f351.zip/node_modules/pkg-dir/",\
          "packageDependencies": [\
            ["pkg-dir", "npm:4.2.0"],\
            ["find-up", "npm:4.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["policyfile", [\
        ["npm:0.0.4", {\
          "packageLocation": "./.yarn/cache/policyfile-npm-0.0.4-b04c2b1558-5a55915696.zip/node_modules/policyfile/",\
          "packageDependencies": [\
            ["policyfile", "npm:0.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["posix-character-classes", [\
        ["npm:0.1.1", {\
          "packageLocation": "./.yarn/cache/posix-character-classes-npm-0.1.1-3e228a6e15-dedb99913c.zip/node_modules/posix-character-classes/",\
          "packageDependencies": [\
            ["posix-character-classes", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss", [\
        ["npm:7.0.39", {\
          "packageLocation": "./.yarn/cache/postcss-npm-7.0.39-0f8737296e-4ac793f506.zip/node_modules/postcss/",\
          "packageDependencies": [\
            ["postcss", "npm:7.0.39"],\
            ["picocolors", "npm:0.2.1"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.4.24", {\
          "packageLocation": "./.yarn/cache/postcss-npm-8.4.24-07c10836e5-814e2126da.zip/node_modules/postcss/",\
          "packageDependencies": [\
            ["postcss", "npm:8.4.24"],\
            ["nanoid", "npm:3.3.6"],\
            ["picocolors", "npm:1.0.0"],\
            ["source-map-js", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss-modules-extract-imports", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/postcss-modules-extract-imports-npm-2.0.0-da9714d21f-154790fe59.zip/node_modules/postcss-modules-extract-imports/",\
          "packageDependencies": [\
            ["postcss-modules-extract-imports", "npm:2.0.0"],\
            ["postcss", "npm:7.0.39"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/postcss-modules-extract-imports-npm-3.0.0-619311282d-4b65f2f138.zip/node_modules/postcss-modules-extract-imports/",\
          "packageDependencies": [\
            ["postcss-modules-extract-imports", "npm:3.0.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:3.0.0", {\
          "packageLocation": "./.yarn/__virtual__/postcss-modules-extract-imports-virtual-a7098ba128/0/cache/postcss-modules-extract-imports-npm-3.0.0-619311282d-4b65f2f138.zip/node_modules/postcss-modules-extract-imports/",\
          "packageDependencies": [\
            ["postcss-modules-extract-imports", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:3.0.0"],\
            ["@types/postcss", null],\
            ["postcss", "npm:8.4.24"]\
          ],\
          "packagePeers": [\
            "@types/postcss",\
            "postcss"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss-modules-local-by-default", [\
        ["npm:3.0.3", {\
          "packageLocation": "./.yarn/cache/postcss-modules-local-by-default-npm-3.0.3-c3456a1d54-0267633eaf.zip/node_modules/postcss-modules-local-by-default/",\
          "packageDependencies": [\
            ["postcss-modules-local-by-default", "npm:3.0.3"],\
            ["icss-utils", "npm:4.1.1"],\
            ["postcss", "npm:7.0.39"],\
            ["postcss-selector-parser", "npm:6.0.13"],\
            ["postcss-value-parser", "npm:4.2.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.3", {\
          "packageLocation": "./.yarn/cache/postcss-modules-local-by-default-npm-4.0.3-f6674d7148-2f8083687f.zip/node_modules/postcss-modules-local-by-default/",\
          "packageDependencies": [\
            ["postcss-modules-local-by-default", "npm:4.0.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:4.0.3", {\
          "packageLocation": "./.yarn/__virtual__/postcss-modules-local-by-default-virtual-1c41d04d9c/0/cache/postcss-modules-local-by-default-npm-4.0.3-f6674d7148-2f8083687f.zip/node_modules/postcss-modules-local-by-default/",\
          "packageDependencies": [\
            ["postcss-modules-local-by-default", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:4.0.3"],\
            ["@types/postcss", null],\
            ["icss-utils", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:5.1.0"],\
            ["postcss", "npm:8.4.24"],\
            ["postcss-selector-parser", "npm:6.0.13"],\
            ["postcss-value-parser", "npm:4.2.0"]\
          ],\
          "packagePeers": [\
            "@types/postcss",\
            "postcss"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss-modules-scope", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/postcss-modules-scope-npm-2.2.0-e243a2b896-c611181df9.zip/node_modules/postcss-modules-scope/",\
          "packageDependencies": [\
            ["postcss-modules-scope", "npm:2.2.0"],\
            ["postcss", "npm:7.0.39"],\
            ["postcss-selector-parser", "npm:6.0.13"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/postcss-modules-scope-npm-3.0.0-0678040a26-330b9398db.zip/node_modules/postcss-modules-scope/",\
          "packageDependencies": [\
            ["postcss-modules-scope", "npm:3.0.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:3.0.0", {\
          "packageLocation": "./.yarn/__virtual__/postcss-modules-scope-virtual-643c457f3b/0/cache/postcss-modules-scope-npm-3.0.0-0678040a26-330b9398db.zip/node_modules/postcss-modules-scope/",\
          "packageDependencies": [\
            ["postcss-modules-scope", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:3.0.0"],\
            ["@types/postcss", null],\
            ["postcss", "npm:8.4.24"],\
            ["postcss-selector-parser", "npm:6.0.13"]\
          ],\
          "packagePeers": [\
            "@types/postcss",\
            "postcss"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss-modules-values", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/postcss-modules-values-npm-3.0.0-bbe8110e5b-f1aea0b9c6.zip/node_modules/postcss-modules-values/",\
          "packageDependencies": [\
            ["postcss-modules-values", "npm:3.0.0"],\
            ["icss-utils", "npm:4.1.1"],\
            ["postcss", "npm:7.0.39"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/postcss-modules-values-npm-4.0.0-63d7ec543a-f7f2cdf14a.zip/node_modules/postcss-modules-values/",\
          "packageDependencies": [\
            ["postcss-modules-values", "npm:4.0.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:4.0.0", {\
          "packageLocation": "./.yarn/__virtual__/postcss-modules-values-virtual-ad9fe120bf/0/cache/postcss-modules-values-npm-4.0.0-63d7ec543a-f7f2cdf14a.zip/node_modules/postcss-modules-values/",\
          "packageDependencies": [\
            ["postcss-modules-values", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:4.0.0"],\
            ["@types/postcss", null],\
            ["icss-utils", "virtual:473f0ea4690adf649615cbfb388f2ef6c3feebdbb85dc302fcf66ed2e0bfa90173269075d07f440b6142bd6e683dd682debf771bb4c12c0191b1bdf7e349cf7c#npm:5.1.0"],\
            ["postcss", "npm:8.4.24"]\
          ],\
          "packagePeers": [\
            "@types/postcss",\
            "postcss"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss-selector-parser", [\
        ["npm:6.0.13", {\
          "packageLocation": "./.yarn/cache/postcss-selector-parser-npm-6.0.13-f732d92326-f89163338a.zip/node_modules/postcss-selector-parser/",\
          "packageDependencies": [\
            ["postcss-selector-parser", "npm:6.0.13"],\
            ["cssesc", "npm:3.0.0"],\
            ["util-deprecate", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["postcss-value-parser", [\
        ["npm:4.2.0", {\
          "packageLocation": "./.yarn/cache/postcss-value-parser-npm-4.2.0-3cef602a6a-819ffab0c9.zip/node_modules/postcss-value-parser/",\
          "packageDependencies": [\
            ["postcss-value-parser", "npm:4.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["prelude-ls", [\
        ["npm:1.1.2", {\
          "packageLocation": "./.yarn/cache/prelude-ls-npm-1.1.2-a0daac0886-c4867c8748.zip/node_modules/prelude-ls/",\
          "packageDependencies": [\
            ["prelude-ls", "npm:1.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.2.1", {\
          "packageLocation": "./.yarn/cache/prelude-ls-npm-1.2.1-3e4d272a55-cd192ec0d0.zip/node_modules/prelude-ls/",\
          "packageDependencies": [\
            ["prelude-ls", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["prettier", [\
        ["npm:2.8.8", {\
          "packageLocation": "./.yarn/cache/prettier-npm-2.8.8-430828a36c-b49e409431.zip/node_modules/prettier/",\
          "packageDependencies": [\
            ["prettier", "npm:2.8.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["prettier-linter-helpers", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/prettier-linter-helpers-npm-1.0.0-6925131a7e-00ce8011cf.zip/node_modules/prettier-linter-helpers/",\
          "packageDependencies": [\
            ["prettier-linter-helpers", "npm:1.0.0"],\
            ["fast-diff", "npm:1.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pretty-format", [\
        ["npm:26.6.2", {\
          "packageLocation": "./.yarn/cache/pretty-format-npm-26.6.2-6edfcf7149-e3b808404d.zip/node_modules/pretty-format/",\
          "packageDependencies": [\
            ["pretty-format", "npm:26.6.2"],\
            ["@jest/types", "npm:26.6.2"],\
            ["ansi-regex", "npm:5.0.1"],\
            ["ansi-styles", "npm:4.3.0"],\
            ["react-is", "npm:17.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["process", [\
        ["npm:0.11.10", {\
          "packageLocation": "./.yarn/cache/process-npm-0.11.10-aeb3b641ae-bfcce49814.zip/node_modules/process/",\
          "packageDependencies": [\
            ["process", "npm:0.11.10"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["process-nextick-args", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/process-nextick-args-npm-2.0.1-b8d7971609-1d38588e52.zip/node_modules/process-nextick-args/",\
          "packageDependencies": [\
            ["process-nextick-args", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["progress", [\
        ["npm:2.0.3", {\
          "packageLocation": "./.yarn/cache/progress-npm-2.0.3-d1f87e2ac6-f67403fe7b.zip/node_modules/progress/",\
          "packageDependencies": [\
            ["progress", "npm:2.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["promise-inflight", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/promise-inflight-npm-1.0.1-5bb925afac-2274948309.zip/node_modules/promise-inflight/",\
          "packageDependencies": [\
            ["promise-inflight", "npm:1.0.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:a7e5239c6ae68bf6359adfd3598326db000e94dbb349bc00a3852ed53a31712a0e2e787228c6e859d3e5cf2fbb872aba1ea4abe4995cef8086a77ef619ae1be6#npm:1.0.1", {\
          "packageLocation": "./.yarn/__virtual__/promise-inflight-virtual-b427a57c8f/0/cache/promise-inflight-npm-1.0.1-5bb925afac-2274948309.zip/node_modules/promise-inflight/",\
          "packageDependencies": [\
            ["promise-inflight", "virtual:a7e5239c6ae68bf6359adfd3598326db000e94dbb349bc00a3852ed53a31712a0e2e787228c6e859d3e5cf2fbb872aba1ea4abe4995cef8086a77ef619ae1be6#npm:1.0.1"],\
            ["@types/bluebird", null],\
            ["bluebird", null]\
          ],\
          "packagePeers": [\
            "@types/bluebird",\
            "bluebird"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["promise-retry", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/promise-retry-npm-2.0.1-871f0b01b7-f96a3f6d90.zip/node_modules/promise-retry/",\
          "packageDependencies": [\
            ["promise-retry", "npm:2.0.1"],\
            ["err-code", "npm:2.0.3"],\
            ["retry", "npm:0.12.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["prompts", [\
        ["npm:2.4.2", {\
          "packageLocation": "./.yarn/cache/prompts-npm-2.4.2-f5d25d5eea-d8fd1fe638.zip/node_modules/prompts/",\
          "packageDependencies": [\
            ["prompts", "npm:2.4.2"],\
            ["kleur", "npm:3.0.3"],\
            ["sisteransi", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["prr", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/prr-npm-1.0.1-608d442761-3bca2db047.zip/node_modules/prr/",\
          "packageDependencies": [\
            ["prr", "npm:1.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["psl", [\
        ["npm:1.9.0", {\
          "packageLocation": "./.yarn/cache/psl-npm-1.9.0-a546edad1a-20c4277f64.zip/node_modules/psl/",\
          "packageDependencies": [\
            ["psl", "npm:1.9.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["pump", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/pump-npm-3.0.0-0080bf6a7a-e42e9229fb.zip/node_modules/pump/",\
          "packageDependencies": [\
            ["pump", "npm:3.0.0"],\
            ["end-of-stream", "npm:1.4.4"],\
            ["once", "npm:1.4.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["punycode", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/punycode-npm-2.3.0-df4bdce06b-39f760e09a.zip/node_modules/punycode/",\
          "packageDependencies": [\
            ["punycode", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["querystringify", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/querystringify-npm-2.2.0-4e77c9f606-5641ea231b.zip/node_modules/querystringify/",\
          "packageDependencies": [\
            ["querystringify", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["randombytes", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/randombytes-npm-2.1.0-e3da76bccf-d779499376.zip/node_modules/randombytes/",\
          "packageDependencies": [\
            ["randombytes", "npm:2.1.0"],\
            ["safe-buffer", "npm:5.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["raw-loader", [\
        ["npm:4.0.2", {\
          "packageLocation": "./.yarn/cache/raw-loader-npm-4.0.2-94da6c700d-51cc1b0d0e.zip/node_modules/raw-loader/",\
          "packageDependencies": [\
            ["raw-loader", "npm:4.0.2"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.0.2", {\
          "packageLocation": "./.yarn/__virtual__/raw-loader-virtual-772782bba8/0/cache/raw-loader-npm-4.0.2-94da6c700d-51cc1b0d0e.zip/node_modules/raw-loader/",\
          "packageDependencies": [\
            ["raw-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.0.2"],\
            ["@types/webpack", null],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:3.3.0"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["react-is", [\
        ["npm:17.0.2", {\
          "packageLocation": "./.yarn/cache/react-is-npm-17.0.2-091bbb8db6-9d6d111d89.zip/node_modules/react-is/",\
          "packageDependencies": [\
            ["react-is", "npm:17.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["read-pkg", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/read-pkg-npm-3.0.0-41471436cb-398903ebae.zip/node_modules/read-pkg/",\
          "packageDependencies": [\
            ["read-pkg", "npm:3.0.0"],\
            ["load-json-file", "npm:4.0.0"],\
            ["normalize-package-data", "npm:2.5.0"],\
            ["path-type", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.2.0", {\
          "packageLocation": "./.yarn/cache/read-pkg-npm-5.2.0-50426bd8dc-eb696e6052.zip/node_modules/read-pkg/",\
          "packageDependencies": [\
            ["read-pkg", "npm:5.2.0"],\
            ["@types/normalize-package-data", "npm:2.4.1"],\
            ["normalize-package-data", "npm:2.5.0"],\
            ["parse-json", "npm:5.2.0"],\
            ["type-fest", "npm:0.6.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["read-pkg-up", [\
        ["npm:7.0.1", {\
          "packageLocation": "./.yarn/cache/read-pkg-up-npm-7.0.1-11895bed9a-e4e93ce70e.zip/node_modules/read-pkg-up/",\
          "packageDependencies": [\
            ["read-pkg-up", "npm:7.0.1"],\
            ["find-up", "npm:4.1.0"],\
            ["read-pkg", "npm:5.2.0"],\
            ["type-fest", "npm:0.8.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["readable-stream", [\
        ["npm:2.3.8", {\
          "packageLocation": "./.yarn/cache/readable-stream-npm-2.3.8-67a94c2cb1-6564546703.zip/node_modules/readable-stream/",\
          "packageDependencies": [\
            ["readable-stream", "npm:2.3.8"],\
            ["core-util-is", "npm:1.0.3"],\
            ["inherits", "npm:2.0.4"],\
            ["isarray", "npm:1.0.0"],\
            ["process-nextick-args", "npm:2.0.1"],\
            ["safe-buffer", "npm:5.1.2"],\
            ["string_decoder", "npm:1.1.1"],\
            ["util-deprecate", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.6.2", {\
          "packageLocation": "./.yarn/cache/readable-stream-npm-3.6.2-d2a6069158-bdcbe6c22e.zip/node_modules/readable-stream/",\
          "packageDependencies": [\
            ["readable-stream", "npm:3.6.2"],\
            ["inherits", "npm:2.0.4"],\
            ["string_decoder", "npm:1.3.0"],\
            ["util-deprecate", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["rechoir", [\
        ["npm:0.7.1", {\
          "packageLocation": "./.yarn/cache/rechoir-npm-0.7.1-0c7e5c1201-2a04aab4e2.zip/node_modules/rechoir/",\
          "packageDependencies": [\
            ["rechoir", "npm:0.7.1"],\
            ["resolve", "patch:resolve@npm%3A1.22.3#~builtin<compat/resolve>::version=1.22.3&hash=c3c19d"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["redis", [\
        ["npm:0.7.3", {\
          "packageLocation": "./.yarn/cache/redis-npm-0.7.3-c0c0235af5-3f62df962f.zip/node_modules/redis/",\
          "packageDependencies": [\
            ["redis", "npm:0.7.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regenerate", [\
        ["npm:1.4.2", {\
          "packageLocation": "./.yarn/cache/regenerate-npm-1.4.2-b296c5b63a-3317a09b2f.zip/node_modules/regenerate/",\
          "packageDependencies": [\
            ["regenerate", "npm:1.4.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regenerate-unicode-properties", [\
        ["npm:10.1.0", {\
          "packageLocation": "./.yarn/cache/regenerate-unicode-properties-npm-10.1.0-f0d5adf0df-b1a8929588.zip/node_modules/regenerate-unicode-properties/",\
          "packageDependencies": [\
            ["regenerate-unicode-properties", "npm:10.1.0"],\
            ["regenerate", "npm:1.4.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regenerator-runtime", [\
        ["npm:0.13.11", {\
          "packageLocation": "./.yarn/cache/regenerator-runtime-npm-0.13.11-90bf536060-27481628d2.zip/node_modules/regenerator-runtime/",\
          "packageDependencies": [\
            ["regenerator-runtime", "npm:0.13.11"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regenerator-transform", [\
        ["npm:0.15.1", {\
          "packageLocation": "./.yarn/cache/regenerator-transform-npm-0.15.1-c43df537f2-2d15bdeadb.zip/node_modules/regenerator-transform/",\
          "packageDependencies": [\
            ["regenerator-transform", "npm:0.15.1"],\
            ["@babel/runtime", "npm:7.22.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regex-not", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/regex-not-npm-1.0.2-06a03c9206-3081403de7.zip/node_modules/regex-not/",\
          "packageDependencies": [\
            ["regex-not", "npm:1.0.2"],\
            ["extend-shallow", "npm:3.0.2"],\
            ["safe-regex", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regexp.prototype.flags", [\
        ["npm:1.5.0", {\
          "packageLocation": "./.yarn/cache/regexp.prototype.flags-npm-1.5.0-5623b9e07f-c541687cdb.zip/node_modules/regexp.prototype.flags/",\
          "packageDependencies": [\
            ["regexp.prototype.flags", "npm:1.5.0"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["functions-have-names", "npm:1.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regexpp", [\
        ["npm:3.2.0", {\
          "packageLocation": "./.yarn/cache/regexpp-npm-3.2.0-2513f32cfc-a78dc5c715.zip/node_modules/regexpp/",\
          "packageDependencies": [\
            ["regexpp", "npm:3.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regexpu-core", [\
        ["npm:5.3.2", {\
          "packageLocation": "./.yarn/cache/regexpu-core-npm-5.3.2-89effc52a2-95bb970884.zip/node_modules/regexpu-core/",\
          "packageDependencies": [\
            ["regexpu-core", "npm:5.3.2"],\
            ["@babel/regjsgen", "npm:0.8.0"],\
            ["regenerate", "npm:1.4.2"],\
            ["regenerate-unicode-properties", "npm:10.1.0"],\
            ["regjsparser", "npm:0.9.1"],\
            ["unicode-match-property-ecmascript", "npm:2.0.0"],\
            ["unicode-match-property-value-ecmascript", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["regjsparser", [\
        ["npm:0.9.1", {\
          "packageLocation": "./.yarn/cache/regjsparser-npm-0.9.1-47cd7c2ee2-5e1b76afe8.zip/node_modules/regjsparser/",\
          "packageDependencies": [\
            ["regjsparser", "npm:0.9.1"],\
            ["jsesc", "npm:0.5.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["remove-trailing-separator", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/remove-trailing-separator-npm-1.1.0-16d7231316-d3c20b5a2d.zip/node_modules/remove-trailing-separator/",\
          "packageDependencies": [\
            ["remove-trailing-separator", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["repeat-element", [\
        ["npm:1.1.4", {\
          "packageLocation": "./.yarn/cache/repeat-element-npm-1.1.4-7e649ab5b1-1edd0301b7.zip/node_modules/repeat-element/",\
          "packageDependencies": [\
            ["repeat-element", "npm:1.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["repeat-string", [\
        ["npm:1.6.1", {\
          "packageLocation": "./.yarn/cache/repeat-string-npm-1.6.1-bc8e388655-1b809fc6db.zip/node_modules/repeat-string/",\
          "packageDependencies": [\
            ["repeat-string", "npm:1.6.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["require-directory", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/require-directory-npm-2.1.1-8608aee50b-fb47e70bf0.zip/node_modules/require-directory/",\
          "packageDependencies": [\
            ["require-directory", "npm:2.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["require-from-string", [\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/require-from-string-npm-2.0.2-8557e0db12-a03ef68954.zip/node_modules/require-from-string/",\
          "packageDependencies": [\
            ["require-from-string", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["require-main-filename", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/require-main-filename-npm-2.0.0-03eef65c84-e9e294695f.zip/node_modules/require-main-filename/",\
          "packageDependencies": [\
            ["require-main-filename", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["requires-port", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/requires-port-npm-1.0.0-fd036b488a-eee0e303ad.zip/node_modules/requires-port/",\
          "packageDependencies": [\
            ["requires-port", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["resolve", [\
        ["patch:resolve@npm%3A1.22.3#~builtin<compat/resolve>::version=1.22.3&hash=c3c19d", {\
          "packageLocation": "./.yarn/cache/resolve-patch-bb62d3c3c5-ad59734723.zip/node_modules/resolve/",\
          "packageDependencies": [\
            ["resolve", "patch:resolve@npm%3A1.22.3#~builtin<compat/resolve>::version=1.22.3&hash=c3c19d"],\
            ["is-core-module", "npm:2.12.1"],\
            ["path-parse", "npm:1.0.7"],\
            ["supports-preserve-symlinks-flag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["resolve-cwd", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/resolve-cwd-npm-3.0.0-e6f4e296bf-546e081601.zip/node_modules/resolve-cwd/",\
          "packageDependencies": [\
            ["resolve-cwd", "npm:3.0.0"],\
            ["resolve-from", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["resolve-from", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/resolve-from-npm-4.0.0-f758ec21bf-f4ba0b8494.zip/node_modules/resolve-from/",\
          "packageDependencies": [\
            ["resolve-from", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/resolve-from-npm-5.0.0-15c9db4d33-4ceeb9113e.zip/node_modules/resolve-from/",\
          "packageDependencies": [\
            ["resolve-from", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["resolve-url", [\
        ["npm:0.2.1", {\
          "packageLocation": "./.yarn/cache/resolve-url-npm-0.2.1-39edb8f908-7b7035b9ed.zip/node_modules/resolve-url/",\
          "packageDependencies": [\
            ["resolve-url", "npm:0.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ret", [\
        ["npm:0.1.15", {\
          "packageLocation": "./.yarn/cache/ret-npm-0.1.15-0d3c19de76-d76a9159eb.zip/node_modules/ret/",\
          "packageDependencies": [\
            ["ret", "npm:0.1.15"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["retry", [\
        ["npm:0.12.0", {\
          "packageLocation": "./.yarn/cache/retry-npm-0.12.0-72ac7fb4cc-623bd7d2e5.zip/node_modules/retry/",\
          "packageDependencies": [\
            ["retry", "npm:0.12.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["rimraf", [\
        ["npm:2.7.1", {\
          "packageLocation": "./.yarn/cache/rimraf-npm-2.7.1-9a71f3cc37-cdc7f6eacb.zip/node_modules/rimraf/",\
          "packageDependencies": [\
            ["rimraf", "npm:2.7.1"],\
            ["glob", "npm:7.2.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.2", {\
          "packageLocation": "./.yarn/cache/rimraf-npm-3.0.2-2cb7dac69a-87f4164e39.zip/node_modules/rimraf/",\
          "packageDependencies": [\
            ["rimraf", "npm:3.0.2"],\
            ["glob", "npm:7.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["rsvp", [\
        ["npm:4.8.5", {\
          "packageLocation": "./.yarn/cache/rsvp-npm-4.8.5-09f3c6ed40-2d8ef30d8f.zip/node_modules/rsvp/",\
          "packageDependencies": [\
            ["rsvp", "npm:4.8.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["safe-buffer", [\
        ["npm:5.1.2", {\
          "packageLocation": "./.yarn/cache/safe-buffer-npm-5.1.2-c27fedf6c4-f2f1f7943c.zip/node_modules/safe-buffer/",\
          "packageDependencies": [\
            ["safe-buffer", "npm:5.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.2.1", {\
          "packageLocation": "./.yarn/cache/safe-buffer-npm-5.2.1-3481c8aa9b-b99c4b41fd.zip/node_modules/safe-buffer/",\
          "packageDependencies": [\
            ["safe-buffer", "npm:5.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["safe-regex", [\
        ["npm:1.1.0", {\
          "packageLocation": "./.yarn/cache/safe-regex-npm-1.1.0-a908e8515c-9a8bba57c8.zip/node_modules/safe-regex/",\
          "packageDependencies": [\
            ["safe-regex", "npm:1.1.0"],\
            ["ret", "npm:0.1.15"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["safe-regex-test", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/safe-regex-test-npm-1.0.0-e94a09b84e-bc566d8beb.zip/node_modules/safe-regex-test/",\
          "packageDependencies": [\
            ["safe-regex-test", "npm:1.0.0"],\
            ["call-bind", "npm:1.0.2"],\
            ["get-intrinsic", "npm:1.2.1"],\
            ["is-regex", "npm:1.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["safer-buffer", [\
        ["npm:2.1.2", {\
          "packageLocation": "./.yarn/cache/safer-buffer-npm-2.1.2-8d5c0b705e-cab8f25ae6.zip/node_modules/safer-buffer/",\
          "packageDependencies": [\
            ["safer-buffer", "npm:2.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["sane", [\
        ["npm:4.1.0", {\
          "packageLocation": "./.yarn/cache/sane-npm-4.1.0-8ab7d6a963-97716502d4.zip/node_modules/sane/",\
          "packageDependencies": [\
            ["sane", "npm:4.1.0"],\
            ["@cnakazawa/watch", "npm:1.0.4"],\
            ["anymatch", "npm:2.0.0"],\
            ["capture-exit", "npm:2.0.0"],\
            ["exec-sh", "npm:0.3.6"],\
            ["execa", "npm:1.0.0"],\
            ["fb-watchman", "npm:2.0.2"],\
            ["micromatch", "npm:3.1.10"],\
            ["minimist", "npm:1.2.8"],\
            ["walker", "npm:1.0.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["sanitize-html", [\
        ["npm:2.11.0", {\
          "packageLocation": "./.yarn/cache/sanitize-html-npm-2.11.0-b1afcdcea6-44807f22b0.zip/node_modules/sanitize-html/",\
          "packageDependencies": [\
            ["sanitize-html", "npm:2.11.0"],\
            ["deepmerge", "npm:4.3.1"],\
            ["escape-string-regexp", "npm:4.0.0"],\
            ["htmlparser2", "npm:8.0.2"],\
            ["is-plain-object", "npm:5.0.0"],\
            ["parse-srcset", "npm:1.0.2"],\
            ["postcss", "npm:8.4.24"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["saxes", [\
        ["npm:5.0.1", {\
          "packageLocation": "./.yarn/cache/saxes-npm-5.0.1-57abf031ae-5636b55cf1.zip/node_modules/saxes/",\
          "packageDependencies": [\
            ["saxes", "npm:5.0.1"],\
            ["xmlchars", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["schema-utils", [\
        ["npm:2.7.1", {\
          "packageLocation": "./.yarn/cache/schema-utils-npm-2.7.1-f84d18c473-32c62fc9e2.zip/node_modules/schema-utils/",\
          "packageDependencies": [\
            ["schema-utils", "npm:2.7.1"],\
            ["@types/json-schema", "npm:7.0.12"],\
            ["ajv", "npm:6.12.6"],\
            ["ajv-keywords", "virtual:f2b36937f163b579815d3163513b3330d7a31aaf0599eea66346382b8838395c613f4204e9809cc2ff6bba09c17ab0c34b37deadcb147de7e2f5e535d6ccc245#npm:3.5.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.3.0", {\
          "packageLocation": "./.yarn/cache/schema-utils-npm-3.3.0-f2b36937f1-ea56971926.zip/node_modules/schema-utils/",\
          "packageDependencies": [\
            ["schema-utils", "npm:3.3.0"],\
            ["@types/json-schema", "npm:7.0.12"],\
            ["ajv", "npm:6.12.6"],\
            ["ajv-keywords", "virtual:f2b36937f163b579815d3163513b3330d7a31aaf0599eea66346382b8838395c613f4204e9809cc2ff6bba09c17ab0c34b37deadcb147de7e2f5e535d6ccc245#npm:3.5.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["semver", [\
        ["npm:5.7.1", {\
          "packageLocation": "./.yarn/cache/semver-npm-5.7.1-40bcea106b-57fd0acfd0.zip/node_modules/semver/",\
          "packageDependencies": [\
            ["semver", "npm:5.7.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.3.0", {\
          "packageLocation": "./.yarn/cache/semver-npm-6.3.0-b3eace8bfd-1b26ecf6db.zip/node_modules/semver/",\
          "packageDependencies": [\
            ["semver", "npm:6.3.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.5.3", {\
          "packageLocation": "./.yarn/cache/semver-npm-7.5.3-275095dbf3-9d58db1652.zip/node_modules/semver/",\
          "packageDependencies": [\
            ["semver", "npm:7.5.3"],\
            ["lru-cache", "npm:6.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["serialize-javascript", [\
        ["npm:5.0.1", {\
          "packageLocation": "./.yarn/cache/serialize-javascript-npm-5.0.1-8ffc173f09-bb45a42769.zip/node_modules/serialize-javascript/",\
          "packageDependencies": [\
            ["serialize-javascript", "npm:5.0.1"],\
            ["randombytes", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.0.1", {\
          "packageLocation": "./.yarn/cache/serialize-javascript-npm-6.0.1-fac87289ed-3c4f4cb61d.zip/node_modules/serialize-javascript/",\
          "packageDependencies": [\
            ["serialize-javascript", "npm:6.0.1"],\
            ["randombytes", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["set-blocking", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/set-blocking-npm-2.0.0-49e2cffa24-6e65a05f7c.zip/node_modules/set-blocking/",\
          "packageDependencies": [\
            ["set-blocking", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["set-value", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/set-value-npm-2.0.1-35da5f8180-09a4bc72c9.zip/node_modules/set-value/",\
          "packageDependencies": [\
            ["set-value", "npm:2.0.1"],\
            ["extend-shallow", "npm:2.0.1"],\
            ["is-extendable", "npm:0.1.1"],\
            ["is-plain-object", "npm:2.0.4"],\
            ["split-string", "npm:3.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["shallow-clone", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/shallow-clone-npm-3.0.1-dab5873d0d-39b3dd9630.zip/node_modules/shallow-clone/",\
          "packageDependencies": [\
            ["shallow-clone", "npm:3.0.1"],\
            ["kind-of", "npm:6.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["shebang-command", [\
        ["npm:1.2.0", {\
          "packageLocation": "./.yarn/cache/shebang-command-npm-1.2.0-8990ba5d1d-9eed175030.zip/node_modules/shebang-command/",\
          "packageDependencies": [\
            ["shebang-command", "npm:1.2.0"],\
            ["shebang-regex", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/shebang-command-npm-2.0.0-eb2b01921d-6b52fe8727.zip/node_modules/shebang-command/",\
          "packageDependencies": [\
            ["shebang-command", "npm:2.0.0"],\
            ["shebang-regex", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["shebang-regex", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/shebang-regex-npm-1.0.0-c3612b74e9-404c5a752c.zip/node_modules/shebang-regex/",\
          "packageDependencies": [\
            ["shebang-regex", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/shebang-regex-npm-3.0.0-899a0cd65e-1a2bcae50d.zip/node_modules/shebang-regex/",\
          "packageDependencies": [\
            ["shebang-regex", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["shell-quote", [\
        ["npm:1.8.1", {\
          "packageLocation": "./.yarn/cache/shell-quote-npm-1.8.1-fcccf06093-5f01201f4e.zip/node_modules/shell-quote/",\
          "packageDependencies": [\
            ["shell-quote", "npm:1.8.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["shellwords", [\
        ["npm:0.1.1", {\
          "packageLocation": "./.yarn/cache/shellwords-npm-0.1.1-a091a78197-8d73a5e986.zip/node_modules/shellwords/",\
          "packageDependencies": [\
            ["shellwords", "npm:0.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["side-channel", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/side-channel-npm-1.0.4-e1f38b9e06-351e41b947.zip/node_modules/side-channel/",\
          "packageDependencies": [\
            ["side-channel", "npm:1.0.4"],\
            ["call-bind", "npm:1.0.2"],\
            ["get-intrinsic", "npm:1.2.1"],\
            ["object-inspect", "npm:1.12.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["signal-exit", [\
        ["npm:3.0.7", {\
          "packageLocation": "./.yarn/cache/signal-exit-npm-3.0.7-bd270458a3-a2f098f247.zip/node_modules/signal-exit/",\
          "packageDependencies": [\
            ["signal-exit", "npm:3.0.7"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.2", {\
          "packageLocation": "./.yarn/cache/signal-exit-npm-4.0.2-e3f0e8ed25-41f5928431.zip/node_modules/signal-exit/",\
          "packageDependencies": [\
            ["signal-exit", "npm:4.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["sisteransi", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/sisteransi-npm-1.0.5-af60cc0cfa-aba6438f46.zip/node_modules/sisteransi/",\
          "packageDependencies": [\
            ["sisteransi", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["slash", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/slash-npm-3.0.0-b87de2279a-94a93fff61.zip/node_modules/slash/",\
          "packageDependencies": [\
            ["slash", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["slice-ansi", [\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/slice-ansi-npm-4.0.0-6eeca1d10e-4a82d7f085.zip/node_modules/slice-ansi/",\
          "packageDependencies": [\
            ["slice-ansi", "npm:4.0.0"],\
            ["ansi-styles", "npm:4.3.0"],\
            ["astral-regex", "npm:2.0.0"],\
            ["is-fullwidth-code-point", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["smart-buffer", [\
        ["npm:4.2.0", {\
          "packageLocation": "./.yarn/cache/smart-buffer-npm-4.2.0-5ac3f668bb-b5167a7142.zip/node_modules/smart-buffer/",\
          "packageDependencies": [\
            ["smart-buffer", "npm:4.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["snapdragon", [\
        ["npm:0.8.2", {\
          "packageLocation": "./.yarn/cache/snapdragon-npm-0.8.2-2bcc47d217-a197f242a8.zip/node_modules/snapdragon/",\
          "packageDependencies": [\
            ["snapdragon", "npm:0.8.2"],\
            ["base", "npm:0.11.2"],\
            ["debug", "virtual:2bcc47d217f870e3d52f0e55493dc34fd3da852877f7db32fa2940cc320151746026495daf6a34a974488cf5a949d93be1e2dc1ffba036faf081a9b15fcd2252#npm:2.6.9"],\
            ["define-property", "npm:0.2.5"],\
            ["extend-shallow", "npm:2.0.1"],\
            ["map-cache", "npm:0.2.2"],\
            ["source-map", "npm:0.5.7"],\
            ["source-map-resolve", "npm:0.5.3"],\
            ["use", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["snapdragon-node", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/snapdragon-node-npm-2.1.1-78bc70e8e2-9bb57d759f.zip/node_modules/snapdragon-node/",\
          "packageDependencies": [\
            ["snapdragon-node", "npm:2.1.1"],\
            ["define-property", "npm:1.0.0"],\
            ["isobject", "npm:3.0.1"],\
            ["snapdragon-util", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["snapdragon-util", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/snapdragon-util-npm-3.0.1-36b5a7829d-684997dbe3.zip/node_modules/snapdragon-util/",\
          "packageDependencies": [\
            ["snapdragon-util", "npm:3.0.1"],\
            ["kind-of", "npm:3.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["socket.io-client", [\
        ["npm:0.9.16", {\
          "packageLocation": "./.yarn/cache/socket.io-client-npm-0.9.16-8afc7ef034-a8d1762eb2.zip/node_modules/socket.io-client/",\
          "packageDependencies": [\
            ["socket.io-client", "npm:0.9.16"],\
            ["active-x-obfuscator", "npm:0.0.1"],\
            ["uglify-js", "npm:1.2.5"],\
            ["ws", "virtual:8afc7ef03460feb4078a4f2fd26d3b578614a68d423938bea12f546ef9796e0320d21afdcde376380f82d29921a800d02b35da41fb2ff1e4098480a377065368#npm:0.4.32"],\
            ["xmlhttprequest", "npm:1.4.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["socks", [\
        ["npm:2.7.1", {\
          "packageLocation": "./.yarn/cache/socks-npm-2.7.1-17f2b53052-259d9e3e8e.zip/node_modules/socks/",\
          "packageDependencies": [\
            ["socks", "npm:2.7.1"],\
            ["ip", "npm:2.0.0"],\
            ["smart-buffer", "npm:4.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["socks-proxy-agent", [\
        ["npm:7.0.0", {\
          "packageLocation": "./.yarn/cache/socks-proxy-agent-npm-7.0.0-7aacf32ea0-7205543701.zip/node_modules/socks-proxy-agent/",\
          "packageDependencies": [\
            ["socks-proxy-agent", "npm:7.0.0"],\
            ["agent-base", "npm:6.0.2"],\
            ["debug", "virtual:071d471ccd267160a6f0ec15e4cdc2874728a012db8f34521faf9e64a516aa918ee9ebab3c4c084d955ab74aaac5b8be9afb7a168813c9f210acdb7d88754ab9#npm:4.3.4"],\
            ["socks", "npm:2.7.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-list-map", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/source-list-map-npm-2.0.1-625c551052-806efc6f75.zip/node_modules/source-list-map/",\
          "packageDependencies": [\
            ["source-list-map", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-map", [\
        ["npm:0.5.7", {\
          "packageLocation": "./.yarn/cache/source-map-npm-0.5.7-7c3f035429-5dc2043b93.zip/node_modules/source-map/",\
          "packageDependencies": [\
            ["source-map", "npm:0.5.7"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.6.1", {\
          "packageLocation": "./.yarn/cache/source-map-npm-0.6.1-1a3621db16-59ce8640cf.zip/node_modules/source-map/",\
          "packageDependencies": [\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.7.4", {\
          "packageLocation": "./.yarn/cache/source-map-npm-0.7.4-bc8d018ab6-01cc5a74b1.zip/node_modules/source-map/",\
          "packageDependencies": [\
            ["source-map", "npm:0.7.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-map-js", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/source-map-js-npm-1.0.2-ee4f9f9b30-c049a7fc4d.zip/node_modules/source-map-js/",\
          "packageDependencies": [\
            ["source-map-js", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-map-loader", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/source-map-loader-npm-1.0.2-d8fe4f70f6-0360b536e9.zip/node_modules/source-map-loader/",\
          "packageDependencies": [\
            ["source-map-loader", "npm:1.0.2"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["npm:1.1.3", {\
          "packageLocation": "./.yarn/cache/source-map-loader-npm-1.1.3-b6dca1709f-0ca16a1458.zip/node_modules/source-map-loader/",\
          "packageDependencies": [\
            ["source-map-loader", "npm:1.1.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.1.3", {\
          "packageLocation": "./.yarn/__virtual__/source-map-loader-virtual-e472220138/0/cache/source-map-loader-npm-1.1.3-b6dca1709f-0ca16a1458.zip/node_modules/source-map-loader/",\
          "packageDependencies": [\
            ["source-map-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.1.3"],\
            ["@types/webpack", null],\
            ["abab", "npm:2.0.6"],\
            ["iconv-lite", "npm:0.6.3"],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:3.3.0"],\
            ["source-map", "npm:0.6.1"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"],\
            ["whatwg-mimetype", "npm:2.3.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:1.0.2", {\
          "packageLocation": "./.yarn/__virtual__/source-map-loader-virtual-c99ed444b2/0/cache/source-map-loader-npm-1.0.2-d8fe4f70f6-0360b536e9.zip/node_modules/source-map-loader/",\
          "packageDependencies": [\
            ["source-map-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:1.0.2"],\
            ["@types/webpack", null],\
            ["data-urls", "npm:2.0.0"],\
            ["iconv-lite", "npm:0.6.3"],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:2.7.1"],\
            ["source-map", "npm:0.6.1"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-map-resolve", [\
        ["npm:0.5.3", {\
          "packageLocation": "./.yarn/cache/source-map-resolve-npm-0.5.3-6502ae65ba-c73fa44ac0.zip/node_modules/source-map-resolve/",\
          "packageDependencies": [\
            ["source-map-resolve", "npm:0.5.3"],\
            ["atob", "npm:2.1.2"],\
            ["decode-uri-component", "npm:0.2.2"],\
            ["resolve-url", "npm:0.2.1"],\
            ["source-map-url", "npm:0.4.1"],\
            ["urix", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-map-support", [\
        ["npm:0.5.21", {\
          "packageLocation": "./.yarn/cache/source-map-support-npm-0.5.21-09ca99e250-43e98d700d.zip/node_modules/source-map-support/",\
          "packageDependencies": [\
            ["source-map-support", "npm:0.5.21"],\
            ["buffer-from", "npm:1.1.2"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["source-map-url", [\
        ["npm:0.4.1", {\
          "packageLocation": "./.yarn/cache/source-map-url-npm-0.4.1-747a1f6eba-64c5c2c77a.zip/node_modules/source-map-url/",\
          "packageDependencies": [\
            ["source-map-url", "npm:0.4.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["spdx-correct", [\
        ["npm:3.2.0", {\
          "packageLocation": "./.yarn/cache/spdx-correct-npm-3.2.0-ffae008484-e9ae98d22f.zip/node_modules/spdx-correct/",\
          "packageDependencies": [\
            ["spdx-correct", "npm:3.2.0"],\
            ["spdx-expression-parse", "npm:3.0.1"],\
            ["spdx-license-ids", "npm:3.0.13"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["spdx-exceptions", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/spdx-exceptions-npm-2.3.0-2b68dad75a-cb69a26fa3.zip/node_modules/spdx-exceptions/",\
          "packageDependencies": [\
            ["spdx-exceptions", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["spdx-expression-parse", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/spdx-expression-parse-npm-3.0.1-b718cbb35a-a1c6e104a2.zip/node_modules/spdx-expression-parse/",\
          "packageDependencies": [\
            ["spdx-expression-parse", "npm:3.0.1"],\
            ["spdx-exceptions", "npm:2.3.0"],\
            ["spdx-license-ids", "npm:3.0.13"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["spdx-license-ids", [\
        ["npm:3.0.13", {\
          "packageLocation": "./.yarn/cache/spdx-license-ids-npm-3.0.13-928dd45e3f-3469d85c65.zip/node_modules/spdx-license-ids/",\
          "packageDependencies": [\
            ["spdx-license-ids", "npm:3.0.13"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["split-string", [\
        ["npm:3.1.0", {\
          "packageLocation": "./.yarn/cache/split-string-npm-3.1.0-df5d83450e-ae5af5c91b.zip/node_modules/split-string/",\
          "packageDependencies": [\
            ["split-string", "npm:3.1.0"],\
            ["extend-shallow", "npm:3.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["sprintf-js", [\
        ["npm:1.0.3", {\
          "packageLocation": "./.yarn/cache/sprintf-js-npm-1.0.3-73f0a322fa-19d79aec21.zip/node_modules/sprintf-js/",\
          "packageDependencies": [\
            ["sprintf-js", "npm:1.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ssri", [\
        ["npm:10.0.4", {\
          "packageLocation": "./.yarn/cache/ssri-npm-10.0.4-f583dafaf3-fb14da9f8a.zip/node_modules/ssri/",\
          "packageDependencies": [\
            ["ssri", "npm:10.0.4"],\
            ["minipass", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.0.1", {\
          "packageLocation": "./.yarn/cache/ssri-npm-8.0.1-a369e72ce2-bc447f5af8.zip/node_modules/ssri/",\
          "packageDependencies": [\
            ["ssri", "npm:8.0.1"],\
            ["minipass", "npm:3.3.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["stack-utils", [\
        ["npm:2.0.6", {\
          "packageLocation": "./.yarn/cache/stack-utils-npm-2.0.6-2be1099696-052bf4d25b.zip/node_modules/stack-utils/",\
          "packageDependencies": [\
            ["stack-utils", "npm:2.0.6"],\
            ["escape-string-regexp", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["static-extend", [\
        ["npm:0.1.2", {\
          "packageLocation": "./.yarn/cache/static-extend-npm-0.1.2-2720ee6882-8657485b83.zip/node_modules/static-extend/",\
          "packageDependencies": [\
            ["static-extend", "npm:0.1.2"],\
            ["define-property", "npm:0.2.5"],\
            ["object-copy", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string-length", [\
        ["npm:4.0.2", {\
          "packageLocation": "./.yarn/cache/string-length-npm-4.0.2-675173c7a2-ce85533ef5.zip/node_modules/string-length/",\
          "packageDependencies": [\
            ["string-length", "npm:4.0.2"],\
            ["char-regex", "npm:1.0.2"],\
            ["strip-ansi", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string-width", [\
        ["npm:4.2.3", {\
          "packageLocation": "./.yarn/cache/string-width-npm-4.2.3-2c27177bae-e52c10dc3f.zip/node_modules/string-width/",\
          "packageDependencies": [\
            ["string-width", "npm:4.2.3"],\
            ["emoji-regex", "npm:8.0.0"],\
            ["is-fullwidth-code-point", "npm:3.0.0"],\
            ["strip-ansi", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.1.2", {\
          "packageLocation": "./.yarn/cache/string-width-npm-5.1.2-bf60531341-7369deaa29.zip/node_modules/string-width/",\
          "packageDependencies": [\
            ["string-width", "npm:5.1.2"],\
            ["eastasianwidth", "npm:0.2.0"],\
            ["emoji-regex", "npm:9.2.2"],\
            ["strip-ansi", "npm:7.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string.prototype.padend", [\
        ["npm:3.1.4", {\
          "packageLocation": "./.yarn/cache/string.prototype.padend-npm-3.1.4-0b31461a40-76e07238fe.zip/node_modules/string.prototype.padend/",\
          "packageDependencies": [\
            ["string.prototype.padend", "npm:3.1.4"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["es-abstract", "npm:1.21.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string.prototype.trim", [\
        ["npm:1.2.7", {\
          "packageLocation": "./.yarn/cache/string.prototype.trim-npm-1.2.7-3fbaf3b9d2-05b7b2d6af.zip/node_modules/string.prototype.trim/",\
          "packageDependencies": [\
            ["string.prototype.trim", "npm:1.2.7"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["es-abstract", "npm:1.21.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string.prototype.trimend", [\
        ["npm:1.0.6", {\
          "packageLocation": "./.yarn/cache/string.prototype.trimend-npm-1.0.6-304246ecc1-0fdc34645a.zip/node_modules/string.prototype.trimend/",\
          "packageDependencies": [\
            ["string.prototype.trimend", "npm:1.0.6"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["es-abstract", "npm:1.21.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string.prototype.trimstart", [\
        ["npm:1.0.6", {\
          "packageLocation": "./.yarn/cache/string.prototype.trimstart-npm-1.0.6-0926caea6c-89080feef4.zip/node_modules/string.prototype.trimstart/",\
          "packageDependencies": [\
            ["string.prototype.trimstart", "npm:1.0.6"],\
            ["call-bind", "npm:1.0.2"],\
            ["define-properties", "npm:1.2.0"],\
            ["es-abstract", "npm:1.21.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["string_decoder", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/string_decoder-npm-1.1.1-e46a6c1353-9ab7e56f9d.zip/node_modules/string_decoder/",\
          "packageDependencies": [\
            ["string_decoder", "npm:1.1.1"],\
            ["safe-buffer", "npm:5.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/string_decoder-npm-1.3.0-2422117fd0-8417646695.zip/node_modules/string_decoder/",\
          "packageDependencies": [\
            ["string_decoder", "npm:1.3.0"],\
            ["safe-buffer", "npm:5.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["strip-ansi", [\
        ["npm:6.0.1", {\
          "packageLocation": "./.yarn/cache/strip-ansi-npm-6.0.1-caddc7cb40-f3cd25890a.zip/node_modules/strip-ansi/",\
          "packageDependencies": [\
            ["strip-ansi", "npm:6.0.1"],\
            ["ansi-regex", "npm:5.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.1.0", {\
          "packageLocation": "./.yarn/cache/strip-ansi-npm-7.1.0-7453b80b79-859c73fcf2.zip/node_modules/strip-ansi/",\
          "packageDependencies": [\
            ["strip-ansi", "npm:7.1.0"],\
            ["ansi-regex", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["strip-bom", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/strip-bom-npm-3.0.0-71e8f81ff9-8d50ff27b7.zip/node_modules/strip-bom/",\
          "packageDependencies": [\
            ["strip-bom", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/strip-bom-npm-4.0.0-97d367a64d-9dbcfbaf50.zip/node_modules/strip-bom/",\
          "packageDependencies": [\
            ["strip-bom", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["strip-eof", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/strip-eof-npm-1.0.0-d82eaf947c-40bc8ddd7e.zip/node_modules/strip-eof/",\
          "packageDependencies": [\
            ["strip-eof", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["strip-final-newline", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/strip-final-newline-npm-2.0.0-340c4f7c66-69412b5e25.zip/node_modules/strip-final-newline/",\
          "packageDependencies": [\
            ["strip-final-newline", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["strip-json-comments", [\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/strip-json-comments-npm-3.1.1-dcb2324823-492f73e272.zip/node_modules/strip-json-comments/",\
          "packageDependencies": [\
            ["strip-json-comments", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["style-loader", [\
        ["npm:1.3.0", {\
          "packageLocation": "./.yarn/cache/style-loader-npm-1.3.0-2047b33843-1be9e87053.zip/node_modules/style-loader/",\
          "packageDependencies": [\
            ["style-loader", "npm:1.3.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/style-loader-npm-2.0.0-b9a5c4a2aa-21425246a5.zip/node_modules/style-loader/",\
          "packageDependencies": [\
            ["style-loader", "npm:2.0.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.3.0", {\
          "packageLocation": "./.yarn/__virtual__/style-loader-virtual-4f3ee71433/0/cache/style-loader-npm-1.3.0-2047b33843-1be9e87053.zip/node_modules/style-loader/",\
          "packageDependencies": [\
            ["style-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:1.3.0"],\
            ["@types/webpack", null],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:2.7.1"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:2.0.0", {\
          "packageLocation": "./.yarn/__virtual__/style-loader-virtual-af5ed36c0e/0/cache/style-loader-npm-2.0.0-b9a5c4a2aa-21425246a5.zip/node_modules/style-loader/",\
          "packageDependencies": [\
            ["style-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:2.0.0"],\
            ["@types/webpack", null],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:3.3.0"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["supports-color", [\
        ["npm:5.5.0", {\
          "packageLocation": "./.yarn/cache/supports-color-npm-5.5.0-183ac537bc-95f6f4ba5a.zip/node_modules/supports-color/",\
          "packageDependencies": [\
            ["supports-color", "npm:5.5.0"],\
            ["has-flag", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.2.0", {\
          "packageLocation": "./.yarn/cache/supports-color-npm-7.2.0-606bfcf7da-3dda818de0.zip/node_modules/supports-color/",\
          "packageDependencies": [\
            ["supports-color", "npm:7.2.0"],\
            ["has-flag", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.1.1", {\
          "packageLocation": "./.yarn/cache/supports-color-npm-8.1.1-289e937149-c052193a7e.zip/node_modules/supports-color/",\
          "packageDependencies": [\
            ["supports-color", "npm:8.1.1"],\
            ["has-flag", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["supports-hyperlinks", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/supports-hyperlinks-npm-2.3.0-d19176eba2-9ee0de3c8c.zip/node_modules/supports-hyperlinks/",\
          "packageDependencies": [\
            ["supports-hyperlinks", "npm:2.3.0"],\
            ["has-flag", "npm:4.0.0"],\
            ["supports-color", "npm:7.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["supports-preserve-symlinks-flag", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/supports-preserve-symlinks-flag-npm-1.0.0-f17c4d0028-53b1e247e6.zip/node_modules/supports-preserve-symlinks-flag/",\
          "packageDependencies": [\
            ["supports-preserve-symlinks-flag", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["svg-url-loader", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/svg-url-loader-npm-6.0.0-c65934e254-ac46cfc897.zip/node_modules/svg-url-loader/",\
          "packageDependencies": [\
            ["svg-url-loader", "npm:6.0.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0", {\
          "packageLocation": "./.yarn/__virtual__/svg-url-loader-virtual-ffac8d648d/0/cache/svg-url-loader-npm-6.0.0-c65934e254-ac46cfc897.zip/node_modules/svg-url-loader/",\
          "packageDependencies": [\
            ["svg-url-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0"],\
            ["@types/webpack", null],\
            ["file-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0"],\
            ["loader-utils", "npm:2.0.4"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["symbol-tree", [\
        ["npm:3.2.4", {\
          "packageLocation": "./.yarn/cache/symbol-tree-npm-3.2.4-fe70cdb75b-6e8fc7e148.zip/node_modules/symbol-tree/",\
          "packageDependencies": [\
            ["symbol-tree", "npm:3.2.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["table", [\
        ["npm:6.8.1", {\
          "packageLocation": "./.yarn/cache/table-npm-6.8.1-83abb79e20-08249c7046.zip/node_modules/table/",\
          "packageDependencies": [\
            ["table", "npm:6.8.1"],\
            ["ajv", "npm:8.12.0"],\
            ["lodash.truncate", "npm:4.4.2"],\
            ["slice-ansi", "npm:4.0.0"],\
            ["string-width", "npm:4.2.3"],\
            ["strip-ansi", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tapable", [\
        ["npm:1.1.3", {\
          "packageLocation": "./.yarn/cache/tapable-npm-1.1.3-f1c2843426-53ff4e7c39.zip/node_modules/tapable/",\
          "packageDependencies": [\
            ["tapable", "npm:1.1.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.2.1", {\
          "packageLocation": "./.yarn/cache/tapable-npm-2.2.1-8cf5ff3039-3b7a1b4d86.zip/node_modules/tapable/",\
          "packageDependencies": [\
            ["tapable", "npm:2.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tar", [\
        ["npm:6.1.15", {\
          "packageLocation": "./.yarn/cache/tar-npm-6.1.15-44c3e71720-f23832fcee.zip/node_modules/tar/",\
          "packageDependencies": [\
            ["tar", "npm:6.1.15"],\
            ["chownr", "npm:2.0.0"],\
            ["fs-minipass", "npm:2.1.0"],\
            ["minipass", "npm:5.0.0"],\
            ["minizlib", "npm:2.1.2"],\
            ["mkdirp", "npm:1.0.4"],\
            ["yallist", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["terminal-link", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/terminal-link-npm-2.1.1-de80341758-ce3d2cd3a4.zip/node_modules/terminal-link/",\
          "packageDependencies": [\
            ["terminal-link", "npm:2.1.1"],\
            ["ansi-escapes", "npm:4.3.2"],\
            ["supports-hyperlinks", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["terser", [\
        ["npm:5.18.1", {\
          "packageLocation": "./.yarn/cache/terser-npm-5.18.1-2ffdc95dfc-15d1af05aa.zip/node_modules/terser/",\
          "packageDependencies": [\
            ["terser", "npm:5.18.1"],\
            ["@jridgewell/source-map", "npm:0.3.3"],\
            ["acorn", "npm:8.9.0"],\
            ["commander", "npm:2.20.3"],\
            ["source-map-support", "npm:0.5.21"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["terser-webpack-plugin", [\
        ["npm:4.2.3", {\
          "packageLocation": "./.yarn/cache/terser-webpack-plugin-npm-4.2.3-3192d0fa12-ec1b3a85e2.zip/node_modules/terser-webpack-plugin/",\
          "packageDependencies": [\
            ["terser-webpack-plugin", "npm:4.2.3"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["npm:5.3.9", {\
          "packageLocation": "./.yarn/cache/terser-webpack-plugin-npm-5.3.9-7ba1eb45f4-41705713d6.zip/node_modules/terser-webpack-plugin/",\
          "packageDependencies": [\
            ["terser-webpack-plugin", "npm:5.3.9"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.2.3", {\
          "packageLocation": "./.yarn/__virtual__/terser-webpack-plugin-virtual-034eb454c0/0/cache/terser-webpack-plugin-npm-4.2.3-3192d0fa12-ec1b3a85e2.zip/node_modules/terser-webpack-plugin/",\
          "packageDependencies": [\
            ["terser-webpack-plugin", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.2.3"],\
            ["@types/webpack", null],\
            ["cacache", "npm:15.3.0"],\
            ["find-cache-dir", "npm:3.3.2"],\
            ["jest-worker", "npm:26.6.2"],\
            ["p-limit", "npm:3.1.0"],\
            ["schema-utils", "npm:3.3.0"],\
            ["serialize-javascript", "npm:5.0.1"],\
            ["source-map", "npm:0.6.1"],\
            ["terser", "npm:5.18.1"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["webpack-sources", "npm:1.4.3"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:a8c2fda196219bb2f79b42a12252ee3317d65cf0d2e525730054f5016e10b6a7de7ffdfd6a671c28c476a270aabdc7f477fa7ae9e2df43bc207110bcc3d8d02b#npm:5.3.9", {\
          "packageLocation": "./.yarn/__virtual__/terser-webpack-plugin-virtual-4849bc2597/0/cache/terser-webpack-plugin-npm-5.3.9-7ba1eb45f4-41705713d6.zip/node_modules/terser-webpack-plugin/",\
          "packageDependencies": [\
            ["terser-webpack-plugin", "virtual:a8c2fda196219bb2f79b42a12252ee3317d65cf0d2e525730054f5016e10b6a7de7ffdfd6a671c28c476a270aabdc7f477fa7ae9e2df43bc207110bcc3d8d02b#npm:5.3.9"],\
            ["@jridgewell/trace-mapping", "npm:0.3.18"],\
            ["@swc/core", null],\
            ["@types/esbuild", null],\
            ["@types/swc__core", null],\
            ["@types/uglify-js", null],\
            ["@types/webpack", null],\
            ["esbuild", null],\
            ["jest-worker", "npm:27.5.1"],\
            ["schema-utils", "npm:3.3.0"],\
            ["serialize-javascript", "npm:6.0.1"],\
            ["terser", "npm:5.18.1"],\
            ["uglify-js", null],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@swc/core",\
            "@types/esbuild",\
            "@types/swc__core",\
            "@types/uglify-js",\
            "@types/webpack",\
            "esbuild",\
            "uglify-js",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:5.3.9", {\
          "packageLocation": "./.yarn/__virtual__/terser-webpack-plugin-virtual-201077c49d/0/cache/terser-webpack-plugin-npm-5.3.9-7ba1eb45f4-41705713d6.zip/node_modules/terser-webpack-plugin/",\
          "packageDependencies": [\
            ["terser-webpack-plugin", "virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:5.3.9"],\
            ["@jridgewell/trace-mapping", "npm:0.3.18"],\
            ["@swc/core", null],\
            ["@types/esbuild", null],\
            ["@types/swc__core", null],\
            ["@types/uglify-js", null],\
            ["@types/webpack", null],\
            ["esbuild", null],\
            ["jest-worker", "npm:27.5.1"],\
            ["schema-utils", "npm:3.3.0"],\
            ["serialize-javascript", "npm:6.0.1"],\
            ["terser", "npm:5.18.1"],\
            ["uglify-js", null],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@swc/core",\
            "@types/esbuild",\
            "@types/swc__core",\
            "@types/uglify-js",\
            "@types/webpack",\
            "esbuild",\
            "uglify-js",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["test-exclude", [\
        ["npm:6.0.0", {\
          "packageLocation": "./.yarn/cache/test-exclude-npm-6.0.0-3fb03d69df-3b34a3d771.zip/node_modules/test-exclude/",\
          "packageDependencies": [\
            ["test-exclude", "npm:6.0.0"],\
            ["@istanbuljs/schema", "npm:0.1.3"],\
            ["glob", "npm:7.2.3"],\
            ["minimatch", "npm:3.1.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["text-table", [\
        ["npm:0.2.0", {\
          "packageLocation": "./.yarn/cache/text-table-npm-0.2.0-d92a778b59-b6937a38c8.zip/node_modules/text-table/",\
          "packageDependencies": [\
            ["text-table", "npm:0.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["throat", [\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/throat-npm-5.0.0-288ce6540a-031ff7f443.zip/node_modules/throat/",\
          "packageDependencies": [\
            ["throat", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tinycolor", [\
        ["npm:0.0.1", {\
          "packageLocation": "./.yarn/cache/tinycolor-npm-0.0.1-f8cee68c73-4a9f2d4c2c.zip/node_modules/tinycolor/",\
          "packageDependencies": [\
            ["tinycolor", "npm:0.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tmpl", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/tmpl-npm-1.0.5-d399ba37e2-cd922d9b85.zip/node_modules/tmpl/",\
          "packageDependencies": [\
            ["tmpl", "npm:1.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["to-fast-properties", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/to-fast-properties-npm-2.0.0-0dc60cc481-be2de62fe5.zip/node_modules/to-fast-properties/",\
          "packageDependencies": [\
            ["to-fast-properties", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["to-object-path", [\
        ["npm:0.3.0", {\
          "packageLocation": "./.yarn/cache/to-object-path-npm-0.3.0-241b5ffa9c-9425effee5.zip/node_modules/to-object-path/",\
          "packageDependencies": [\
            ["to-object-path", "npm:0.3.0"],\
            ["kind-of", "npm:3.2.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["to-regex", [\
        ["npm:3.0.2", {\
          "packageLocation": "./.yarn/cache/to-regex-npm-3.0.2-3af893c972-4ed4a61905.zip/node_modules/to-regex/",\
          "packageDependencies": [\
            ["to-regex", "npm:3.0.2"],\
            ["define-property", "npm:2.0.2"],\
            ["extend-shallow", "npm:3.0.2"],\
            ["regex-not", "npm:1.0.2"],\
            ["safe-regex", "npm:1.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["to-regex-range", [\
        ["npm:2.1.1", {\
          "packageLocation": "./.yarn/cache/to-regex-range-npm-2.1.1-60af4c593e-46093cc14b.zip/node_modules/to-regex-range/",\
          "packageDependencies": [\
            ["to-regex-range", "npm:2.1.1"],\
            ["is-number", "npm:3.0.0"],\
            ["repeat-string", "npm:1.6.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.0.1", {\
          "packageLocation": "./.yarn/cache/to-regex-range-npm-5.0.1-f1e8263b00-f76fa01b3d.zip/node_modules/to-regex-range/",\
          "packageDependencies": [\
            ["to-regex-range", "npm:5.0.1"],\
            ["is-number", "npm:7.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["to-string-loader", [\
        ["npm:1.2.0", {\
          "packageLocation": "./.yarn/cache/to-string-loader-npm-1.2.0-b3f267261b-738d51379a.zip/node_modules/to-string-loader/",\
          "packageDependencies": [\
            ["to-string-loader", "npm:1.2.0"],\
            ["loader-utils", "npm:1.4.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tough-cookie", [\
        ["npm:4.1.3", {\
          "packageLocation": "./.yarn/cache/tough-cookie-npm-4.1.3-08285e2518-c9226afff3.zip/node_modules/tough-cookie/",\
          "packageDependencies": [\
            ["tough-cookie", "npm:4.1.3"],\
            ["psl", "npm:1.9.0"],\
            ["punycode", "npm:2.3.0"],\
            ["universalify", "npm:0.2.0"],\
            ["url-parse", "npm:1.5.10"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tr46", [\
        ["npm:0.0.3", {\
          "packageLocation": "./.yarn/cache/tr46-npm-0.0.3-de53018915-726321c5ea.zip/node_modules/tr46/",\
          "packageDependencies": [\
            ["tr46", "npm:0.0.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/tr46-npm-2.1.0-00af583f4f-ffe6049b9d.zip/node_modules/tr46/",\
          "packageDependencies": [\
            ["tr46", "npm:2.1.0"],\
            ["punycode", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ts-jest", [\
        ["npm:26.5.6", {\
          "packageLocation": "./.yarn/cache/ts-jest-npm-26.5.6-fa1909416a-6f65ad4fe6.zip/node_modules/ts-jest/",\
          "packageDependencies": [\
            ["ts-jest", "npm:26.5.6"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:26.5.6", {\
          "packageLocation": "./.yarn/__virtual__/ts-jest-virtual-1c50afe85f/0/cache/ts-jest-npm-26.5.6-fa1909416a-6f65ad4fe6.zip/node_modules/ts-jest/",\
          "packageDependencies": [\
            ["ts-jest", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:26.5.6"],\
            ["@types/jest", "npm:26.0.24"],\
            ["@types/typescript", null],\
            ["bs-logger", "npm:0.2.6"],\
            ["buffer-from", "npm:1.1.2"],\
            ["fast-json-stable-stringify", "npm:2.1.0"],\
            ["jest", "npm:26.6.3"],\
            ["jest-util", "npm:26.6.2"],\
            ["json5", "npm:2.2.3"],\
            ["lodash", "npm:4.17.21"],\
            ["make-error", "npm:1.3.6"],\
            ["mkdirp", "npm:1.0.4"],\
            ["semver", "npm:7.5.3"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"],\
            ["yargs-parser", "npm:20.2.9"]\
          ],\
          "packagePeers": [\
            "@types/jest",\
            "@types/typescript",\
            "jest",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ts-loader", [\
        ["npm:8.4.0", {\
          "packageLocation": "./.yarn/cache/ts-loader-npm-8.4.0-8da6fd87a8-79da0f364c.zip/node_modules/ts-loader/",\
          "packageDependencies": [\
            ["ts-loader", "npm:8.4.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:8.4.0", {\
          "packageLocation": "./.yarn/__virtual__/ts-loader-virtual-bd50a0a825/0/cache/ts-loader-npm-8.4.0-8da6fd87a8-79da0f364c.zip/node_modules/ts-loader/",\
          "packageDependencies": [\
            ["ts-loader", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:8.4.0"],\
            ["@types/typescript", null],\
            ["@types/webpack", null],\
            ["chalk", "npm:4.1.2"],\
            ["enhanced-resolve", "npm:4.5.0"],\
            ["loader-utils", "npm:2.0.4"],\
            ["micromatch", "npm:4.0.5"],\
            ["semver", "npm:7.5.3"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/typescript",\
            "@types/webpack",\
            "typescript",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tslib", [\
        ["npm:1.14.1", {\
          "packageLocation": "./.yarn/cache/tslib-npm-1.14.1-102499115e-dbe628ef87.zip/node_modules/tslib/",\
          "packageDependencies": [\
            ["tslib", "npm:1.14.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["tsutils", [\
        ["npm:3.21.0", {\
          "packageLocation": "./.yarn/cache/tsutils-npm-3.21.0-347e6636c5-1843f4c1b2.zip/node_modules/tsutils/",\
          "packageDependencies": [\
            ["tsutils", "npm:3.21.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:4e46a89bdf2ac87e21276dc3d81d30cea0ffd5e1ff3d5417df1e96f79ae422194dfcfa17f2a3469ab76696bae0f7f47086ff12110b9307494abd02932d6722d4#npm:3.21.0", {\
          "packageLocation": "./.yarn/__virtual__/tsutils-virtual-d2cc985a11/0/cache/tsutils-npm-3.21.0-347e6636c5-1843f4c1b2.zip/node_modules/tsutils/",\
          "packageDependencies": [\
            ["tsutils", "virtual:4e46a89bdf2ac87e21276dc3d81d30cea0ffd5e1ff3d5417df1e96f79ae422194dfcfa17f2a3469ab76696bae0f7f47086ff12110b9307494abd02932d6722d4#npm:3.21.0"],\
            ["@types/typescript", null],\
            ["tslib", "npm:1.14.1"],\
            ["typescript", null]\
          ],\
          "packagePeers": [\
            "@types/typescript",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.21.0", {\
          "packageLocation": "./.yarn/__virtual__/tsutils-virtual-991fb42f34/0/cache/tsutils-npm-3.21.0-347e6636c5-1843f4c1b2.zip/node_modules/tsutils/",\
          "packageDependencies": [\
            ["tsutils", "virtual:60dbc451ed742a487455614de798ab170b18975deaff688919a023f5c9ac9d116706bfef4f643b5fd6ebdd92aa99be062373168cee5a0304fcc76b03d3535783#npm:3.21.0"],\
            ["@types/typescript", null],\
            ["tslib", "npm:1.14.1"],\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"]\
          ],\
          "packagePeers": [\
            "@types/typescript",\
            "typescript"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["type-check", [\
        ["npm:0.3.2", {\
          "packageLocation": "./.yarn/cache/type-check-npm-0.3.2-a4a38bb0b6-dd3b149564.zip/node_modules/type-check/",\
          "packageDependencies": [\
            ["type-check", "npm:0.3.2"],\
            ["prelude-ls", "npm:1.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.4.0", {\
          "packageLocation": "./.yarn/cache/type-check-npm-0.4.0-60565800ce-ec688ebfc9.zip/node_modules/type-check/",\
          "packageDependencies": [\
            ["type-check", "npm:0.4.0"],\
            ["prelude-ls", "npm:1.2.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["type-detect", [\
        ["npm:4.0.8", {\
          "packageLocation": "./.yarn/cache/type-detect-npm-4.0.8-8d8127b901-62b5628bff.zip/node_modules/type-detect/",\
          "packageDependencies": [\
            ["type-detect", "npm:4.0.8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["type-fest", [\
        ["npm:0.20.2", {\
          "packageLocation": "./.yarn/cache/type-fest-npm-0.20.2-b36432617f-4fb3272df2.zip/node_modules/type-fest/",\
          "packageDependencies": [\
            ["type-fest", "npm:0.20.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.21.3", {\
          "packageLocation": "./.yarn/cache/type-fest-npm-0.21.3-5ff2a9c6fd-e6b32a3b38.zip/node_modules/type-fest/",\
          "packageDependencies": [\
            ["type-fest", "npm:0.21.3"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.6.0", {\
          "packageLocation": "./.yarn/cache/type-fest-npm-0.6.0-76b229965b-b2188e6e4b.zip/node_modules/type-fest/",\
          "packageDependencies": [\
            ["type-fest", "npm:0.6.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.8.1", {\
          "packageLocation": "./.yarn/cache/type-fest-npm-0.8.1-351ad028fe-d61c4b2eba.zip/node_modules/type-fest/",\
          "packageDependencies": [\
            ["type-fest", "npm:0.8.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["typed-array-length", [\
        ["npm:1.0.4", {\
          "packageLocation": "./.yarn/cache/typed-array-length-npm-1.0.4-92771b81fc-2228febc93.zip/node_modules/typed-array-length/",\
          "packageDependencies": [\
            ["typed-array-length", "npm:1.0.4"],\
            ["call-bind", "npm:1.0.2"],\
            ["for-each", "npm:0.3.3"],\
            ["is-typed-array", "npm:1.1.10"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["typedarray-to-buffer", [\
        ["npm:3.1.5", {\
          "packageLocation": "./.yarn/cache/typedarray-to-buffer-npm-3.1.5-aadc11995e-99c11aaa8f.zip/node_modules/typedarray-to-buffer/",\
          "packageDependencies": [\
            ["typedarray-to-buffer", "npm:3.1.5"],\
            ["is-typedarray", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["typescript", [\
        ["patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8", {\
          "packageLocation": "./.yarn/cache/typescript-patch-dac367b9b3-3bd9915f23.zip/node_modules/typescript/",\
          "packageDependencies": [\
            ["typescript", "patch:typescript@npm%3A4.1.6#~builtin<compat/typescript>::version=4.1.6&hash=4a8eb8"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["uglify-js", [\
        ["npm:1.2.5", {\
          "packageLocation": "./.yarn/cache/uglify-js-npm-1.2.5-b0ec7e02f8-0ccc209cdc.zip/node_modules/uglify-js/",\
          "packageDependencies": [\
            ["uglify-js", "npm:1.2.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unbox-primitive", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/unbox-primitive-npm-1.0.2-cb56a05066-b7a1cf5862.zip/node_modules/unbox-primitive/",\
          "packageDependencies": [\
            ["unbox-primitive", "npm:1.0.2"],\
            ["call-bind", "npm:1.0.2"],\
            ["has-bigints", "npm:1.0.2"],\
            ["has-symbols", "npm:1.0.3"],\
            ["which-boxed-primitive", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["underscore", [\
        ["npm:1.13.6", {\
          "packageLocation": "./.yarn/cache/underscore-npm-1.13.6-3ebe9d92fb-d5cedd14a9.zip/node_modules/underscore/",\
          "packageDependencies": [\
            ["underscore", "npm:1.13.6"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unicode-canonical-property-names-ecmascript", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/unicode-canonical-property-names-ecmascript-npm-2.0.0-d2d8554a14-39be078afd.zip/node_modules/unicode-canonical-property-names-ecmascript/",\
          "packageDependencies": [\
            ["unicode-canonical-property-names-ecmascript", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unicode-match-property-ecmascript", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/unicode-match-property-ecmascript-npm-2.0.0-97a00fd52c-1f34a7434a.zip/node_modules/unicode-match-property-ecmascript/",\
          "packageDependencies": [\
            ["unicode-match-property-ecmascript", "npm:2.0.0"],\
            ["unicode-canonical-property-names-ecmascript", "npm:2.0.0"],\
            ["unicode-property-aliases-ecmascript", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unicode-match-property-value-ecmascript", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/unicode-match-property-value-ecmascript-npm-2.1.0-65e24443e6-8d6f5f586b.zip/node_modules/unicode-match-property-value-ecmascript/",\
          "packageDependencies": [\
            ["unicode-match-property-value-ecmascript", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unicode-property-aliases-ecmascript", [\
        ["npm:2.1.0", {\
          "packageLocation": "./.yarn/cache/unicode-property-aliases-ecmascript-npm-2.1.0-46779595f4-2435244318.zip/node_modules/unicode-property-aliases-ecmascript/",\
          "packageDependencies": [\
            ["unicode-property-aliases-ecmascript", "npm:2.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["union-value", [\
        ["npm:1.0.1", {\
          "packageLocation": "./.yarn/cache/union-value-npm-1.0.1-76c6e8a88f-a3464097d3.zip/node_modules/union-value/",\
          "packageDependencies": [\
            ["union-value", "npm:1.0.1"],\
            ["arr-union", "npm:3.1.0"],\
            ["get-value", "npm:2.0.6"],\
            ["is-extendable", "npm:0.1.1"],\
            ["set-value", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unique-filename", [\
        ["npm:1.1.1", {\
          "packageLocation": "./.yarn/cache/unique-filename-npm-1.1.1-c885c5095b-cf4998c922.zip/node_modules/unique-filename/",\
          "packageDependencies": [\
            ["unique-filename", "npm:1.1.1"],\
            ["unique-slug", "npm:2.0.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/unique-filename-npm-3.0.0-77d68e0a45-8e2f59b356.zip/node_modules/unique-filename/",\
          "packageDependencies": [\
            ["unique-filename", "npm:3.0.0"],\
            ["unique-slug", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unique-slug", [\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/unique-slug-npm-2.0.2-f6ba1ddeb7-5b6876a645.zip/node_modules/unique-slug/",\
          "packageDependencies": [\
            ["unique-slug", "npm:2.0.2"],\
            ["imurmurhash", "npm:0.1.4"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/unique-slug-npm-4.0.0-e6b08f28aa-0884b58365.zip/node_modules/unique-slug/",\
          "packageDependencies": [\
            ["unique-slug", "npm:4.0.0"],\
            ["imurmurhash", "npm:0.1.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["universalify", [\
        ["npm:0.1.2", {\
          "packageLocation": "./.yarn/cache/universalify-npm-0.1.2-9b22d31d2d-40cdc60f6e.zip/node_modules/universalify/",\
          "packageDependencies": [\
            ["universalify", "npm:0.1.2"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:0.2.0", {\
          "packageLocation": "./.yarn/cache/universalify-npm-0.2.0-9984e61c10-e86134cb12.zip/node_modules/universalify/",\
          "packageDependencies": [\
            ["universalify", "npm:0.2.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/universalify-npm-2.0.0-03b8b418a8-2406a4edf4.zip/node_modules/universalify/",\
          "packageDependencies": [\
            ["universalify", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["unset-value", [\
        ["npm:1.0.0", {\
          "packageLocation": "./.yarn/cache/unset-value-npm-1.0.0-2af803b920-5990ecf660.zip/node_modules/unset-value/",\
          "packageDependencies": [\
            ["unset-value", "npm:1.0.0"],\
            ["has-value", "npm:0.3.1"],\
            ["isobject", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["update-browserslist-db", [\
        ["npm:1.0.11", {\
          "packageLocation": "./.yarn/cache/update-browserslist-db-npm-1.0.11-2c8e64258f-b98327518f.zip/node_modules/update-browserslist-db/",\
          "packageDependencies": [\
            ["update-browserslist-db", "npm:1.0.11"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:f6128308c12e2b37fe427262226a88e213eb66128b3445db14a7af8a6dafb815a9c4562e232b574e216fb796ee19313d71844b195e41ad82170480289a6b575c#npm:1.0.11", {\
          "packageLocation": "./.yarn/__virtual__/update-browserslist-db-virtual-3718aa891c/0/cache/update-browserslist-db-npm-1.0.11-2c8e64258f-b98327518f.zip/node_modules/update-browserslist-db/",\
          "packageDependencies": [\
            ["update-browserslist-db", "virtual:f6128308c12e2b37fe427262226a88e213eb66128b3445db14a7af8a6dafb815a9c4562e232b574e216fb796ee19313d71844b195e41ad82170480289a6b575c#npm:1.0.11"],\
            ["@types/browserslist", null],\
            ["browserslist", "npm:4.21.9"],\
            ["escalade", "npm:3.1.1"],\
            ["picocolors", "npm:1.0.0"]\
          ],\
          "packagePeers": [\
            "@types/browserslist",\
            "browserslist"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["uri-js", [\
        ["npm:4.4.1", {\
          "packageLocation": "./.yarn/cache/uri-js-npm-4.4.1-66d11cbcaf-7167432de6.zip/node_modules/uri-js/",\
          "packageDependencies": [\
            ["uri-js", "npm:4.4.1"],\
            ["punycode", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["urix", [\
        ["npm:0.1.0", {\
          "packageLocation": "./.yarn/cache/urix-npm-0.1.0-bd5e55a13a-4c076ecfbf.zip/node_modules/urix/",\
          "packageDependencies": [\
            ["urix", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["url-loader", [\
        ["npm:4.1.1", {\
          "packageLocation": "./.yarn/cache/url-loader-npm-4.1.1-568e22ebcb-c1122a992c.zip/node_modules/url-loader/",\
          "packageDependencies": [\
            ["url-loader", "npm:4.1.1"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.1.1", {\
          "packageLocation": "./.yarn/__virtual__/url-loader-virtual-e5eba94ebd/0/cache/url-loader-npm-4.1.1-568e22ebcb-c1122a992c.zip/node_modules/url-loader/",\
          "packageDependencies": [\
            ["url-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.1.1"],\
            ["@types/file-loader", null],\
            ["@types/webpack", null],\
            ["file-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:6.0.0"],\
            ["loader-utils", "npm:2.0.4"],\
            ["mime-types", "npm:2.1.35"],\
            ["schema-utils", "npm:3.3.0"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/file-loader",\
            "@types/webpack",\
            "file-loader",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["url-parse", [\
        ["npm:1.5.10", {\
          "packageLocation": "./.yarn/cache/url-parse-npm-1.5.10-64fa2bcd6d-fbdba6b1d8.zip/node_modules/url-parse/",\
          "packageDependencies": [\
            ["url-parse", "npm:1.5.10"],\
            ["querystringify", "npm:2.2.0"],\
            ["requires-port", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["use", [\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/use-npm-3.1.1-7ba643714c-08a130289f.zip/node_modules/use/",\
          "packageDependencies": [\
            ["use", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["util-deprecate", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/util-deprecate-npm-1.0.2-e3fe1a219c-474acf1146.zip/node_modules/util-deprecate/",\
          "packageDependencies": [\
            ["util-deprecate", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["uuid", [\
        ["npm:8.3.2", {\
          "packageLocation": "./.yarn/cache/uuid-npm-8.3.2-eca0baba53-5575a8a75c.zip/node_modules/uuid/",\
          "packageDependencies": [\
            ["uuid", "npm:8.3.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["v8-compile-cache", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/v8-compile-cache-npm-2.3.0-961375f150-adb0a271ea.zip/node_modules/v8-compile-cache/",\
          "packageDependencies": [\
            ["v8-compile-cache", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["v8-to-istanbul", [\
        ["npm:7.1.2", {\
          "packageLocation": "./.yarn/cache/v8-to-istanbul-npm-7.1.2-f29c35e9aa-e52b48764f.zip/node_modules/v8-to-istanbul/",\
          "packageDependencies": [\
            ["v8-to-istanbul", "npm:7.1.2"],\
            ["@types/istanbul-lib-coverage", "npm:2.0.4"],\
            ["convert-source-map", "npm:1.9.0"],\
            ["source-map", "npm:0.7.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["validate-npm-package-license", [\
        ["npm:3.0.4", {\
          "packageLocation": "./.yarn/cache/validate-npm-package-license-npm-3.0.4-7af8adc7a8-35703ac889.zip/node_modules/validate-npm-package-license/",\
          "packageDependencies": [\
            ["validate-npm-package-license", "npm:3.0.4"],\
            ["spdx-correct", "npm:3.2.0"],\
            ["spdx-expression-parse", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["w3c-hr-time", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/w3c-hr-time-npm-1.0.2-87f88e51d9-ec3c2dacbf.zip/node_modules/w3c-hr-time/",\
          "packageDependencies": [\
            ["w3c-hr-time", "npm:1.0.2"],\
            ["browser-process-hrtime", "npm:1.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["w3c-xmlserializer", [\
        ["npm:2.0.0", {\
          "packageLocation": "./.yarn/cache/w3c-xmlserializer-npm-2.0.0-f8f7bc8b42-ae25c51cf7.zip/node_modules/w3c-xmlserializer/",\
          "packageDependencies": [\
            ["w3c-xmlserializer", "npm:2.0.0"],\
            ["xml-name-validator", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["walker", [\
        ["npm:1.0.8", {\
          "packageLocation": "./.yarn/cache/walker-npm-1.0.8-b0a05b9478-ad7a257ea1.zip/node_modules/walker/",\
          "packageDependencies": [\
            ["walker", "npm:1.0.8"],\
            ["makeerror", "npm:1.0.12"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["watchpack", [\
        ["npm:2.4.0", {\
          "packageLocation": "./.yarn/cache/watchpack-npm-2.4.0-7ec4b9cc65-23d4bc5863.zip/node_modules/watchpack/",\
          "packageDependencies": [\
            ["watchpack", "npm:2.4.0"],\
            ["glob-to-regexp", "npm:0.4.1"],\
            ["graceful-fs", "npm:4.2.11"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["webidl-conversions", [\
        ["npm:3.0.1", {\
          "packageLocation": "./.yarn/cache/webidl-conversions-npm-3.0.1-60310f6a2b-c92a0a6ab9.zip/node_modules/webidl-conversions/",\
          "packageDependencies": [\
            ["webidl-conversions", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/webidl-conversions-npm-5.0.0-9649787484-ccf1ec2ca7.zip/node_modules/webidl-conversions/",\
          "packageDependencies": [\
            ["webidl-conversions", "npm:5.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:6.1.0", {\
          "packageLocation": "./.yarn/cache/webidl-conversions-npm-6.1.0-0594fd577c-1f526507aa.zip/node_modules/webidl-conversions/",\
          "packageDependencies": [\
            ["webidl-conversions", "npm:6.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["webpack", [\
        ["npm:5.88.0", {\
          "packageLocation": "./.yarn/cache/webpack-npm-5.88.0-47177f9f58-9fd1568b34.zip/node_modules/webpack/",\
          "packageDependencies": [\
            ["webpack", "npm:5.88.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0", {\
          "packageLocation": "./.yarn/__virtual__/webpack-virtual-a8c2fda196/0/cache/webpack-npm-5.88.0-47177f9f58-9fd1568b34.zip/node_modules/webpack/",\
          "packageDependencies": [\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"],\
            ["@types/eslint-scope", "npm:3.7.4"],\
            ["@types/estree", "npm:1.0.1"],\
            ["@types/webpack-cli", null],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-edit", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-parser", "npm:1.11.6"],\
            ["acorn", "npm:8.9.0"],\
            ["acorn-import-assertions", "virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:1.9.0"],\
            ["browserslist", "npm:4.21.9"],\
            ["chrome-trace-event", "npm:1.0.3"],\
            ["enhanced-resolve", "npm:5.15.0"],\
            ["es-module-lexer", "npm:1.3.0"],\
            ["eslint-scope", "npm:5.1.1"],\
            ["events", "npm:3.3.0"],\
            ["glob-to-regexp", "npm:0.4.1"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["json-parse-even-better-errors", "npm:2.3.1"],\
            ["loader-runner", "npm:4.3.0"],\
            ["mime-types", "npm:2.1.35"],\
            ["neo-async", "npm:2.6.2"],\
            ["schema-utils", "npm:3.3.0"],\
            ["tapable", "npm:2.2.1"],\
            ["terser-webpack-plugin", "virtual:a8c2fda196219bb2f79b42a12252ee3317d65cf0d2e525730054f5016e10b6a7de7ffdfd6a671c28c476a270aabdc7f477fa7ae9e2df43bc207110bcc3d8d02b#npm:5.3.9"],\
            ["watchpack", "npm:2.4.0"],\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"],\
            ["webpack-sources", "npm:3.2.3"]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "webpack-cli"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0", {\
          "packageLocation": "./.yarn/__virtual__/webpack-virtual-ae9128e019/0/cache/webpack-npm-5.88.0-47177f9f58-9fd1568b34.zip/node_modules/webpack/",\
          "packageDependencies": [\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["@types/eslint-scope", "npm:3.7.4"],\
            ["@types/estree", "npm:1.0.1"],\
            ["@types/webpack-cli", null],\
            ["@webassemblyjs/ast", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-edit", "npm:1.11.6"],\
            ["@webassemblyjs/wasm-parser", "npm:1.11.6"],\
            ["acorn", "npm:8.9.0"],\
            ["acorn-import-assertions", "virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:1.9.0"],\
            ["browserslist", "npm:4.21.9"],\
            ["chrome-trace-event", "npm:1.0.3"],\
            ["enhanced-resolve", "npm:5.15.0"],\
            ["es-module-lexer", "npm:1.3.0"],\
            ["eslint-scope", "npm:5.1.1"],\
            ["events", "npm:3.3.0"],\
            ["glob-to-regexp", "npm:0.4.1"],\
            ["graceful-fs", "npm:4.2.11"],\
            ["json-parse-even-better-errors", "npm:2.3.1"],\
            ["loader-runner", "npm:4.3.0"],\
            ["mime-types", "npm:2.1.35"],\
            ["neo-async", "npm:2.6.2"],\
            ["schema-utils", "npm:3.3.0"],\
            ["tapable", "npm:2.2.1"],\
            ["terser-webpack-plugin", "virtual:ae9128e0193ad578088fc20229e9f00db6436a463b4f6ae6d290f13eac5adc16a1b2c53dc86cc3550bfa3e4c4fe3b460fb2236bba4ba086aa9801096aa0467e7#npm:5.3.9"],\
            ["watchpack", "npm:2.4.0"],\
            ["webpack-cli", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0"],\
            ["webpack-sources", "npm:3.2.3"]\
          ],\
          "packagePeers": [\
            "@types/webpack-cli",\
            "webpack-cli"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["webpack-cli", [\
        ["npm:4.10.0", {\
          "packageLocation": "./.yarn/cache/webpack-cli-npm-4.10.0-09cee8c457-2ff5355ac3.zip/node_modules/webpack-cli/",\
          "packageDependencies": [\
            ["webpack-cli", "npm:4.10.0"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0", {\
          "packageLocation": "./.yarn/__virtual__/webpack-cli-virtual-f8c4ac2936/0/cache/webpack-cli-npm-4.10.0-09cee8c457-2ff5355ac3.zip/node_modules/webpack-cli/",\
          "packageDependencies": [\
            ["webpack-cli", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:4.10.0"],\
            ["@discoveryjs/json-ext", "npm:0.5.7"],\
            ["@types/webpack", null],\
            ["@types/webpack-bundle-analyzer", null],\
            ["@types/webpack-cli__generators", null],\
            ["@types/webpack-cli__migrate", null],\
            ["@types/webpack-dev-server", null],\
            ["@webpack-cli/configtest", "virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.2.0"],\
            ["@webpack-cli/generators", null],\
            ["@webpack-cli/info", "virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.5.0"],\
            ["@webpack-cli/migrate", null],\
            ["@webpack-cli/serve", "virtual:f8c4ac29369ce0e4ab33fe455327889ff446ca2cf4e4fd792c5323c3232f9c1f601ffaf33d7663cdedf54470973300b83036a2503b6e5fc0457703f6a1e64389#npm:1.7.0"],\
            ["colorette", "npm:2.0.20"],\
            ["commander", "npm:7.2.0"],\
            ["cross-spawn", "npm:7.0.3"],\
            ["fastest-levenshtein", "npm:1.0.16"],\
            ["import-local", "npm:3.1.0"],\
            ["interpret", "npm:2.2.0"],\
            ["rechoir", "npm:0.7.1"],\
            ["webpack", "virtual:33696cd96458f016c94bd4b81919d976ce3804ae0c05679efb65513a9b55155210a35f8a07a44bfce0fd873cf6fc3033162ec62b46a683234f4dd59b9c9b5717#npm:5.88.0"],\
            ["webpack-bundle-analyzer", null],\
            ["webpack-dev-server", null],\
            ["webpack-merge", "npm:5.9.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack-bundle-analyzer",\
            "@types/webpack-cli__generators",\
            "@types/webpack-cli__migrate",\
            "@types/webpack-dev-server",\
            "@types/webpack",\
            "@webpack-cli/generators",\
            "@webpack-cli/migrate",\
            "webpack-bundle-analyzer",\
            "webpack-dev-server",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0", {\
          "packageLocation": "./.yarn/__virtual__/webpack-cli-virtual-6d62a8ba00/0/cache/webpack-cli-npm-4.10.0-09cee8c457-2ff5355ac3.zip/node_modules/webpack-cli/",\
          "packageDependencies": [\
            ["webpack-cli", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:4.10.0"],\
            ["@discoveryjs/json-ext", "npm:0.5.7"],\
            ["@types/webpack", null],\
            ["@types/webpack-bundle-analyzer", null],\
            ["@types/webpack-cli__generators", null],\
            ["@types/webpack-cli__migrate", null],\
            ["@types/webpack-dev-server", null],\
            ["@webpack-cli/configtest", "virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.2.0"],\
            ["@webpack-cli/generators", null],\
            ["@webpack-cli/info", "virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.5.0"],\
            ["@webpack-cli/migrate", null],\
            ["@webpack-cli/serve", "virtual:6d62a8ba00845dfc83f8877e325da1880b7d4adb6ba7e085553d3bcc243ed6c6117df9f5a6d841e7c8c08f88eaa266a9a969a1528b27ecdc3f0c87ac2cad438e#npm:1.7.0"],\
            ["colorette", "npm:2.0.20"],\
            ["commander", "npm:7.2.0"],\
            ["cross-spawn", "npm:7.0.3"],\
            ["fastest-levenshtein", "npm:1.0.16"],\
            ["import-local", "npm:3.1.0"],\
            ["interpret", "npm:2.2.0"],\
            ["rechoir", "npm:0.7.1"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"],\
            ["webpack-bundle-analyzer", null],\
            ["webpack-dev-server", null],\
            ["webpack-merge", "npm:5.9.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack-bundle-analyzer",\
            "@types/webpack-cli__generators",\
            "@types/webpack-cli__migrate",\
            "@types/webpack-dev-server",\
            "@types/webpack",\
            "@webpack-cli/generators",\
            "@webpack-cli/migrate",\
            "webpack-bundle-analyzer",\
            "webpack-dev-server",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["webpack-merge", [\
        ["npm:5.9.0", {\
          "packageLocation": "./.yarn/cache/webpack-merge-npm-5.9.0-9110e650de-64fe2c23aa.zip/node_modules/webpack-merge/",\
          "packageDependencies": [\
            ["webpack-merge", "npm:5.9.0"],\
            ["clone-deep", "npm:4.0.1"],\
            ["wildcard", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["webpack-sources", [\
        ["npm:1.4.3", {\
          "packageLocation": "./.yarn/cache/webpack-sources-npm-1.4.3-2b3a9b1de0-37463dad8d.zip/node_modules/webpack-sources/",\
          "packageDependencies": [\
            ["webpack-sources", "npm:1.4.3"],\
            ["source-list-map", "npm:2.0.1"],\
            ["source-map", "npm:0.6.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:3.2.3", {\
          "packageLocation": "./.yarn/cache/webpack-sources-npm-3.2.3-6bfb5d9563-989e401b9f.zip/node_modules/webpack-sources/",\
          "packageDependencies": [\
            ["webpack-sources", "npm:3.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["whatwg-encoding", [\
        ["npm:1.0.5", {\
          "packageLocation": "./.yarn/cache/whatwg-encoding-npm-1.0.5-85e0fb7d7d-5be4efe111.zip/node_modules/whatwg-encoding/",\
          "packageDependencies": [\
            ["whatwg-encoding", "npm:1.0.5"],\
            ["iconv-lite", "npm:0.4.24"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["whatwg-mimetype", [\
        ["npm:2.3.0", {\
          "packageLocation": "./.yarn/cache/whatwg-mimetype-npm-2.3.0-52eaa1d941-23eb885940.zip/node_modules/whatwg-mimetype/",\
          "packageDependencies": [\
            ["whatwg-mimetype", "npm:2.3.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["whatwg-url", [\
        ["npm:5.0.0", {\
          "packageLocation": "./.yarn/cache/whatwg-url-npm-5.0.0-374fb45e60-b8daed4ad3.zip/node_modules/whatwg-url/",\
          "packageDependencies": [\
            ["whatwg-url", "npm:5.0.0"],\
            ["tr46", "npm:0.0.3"],\
            ["webidl-conversions", "npm:3.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.7.0", {\
          "packageLocation": "./.yarn/cache/whatwg-url-npm-8.7.0-67af66db8f-a87abcc6ce.zip/node_modules/whatwg-url/",\
          "packageDependencies": [\
            ["whatwg-url", "npm:8.7.0"],\
            ["lodash", "npm:4.17.21"],\
            ["tr46", "npm:2.1.0"],\
            ["webidl-conversions", "npm:6.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["which", [\
        ["npm:1.3.1", {\
          "packageLocation": "./.yarn/cache/which-npm-1.3.1-f0ebb8bdd8-f2e185c624.zip/node_modules/which/",\
          "packageDependencies": [\
            ["which", "npm:1.3.1"],\
            ["isexe", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:2.0.2", {\
          "packageLocation": "./.yarn/cache/which-npm-2.0.2-320ddf72f7-1a5c563d3c.zip/node_modules/which/",\
          "packageDependencies": [\
            ["which", "npm:2.0.2"],\
            ["isexe", "npm:2.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["which-boxed-primitive", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/which-boxed-primitive-npm-1.0.2-e214f9ae5a-53ce774c73.zip/node_modules/which-boxed-primitive/",\
          "packageDependencies": [\
            ["which-boxed-primitive", "npm:1.0.2"],\
            ["is-bigint", "npm:1.0.4"],\
            ["is-boolean-object", "npm:1.1.2"],\
            ["is-number-object", "npm:1.0.7"],\
            ["is-string", "npm:1.0.7"],\
            ["is-symbol", "npm:1.0.4"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["which-module", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/which-module-npm-2.0.1-90f889f6f6-1967b7ce17.zip/node_modules/which-module/",\
          "packageDependencies": [\
            ["which-module", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["which-typed-array", [\
        ["npm:1.1.9", {\
          "packageLocation": "./.yarn/cache/which-typed-array-npm-1.1.9-9559c95dfc-fe0178ca44.zip/node_modules/which-typed-array/",\
          "packageDependencies": [\
            ["which-typed-array", "npm:1.1.9"],\
            ["available-typed-arrays", "npm:1.0.5"],\
            ["call-bind", "npm:1.0.2"],\
            ["for-each", "npm:0.3.3"],\
            ["gopd", "npm:1.0.1"],\
            ["has-tostringtag", "npm:1.0.0"],\
            ["is-typed-array", "npm:1.1.10"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["wide-align", [\
        ["npm:1.1.5", {\
          "packageLocation": "./.yarn/cache/wide-align-npm-1.1.5-889d77e592-d5fc37cd56.zip/node_modules/wide-align/",\
          "packageDependencies": [\
            ["wide-align", "npm:1.1.5"],\
            ["string-width", "npm:4.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["wildcard", [\
        ["npm:2.0.1", {\
          "packageLocation": "./.yarn/cache/wildcard-npm-2.0.1-7c6a3a3365-e0c60a12a2.zip/node_modules/wildcard/",\
          "packageDependencies": [\
            ["wildcard", "npm:2.0.1"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["word-wrap", [\
        ["npm:1.2.3", {\
          "packageLocation": "./.yarn/cache/word-wrap-npm-1.2.3-7fb15ab002-30b48f91fc.zip/node_modules/word-wrap/",\
          "packageDependencies": [\
            ["word-wrap", "npm:1.2.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["worker-loader", [\
        ["npm:3.0.8", {\
          "packageLocation": "./.yarn/cache/worker-loader-npm-3.0.8-6423aa0002-84f4a7eeb2.zip/node_modules/worker-loader/",\
          "packageDependencies": [\
            ["worker-loader", "npm:3.0.8"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:3.0.8", {\
          "packageLocation": "./.yarn/__virtual__/worker-loader-virtual-52716a8f04/0/cache/worker-loader-npm-3.0.8-6423aa0002-84f4a7eeb2.zip/node_modules/worker-loader/",\
          "packageDependencies": [\
            ["worker-loader", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:3.0.8"],\
            ["@types/webpack", null],\
            ["loader-utils", "npm:2.0.4"],\
            ["schema-utils", "npm:3.3.0"],\
            ["webpack", "virtual:80d0dfd7517920160e47673f54a7ec82767652560c62c965326de7b850c4cce4808598573863d51bb628f65a68200a88cd5508501e63dad8a87d365620d7de73#npm:5.88.0"]\
          ],\
          "packagePeers": [\
            "@types/webpack",\
            "webpack"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["wrap-ansi", [\
        ["npm:6.2.0", {\
          "packageLocation": "./.yarn/cache/wrap-ansi-npm-6.2.0-439a7246d8-6cd96a4101.zip/node_modules/wrap-ansi/",\
          "packageDependencies": [\
            ["wrap-ansi", "npm:6.2.0"],\
            ["ansi-styles", "npm:4.3.0"],\
            ["string-width", "npm:4.2.3"],\
            ["strip-ansi", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:7.0.0", {\
          "packageLocation": "./.yarn/cache/wrap-ansi-npm-7.0.0-ad6e1a0554-a790b846fd.zip/node_modules/wrap-ansi/",\
          "packageDependencies": [\
            ["wrap-ansi", "npm:7.0.0"],\
            ["ansi-styles", "npm:4.3.0"],\
            ["string-width", "npm:4.2.3"],\
            ["strip-ansi", "npm:6.0.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:8.1.0", {\
          "packageLocation": "./.yarn/cache/wrap-ansi-npm-8.1.0-26a4e6ae28-371733296d.zip/node_modules/wrap-ansi/",\
          "packageDependencies": [\
            ["wrap-ansi", "npm:8.1.0"],\
            ["ansi-styles", "npm:6.2.1"],\
            ["string-width", "npm:5.1.2"],\
            ["strip-ansi", "npm:7.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["wrappy", [\
        ["npm:1.0.2", {\
          "packageLocation": "./.yarn/cache/wrappy-npm-1.0.2-916de4d4b3-159da4805f.zip/node_modules/wrappy/",\
          "packageDependencies": [\
            ["wrappy", "npm:1.0.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["write-file-atomic", [\
        ["npm:3.0.3", {\
          "packageLocation": "./.yarn/cache/write-file-atomic-npm-3.0.3-d948a237da-c55b24617c.zip/node_modules/write-file-atomic/",\
          "packageDependencies": [\
            ["write-file-atomic", "npm:3.0.3"],\
            ["imurmurhash", "npm:0.1.4"],\
            ["is-typedarray", "npm:1.0.0"],\
            ["signal-exit", "npm:3.0.7"],\
            ["typedarray-to-buffer", "npm:3.1.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["ws", [\
        ["npm:0.4.32", {\
          "packageLocation": "./.yarn/unplugged/ws-virtual-21f55c7998/node_modules/ws/",\
          "packageDependencies": [\
            ["ws", "npm:0.4.32"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["npm:7.5.9", {\
          "packageLocation": "./.yarn/cache/ws-npm-7.5.9-26f12a5ed6-c3c100a181.zip/node_modules/ws/",\
          "packageDependencies": [\
            ["ws", "npm:7.5.9"]\
          ],\
          "linkType": "SOFT"\
        }],\
        ["virtual:8afc7ef03460feb4078a4f2fd26d3b578614a68d423938bea12f546ef9796e0320d21afdcde376380f82d29921a800d02b35da41fb2ff1e4098480a377065368#npm:0.4.32", {\
          "packageLocation": "./.yarn/unplugged/ws-virtual-21f55c7998/node_modules/ws/",\
          "packageDependencies": [\
            ["ws", "virtual:8afc7ef03460feb4078a4f2fd26d3b578614a68d423938bea12f546ef9796e0320d21afdcde376380f82d29921a800d02b35da41fb2ff1e4098480a377065368#npm:0.4.32"],\
            ["@types/bufferutil", null],\
            ["@types/utf-8-validate", null],\
            ["bufferutil", null],\
            ["commander", "npm:2.1.0"],\
            ["nan", "npm:1.0.0"],\
            ["node-gyp", "npm:9.4.0"],\
            ["options", "npm:0.0.6"],\
            ["tinycolor", "npm:0.0.1"],\
            ["utf-8-validate", null]\
          ],\
          "packagePeers": [\
            "@types/bufferutil",\
            "@types/utf-8-validate",\
            "bufferutil",\
            "utf-8-validate"\
          ],\
          "linkType": "HARD"\
        }],\
        ["virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:7.5.9", {\
          "packageLocation": "./.yarn/__virtual__/ws-virtual-ceed7510c7/0/cache/ws-npm-7.5.9-26f12a5ed6-c3c100a181.zip/node_modules/ws/",\
          "packageDependencies": [\
            ["ws", "virtual:dce30a2f17982ee01d94c50294685d046bce044ee3caac180965d83c43eb87c02597abbc5718365f5dca4c373efae9a08489b882b6a52c7643e88ab6f4bfd9d5#npm:7.5.9"],\
            ["@types/bufferutil", null],\
            ["@types/utf-8-validate", null],\
            ["bufferutil", null],\
            ["utf-8-validate", null]\
          ],\
          "packagePeers": [\
            "@types/bufferutil",\
            "@types/utf-8-validate",\
            "bufferutil",\
            "utf-8-validate"\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["xml-name-validator", [\
        ["npm:3.0.0", {\
          "packageLocation": "./.yarn/cache/xml-name-validator-npm-3.0.0-10e74a38ea-b3ac459afe.zip/node_modules/xml-name-validator/",\
          "packageDependencies": [\
            ["xml-name-validator", "npm:3.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["xmlchars", [\
        ["npm:2.2.0", {\
          "packageLocation": "./.yarn/cache/xmlchars-npm-2.2.0-8b78f0f5e4-8c70ac9407.zip/node_modules/xmlchars/",\
          "packageDependencies": [\
            ["xmlchars", "npm:2.2.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["xmlhttprequest", [\
        ["npm:1.4.2", {\
          "packageLocation": "./.yarn/cache/xmlhttprequest-npm-1.4.2-01a0cf7be3-fb93109425.zip/node_modules/xmlhttprequest/",\
          "packageDependencies": [\
            ["xmlhttprequest", "npm:1.4.2"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["y18n", [\
        ["npm:4.0.3", {\
          "packageLocation": "./.yarn/cache/y18n-npm-4.0.3-ced95acdbc-014dfcd9b5.zip/node_modules/y18n/",\
          "packageDependencies": [\
            ["y18n", "npm:4.0.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["yallist", [\
        ["npm:3.1.1", {\
          "packageLocation": "./.yarn/cache/yallist-npm-3.1.1-a568a556b4-48f7bb00dc.zip/node_modules/yallist/",\
          "packageDependencies": [\
            ["yallist", "npm:3.1.1"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:4.0.0", {\
          "packageLocation": "./.yarn/cache/yallist-npm-4.0.0-b493d9e907-343617202a.zip/node_modules/yallist/",\
          "packageDependencies": [\
            ["yallist", "npm:4.0.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["yargs", [\
        ["npm:15.4.1", {\
          "packageLocation": "./.yarn/cache/yargs-npm-15.4.1-ca1c444de1-40b974f508.zip/node_modules/yargs/",\
          "packageDependencies": [\
            ["yargs", "npm:15.4.1"],\
            ["cliui", "npm:6.0.0"],\
            ["decamelize", "npm:1.2.0"],\
            ["find-up", "npm:4.1.0"],\
            ["get-caller-file", "npm:2.0.5"],\
            ["require-directory", "npm:2.1.1"],\
            ["require-main-filename", "npm:2.0.0"],\
            ["set-blocking", "npm:2.0.0"],\
            ["string-width", "npm:4.2.3"],\
            ["which-module", "npm:2.0.1"],\
            ["y18n", "npm:4.0.3"],\
            ["yargs-parser", "npm:18.1.3"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["yargs-parser", [\
        ["npm:18.1.3", {\
          "packageLocation": "./.yarn/cache/yargs-parser-npm-18.1.3-0ba9c4f088-60e8c7d1b8.zip/node_modules/yargs-parser/",\
          "packageDependencies": [\
            ["yargs-parser", "npm:18.1.3"],\
            ["camelcase", "npm:5.3.1"],\
            ["decamelize", "npm:1.2.0"]\
          ],\
          "linkType": "HARD"\
        }],\
        ["npm:20.2.9", {\
          "packageLocation": "./.yarn/cache/yargs-parser-npm-20.2.9-a1d19e598d-8bb69015f2.zip/node_modules/yargs-parser/",\
          "packageDependencies": [\
            ["yargs-parser", "npm:20.2.9"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["yocto-queue", [\
        ["npm:0.1.0", {\
          "packageLocation": "./.yarn/cache/yocto-queue-npm-0.1.0-c6c9a7db29-f77b3d8d00.zip/node_modules/yocto-queue/",\
          "packageDependencies": [\
            ["yocto-queue", "npm:0.1.0"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]],\
      ["zeparser", [\
        ["npm:0.0.5", {\
          "packageLocation": "./.yarn/cache/zeparser-npm-0.0.5-b2108c54c9-4325ab8716.zip/node_modules/zeparser/",\
          "packageDependencies": [\
            ["zeparser", "npm:0.0.5"]\
          ],\
          "linkType": "HARD"\
        }]\
      ]]\
    ]\
  }'), {basePath: basePath || __dirname});
  }

const fs = require('fs');
const path = require('path');
const require$$0 = require('module');
const StringDecoder = require('string_decoder');
const url = require('url');
const os = require('os');
const nodeUtils = require('util');
const readline = require('readline');
const assert = require('assert');
const stream = require('stream');
const zlib = require('zlib');
const events = require('events');

const _interopDefaultLegacy = e => e && typeof e === 'object' && 'default' in e ? e : { default: e };

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  const n = Object.create(null);
  if (e) {
    for (const k in e) {
      if (k !== 'default') {
        const d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: () => e[k]
        });
      }
    }
  }
  n.default = e;
  return Object.freeze(n);
}

const fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
const path__default = /*#__PURE__*/_interopDefaultLegacy(path);
const require$$0__default = /*#__PURE__*/_interopDefaultLegacy(require$$0);
const StringDecoder__default = /*#__PURE__*/_interopDefaultLegacy(StringDecoder);
const nodeUtils__namespace = /*#__PURE__*/_interopNamespace(nodeUtils);
const assert__default = /*#__PURE__*/_interopDefaultLegacy(assert);
const zlib__default = /*#__PURE__*/_interopDefaultLegacy(zlib);

const S_IFMT = 61440;
const S_IFDIR = 16384;
const S_IFREG = 32768;
const S_IFLNK = 40960;
const SAFE_TIME = 456789e3;

const DEFAULT_MODE = S_IFREG | 420;
class StatEntry {
  constructor() {
    this.uid = 0;
    this.gid = 0;
    this.size = 0;
    this.blksize = 0;
    this.atimeMs = 0;
    this.mtimeMs = 0;
    this.ctimeMs = 0;
    this.birthtimeMs = 0;
    this.atime = new Date(0);
    this.mtime = new Date(0);
    this.ctime = new Date(0);
    this.birthtime = new Date(0);
    this.dev = 0;
    this.ino = 0;
    this.mode = DEFAULT_MODE;
    this.nlink = 1;
    this.rdev = 0;
    this.blocks = 1;
  }
  isBlockDevice() {
    return false;
  }
  isCharacterDevice() {
    return false;
  }
  isDirectory() {
    return (this.mode & S_IFMT) === S_IFDIR;
  }
  isFIFO() {
    return false;
  }
  isFile() {
    return (this.mode & S_IFMT) === S_IFREG;
  }
  isSocket() {
    return false;
  }
  isSymbolicLink() {
    return (this.mode & S_IFMT) === S_IFLNK;
  }
}
class BigIntStatsEntry {
  constructor() {
    this.uid = BigInt(0);
    this.gid = BigInt(0);
    this.size = BigInt(0);
    this.blksize = BigInt(0);
    this.atimeMs = BigInt(0);
    this.mtimeMs = BigInt(0);
    this.ctimeMs = BigInt(0);
    this.birthtimeMs = BigInt(0);
    this.atimeNs = BigInt(0);
    this.mtimeNs = BigInt(0);
    this.ctimeNs = BigInt(0);
    this.birthtimeNs = BigInt(0);
    this.atime = new Date(0);
    this.mtime = new Date(0);
    this.ctime = new Date(0);
    this.birthtime = new Date(0);
    this.dev = BigInt(0);
    this.ino = BigInt(0);
    this.mode = BigInt(DEFAULT_MODE);
    this.nlink = BigInt(1);
    this.rdev = BigInt(0);
    this.blocks = BigInt(1);
  }
  isBlockDevice() {
    return false;
  }
  isCharacterDevice() {
    return false;
  }
  isDirectory() {
    return (this.mode & BigInt(S_IFMT)) === BigInt(S_IFDIR);
  }
  isFIFO() {
    return false;
  }
  isFile() {
    return (this.mode & BigInt(S_IFMT)) === BigInt(S_IFREG);
  }
  isSocket() {
    return false;
  }
  isSymbolicLink() {
    return (this.mode & BigInt(S_IFMT)) === BigInt(S_IFLNK);
  }
}
function makeDefaultStats() {
  return new StatEntry();
}
function clearStats(stats) {
  for (const key in stats) {
    if (Object.prototype.hasOwnProperty.call(stats, key)) {
      const element = stats[key];
      if (typeof element === `number`) {
        stats[key] = 0;
      } else if (typeof element === `bigint`) {
        stats[key] = BigInt(0);
      } else if (nodeUtils__namespace.types.isDate(element)) {
        stats[key] = new Date(0);
      }
    }
  }
  return stats;
}
function convertToBigIntStats(stats) {
  const bigintStats = new BigIntStatsEntry();
  for (const key in stats) {
    if (Object.prototype.hasOwnProperty.call(stats, key)) {
      const element = stats[key];
      if (typeof element === `number`) {
        bigintStats[key] = BigInt(element);
      } else if (nodeUtils__namespace.types.isDate(element)) {
        bigintStats[key] = new Date(element);
      }
    }
  }
  bigintStats.atimeNs = bigintStats.atimeMs * BigInt(1e6);
  bigintStats.mtimeNs = bigintStats.mtimeMs * BigInt(1e6);
  bigintStats.ctimeNs = bigintStats.ctimeMs * BigInt(1e6);
  bigintStats.birthtimeNs = bigintStats.birthtimeMs * BigInt(1e6);
  return bigintStats;
}
function areStatsEqual(a, b) {
  if (a.atimeMs !== b.atimeMs)
    return false;
  if (a.birthtimeMs !== b.birthtimeMs)
    return false;
  if (a.blksize !== b.blksize)
    return false;
  if (a.blocks !== b.blocks)
    return false;
  if (a.ctimeMs !== b.ctimeMs)
    return false;
  if (a.dev !== b.dev)
    return false;
  if (a.gid !== b.gid)
    return false;
  if (a.ino !== b.ino)
    return false;
  if (a.isBlockDevice() !== b.isBlockDevice())
    return false;
  if (a.isCharacterDevice() !== b.isCharacterDevice())
    return false;
  if (a.isDirectory() !== b.isDirectory())
    return false;
  if (a.isFIFO() !== b.isFIFO())
    return false;
  if (a.isFile() !== b.isFile())
    return false;
  if (a.isSocket() !== b.isSocket())
    return false;
  if (a.isSymbolicLink() !== b.isSymbolicLink())
    return false;
  if (a.mode !== b.mode)
    return false;
  if (a.mtimeMs !== b.mtimeMs)
    return false;
  if (a.nlink !== b.nlink)
    return false;
  if (a.rdev !== b.rdev)
    return false;
  if (a.size !== b.size)
    return false;
  if (a.uid !== b.uid)
    return false;
  const aN = a;
  const bN = b;
  if (aN.atimeNs !== bN.atimeNs)
    return false;
  if (aN.mtimeNs !== bN.mtimeNs)
    return false;
  if (aN.ctimeNs !== bN.ctimeNs)
    return false;
  if (aN.birthtimeNs !== bN.birthtimeNs)
    return false;
  return true;
}

const PortablePath = {
  root: `/`,
  dot: `.`,
  parent: `..`
};
const Filename = {
  nodeModules: `node_modules`,
  manifest: `package.json`,
  lockfile: `yarn.lock`,
  virtual: `__virtual__`,
  pnpJs: `.pnp.js`,
  pnpCjs: `.pnp.cjs`,
  rc: `.yarnrc.yml`
};
const npath = Object.create(path__default.default);
const ppath = Object.create(path__default.default.posix);
npath.cwd = () => process.cwd();
ppath.cwd = () => toPortablePath(process.cwd());
ppath.resolve = (...segments) => {
  if (segments.length > 0 && ppath.isAbsolute(segments[0])) {
    return path__default.default.posix.resolve(...segments);
  } else {
    return path__default.default.posix.resolve(ppath.cwd(), ...segments);
  }
};
const contains = function(pathUtils, from, to) {
  from = pathUtils.normalize(from);
  to = pathUtils.normalize(to);
  if (from === to)
    return `.`;
  if (!from.endsWith(pathUtils.sep))
    from = from + pathUtils.sep;
  if (to.startsWith(from)) {
    return to.slice(from.length);
  } else {
    return null;
  }
};
npath.fromPortablePath = fromPortablePath;
npath.toPortablePath = toPortablePath;
npath.contains = (from, to) => contains(npath, from, to);
ppath.contains = (from, to) => contains(ppath, from, to);
const WINDOWS_PATH_REGEXP = /^([a-zA-Z]:.*)$/;
const UNC_WINDOWS_PATH_REGEXP = /^\/\/(\.\/)?(.*)$/;
const PORTABLE_PATH_REGEXP = /^\/([a-zA-Z]:.*)$/;
const UNC_PORTABLE_PATH_REGEXP = /^\/unc\/(\.dot\/)?(.*)$/;
function fromPortablePath(p) {
  if (process.platform !== `win32`)
    return p;
  let portablePathMatch, uncPortablePathMatch;
  if (portablePathMatch = p.match(PORTABLE_PATH_REGEXP))
    p = portablePathMatch[1];
  else if (uncPortablePathMatch = p.match(UNC_PORTABLE_PATH_REGEXP))
    p = `\\\\${uncPortablePathMatch[1] ? `.\\` : ``}${uncPortablePathMatch[2]}`;
  else
    return p;
  return p.replace(/\//g, `\\`);
}
function toPortablePath(p) {
  if (process.platform !== `win32`)
    return p;
  p = p.replace(/\\/g, `/`);
  let windowsPathMatch, uncWindowsPathMatch;
  if (windowsPathMatch = p.match(WINDOWS_PATH_REGEXP))
    p = `/${windowsPathMatch[1]}`;
  else if (uncWindowsPathMatch = p.match(UNC_WINDOWS_PATH_REGEXP))
    p = `/unc/${uncWindowsPathMatch[1] ? `.dot/` : ``}${uncWindowsPathMatch[2]}`;
  return p;
}
function convertPath(targetPathUtils, sourcePath) {
  return targetPathUtils === npath ? fromPortablePath(sourcePath) : toPortablePath(sourcePath);
}

const defaultTime = new Date(SAFE_TIME * 1e3);
async function copyPromise(destinationFs, destination, sourceFs, source, opts) {
  const normalizedDestination = destinationFs.pathUtils.normalize(destination);
  const normalizedSource = sourceFs.pathUtils.normalize(source);
  const prelayout = [];
  const postlayout = [];
  const { atime, mtime } = opts.stableTime ? { atime: defaultTime, mtime: defaultTime } : await sourceFs.lstatPromise(normalizedSource);
  await destinationFs.mkdirpPromise(destinationFs.pathUtils.dirname(destination), { utimes: [atime, mtime] });
  const updateTime = typeof destinationFs.lutimesPromise === `function` ? destinationFs.lutimesPromise.bind(destinationFs) : destinationFs.utimesPromise.bind(destinationFs);
  await copyImpl(prelayout, postlayout, updateTime, destinationFs, normalizedDestination, sourceFs, normalizedSource, { ...opts, didParentExist: true });
  for (const operation of prelayout)
    await operation();
  await Promise.all(postlayout.map((operation) => {
    return operation();
  }));
}
async function copyImpl(prelayout, postlayout, updateTime, destinationFs, destination, sourceFs, source, opts) {
  var _a, _b;
  const destinationStat = opts.didParentExist ? await maybeLStat(destinationFs, destination) : null;
  const sourceStat = await sourceFs.lstatPromise(source);
  const { atime, mtime } = opts.stableTime ? { atime: defaultTime, mtime: defaultTime } : sourceStat;
  let updated;
  switch (true) {
    case sourceStat.isDirectory():
      {
        updated = await copyFolder(prelayout, postlayout, updateTime, destinationFs, destination, destinationStat, sourceFs, source, sourceStat, opts);
      }
      break;
    case sourceStat.isFile():
      {
        updated = await copyFile(prelayout, postlayout, updateTime, destinationFs, destination, destinationStat, sourceFs, source, sourceStat, opts);
      }
      break;
    case sourceStat.isSymbolicLink():
      {
        updated = await copySymlink(prelayout, postlayout, updateTime, destinationFs, destination, destinationStat, sourceFs, source, sourceStat, opts);
      }
      break;
    default:
      {
        throw new Error(`Unsupported file type (${sourceStat.mode})`);
      }
  }
  if (updated || ((_a = destinationStat == null ? void 0 : destinationStat.mtime) == null ? void 0 : _a.getTime()) !== mtime.getTime() || ((_b = destinationStat == null ? void 0 : destinationStat.atime) == null ? void 0 : _b.getTime()) !== atime.getTime()) {
    postlayout.push(() => updateTime(destination, atime, mtime));
    updated = true;
  }
  if (destinationStat === null || (destinationStat.mode & 511) !== (sourceStat.mode & 511)) {
    postlayout.push(() => destinationFs.chmodPromise(destination, sourceStat.mode & 511));
    updated = true;
  }
  return updated;
}
async function maybeLStat(baseFs, p) {
  try {
    return await baseFs.lstatPromise(p);
  } catch (e) {
    return null;
  }
}
async function copyFolder(prelayout, postlayout, updateTime, destinationFs, destination, destinationStat, sourceFs, source, sourceStat, opts) {
  if (destinationStat !== null && !destinationStat.isDirectory()) {
    if (opts.overwrite) {
      prelayout.push(async () => destinationFs.removePromise(destination));
      destinationStat = null;
    } else {
      return false;
    }
  }
  let updated = false;
  if (destinationStat === null) {
    prelayout.push(async () => {
      try {
        await destinationFs.mkdirPromise(destination, { mode: sourceStat.mode });
      } catch (err) {
        if (err.code !== `EEXIST`) {
          throw err;
        }
      }
    });
    updated = true;
  }
  const entries = await sourceFs.readdirPromise(source);
  const nextOpts = opts.didParentExist && !destinationStat ? { ...opts, didParentExist: false } : opts;
  if (opts.stableSort) {
    for (const entry of entries.sort()) {
      if (await copyImpl(prelayout, postlayout, updateTime, destinationFs, destinationFs.pathUtils.join(destination, entry), sourceFs, sourceFs.pathUtils.join(source, entry), nextOpts)) {
        updated = true;
      }
    }
  } else {
    const entriesUpdateStatus = await Promise.all(entries.map(async (entry) => {
      await copyImpl(prelayout, postlayout, updateTime, destinationFs, destinationFs.pathUtils.join(destination, entry), sourceFs, sourceFs.pathUtils.join(source, entry), nextOpts);
    }));
    if (entriesUpdateStatus.some((status) => status)) {
      updated = true;
    }
  }
  return updated;
}
const isCloneSupportedCache = /* @__PURE__ */ new WeakMap();
function makeLinkOperation(opFs, destination, source, sourceStat, linkStrategy) {
  return async () => {
    await opFs.linkPromise(source, destination);
    if (linkStrategy === "readOnly" /* ReadOnly */) {
      sourceStat.mode &= ~146;
      await opFs.chmodPromise(destination, sourceStat.mode);
    }
  };
}
function makeCloneLinkOperation(opFs, destination, source, sourceStat, linkStrategy) {
  const isCloneSupported = isCloneSupportedCache.get(opFs);
  if (typeof isCloneSupported === `undefined`) {
    return async () => {
      try {
        await opFs.copyFilePromise(source, destination, fs__default.default.constants.COPYFILE_FICLONE_FORCE);
        isCloneSupportedCache.set(opFs, true);
      } catch (err) {
        if (err.code === `ENOSYS` || err.code === `ENOTSUP`) {
          isCloneSupportedCache.set(opFs, false);
          await makeLinkOperation(opFs, destination, source, sourceStat, linkStrategy)();
        } else {
          throw err;
        }
      }
    };
  } else {
    if (isCloneSupported) {
      return async () => opFs.copyFilePromise(source, destination, fs__default.default.constants.COPYFILE_FICLONE_FORCE);
    } else {
      return makeLinkOperation(opFs, destination, source, sourceStat, linkStrategy);
    }
  }
}
async function copyFile(prelayout, postlayout, updateTime, destinationFs, destination, destinationStat, sourceFs, source, sourceStat, opts) {
  var _a;
  if (destinationStat !== null) {
    if (opts.overwrite) {
      prelayout.push(async () => destinationFs.removePromise(destination));
      destinationStat = null;
    } else {
      return false;
    }
  }
  const linkStrategy = (_a = opts.linkStrategy) != null ? _a : null;
  const op = destinationFs === sourceFs ? linkStrategy !== null ? makeCloneLinkOperation(destinationFs, destination, source, sourceStat, linkStrategy) : async () => destinationFs.copyFilePromise(source, destination, fs__default.default.constants.COPYFILE_FICLONE) : linkStrategy !== null ? makeLinkOperation(destinationFs, destination, source, sourceStat, linkStrategy) : async () => destinationFs.writeFilePromise(destination, await sourceFs.readFilePromise(source));
  prelayout.push(async () => op());
  return true;
}
async function copySymlink(prelayout, postlayout, updateTime, destinationFs, destination, destinationStat, sourceFs, source, sourceStat, opts) {
  if (destinationStat !== null) {
    if (opts.overwrite) {
      prelayout.push(async () => destinationFs.removePromise(destination));
      destinationStat = null;
    } else {
      return false;
    }
  }
  prelayout.push(async () => {
    await destinationFs.symlinkPromise(convertPath(destinationFs.pathUtils, await sourceFs.readlinkPromise(source)), destination);
  });
  return true;
}

function makeError$1(code, message) {
  return Object.assign(new Error(`${code}: ${message}`), { code });
}
function EBUSY(message) {
  return makeError$1(`EBUSY`, message);
}
function ENOSYS(message, reason) {
  return makeError$1(`ENOSYS`, `${message}, ${reason}`);
}
function EINVAL(reason) {
  return makeError$1(`EINVAL`, `invalid argument, ${reason}`);
}
function EBADF(reason) {
  return makeError$1(`EBADF`, `bad file descriptor, ${reason}`);
}
function ENOENT(reason) {
  return makeError$1(`ENOENT`, `no such file or directory, ${reason}`);
}
function ENOTDIR(reason) {
  return makeError$1(`ENOTDIR`, `not a directory, ${reason}`);
}
function EISDIR(reason) {
  return makeError$1(`EISDIR`, `illegal operation on a directory, ${reason}`);
}
function EEXIST(reason) {
  return makeError$1(`EEXIST`, `file already exists, ${reason}`);
}
function EROFS(reason) {
  return makeError$1(`EROFS`, `read-only filesystem, ${reason}`);
}
function ENOTEMPTY(reason) {
  return makeError$1(`ENOTEMPTY`, `directory not empty, ${reason}`);
}
function EOPNOTSUPP(reason) {
  return makeError$1(`EOPNOTSUPP`, `operation not supported, ${reason}`);
}
function ERR_DIR_CLOSED() {
  return makeError$1(`ERR_DIR_CLOSED`, `Directory handle was closed`);
}
class LibzipError extends Error {
  constructor(message, code) {
    super(message);
    this.name = `Libzip Error`;
    this.code = code;
  }
}

class CustomDir {
  constructor(path, nextDirent, opts = {}) {
    this.path = path;
    this.nextDirent = nextDirent;
    this.opts = opts;
    this.closed = false;
  }
  throwIfClosed() {
    if (this.closed) {
      throw ERR_DIR_CLOSED();
    }
  }
  async *[Symbol.asyncIterator]() {
    try {
      let dirent;
      while ((dirent = await this.read()) !== null) {
        yield dirent;
      }
    } finally {
      await this.close();
    }
  }
  read(cb) {
    const dirent = this.readSync();
    if (typeof cb !== `undefined`)
      return cb(null, dirent);
    return Promise.resolve(dirent);
  }
  readSync() {
    this.throwIfClosed();
    return this.nextDirent();
  }
  close(cb) {
    this.closeSync();
    if (typeof cb !== `undefined`)
      return cb(null);
    return Promise.resolve();
  }
  closeSync() {
    var _a, _b;
    this.throwIfClosed();
    (_b = (_a = this.opts).onClose) == null ? void 0 : _b.call(_a);
    this.closed = true;
  }
}
function opendir(fakeFs, path, entries, opts) {
  const nextDirent = () => {
    const filename = entries.shift();
    if (typeof filename === `undefined`)
      return null;
    return Object.assign(fakeFs.statSync(fakeFs.pathUtils.join(path, filename)), {
      name: filename
    });
  };
  return new CustomDir(path, nextDirent, opts);
}

class FakeFS {
  constructor(pathUtils) {
    this.pathUtils = pathUtils;
  }
  async *genTraversePromise(init, { stableSort = false } = {}) {
    const stack = [init];
    while (stack.length > 0) {
      const p = stack.shift();
      const entry = await this.lstatPromise(p);
      if (entry.isDirectory()) {
        const entries = await this.readdirPromise(p);
        if (stableSort) {
          for (const entry2 of entries.sort()) {
            stack.push(this.pathUtils.join(p, entry2));
          }
        } else {
          throw new Error(`Not supported`);
        }
      } else {
        yield p;
      }
    }
  }
  async removePromise(p, { recursive = true, maxRetries = 5 } = {}) {
    let stat;
    try {
      stat = await this.lstatPromise(p);
    } catch (error) {
      if (error.code === `ENOENT`) {
        return;
      } else {
        throw error;
      }
    }
    if (stat.isDirectory()) {
      if (recursive) {
        const entries = await this.readdirPromise(p);
        await Promise.all(entries.map((entry) => {
          return this.removePromise(this.pathUtils.resolve(p, entry));
        }));
      }
      for (let t = 0; t <= maxRetries; t++) {
        try {
          await this.rmdirPromise(p);
          break;
        } catch (error) {
          if (error.code !== `EBUSY` && error.code !== `ENOTEMPTY`) {
            throw error;
          } else if (t < maxRetries) {
            await new Promise((resolve) => setTimeout(resolve, t * 100));
          }
        }
      }
    } else {
      await this.unlinkPromise(p);
    }
  }
  removeSync(p, { recursive = true } = {}) {
    let stat;
    try {
      stat = this.lstatSync(p);
    } catch (error) {
      if (error.code === `ENOENT`) {
        return;
      } else {
        throw error;
      }
    }
    if (stat.isDirectory()) {
      if (recursive)
        for (const entry of this.readdirSync(p))
          this.removeSync(this.pathUtils.resolve(p, entry));
      this.rmdirSync(p);
    } else {
      this.unlinkSync(p);
    }
  }
  async mkdirpPromise(p, { chmod, utimes } = {}) {
    p = this.resolve(p);
    if (p === this.pathUtils.dirname(p))
      return void 0;
    const parts = p.split(this.pathUtils.sep);
    let createdDirectory;
    for (let u = 2; u <= parts.length; ++u) {
      const subPath = parts.slice(0, u).join(this.pathUtils.sep);
      if (!this.existsSync(subPath)) {
        try {
          await this.mkdirPromise(subPath);
        } catch (error) {
          if (error.code === `EEXIST`) {
            continue;
          } else {
            throw error;
          }
        }
        createdDirectory != null ? createdDirectory : createdDirectory = subPath;
        if (chmod != null)
          await this.chmodPromise(subPath, chmod);
        if (utimes != null) {
          await this.utimesPromise(subPath, utimes[0], utimes[1]);
        } else {
          const parentStat = await this.statPromise(this.pathUtils.dirname(subPath));
          await this.utimesPromise(subPath, parentStat.atime, parentStat.mtime);
        }
      }
    }
    return createdDirectory;
  }
  mkdirpSync(p, { chmod, utimes } = {}) {
    p = this.resolve(p);
    if (p === this.pathUtils.dirname(p))
      return void 0;
    const parts = p.split(this.pathUtils.sep);
    let createdDirectory;
    for (let u = 2; u <= parts.length; ++u) {
      const subPath = parts.slice(0, u).join(this.pathUtils.sep);
      if (!this.existsSync(subPath)) {
        try {
          this.mkdirSync(subPath);
        } catch (error) {
          if (error.code === `EEXIST`) {
            continue;
          } else {
            throw error;
          }
        }
        createdDirectory != null ? createdDirectory : createdDirectory = subPath;
        if (chmod != null)
          this.chmodSync(subPath, chmod);
        if (utimes != null) {
          this.utimesSync(subPath, utimes[0], utimes[1]);
        } else {
          const parentStat = this.statSync(this.pathUtils.dirname(subPath));
          this.utimesSync(subPath, parentStat.atime, parentStat.mtime);
        }
      }
    }
    return createdDirectory;
  }
  async copyPromise(destination, source, { baseFs = this, overwrite = true, stableSort = false, stableTime = false, linkStrategy = null } = {}) {
    return await copyPromise(this, destination, baseFs, source, { overwrite, stableSort, stableTime, linkStrategy });
  }
  copySync(destination, source, { baseFs = this, overwrite = true } = {}) {
    const stat = baseFs.lstatSync(source);
    const exists = this.existsSync(destination);
    if (stat.isDirectory()) {
      this.mkdirpSync(destination);
      const directoryListing = baseFs.readdirSync(source);
      for (const entry of directoryListing) {
        this.copySync(this.pathUtils.join(destination, entry), baseFs.pathUtils.join(source, entry), { baseFs, overwrite });
      }
    } else if (stat.isFile()) {
      if (!exists || overwrite) {
        if (exists)
          this.removeSync(destination);
        const content = baseFs.readFileSync(source);
        this.writeFileSync(destination, content);
      }
    } else if (stat.isSymbolicLink()) {
      if (!exists || overwrite) {
        if (exists)
          this.removeSync(destination);
        const target = baseFs.readlinkSync(source);
        this.symlinkSync(convertPath(this.pathUtils, target), destination);
      }
    } else {
      throw new Error(`Unsupported file type (file: ${source}, mode: 0o${stat.mode.toString(8).padStart(6, `0`)})`);
    }
    const mode = stat.mode & 511;
    this.chmodSync(destination, mode);
  }
  async changeFilePromise(p, content, opts = {}) {
    if (Buffer.isBuffer(content)) {
      return this.changeFileBufferPromise(p, content, opts);
    } else {
      return this.changeFileTextPromise(p, content, opts);
    }
  }
  async changeFileBufferPromise(p, content, { mode } = {}) {
    let current = Buffer.alloc(0);
    try {
      current = await this.readFilePromise(p);
    } catch (error) {
    }
    if (Buffer.compare(current, content) === 0)
      return;
    await this.writeFilePromise(p, content, { mode });
  }
  async changeFileTextPromise(p, content, { automaticNewlines, mode } = {}) {
    let current = ``;
    try {
      current = await this.readFilePromise(p, `utf8`);
    } catch (error) {
    }
    const normalizedContent = automaticNewlines ? normalizeLineEndings(current, content) : content;
    if (current === normalizedContent)
      return;
    await this.writeFilePromise(p, normalizedContent, { mode });
  }
  changeFileSync(p, content, opts = {}) {
    if (Buffer.isBuffer(content)) {
      return this.changeFileBufferSync(p, content, opts);
    } else {
      return this.changeFileTextSync(p, content, opts);
    }
  }
  changeFileBufferSync(p, content, { mode } = {}) {
    let current = Buffer.alloc(0);
    try {
      current = this.readFileSync(p);
    } catch (error) {
    }
    if (Buffer.compare(current, content) === 0)
      return;
    this.writeFileSync(p, content, { mode });
  }
  changeFileTextSync(p, content, { automaticNewlines = false, mode } = {}) {
    let current = ``;
    try {
      current = this.readFileSync(p, `utf8`);
    } catch (error) {
    }
    const normalizedContent = automaticNewlines ? normalizeLineEndings(current, content) : content;
    if (current === normalizedContent)
      return;
    this.writeFileSync(p, normalizedContent, { mode });
  }
  async movePromise(fromP, toP) {
    try {
      await this.renamePromise(fromP, toP);
    } catch (error) {
      if (error.code === `EXDEV`) {
        await this.copyPromise(toP, fromP);
        await this.removePromise(fromP);
      } else {
        throw error;
      }
    }
  }
  moveSync(fromP, toP) {
    try {
      this.renameSync(fromP, toP);
    } catch (error) {
      if (error.code === `EXDEV`) {
        this.copySync(toP, fromP);
        this.removeSync(fromP);
      } else {
        throw error;
      }
    }
  }
  async lockPromise(affectedPath, callback) {
    const lockPath = `${affectedPath}.flock`;
    const interval = 1e3 / 60;
    const startTime = Date.now();
    let fd = null;
    const isAlive = async () => {
      let pid;
      try {
        [pid] = await this.readJsonPromise(lockPath);
      } catch (error) {
        return Date.now() - startTime < 500;
      }
      try {
        process.kill(pid, 0);
        return true;
      } catch (error) {
        return false;
      }
    };
    while (fd === null) {
      try {
        fd = await this.openPromise(lockPath, `wx`);
      } catch (error) {
        if (error.code === `EEXIST`) {
          if (!await isAlive()) {
            try {
              await this.unlinkPromise(lockPath);
              continue;
            } catch (error2) {
            }
          }
          if (Date.now() - startTime < 60 * 1e3) {
            await new Promise((resolve) => setTimeout(resolve, interval));
          } else {
            throw new Error(`Couldn't acquire a lock in a reasonable time (via ${lockPath})`);
          }
        } else {
          throw error;
        }
      }
    }
    await this.writePromise(fd, JSON.stringify([process.pid]));
    try {
      return await callback();
    } finally {
      try {
        await this.closePromise(fd);
        await this.unlinkPromise(lockPath);
      } catch (error) {
      }
    }
  }
  async readJsonPromise(p) {
    const content = await this.readFilePromise(p, `utf8`);
    try {
      return JSON.parse(content);
    } catch (error) {
      error.message += ` (in ${p})`;
      throw error;
    }
  }
  readJsonSync(p) {
    const content = this.readFileSync(p, `utf8`);
    try {
      return JSON.parse(content);
    } catch (error) {
      error.message += ` (in ${p})`;
      throw error;
    }
  }
  async writeJsonPromise(p, data) {
    return await this.writeFilePromise(p, `${JSON.stringify(data, null, 2)}
`);
  }
  writeJsonSync(p, data) {
    return this.writeFileSync(p, `${JSON.stringify(data, null, 2)}
`);
  }
  async preserveTimePromise(p, cb) {
    const stat = await this.lstatPromise(p);
    const result = await cb();
    if (typeof result !== `undefined`)
      p = result;
    if (this.lutimesPromise) {
      await this.lutimesPromise(p, stat.atime, stat.mtime);
    } else if (!stat.isSymbolicLink()) {
      await this.utimesPromise(p, stat.atime, stat.mtime);
    }
  }
  async preserveTimeSync(p, cb) {
    const stat = this.lstatSync(p);
    const result = cb();
    if (typeof result !== `undefined`)
      p = result;
    if (this.lutimesSync) {
      this.lutimesSync(p, stat.atime, stat.mtime);
    } else if (!stat.isSymbolicLink()) {
      this.utimesSync(p, stat.atime, stat.mtime);
    }
  }
}
class BasePortableFakeFS extends FakeFS {
  constructor() {
    super(ppath);
  }
}
function getEndOfLine(content) {
  const matches = content.match(/\r?\n/g);
  if (matches === null)
    return os.EOL;
  const crlf = matches.filter((nl) => nl === `\r
`).length;
  const lf = matches.length - crlf;
  return crlf > lf ? `\r
` : `
`;
}
function normalizeLineEndings(originalContent, newContent) {
  return newContent.replace(/\r?\n/g, getEndOfLine(originalContent));
}

class NodeFS extends BasePortableFakeFS {
  constructor(realFs = fs__default.default) {
    super();
    this.realFs = realFs;
    if (typeof this.realFs.lutimes !== `undefined`) {
      this.lutimesPromise = this.lutimesPromiseImpl;
      this.lutimesSync = this.lutimesSyncImpl;
    }
  }
  getExtractHint() {
    return false;
  }
  getRealPath() {
    return PortablePath.root;
  }
  resolve(p) {
    return ppath.resolve(p);
  }
  async openPromise(p, flags, mode) {
    return await new Promise((resolve, reject) => {
      this.realFs.open(npath.fromPortablePath(p), flags, mode, this.makeCallback(resolve, reject));
    });
  }
  openSync(p, flags, mode) {
    return this.realFs.openSync(npath.fromPortablePath(p), flags, mode);
  }
  async opendirPromise(p, opts) {
    return await new Promise((resolve, reject) => {
      if (typeof opts !== `undefined`) {
        this.realFs.opendir(npath.fromPortablePath(p), opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.opendir(npath.fromPortablePath(p), this.makeCallback(resolve, reject));
      }
    }).then((dir) => {
      return Object.defineProperty(dir, `path`, { value: p, configurable: true, writable: true });
    });
  }
  opendirSync(p, opts) {
    const dir = typeof opts !== `undefined` ? this.realFs.opendirSync(npath.fromPortablePath(p), opts) : this.realFs.opendirSync(npath.fromPortablePath(p));
    return Object.defineProperty(dir, `path`, { value: p, configurable: true, writable: true });
  }
  async readPromise(fd, buffer, offset = 0, length = 0, position = -1) {
    return await new Promise((resolve, reject) => {
      this.realFs.read(fd, buffer, offset, length, position, (error, bytesRead) => {
        if (error) {
          reject(error);
        } else {
          resolve(bytesRead);
        }
      });
    });
  }
  readSync(fd, buffer, offset, length, position) {
    return this.realFs.readSync(fd, buffer, offset, length, position);
  }
  async writePromise(fd, buffer, offset, length, position) {
    return await new Promise((resolve, reject) => {
      if (typeof buffer === `string`) {
        return this.realFs.write(fd, buffer, offset, this.makeCallback(resolve, reject));
      } else {
        return this.realFs.write(fd, buffer, offset, length, position, this.makeCallback(resolve, reject));
      }
    });
  }
  writeSync(fd, buffer, offset, length, position) {
    if (typeof buffer === `string`) {
      return this.realFs.writeSync(fd, buffer, offset);
    } else {
      return this.realFs.writeSync(fd, buffer, offset, length, position);
    }
  }
  async closePromise(fd) {
    await new Promise((resolve, reject) => {
      this.realFs.close(fd, this.makeCallback(resolve, reject));
    });
  }
  closeSync(fd) {
    this.realFs.closeSync(fd);
  }
  createReadStream(p, opts) {
    const realPath = p !== null ? npath.fromPortablePath(p) : p;
    return this.realFs.createReadStream(realPath, opts);
  }
  createWriteStream(p, opts) {
    const realPath = p !== null ? npath.fromPortablePath(p) : p;
    return this.realFs.createWriteStream(realPath, opts);
  }
  async realpathPromise(p) {
    return await new Promise((resolve, reject) => {
      this.realFs.realpath(npath.fromPortablePath(p), {}, this.makeCallback(resolve, reject));
    }).then((path) => {
      return npath.toPortablePath(path);
    });
  }
  realpathSync(p) {
    return npath.toPortablePath(this.realFs.realpathSync(npath.fromPortablePath(p), {}));
  }
  async existsPromise(p) {
    return await new Promise((resolve) => {
      this.realFs.exists(npath.fromPortablePath(p), resolve);
    });
  }
  accessSync(p, mode) {
    return this.realFs.accessSync(npath.fromPortablePath(p), mode);
  }
  async accessPromise(p, mode) {
    return await new Promise((resolve, reject) => {
      this.realFs.access(npath.fromPortablePath(p), mode, this.makeCallback(resolve, reject));
    });
  }
  existsSync(p) {
    return this.realFs.existsSync(npath.fromPortablePath(p));
  }
  async statPromise(p, opts) {
    return await new Promise((resolve, reject) => {
      if (opts) {
        this.realFs.stat(npath.fromPortablePath(p), opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.stat(npath.fromPortablePath(p), this.makeCallback(resolve, reject));
      }
    });
  }
  statSync(p, opts) {
    if (opts) {
      return this.realFs.statSync(npath.fromPortablePath(p), opts);
    } else {
      return this.realFs.statSync(npath.fromPortablePath(p));
    }
  }
  async fstatPromise(fd, opts) {
    return await new Promise((resolve, reject) => {
      if (opts) {
        this.realFs.fstat(fd, opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.fstat(fd, this.makeCallback(resolve, reject));
      }
    });
  }
  fstatSync(fd, opts) {
    if (opts) {
      return this.realFs.fstatSync(fd, opts);
    } else {
      return this.realFs.fstatSync(fd);
    }
  }
  async lstatPromise(p, opts) {
    return await new Promise((resolve, reject) => {
      if (opts) {
        this.realFs.lstat(npath.fromPortablePath(p), opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.lstat(npath.fromPortablePath(p), this.makeCallback(resolve, reject));
      }
    });
  }
  lstatSync(p, opts) {
    if (opts) {
      return this.realFs.lstatSync(npath.fromPortablePath(p), opts);
    } else {
      return this.realFs.lstatSync(npath.fromPortablePath(p));
    }
  }
  async fchmodPromise(fd, mask) {
    return await new Promise((resolve, reject) => {
      this.realFs.fchmod(fd, mask, this.makeCallback(resolve, reject));
    });
  }
  fchmodSync(fd, mask) {
    return this.realFs.fchmodSync(fd, mask);
  }
  async chmodPromise(p, mask) {
    return await new Promise((resolve, reject) => {
      this.realFs.chmod(npath.fromPortablePath(p), mask, this.makeCallback(resolve, reject));
    });
  }
  chmodSync(p, mask) {
    return this.realFs.chmodSync(npath.fromPortablePath(p), mask);
  }
  async fchownPromise(fd, uid, gid) {
    return await new Promise((resolve, reject) => {
      this.realFs.fchown(fd, uid, gid, this.makeCallback(resolve, reject));
    });
  }
  fchownSync(fd, uid, gid) {
    return this.realFs.fchownSync(fd, uid, gid);
  }
  async chownPromise(p, uid, gid) {
    return await new Promise((resolve, reject) => {
      this.realFs.chown(npath.fromPortablePath(p), uid, gid, this.makeCallback(resolve, reject));
    });
  }
  chownSync(p, uid, gid) {
    return this.realFs.chownSync(npath.fromPortablePath(p), uid, gid);
  }
  async renamePromise(oldP, newP) {
    return await new Promise((resolve, reject) => {
      this.realFs.rename(npath.fromPortablePath(oldP), npath.fromPortablePath(newP), this.makeCallback(resolve, reject));
    });
  }
  renameSync(oldP, newP) {
    return this.realFs.renameSync(npath.fromPortablePath(oldP), npath.fromPortablePath(newP));
  }
  async copyFilePromise(sourceP, destP, flags = 0) {
    return await new Promise((resolve, reject) => {
      this.realFs.copyFile(npath.fromPortablePath(sourceP), npath.fromPortablePath(destP), flags, this.makeCallback(resolve, reject));
    });
  }
  copyFileSync(sourceP, destP, flags = 0) {
    return this.realFs.copyFileSync(npath.fromPortablePath(sourceP), npath.fromPortablePath(destP), flags);
  }
  async appendFilePromise(p, content, opts) {
    return await new Promise((resolve, reject) => {
      const fsNativePath = typeof p === `string` ? npath.fromPortablePath(p) : p;
      if (opts) {
        this.realFs.appendFile(fsNativePath, content, opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.appendFile(fsNativePath, content, this.makeCallback(resolve, reject));
      }
    });
  }
  appendFileSync(p, content, opts) {
    const fsNativePath = typeof p === `string` ? npath.fromPortablePath(p) : p;
    if (opts) {
      this.realFs.appendFileSync(fsNativePath, content, opts);
    } else {
      this.realFs.appendFileSync(fsNativePath, content);
    }
  }
  async writeFilePromise(p, content, opts) {
    return await new Promise((resolve, reject) => {
      const fsNativePath = typeof p === `string` ? npath.fromPortablePath(p) : p;
      if (opts) {
        this.realFs.writeFile(fsNativePath, content, opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.writeFile(fsNativePath, content, this.makeCallback(resolve, reject));
      }
    });
  }
  writeFileSync(p, content, opts) {
    const fsNativePath = typeof p === `string` ? npath.fromPortablePath(p) : p;
    if (opts) {
      this.realFs.writeFileSync(fsNativePath, content, opts);
    } else {
      this.realFs.writeFileSync(fsNativePath, content);
    }
  }
  async unlinkPromise(p) {
    return await new Promise((resolve, reject) => {
      this.realFs.unlink(npath.fromPortablePath(p), this.makeCallback(resolve, reject));
    });
  }
  unlinkSync(p) {
    return this.realFs.unlinkSync(npath.fromPortablePath(p));
  }
  async utimesPromise(p, atime, mtime) {
    return await new Promise((resolve, reject) => {
      this.realFs.utimes(npath.fromPortablePath(p), atime, mtime, this.makeCallback(resolve, reject));
    });
  }
  utimesSync(p, atime, mtime) {
    this.realFs.utimesSync(npath.fromPortablePath(p), atime, mtime);
  }
  async lutimesPromiseImpl(p, atime, mtime) {
    const lutimes = this.realFs.lutimes;
    if (typeof lutimes === `undefined`)
      throw ENOSYS(`unavailable Node binding`, `lutimes '${p}'`);
    return await new Promise((resolve, reject) => {
      lutimes.call(this.realFs, npath.fromPortablePath(p), atime, mtime, this.makeCallback(resolve, reject));
    });
  }
  lutimesSyncImpl(p, atime, mtime) {
    const lutimesSync = this.realFs.lutimesSync;
    if (typeof lutimesSync === `undefined`)
      throw ENOSYS(`unavailable Node binding`, `lutimes '${p}'`);
    lutimesSync.call(this.realFs, npath.fromPortablePath(p), atime, mtime);
  }
  async mkdirPromise(p, opts) {
    return await new Promise((resolve, reject) => {
      this.realFs.mkdir(npath.fromPortablePath(p), opts, this.makeCallback(resolve, reject));
    });
  }
  mkdirSync(p, opts) {
    return this.realFs.mkdirSync(npath.fromPortablePath(p), opts);
  }
  async rmdirPromise(p, opts) {
    return await new Promise((resolve, reject) => {
      if (opts) {
        this.realFs.rmdir(npath.fromPortablePath(p), opts, this.makeCallback(resolve, reject));
      } else {
        this.realFs.rmdir(npath.fromPortablePath(p), this.makeCallback(resolve, reject));
      }
    });
  }
  rmdirSync(p, opts) {
    return this.realFs.rmdirSync(npath.fromPortablePath(p), opts);
  }
  async linkPromise(existingP, newP) {
    return await new Promise((resolve, reject) => {
      this.realFs.link(npath.fromPortablePath(existingP), npath.fromPortablePath(newP), this.makeCallback(resolve, reject));
    });
  }
  linkSync(existingP, newP) {
    return this.realFs.linkSync(npath.fromPortablePath(existingP), npath.fromPortablePath(newP));
  }
  async symlinkPromise(target, p, type) {
    return await new Promise((resolve, reject) => {
      this.realFs.symlink(npath.fromPortablePath(target.replace(/\/+$/, ``)), npath.fromPortablePath(p), type, this.makeCallback(resolve, reject));
    });
  }
  symlinkSync(target, p, type) {
    return this.realFs.symlinkSync(npath.fromPortablePath(target.replace(/\/+$/, ``)), npath.fromPortablePath(p), type);
  }
  async readFilePromise(p, encoding) {
    return await new Promise((resolve, reject) => {
      const fsNativePath = typeof p === `string` ? npath.fromPortablePath(p) : p;
      this.realFs.readFile(fsNativePath, encoding, this.makeCallback(resolve, reject));
    });
  }
  readFileSync(p, encoding) {
    const fsNativePath = typeof p === `string` ? npath.fromPortablePath(p) : p;
    return this.realFs.readFileSync(fsNativePath, encoding);
  }
  async readdirPromise(p, opts) {
    return await new Promise((resolve, reject) => {
      if (opts == null ? void 0 : opts.withFileTypes) {
        this.realFs.readdir(npath.fromPortablePath(p), { withFileTypes: true }, this.makeCallback(resolve, reject));
      } else {
        this.realFs.readdir(npath.fromPortablePath(p), this.makeCallback((value) => resolve(value), reject));
      }
    });
  }
  readdirSync(p, opts) {
    if (opts == null ? void 0 : opts.withFileTypes) {
      return this.realFs.readdirSync(npath.fromPortablePath(p), { withFileTypes: true });
    } else {
      return this.realFs.readdirSync(npath.fromPortablePath(p));
    }
  }
  async readlinkPromise(p) {
    return await new Promise((resolve, reject) => {
      this.realFs.readlink(npath.fromPortablePath(p), this.makeCallback(resolve, reject));
    }).then((path) => {
      return npath.toPortablePath(path);
    });
  }
  readlinkSync(p) {
    return npath.toPortablePath(this.realFs.readlinkSync(npath.fromPortablePath(p)));
  }
  async truncatePromise(p, len) {
    return await new Promise((resolve, reject) => {
      this.realFs.truncate(npath.fromPortablePath(p), len, this.makeCallback(resolve, reject));
    });
  }
  truncateSync(p, len) {
    return this.realFs.truncateSync(npath.fromPortablePath(p), len);
  }
  async ftruncatePromise(fd, len) {
    return await new Promise((resolve, reject) => {
      this.realFs.ftruncate(fd, len, this.makeCallback(resolve, reject));
    });
  }
  ftruncateSync(fd, len) {
    return this.realFs.ftruncateSync(fd, len);
  }
  watch(p, a, b) {
    return this.realFs.watch(
      npath.fromPortablePath(p),
      a,
      b
    );
  }
  watchFile(p, a, b) {
    return this.realFs.watchFile(
      npath.fromPortablePath(p),
      a,
      b
    );
  }
  unwatchFile(p, cb) {
    return this.realFs.unwatchFile(npath.fromPortablePath(p), cb);
  }
  makeCallback(resolve, reject) {
    return (err, result) => {
      if (err) {
        reject(err);
      } else {
        resolve(result);
      }
    };
  }
}

function assertStatus(current, expected) {
  if (current !== expected) {
    throw new Error(`Invalid StatWatcher status: expected '${expected}', got '${current}'`);
  }
}
class CustomStatWatcher extends events.EventEmitter {
  constructor(fakeFs, path, { bigint = false } = {}) {
    super();
    this.status = "ready" /* Ready */;
    this.changeListeners = /* @__PURE__ */ new Map();
    this.startTimeout = null;
    this.fakeFs = fakeFs;
    this.path = path;
    this.bigint = bigint;
    this.lastStats = this.stat();
  }
  static create(fakeFs, path, opts) {
    const statWatcher = new CustomStatWatcher(fakeFs, path, opts);
    statWatcher.start();
    return statWatcher;
  }
  start() {
    assertStatus(this.status, "ready" /* Ready */);
    this.status = "running" /* Running */;
    this.startTimeout = setTimeout(() => {
      this.startTimeout = null;
      if (!this.fakeFs.existsSync(this.path)) {
        this.emit("change" /* Change */, this.lastStats, this.lastStats);
      }
    }, 3);
  }
  stop() {
    assertStatus(this.status, "running" /* Running */);
    this.status = "stopped" /* Stopped */;
    if (this.startTimeout !== null) {
      clearTimeout(this.startTimeout);
      this.startTimeout = null;
    }
    this.emit("stop" /* Stop */);
  }
  stat() {
    try {
      return this.fakeFs.statSync(this.path, { bigint: this.bigint });
    } catch (error) {
      const statInstance = this.bigint ? new BigIntStatsEntry() : new StatEntry();
      return clearStats(statInstance);
    }
  }
  makeInterval(opts) {
    const interval = setInterval(() => {
      const currentStats = this.stat();
      const previousStats = this.lastStats;
      if (areStatsEqual(currentStats, previousStats))
        return;
      this.lastStats = currentStats;
      this.emit("change" /* Change */, currentStats, previousStats);
    }, opts.interval);
    return opts.persistent ? interval : interval.unref();
  }
  registerChangeListener(listener, opts) {
    this.addListener("change" /* Change */, listener);
    this.changeListeners.set(listener, this.makeInterval(opts));
  }
  unregisterChangeListener(listener) {
    this.removeListener("change" /* Change */, listener);
    const interval = this.changeListeners.get(listener);
    if (typeof interval !== `undefined`)
      clearInterval(interval);
    this.changeListeners.delete(listener);
  }
  unregisterAllChangeListeners() {
    for (const listener of this.changeListeners.keys()) {
      this.unregisterChangeListener(listener);
    }
  }
  hasChangeListeners() {
    return this.changeListeners.size > 0;
  }
  ref() {
    for (const interval of this.changeListeners.values())
      interval.ref();
    return this;
  }
  unref() {
    for (const interval of this.changeListeners.values())
      interval.unref();
    return this;
  }
}

const statWatchersByFakeFS = /* @__PURE__ */ new WeakMap();
function watchFile(fakeFs, path, a, b) {
  let bigint;
  let persistent;
  let interval;
  let listener;
  switch (typeof a) {
    case `function`:
      {
        bigint = false;
        persistent = true;
        interval = 5007;
        listener = a;
      }
      break;
    default:
      {
        ({
          bigint = false,
          persistent = true,
          interval = 5007
        } = a);
        listener = b;
      }
      break;
  }
  let statWatchers = statWatchersByFakeFS.get(fakeFs);
  if (typeof statWatchers === `undefined`)
    statWatchersByFakeFS.set(fakeFs, statWatchers = /* @__PURE__ */ new Map());
  let statWatcher = statWatchers.get(path);
  if (typeof statWatcher === `undefined`) {
    statWatcher = CustomStatWatcher.create(fakeFs, path, { bigint });
    statWatchers.set(path, statWatcher);
  }
  statWatcher.registerChangeListener(listener, { persistent, interval });
  return statWatcher;
}
function unwatchFile(fakeFs, path, cb) {
  const statWatchers = statWatchersByFakeFS.get(fakeFs);
  if (typeof statWatchers === `undefined`)
    return;
  const statWatcher = statWatchers.get(path);
  if (typeof statWatcher === `undefined`)
    return;
  if (typeof cb === `undefined`)
    statWatcher.unregisterAllChangeListeners();
  else
    statWatcher.unregisterChangeListener(cb);
  if (!statWatcher.hasChangeListeners()) {
    statWatcher.stop();
    statWatchers.delete(path);
  }
}
function unwatchAllFiles(fakeFs) {
  const statWatchers = statWatchersByFakeFS.get(fakeFs);
  if (typeof statWatchers === `undefined`)
    return;
  for (const path of statWatchers.keys()) {
    unwatchFile(fakeFs, path);
  }
}

const DEFAULT_COMPRESSION_LEVEL = `mixed`;
function toUnixTimestamp(time) {
  if (typeof time === `string` && String(+time) === time)
    return +time;
  if (typeof time === `number` && Number.isFinite(time)) {
    if (time < 0) {
      return Date.now() / 1e3;
    } else {
      return time;
    }
  }
  if (nodeUtils.types.isDate(time))
    return time.getTime() / 1e3;
  throw new Error(`Invalid time`);
}
function makeEmptyArchive() {
  return Buffer.from([
    80,
    75,
    5,
    6,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0
  ]);
}
class ZipFS extends BasePortableFakeFS {
  constructor(source, opts) {
    super();
    this.lzSource = null;
    this.listings = /* @__PURE__ */ new Map();
    this.entries = /* @__PURE__ */ new Map();
    this.fileSources = /* @__PURE__ */ new Map();
    this.fds = /* @__PURE__ */ new Map();
    this.nextFd = 0;
    this.ready = false;
    this.readOnly = false;
    this.libzip = opts.libzip;
    const pathOptions = opts;
    this.level = typeof pathOptions.level !== `undefined` ? pathOptions.level : DEFAULT_COMPRESSION_LEVEL;
    source != null ? source : source = makeEmptyArchive();
    if (typeof source === `string`) {
      const { baseFs = new NodeFS() } = pathOptions;
      this.baseFs = baseFs;
      this.path = source;
    } else {
      this.path = null;
      this.baseFs = null;
    }
    if (opts.stats) {
      this.stats = opts.stats;
    } else {
      if (typeof source === `string`) {
        try {
          this.stats = this.baseFs.statSync(source);
        } catch (error) {
          if (error.code === `ENOENT` && pathOptions.create) {
            this.stats = makeDefaultStats();
          } else {
            throw error;
          }
        }
      } else {
        this.stats = makeDefaultStats();
      }
    }
    const errPtr = this.libzip.malloc(4);
    try {
      let flags = 0;
      if (typeof source === `string` && pathOptions.create)
        flags |= this.libzip.ZIP_CREATE | this.libzip.ZIP_TRUNCATE;
      if (opts.readOnly) {
        flags |= this.libzip.ZIP_RDONLY;
        this.readOnly = true;
      }
      if (typeof source === `string`) {
        this.zip = this.libzip.open(npath.fromPortablePath(source), flags, errPtr);
      } else {
        const lzSource = this.allocateUnattachedSource(source);
        try {
          this.zip = this.libzip.openFromSource(lzSource, flags, errPtr);
          this.lzSource = lzSource;
        } catch (error) {
          this.libzip.source.free(lzSource);
          throw error;
        }
      }
      if (this.zip === 0) {
        const error = this.libzip.struct.errorS();
        this.libzip.error.initWithCode(error, this.libzip.getValue(errPtr, `i32`));
        throw this.makeLibzipError(error);
      }
    } finally {
      this.libzip.free(errPtr);
    }
    this.listings.set(PortablePath.root, /* @__PURE__ */ new Set());
    const entryCount = this.libzip.getNumEntries(this.zip, 0);
    for (let t = 0; t < entryCount; ++t) {
      const raw = this.libzip.getName(this.zip, t, 0);
      if (ppath.isAbsolute(raw))
        continue;
      const p = ppath.resolve(PortablePath.root, raw);
      this.registerEntry(p, t);
      if (raw.endsWith(`/`)) {
        this.registerListing(p);
      }
    }
    this.symlinkCount = this.libzip.ext.countSymlinks(this.zip);
    if (this.symlinkCount === -1)
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    this.ready = true;
  }
  makeLibzipError(error) {
    const errorCode = this.libzip.struct.errorCodeZip(error);
    const strerror = this.libzip.error.strerror(error);
    const libzipError = new LibzipError(strerror, this.libzip.errors[errorCode]);
    if (errorCode === this.libzip.errors.ZIP_ER_CHANGED)
      throw new Error(`Assertion failed: Unexpected libzip error: ${libzipError.message}`);
    return libzipError;
  }
  getExtractHint(hints) {
    for (const fileName of this.entries.keys()) {
      const ext = this.pathUtils.extname(fileName);
      if (hints.relevantExtensions.has(ext)) {
        return true;
      }
    }
    return false;
  }
  getAllFiles() {
    return Array.from(this.entries.keys());
  }
  getRealPath() {
    if (!this.path)
      throw new Error(`ZipFS don't have real paths when loaded from a buffer`);
    return this.path;
  }
  getBufferAndClose() {
    this.prepareClose();
    if (!this.lzSource)
      throw new Error(`ZipFS was not created from a Buffer`);
    if (this.entries.size === 0) {
      this.discardAndClose();
      return makeEmptyArchive();
    }
    try {
      this.libzip.source.keep(this.lzSource);
      if (this.libzip.close(this.zip) === -1)
        throw this.makeLibzipError(this.libzip.getError(this.zip));
      if (this.libzip.source.open(this.lzSource) === -1)
        throw this.makeLibzipError(this.libzip.source.error(this.lzSource));
      if (this.libzip.source.seek(this.lzSource, 0, 0, this.libzip.SEEK_END) === -1)
        throw this.makeLibzipError(this.libzip.source.error(this.lzSource));
      const size = this.libzip.source.tell(this.lzSource);
      if (size === -1)
        throw this.makeLibzipError(this.libzip.source.error(this.lzSource));
      if (this.libzip.source.seek(this.lzSource, 0, 0, this.libzip.SEEK_SET) === -1)
        throw this.makeLibzipError(this.libzip.source.error(this.lzSource));
      const buffer = this.libzip.malloc(size);
      if (!buffer)
        throw new Error(`Couldn't allocate enough memory`);
      try {
        const rc = this.libzip.source.read(this.lzSource, buffer, size);
        if (rc === -1)
          throw this.makeLibzipError(this.libzip.source.error(this.lzSource));
        else if (rc < size)
          throw new Error(`Incomplete read`);
        else if (rc > size)
          throw new Error(`Overread`);
        const memory = this.libzip.HEAPU8.subarray(buffer, buffer + size);
        return Buffer.from(memory);
      } finally {
        this.libzip.free(buffer);
      }
    } finally {
      this.libzip.source.close(this.lzSource);
      this.libzip.source.free(this.lzSource);
      this.ready = false;
    }
  }
  prepareClose() {
    if (!this.ready)
      throw EBUSY(`archive closed, close`);
    unwatchAllFiles(this);
  }
  saveAndClose() {
    if (!this.path || !this.baseFs)
      throw new Error(`ZipFS cannot be saved and must be discarded when loaded from a buffer`);
    this.prepareClose();
    if (this.readOnly) {
      this.discardAndClose();
      return;
    }
    const newMode = this.baseFs.existsSync(this.path) || this.stats.mode === DEFAULT_MODE ? void 0 : this.stats.mode;
    if (this.entries.size === 0) {
      this.discardAndClose();
      this.baseFs.writeFileSync(this.path, makeEmptyArchive(), { mode: newMode });
    } else {
      const rc = this.libzip.close(this.zip);
      if (rc === -1)
        throw this.makeLibzipError(this.libzip.getError(this.zip));
      if (typeof newMode !== `undefined`) {
        this.baseFs.chmodSync(this.path, newMode);
      }
    }
    this.ready = false;
  }
  discardAndClose() {
    this.prepareClose();
    this.libzip.discard(this.zip);
    this.ready = false;
  }
  resolve(p) {
    return ppath.resolve(PortablePath.root, p);
  }
  async openPromise(p, flags, mode) {
    return this.openSync(p, flags, mode);
  }
  openSync(p, flags, mode) {
    const fd = this.nextFd++;
    this.fds.set(fd, { cursor: 0, p });
    return fd;
  }
  hasOpenFileHandles() {
    return !!this.fds.size;
  }
  async opendirPromise(p, opts) {
    return this.opendirSync(p, opts);
  }
  opendirSync(p, opts = {}) {
    const resolvedP = this.resolveFilename(`opendir '${p}'`, p);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`opendir '${p}'`);
    const directoryListing = this.listings.get(resolvedP);
    if (!directoryListing)
      throw ENOTDIR(`opendir '${p}'`);
    const entries = [...directoryListing];
    const fd = this.openSync(resolvedP, `r`);
    const onClose = () => {
      this.closeSync(fd);
    };
    return opendir(this, resolvedP, entries, { onClose });
  }
  async readPromise(fd, buffer, offset, length, position) {
    return this.readSync(fd, buffer, offset, length, position);
  }
  readSync(fd, buffer, offset = 0, length = buffer.byteLength, position = -1) {
    const entry = this.fds.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`read`);
    const realPosition = position === -1 || position === null ? entry.cursor : position;
    const source = this.readFileSync(entry.p);
    source.copy(buffer, offset, realPosition, realPosition + length);
    const bytesRead = Math.max(0, Math.min(source.length - realPosition, length));
    if (position === -1 || position === null)
      entry.cursor += bytesRead;
    return bytesRead;
  }
  async writePromise(fd, buffer, offset, length, position) {
    if (typeof buffer === `string`) {
      return this.writeSync(fd, buffer, position);
    } else {
      return this.writeSync(fd, buffer, offset, length, position);
    }
  }
  writeSync(fd, buffer, offset, length, position) {
    const entry = this.fds.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`read`);
    throw new Error(`Unimplemented`);
  }
  async closePromise(fd) {
    return this.closeSync(fd);
  }
  closeSync(fd) {
    const entry = this.fds.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`read`);
    this.fds.delete(fd);
  }
  createReadStream(p, { encoding } = {}) {
    if (p === null)
      throw new Error(`Unimplemented`);
    const fd = this.openSync(p, `r`);
    const stream$1 = Object.assign(
      new stream.PassThrough({
        emitClose: true,
        autoDestroy: true,
        destroy: (error, callback) => {
          clearImmediate(immediate);
          this.closeSync(fd);
          callback(error);
        }
      }),
      {
        close() {
          stream$1.destroy();
        },
        bytesRead: 0,
        path: p
      }
    );
    const immediate = setImmediate(async () => {
      try {
        const data = await this.readFilePromise(p, encoding);
        stream$1.bytesRead = data.length;
        stream$1.end(data);
      } catch (error) {
        stream$1.destroy(error);
      }
    });
    return stream$1;
  }
  createWriteStream(p, { encoding } = {}) {
    if (this.readOnly)
      throw EROFS(`open '${p}'`);
    if (p === null)
      throw new Error(`Unimplemented`);
    const chunks = [];
    const fd = this.openSync(p, `w`);
    const stream$1 = Object.assign(
      new stream.PassThrough({
        autoDestroy: true,
        emitClose: true,
        destroy: (error, callback) => {
          try {
            if (error) {
              callback(error);
            } else {
              this.writeFileSync(p, Buffer.concat(chunks), encoding);
              callback(null);
            }
          } catch (err) {
            callback(err);
          } finally {
            this.closeSync(fd);
          }
        }
      }),
      {
        bytesWritten: 0,
        path: p,
        close() {
          stream$1.destroy();
        }
      }
    );
    stream$1.on(`data`, (chunk) => {
      const chunkBuffer = Buffer.from(chunk);
      stream$1.bytesWritten += chunkBuffer.length;
      chunks.push(chunkBuffer);
    });
    return stream$1;
  }
  async realpathPromise(p) {
    return this.realpathSync(p);
  }
  realpathSync(p) {
    const resolvedP = this.resolveFilename(`lstat '${p}'`, p);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`lstat '${p}'`);
    return resolvedP;
  }
  async existsPromise(p) {
    return this.existsSync(p);
  }
  existsSync(p) {
    if (!this.ready)
      throw EBUSY(`archive closed, existsSync '${p}'`);
    if (this.symlinkCount === 0) {
      const resolvedP2 = ppath.resolve(PortablePath.root, p);
      return this.entries.has(resolvedP2) || this.listings.has(resolvedP2);
    }
    let resolvedP;
    try {
      resolvedP = this.resolveFilename(`stat '${p}'`, p, void 0, false);
    } catch (error) {
      return false;
    }
    if (resolvedP === void 0)
      return false;
    return this.entries.has(resolvedP) || this.listings.has(resolvedP);
  }
  async accessPromise(p, mode) {
    return this.accessSync(p, mode);
  }
  accessSync(p, mode = fs.constants.F_OK) {
    const resolvedP = this.resolveFilename(`access '${p}'`, p);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`access '${p}'`);
    if (this.readOnly && mode & fs.constants.W_OK) {
      throw EROFS(`access '${p}'`);
    }
  }
  async statPromise(p, opts = { bigint: false }) {
    if (opts.bigint)
      return this.statSync(p, { bigint: true });
    return this.statSync(p);
  }
  statSync(p, opts = { bigint: false, throwIfNoEntry: true }) {
    const resolvedP = this.resolveFilename(`stat '${p}'`, p, void 0, opts.throwIfNoEntry);
    if (resolvedP === void 0)
      return void 0;
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP)) {
      if (opts.throwIfNoEntry === false)
        return void 0;
      throw ENOENT(`stat '${p}'`);
    }
    if (p[p.length - 1] === `/` && !this.listings.has(resolvedP))
      throw ENOTDIR(`stat '${p}'`);
    return this.statImpl(`stat '${p}'`, resolvedP, opts);
  }
  async fstatPromise(fd, opts) {
    return this.fstatSync(fd, opts);
  }
  fstatSync(fd, opts) {
    const entry = this.fds.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fstatSync`);
    const { p } = entry;
    const resolvedP = this.resolveFilename(`stat '${p}'`, p);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`stat '${p}'`);
    if (p[p.length - 1] === `/` && !this.listings.has(resolvedP))
      throw ENOTDIR(`stat '${p}'`);
    return this.statImpl(`fstat '${p}'`, resolvedP, opts);
  }
  async lstatPromise(p, opts = { bigint: false }) {
    if (opts.bigint)
      return this.lstatSync(p, { bigint: true });
    return this.lstatSync(p);
  }
  lstatSync(p, opts = { bigint: false, throwIfNoEntry: true }) {
    const resolvedP = this.resolveFilename(`lstat '${p}'`, p, false, opts.throwIfNoEntry);
    if (resolvedP === void 0)
      return void 0;
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP)) {
      if (opts.throwIfNoEntry === false)
        return void 0;
      throw ENOENT(`lstat '${p}'`);
    }
    if (p[p.length - 1] === `/` && !this.listings.has(resolvedP))
      throw ENOTDIR(`lstat '${p}'`);
    return this.statImpl(`lstat '${p}'`, resolvedP, opts);
  }
  statImpl(reason, p, opts = {}) {
    const entry = this.entries.get(p);
    if (typeof entry !== `undefined`) {
      const stat = this.libzip.struct.statS();
      const rc = this.libzip.statIndex(this.zip, entry, 0, 0, stat);
      if (rc === -1)
        throw this.makeLibzipError(this.libzip.getError(this.zip));
      const uid = this.stats.uid;
      const gid = this.stats.gid;
      const size = this.libzip.struct.statSize(stat) >>> 0;
      const blksize = 512;
      const blocks = Math.ceil(size / blksize);
      const mtimeMs = (this.libzip.struct.statMtime(stat) >>> 0) * 1e3;
      const atimeMs = mtimeMs;
      const birthtimeMs = mtimeMs;
      const ctimeMs = mtimeMs;
      const atime = new Date(atimeMs);
      const birthtime = new Date(birthtimeMs);
      const ctime = new Date(ctimeMs);
      const mtime = new Date(mtimeMs);
      const type = this.listings.has(p) ? S_IFDIR : this.isSymbolicLink(entry) ? S_IFLNK : S_IFREG;
      const defaultMode = type === S_IFDIR ? 493 : 420;
      const mode = type | this.getUnixMode(entry, defaultMode) & 511;
      const crc = this.libzip.struct.statCrc(stat);
      const statInstance = Object.assign(new StatEntry(), { uid, gid, size, blksize, blocks, atime, birthtime, ctime, mtime, atimeMs, birthtimeMs, ctimeMs, mtimeMs, mode, crc });
      return opts.bigint === true ? convertToBigIntStats(statInstance) : statInstance;
    }
    if (this.listings.has(p)) {
      const uid = this.stats.uid;
      const gid = this.stats.gid;
      const size = 0;
      const blksize = 512;
      const blocks = 0;
      const atimeMs = this.stats.mtimeMs;
      const birthtimeMs = this.stats.mtimeMs;
      const ctimeMs = this.stats.mtimeMs;
      const mtimeMs = this.stats.mtimeMs;
      const atime = new Date(atimeMs);
      const birthtime = new Date(birthtimeMs);
      const ctime = new Date(ctimeMs);
      const mtime = new Date(mtimeMs);
      const mode = S_IFDIR | 493;
      const crc = 0;
      const statInstance = Object.assign(new StatEntry(), { uid, gid, size, blksize, blocks, atime, birthtime, ctime, mtime, atimeMs, birthtimeMs, ctimeMs, mtimeMs, mode, crc });
      return opts.bigint === true ? convertToBigIntStats(statInstance) : statInstance;
    }
    throw new Error(`Unreachable`);
  }
  getUnixMode(index, defaultMode) {
    const rc = this.libzip.file.getExternalAttributes(this.zip, index, 0, 0, this.libzip.uint08S, this.libzip.uint32S);
    if (rc === -1)
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    const opsys = this.libzip.getValue(this.libzip.uint08S, `i8`) >>> 0;
    if (opsys !== this.libzip.ZIP_OPSYS_UNIX)
      return defaultMode;
    return this.libzip.getValue(this.libzip.uint32S, `i32`) >>> 16;
  }
  registerListing(p) {
    const existingListing = this.listings.get(p);
    if (existingListing)
      return existingListing;
    const parentListing = this.registerListing(ppath.dirname(p));
    parentListing.add(ppath.basename(p));
    const newListing = /* @__PURE__ */ new Set();
    this.listings.set(p, newListing);
    return newListing;
  }
  registerEntry(p, index) {
    const parentListing = this.registerListing(ppath.dirname(p));
    parentListing.add(ppath.basename(p));
    this.entries.set(p, index);
  }
  unregisterListing(p) {
    this.listings.delete(p);
    const parentListing = this.listings.get(ppath.dirname(p));
    parentListing == null ? void 0 : parentListing.delete(ppath.basename(p));
  }
  unregisterEntry(p) {
    this.unregisterListing(p);
    const entry = this.entries.get(p);
    this.entries.delete(p);
    if (typeof entry === `undefined`)
      return;
    this.fileSources.delete(entry);
    if (this.isSymbolicLink(entry)) {
      this.symlinkCount--;
    }
  }
  deleteEntry(p, index) {
    this.unregisterEntry(p);
    const rc = this.libzip.delete(this.zip, index);
    if (rc === -1) {
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    }
  }
  resolveFilename(reason, p, resolveLastComponent = true, throwIfNoEntry = true) {
    if (!this.ready)
      throw EBUSY(`archive closed, ${reason}`);
    let resolvedP = ppath.resolve(PortablePath.root, p);
    if (resolvedP === `/`)
      return PortablePath.root;
    const fileIndex = this.entries.get(resolvedP);
    if (resolveLastComponent && fileIndex !== void 0) {
      if (this.symlinkCount !== 0 && this.isSymbolicLink(fileIndex)) {
        const target = this.getFileSource(fileIndex).toString();
        return this.resolveFilename(reason, ppath.resolve(ppath.dirname(resolvedP), target), true, throwIfNoEntry);
      } else {
        return resolvedP;
      }
    }
    while (true) {
      const parentP = this.resolveFilename(reason, ppath.dirname(resolvedP), true, throwIfNoEntry);
      if (parentP === void 0)
        return parentP;
      const isDir = this.listings.has(parentP);
      const doesExist = this.entries.has(parentP);
      if (!isDir && !doesExist) {
        if (throwIfNoEntry === false)
          return void 0;
        throw ENOENT(reason);
      }
      if (!isDir)
        throw ENOTDIR(reason);
      resolvedP = ppath.resolve(parentP, ppath.basename(resolvedP));
      if (!resolveLastComponent || this.symlinkCount === 0)
        break;
      const index = this.libzip.name.locate(this.zip, resolvedP.slice(1), 0);
      if (index === -1)
        break;
      if (this.isSymbolicLink(index)) {
        const target = this.getFileSource(index).toString();
        resolvedP = ppath.resolve(ppath.dirname(resolvedP), target);
      } else {
        break;
      }
    }
    return resolvedP;
  }
  allocateBuffer(content) {
    if (!Buffer.isBuffer(content))
      content = Buffer.from(content);
    const buffer = this.libzip.malloc(content.byteLength);
    if (!buffer)
      throw new Error(`Couldn't allocate enough memory`);
    const heap = new Uint8Array(this.libzip.HEAPU8.buffer, buffer, content.byteLength);
    heap.set(content);
    return { buffer, byteLength: content.byteLength };
  }
  allocateUnattachedSource(content) {
    const error = this.libzip.struct.errorS();
    const { buffer, byteLength } = this.allocateBuffer(content);
    const source = this.libzip.source.fromUnattachedBuffer(buffer, byteLength, 0, 1, error);
    if (source === 0) {
      this.libzip.free(error);
      throw this.makeLibzipError(error);
    }
    return source;
  }
  allocateSource(content) {
    const { buffer, byteLength } = this.allocateBuffer(content);
    const source = this.libzip.source.fromBuffer(this.zip, buffer, byteLength, 0, 1);
    if (source === 0) {
      this.libzip.free(buffer);
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    }
    return source;
  }
  setFileSource(p, content) {
    const buffer = Buffer.isBuffer(content) ? content : Buffer.from(content);
    const target = ppath.relative(PortablePath.root, p);
    const lzSource = this.allocateSource(content);
    try {
      const newIndex = this.libzip.file.add(this.zip, target, lzSource, this.libzip.ZIP_FL_OVERWRITE);
      if (newIndex === -1)
        throw this.makeLibzipError(this.libzip.getError(this.zip));
      if (this.level !== `mixed`) {
        const method = this.level === 0 ? this.libzip.ZIP_CM_STORE : this.libzip.ZIP_CM_DEFLATE;
        const rc = this.libzip.file.setCompression(this.zip, newIndex, 0, method, this.level);
        if (rc === -1) {
          throw this.makeLibzipError(this.libzip.getError(this.zip));
        }
      }
      this.fileSources.set(newIndex, buffer);
      return newIndex;
    } catch (error) {
      this.libzip.source.free(lzSource);
      throw error;
    }
  }
  isSymbolicLink(index) {
    if (this.symlinkCount === 0)
      return false;
    const attrs = this.libzip.file.getExternalAttributes(this.zip, index, 0, 0, this.libzip.uint08S, this.libzip.uint32S);
    if (attrs === -1)
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    const opsys = this.libzip.getValue(this.libzip.uint08S, `i8`) >>> 0;
    if (opsys !== this.libzip.ZIP_OPSYS_UNIX)
      return false;
    const attributes = this.libzip.getValue(this.libzip.uint32S, `i32`) >>> 16;
    return (attributes & S_IFMT) === S_IFLNK;
  }
  getFileSource(index, opts = { asyncDecompress: false }) {
    const cachedFileSource = this.fileSources.get(index);
    if (typeof cachedFileSource !== `undefined`)
      return cachedFileSource;
    const stat = this.libzip.struct.statS();
    const rc = this.libzip.statIndex(this.zip, index, 0, 0, stat);
    if (rc === -1)
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    const size = this.libzip.struct.statCompSize(stat);
    const compressionMethod = this.libzip.struct.statCompMethod(stat);
    const buffer = this.libzip.malloc(size);
    try {
      const file = this.libzip.fopenIndex(this.zip, index, 0, this.libzip.ZIP_FL_COMPRESSED);
      if (file === 0)
        throw this.makeLibzipError(this.libzip.getError(this.zip));
      try {
        const rc2 = this.libzip.fread(file, buffer, size, 0);
        if (rc2 === -1)
          throw this.makeLibzipError(this.libzip.file.getError(file));
        else if (rc2 < size)
          throw new Error(`Incomplete read`);
        else if (rc2 > size)
          throw new Error(`Overread`);
        const memory = this.libzip.HEAPU8.subarray(buffer, buffer + size);
        const data = Buffer.from(memory);
        if (compressionMethod === 0) {
          this.fileSources.set(index, data);
          return data;
        } else if (opts.asyncDecompress) {
          return new Promise((resolve, reject) => {
            zlib__default.default.inflateRaw(data, (error, result) => {
              if (error) {
                reject(error);
              } else {
                this.fileSources.set(index, result);
                resolve(result);
              }
            });
          });
        } else {
          const decompressedData = zlib__default.default.inflateRawSync(data);
          this.fileSources.set(index, decompressedData);
          return decompressedData;
        }
      } finally {
        this.libzip.fclose(file);
      }
    } finally {
      this.libzip.free(buffer);
    }
  }
  async fchmodPromise(fd, mask) {
    return this.chmodPromise(this.fdToPath(fd, `fchmod`), mask);
  }
  fchmodSync(fd, mask) {
    return this.chmodSync(this.fdToPath(fd, `fchmodSync`), mask);
  }
  async chmodPromise(p, mask) {
    return this.chmodSync(p, mask);
  }
  chmodSync(p, mask) {
    if (this.readOnly)
      throw EROFS(`chmod '${p}'`);
    mask &= 493;
    const resolvedP = this.resolveFilename(`chmod '${p}'`, p, false);
    const entry = this.entries.get(resolvedP);
    if (typeof entry === `undefined`)
      throw new Error(`Assertion failed: The entry should have been registered (${resolvedP})`);
    const oldMod = this.getUnixMode(entry, S_IFREG | 0);
    const newMod = oldMod & ~511 | mask;
    const rc = this.libzip.file.setExternalAttributes(this.zip, entry, 0, 0, this.libzip.ZIP_OPSYS_UNIX, newMod << 16);
    if (rc === -1) {
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    }
  }
  async fchownPromise(fd, uid, gid) {
    return this.chownPromise(this.fdToPath(fd, `fchown`), uid, gid);
  }
  fchownSync(fd, uid, gid) {
    return this.chownSync(this.fdToPath(fd, `fchownSync`), uid, gid);
  }
  async chownPromise(p, uid, gid) {
    return this.chownSync(p, uid, gid);
  }
  chownSync(p, uid, gid) {
    throw new Error(`Unimplemented`);
  }
  async renamePromise(oldP, newP) {
    return this.renameSync(oldP, newP);
  }
  renameSync(oldP, newP) {
    throw new Error(`Unimplemented`);
  }
  async copyFilePromise(sourceP, destP, flags) {
    const { indexSource, indexDest, resolvedDestP } = this.prepareCopyFile(sourceP, destP, flags);
    const source = await this.getFileSource(indexSource, { asyncDecompress: true });
    const newIndex = this.setFileSource(resolvedDestP, source);
    if (newIndex !== indexDest) {
      this.registerEntry(resolvedDestP, newIndex);
    }
  }
  copyFileSync(sourceP, destP, flags = 0) {
    const { indexSource, indexDest, resolvedDestP } = this.prepareCopyFile(sourceP, destP, flags);
    const source = this.getFileSource(indexSource);
    const newIndex = this.setFileSource(resolvedDestP, source);
    if (newIndex !== indexDest) {
      this.registerEntry(resolvedDestP, newIndex);
    }
  }
  prepareCopyFile(sourceP, destP, flags = 0) {
    if (this.readOnly)
      throw EROFS(`copyfile '${sourceP} -> '${destP}'`);
    if ((flags & fs.constants.COPYFILE_FICLONE_FORCE) !== 0)
      throw ENOSYS(`unsupported clone operation`, `copyfile '${sourceP}' -> ${destP}'`);
    const resolvedSourceP = this.resolveFilename(`copyfile '${sourceP} -> ${destP}'`, sourceP);
    const indexSource = this.entries.get(resolvedSourceP);
    if (typeof indexSource === `undefined`)
      throw EINVAL(`copyfile '${sourceP}' -> '${destP}'`);
    const resolvedDestP = this.resolveFilename(`copyfile '${sourceP}' -> ${destP}'`, destP);
    const indexDest = this.entries.get(resolvedDestP);
    if ((flags & (fs.constants.COPYFILE_EXCL | fs.constants.COPYFILE_FICLONE_FORCE)) !== 0 && typeof indexDest !== `undefined`)
      throw EEXIST(`copyfile '${sourceP}' -> '${destP}'`);
    return {
      indexSource,
      resolvedDestP,
      indexDest
    };
  }
  async appendFilePromise(p, content, opts) {
    if (this.readOnly)
      throw EROFS(`open '${p}'`);
    if (typeof opts === `undefined`)
      opts = { flag: `a` };
    else if (typeof opts === `string`)
      opts = { flag: `a`, encoding: opts };
    else if (typeof opts.flag === `undefined`)
      opts = { flag: `a`, ...opts };
    return this.writeFilePromise(p, content, opts);
  }
  appendFileSync(p, content, opts = {}) {
    if (this.readOnly)
      throw EROFS(`open '${p}'`);
    if (typeof opts === `undefined`)
      opts = { flag: `a` };
    else if (typeof opts === `string`)
      opts = { flag: `a`, encoding: opts };
    else if (typeof opts.flag === `undefined`)
      opts = { flag: `a`, ...opts };
    return this.writeFileSync(p, content, opts);
  }
  fdToPath(fd, reason) {
    var _a;
    const path = (_a = this.fds.get(fd)) == null ? void 0 : _a.p;
    if (typeof path === `undefined`)
      throw EBADF(reason);
    return path;
  }
  async writeFilePromise(p, content, opts) {
    const { encoding, mode, index, resolvedP } = this.prepareWriteFile(p, opts);
    if (index !== void 0 && typeof opts === `object` && opts.flag && opts.flag.includes(`a`))
      content = Buffer.concat([await this.getFileSource(index, { asyncDecompress: true }), Buffer.from(content)]);
    if (encoding !== null)
      content = content.toString(encoding);
    const newIndex = this.setFileSource(resolvedP, content);
    if (newIndex !== index)
      this.registerEntry(resolvedP, newIndex);
    if (mode !== null) {
      await this.chmodPromise(resolvedP, mode);
    }
  }
  writeFileSync(p, content, opts) {
    const { encoding, mode, index, resolvedP } = this.prepareWriteFile(p, opts);
    if (index !== void 0 && typeof opts === `object` && opts.flag && opts.flag.includes(`a`))
      content = Buffer.concat([this.getFileSource(index), Buffer.from(content)]);
    if (encoding !== null)
      content = content.toString(encoding);
    const newIndex = this.setFileSource(resolvedP, content);
    if (newIndex !== index)
      this.registerEntry(resolvedP, newIndex);
    if (mode !== null) {
      this.chmodSync(resolvedP, mode);
    }
  }
  prepareWriteFile(p, opts) {
    if (typeof p === `number`)
      p = this.fdToPath(p, `read`);
    if (this.readOnly)
      throw EROFS(`open '${p}'`);
    const resolvedP = this.resolveFilename(`open '${p}'`, p);
    if (this.listings.has(resolvedP))
      throw EISDIR(`open '${p}'`);
    let encoding = null, mode = null;
    if (typeof opts === `string`) {
      encoding = opts;
    } else if (typeof opts === `object`) {
      ({
        encoding = null,
        mode = null
      } = opts);
    }
    const index = this.entries.get(resolvedP);
    return {
      encoding,
      mode,
      resolvedP,
      index
    };
  }
  async unlinkPromise(p) {
    return this.unlinkSync(p);
  }
  unlinkSync(p) {
    if (this.readOnly)
      throw EROFS(`unlink '${p}'`);
    const resolvedP = this.resolveFilename(`unlink '${p}'`, p);
    if (this.listings.has(resolvedP))
      throw EISDIR(`unlink '${p}'`);
    const index = this.entries.get(resolvedP);
    if (typeof index === `undefined`)
      throw EINVAL(`unlink '${p}'`);
    this.deleteEntry(resolvedP, index);
  }
  async utimesPromise(p, atime, mtime) {
    return this.utimesSync(p, atime, mtime);
  }
  utimesSync(p, atime, mtime) {
    if (this.readOnly)
      throw EROFS(`utimes '${p}'`);
    const resolvedP = this.resolveFilename(`utimes '${p}'`, p);
    this.utimesImpl(resolvedP, mtime);
  }
  async lutimesPromise(p, atime, mtime) {
    return this.lutimesSync(p, atime, mtime);
  }
  lutimesSync(p, atime, mtime) {
    if (this.readOnly)
      throw EROFS(`lutimes '${p}'`);
    const resolvedP = this.resolveFilename(`utimes '${p}'`, p, false);
    this.utimesImpl(resolvedP, mtime);
  }
  utimesImpl(resolvedP, mtime) {
    if (this.listings.has(resolvedP)) {
      if (!this.entries.has(resolvedP))
        this.hydrateDirectory(resolvedP);
    }
    const entry = this.entries.get(resolvedP);
    if (entry === void 0)
      throw new Error(`Unreachable`);
    const rc = this.libzip.file.setMtime(this.zip, entry, 0, toUnixTimestamp(mtime), 0);
    if (rc === -1) {
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    }
  }
  async mkdirPromise(p, opts) {
    return this.mkdirSync(p, opts);
  }
  mkdirSync(p, { mode = 493, recursive = false } = {}) {
    if (recursive)
      return this.mkdirpSync(p, { chmod: mode });
    if (this.readOnly)
      throw EROFS(`mkdir '${p}'`);
    const resolvedP = this.resolveFilename(`mkdir '${p}'`, p);
    if (this.entries.has(resolvedP) || this.listings.has(resolvedP))
      throw EEXIST(`mkdir '${p}'`);
    this.hydrateDirectory(resolvedP);
    this.chmodSync(resolvedP, mode);
    return void 0;
  }
  async rmdirPromise(p, opts) {
    return this.rmdirSync(p, opts);
  }
  rmdirSync(p, { recursive = false } = {}) {
    if (this.readOnly)
      throw EROFS(`rmdir '${p}'`);
    if (recursive) {
      this.removeSync(p);
      return;
    }
    const resolvedP = this.resolveFilename(`rmdir '${p}'`, p);
    const directoryListing = this.listings.get(resolvedP);
    if (!directoryListing)
      throw ENOTDIR(`rmdir '${p}'`);
    if (directoryListing.size > 0)
      throw ENOTEMPTY(`rmdir '${p}'`);
    const index = this.entries.get(resolvedP);
    if (typeof index === `undefined`)
      throw EINVAL(`rmdir '${p}'`);
    this.deleteEntry(p, index);
  }
  hydrateDirectory(resolvedP) {
    const index = this.libzip.dir.add(this.zip, ppath.relative(PortablePath.root, resolvedP));
    if (index === -1)
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    this.registerListing(resolvedP);
    this.registerEntry(resolvedP, index);
    return index;
  }
  async linkPromise(existingP, newP) {
    return this.linkSync(existingP, newP);
  }
  linkSync(existingP, newP) {
    throw EOPNOTSUPP(`link '${existingP}' -> '${newP}'`);
  }
  async symlinkPromise(target, p) {
    return this.symlinkSync(target, p);
  }
  symlinkSync(target, p) {
    if (this.readOnly)
      throw EROFS(`symlink '${target}' -> '${p}'`);
    const resolvedP = this.resolveFilename(`symlink '${target}' -> '${p}'`, p);
    if (this.listings.has(resolvedP))
      throw EISDIR(`symlink '${target}' -> '${p}'`);
    if (this.entries.has(resolvedP))
      throw EEXIST(`symlink '${target}' -> '${p}'`);
    const index = this.setFileSource(resolvedP, target);
    this.registerEntry(resolvedP, index);
    const rc = this.libzip.file.setExternalAttributes(this.zip, index, 0, 0, this.libzip.ZIP_OPSYS_UNIX, (S_IFLNK | 511) << 16);
    if (rc === -1)
      throw this.makeLibzipError(this.libzip.getError(this.zip));
    this.symlinkCount += 1;
  }
  async readFilePromise(p, encoding) {
    if (typeof encoding === `object`)
      encoding = encoding ? encoding.encoding : void 0;
    const data = await this.readFileBuffer(p, { asyncDecompress: true });
    return encoding ? data.toString(encoding) : data;
  }
  readFileSync(p, encoding) {
    if (typeof encoding === `object`)
      encoding = encoding ? encoding.encoding : void 0;
    const data = this.readFileBuffer(p);
    return encoding ? data.toString(encoding) : data;
  }
  readFileBuffer(p, opts = { asyncDecompress: false }) {
    if (typeof p === `number`)
      p = this.fdToPath(p, `read`);
    const resolvedP = this.resolveFilename(`open '${p}'`, p);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`open '${p}'`);
    if (p[p.length - 1] === `/` && !this.listings.has(resolvedP))
      throw ENOTDIR(`open '${p}'`);
    if (this.listings.has(resolvedP))
      throw EISDIR(`read`);
    const entry = this.entries.get(resolvedP);
    if (entry === void 0)
      throw new Error(`Unreachable`);
    return this.getFileSource(entry, opts);
  }
  async readdirPromise(p, opts) {
    return this.readdirSync(p, opts);
  }
  readdirSync(p, opts) {
    const resolvedP = this.resolveFilename(`scandir '${p}'`, p);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`scandir '${p}'`);
    const directoryListing = this.listings.get(resolvedP);
    if (!directoryListing)
      throw ENOTDIR(`scandir '${p}'`);
    const entries = [...directoryListing];
    if (!(opts == null ? void 0 : opts.withFileTypes))
      return entries;
    return entries.map((name) => {
      return Object.assign(this.statImpl(`lstat`, ppath.join(p, name)), {
        name
      });
    });
  }
  async readlinkPromise(p) {
    const entry = this.prepareReadlink(p);
    return (await this.getFileSource(entry, { asyncDecompress: true })).toString();
  }
  readlinkSync(p) {
    const entry = this.prepareReadlink(p);
    return this.getFileSource(entry).toString();
  }
  prepareReadlink(p) {
    const resolvedP = this.resolveFilename(`readlink '${p}'`, p, false);
    if (!this.entries.has(resolvedP) && !this.listings.has(resolvedP))
      throw ENOENT(`readlink '${p}'`);
    if (p[p.length - 1] === `/` && !this.listings.has(resolvedP))
      throw ENOTDIR(`open '${p}'`);
    if (this.listings.has(resolvedP))
      throw EINVAL(`readlink '${p}'`);
    const entry = this.entries.get(resolvedP);
    if (entry === void 0)
      throw new Error(`Unreachable`);
    if (!this.isSymbolicLink(entry))
      throw EINVAL(`readlink '${p}'`);
    return entry;
  }
  async truncatePromise(p, len = 0) {
    const resolvedP = this.resolveFilename(`open '${p}'`, p);
    const index = this.entries.get(resolvedP);
    if (typeof index === `undefined`)
      throw EINVAL(`open '${p}'`);
    const source = await this.getFileSource(index, { asyncDecompress: true });
    const truncated = Buffer.alloc(len, 0);
    source.copy(truncated);
    return await this.writeFilePromise(p, truncated);
  }
  truncateSync(p, len = 0) {
    const resolvedP = this.resolveFilename(`open '${p}'`, p);
    const index = this.entries.get(resolvedP);
    if (typeof index === `undefined`)
      throw EINVAL(`open '${p}'`);
    const source = this.getFileSource(index);
    const truncated = Buffer.alloc(len, 0);
    source.copy(truncated);
    return this.writeFileSync(p, truncated);
  }
  async ftruncatePromise(fd, len) {
    return this.truncatePromise(this.fdToPath(fd, `ftruncate`), len);
  }
  ftruncateSync(fd, len) {
    return this.truncateSync(this.fdToPath(fd, `ftruncateSync`), len);
  }
  watch(p, a, b) {
    let persistent;
    switch (typeof a) {
      case `function`:
      case `string`:
      case `undefined`:
        {
          persistent = true;
        }
        break;
      default:
        {
          ({ persistent = true } = a);
        }
        break;
    }
    if (!persistent)
      return { on: () => {
      }, close: () => {
      } };
    const interval = setInterval(() => {
    }, 24 * 60 * 60 * 1e3);
    return { on: () => {
    }, close: () => {
      clearInterval(interval);
    } };
  }
  watchFile(p, a, b) {
    const resolvedP = ppath.resolve(PortablePath.root, p);
    return watchFile(this, resolvedP, a, b);
  }
  unwatchFile(p, cb) {
    const resolvedP = ppath.resolve(PortablePath.root, p);
    return unwatchFile(this, resolvedP, cb);
  }
}

class ProxiedFS extends FakeFS {
  getExtractHint(hints) {
    return this.baseFs.getExtractHint(hints);
  }
  resolve(path) {
    return this.mapFromBase(this.baseFs.resolve(this.mapToBase(path)));
  }
  getRealPath() {
    return this.mapFromBase(this.baseFs.getRealPath());
  }
  async openPromise(p, flags, mode) {
    return this.baseFs.openPromise(this.mapToBase(p), flags, mode);
  }
  openSync(p, flags, mode) {
    return this.baseFs.openSync(this.mapToBase(p), flags, mode);
  }
  async opendirPromise(p, opts) {
    return Object.assign(await this.baseFs.opendirPromise(this.mapToBase(p), opts), { path: p });
  }
  opendirSync(p, opts) {
    return Object.assign(this.baseFs.opendirSync(this.mapToBase(p), opts), { path: p });
  }
  async readPromise(fd, buffer, offset, length, position) {
    return await this.baseFs.readPromise(fd, buffer, offset, length, position);
  }
  readSync(fd, buffer, offset, length, position) {
    return this.baseFs.readSync(fd, buffer, offset, length, position);
  }
  async writePromise(fd, buffer, offset, length, position) {
    if (typeof buffer === `string`) {
      return await this.baseFs.writePromise(fd, buffer, offset);
    } else {
      return await this.baseFs.writePromise(fd, buffer, offset, length, position);
    }
  }
  writeSync(fd, buffer, offset, length, position) {
    if (typeof buffer === `string`) {
      return this.baseFs.writeSync(fd, buffer, offset);
    } else {
      return this.baseFs.writeSync(fd, buffer, offset, length, position);
    }
  }
  async closePromise(fd) {
    return this.baseFs.closePromise(fd);
  }
  closeSync(fd) {
    this.baseFs.closeSync(fd);
  }
  createReadStream(p, opts) {
    return this.baseFs.createReadStream(p !== null ? this.mapToBase(p) : p, opts);
  }
  createWriteStream(p, opts) {
    return this.baseFs.createWriteStream(p !== null ? this.mapToBase(p) : p, opts);
  }
  async realpathPromise(p) {
    return this.mapFromBase(await this.baseFs.realpathPromise(this.mapToBase(p)));
  }
  realpathSync(p) {
    return this.mapFromBase(this.baseFs.realpathSync(this.mapToBase(p)));
  }
  async existsPromise(p) {
    return this.baseFs.existsPromise(this.mapToBase(p));
  }
  existsSync(p) {
    return this.baseFs.existsSync(this.mapToBase(p));
  }
  accessSync(p, mode) {
    return this.baseFs.accessSync(this.mapToBase(p), mode);
  }
  async accessPromise(p, mode) {
    return this.baseFs.accessPromise(this.mapToBase(p), mode);
  }
  async statPromise(p, opts) {
    return this.baseFs.statPromise(this.mapToBase(p), opts);
  }
  statSync(p, opts) {
    return this.baseFs.statSync(this.mapToBase(p), opts);
  }
  async fstatPromise(fd, opts) {
    return this.baseFs.fstatPromise(fd, opts);
  }
  fstatSync(fd, opts) {
    return this.baseFs.fstatSync(fd, opts);
  }
  lstatPromise(p, opts) {
    return this.baseFs.lstatPromise(this.mapToBase(p), opts);
  }
  lstatSync(p, opts) {
    return this.baseFs.lstatSync(this.mapToBase(p), opts);
  }
  async fchmodPromise(fd, mask) {
    return this.baseFs.fchmodPromise(fd, mask);
  }
  fchmodSync(fd, mask) {
    return this.baseFs.fchmodSync(fd, mask);
  }
  async chmodPromise(p, mask) {
    return this.baseFs.chmodPromise(this.mapToBase(p), mask);
  }
  chmodSync(p, mask) {
    return this.baseFs.chmodSync(this.mapToBase(p), mask);
  }
  async fchownPromise(fd, uid, gid) {
    return this.baseFs.fchownPromise(fd, uid, gid);
  }
  fchownSync(fd, uid, gid) {
    return this.baseFs.fchownSync(fd, uid, gid);
  }
  async chownPromise(p, uid, gid) {
    return this.baseFs.chownPromise(this.mapToBase(p), uid, gid);
  }
  chownSync(p, uid, gid) {
    return this.baseFs.chownSync(this.mapToBase(p), uid, gid);
  }
  async renamePromise(oldP, newP) {
    return this.baseFs.renamePromise(this.mapToBase(oldP), this.mapToBase(newP));
  }
  renameSync(oldP, newP) {
    return this.baseFs.renameSync(this.mapToBase(oldP), this.mapToBase(newP));
  }
  async copyFilePromise(sourceP, destP, flags = 0) {
    return this.baseFs.copyFilePromise(this.mapToBase(sourceP), this.mapToBase(destP), flags);
  }
  copyFileSync(sourceP, destP, flags = 0) {
    return this.baseFs.copyFileSync(this.mapToBase(sourceP), this.mapToBase(destP), flags);
  }
  async appendFilePromise(p, content, opts) {
    return this.baseFs.appendFilePromise(this.fsMapToBase(p), content, opts);
  }
  appendFileSync(p, content, opts) {
    return this.baseFs.appendFileSync(this.fsMapToBase(p), content, opts);
  }
  async writeFilePromise(p, content, opts) {
    return this.baseFs.writeFilePromise(this.fsMapToBase(p), content, opts);
  }
  writeFileSync(p, content, opts) {
    return this.baseFs.writeFileSync(this.fsMapToBase(p), content, opts);
  }
  async unlinkPromise(p) {
    return this.baseFs.unlinkPromise(this.mapToBase(p));
  }
  unlinkSync(p) {
    return this.baseFs.unlinkSync(this.mapToBase(p));
  }
  async utimesPromise(p, atime, mtime) {
    return this.baseFs.utimesPromise(this.mapToBase(p), atime, mtime);
  }
  utimesSync(p, atime, mtime) {
    return this.baseFs.utimesSync(this.mapToBase(p), atime, mtime);
  }
  async mkdirPromise(p, opts) {
    return this.baseFs.mkdirPromise(this.mapToBase(p), opts);
  }
  mkdirSync(p, opts) {
    return this.baseFs.mkdirSync(this.mapToBase(p), opts);
  }
  async rmdirPromise(p, opts) {
    return this.baseFs.rmdirPromise(this.mapToBase(p), opts);
  }
  rmdirSync(p, opts) {
    return this.baseFs.rmdirSync(this.mapToBase(p), opts);
  }
  async linkPromise(existingP, newP) {
    return this.baseFs.linkPromise(this.mapToBase(existingP), this.mapToBase(newP));
  }
  linkSync(existingP, newP) {
    return this.baseFs.linkSync(this.mapToBase(existingP), this.mapToBase(newP));
  }
  async symlinkPromise(target, p, type) {
    const mappedP = this.mapToBase(p);
    if (this.pathUtils.isAbsolute(target))
      return this.baseFs.symlinkPromise(this.mapToBase(target), mappedP, type);
    const mappedAbsoluteTarget = this.mapToBase(this.pathUtils.join(this.pathUtils.dirname(p), target));
    const mappedTarget = this.baseFs.pathUtils.relative(this.baseFs.pathUtils.dirname(mappedP), mappedAbsoluteTarget);
    return this.baseFs.symlinkPromise(mappedTarget, mappedP, type);
  }
  symlinkSync(target, p, type) {
    const mappedP = this.mapToBase(p);
    if (this.pathUtils.isAbsolute(target))
      return this.baseFs.symlinkSync(this.mapToBase(target), mappedP, type);
    const mappedAbsoluteTarget = this.mapToBase(this.pathUtils.join(this.pathUtils.dirname(p), target));
    const mappedTarget = this.baseFs.pathUtils.relative(this.baseFs.pathUtils.dirname(mappedP), mappedAbsoluteTarget);
    return this.baseFs.symlinkSync(mappedTarget, mappedP, type);
  }
  async readFilePromise(p, encoding) {
    if (encoding === `utf8`) {
      return this.baseFs.readFilePromise(this.fsMapToBase(p), encoding);
    } else {
      return this.baseFs.readFilePromise(this.fsMapToBase(p), encoding);
    }
  }
  readFileSync(p, encoding) {
    if (encoding === `utf8`) {
      return this.baseFs.readFileSync(this.fsMapToBase(p), encoding);
    } else {
      return this.baseFs.readFileSync(this.fsMapToBase(p), encoding);
    }
  }
  async readdirPromise(p, opts) {
    return this.baseFs.readdirPromise(this.mapToBase(p), opts);
  }
  readdirSync(p, opts) {
    return this.baseFs.readdirSync(this.mapToBase(p), opts);
  }
  async readlinkPromise(p) {
    return this.mapFromBase(await this.baseFs.readlinkPromise(this.mapToBase(p)));
  }
  readlinkSync(p) {
    return this.mapFromBase(this.baseFs.readlinkSync(this.mapToBase(p)));
  }
  async truncatePromise(p, len) {
    return this.baseFs.truncatePromise(this.mapToBase(p), len);
  }
  truncateSync(p, len) {
    return this.baseFs.truncateSync(this.mapToBase(p), len);
  }
  async ftruncatePromise(fd, len) {
    return this.baseFs.ftruncatePromise(fd, len);
  }
  ftruncateSync(fd, len) {
    return this.baseFs.ftruncateSync(fd, len);
  }
  watch(p, a, b) {
    return this.baseFs.watch(
      this.mapToBase(p),
      a,
      b
    );
  }
  watchFile(p, a, b) {
    return this.baseFs.watchFile(
      this.mapToBase(p),
      a,
      b
    );
  }
  unwatchFile(p, cb) {
    return this.baseFs.unwatchFile(this.mapToBase(p), cb);
  }
  fsMapToBase(p) {
    if (typeof p === `number`) {
      return p;
    } else {
      return this.mapToBase(p);
    }
  }
}

class PosixFS extends ProxiedFS {
  constructor(baseFs) {
    super(npath);
    this.baseFs = baseFs;
  }
  mapFromBase(path) {
    return npath.fromPortablePath(path);
  }
  mapToBase(path) {
    return npath.toPortablePath(path);
  }
}

const NUMBER_REGEXP = /^[0-9]+$/;
const VIRTUAL_REGEXP = /^(\/(?:[^/]+\/)*?(?:\$\$virtual|__virtual__))((?:\/((?:[^/]+-)?[a-f0-9]+)(?:\/([^/]+))?)?((?:\/.*)?))$/;
const VALID_COMPONENT = /^([^/]+-)?[a-f0-9]+$/;
class VirtualFS extends ProxiedFS {
  constructor({ baseFs = new NodeFS() } = {}) {
    super(ppath);
    this.baseFs = baseFs;
  }
  static makeVirtualPath(base, component, to) {
    if (ppath.basename(base) !== `__virtual__`)
      throw new Error(`Assertion failed: Virtual folders must be named "__virtual__"`);
    if (!ppath.basename(component).match(VALID_COMPONENT))
      throw new Error(`Assertion failed: Virtual components must be ended by an hexadecimal hash`);
    const target = ppath.relative(ppath.dirname(base), to);
    const segments = target.split(`/`);
    let depth = 0;
    while (depth < segments.length && segments[depth] === `..`)
      depth += 1;
    const finalSegments = segments.slice(depth);
    const fullVirtualPath = ppath.join(base, component, String(depth), ...finalSegments);
    return fullVirtualPath;
  }
  static resolveVirtual(p) {
    const match = p.match(VIRTUAL_REGEXP);
    if (!match || !match[3] && match[5])
      return p;
    const target = ppath.dirname(match[1]);
    if (!match[3] || !match[4])
      return target;
    const isnum = NUMBER_REGEXP.test(match[4]);
    if (!isnum)
      return p;
    const depth = Number(match[4]);
    const backstep = `../`.repeat(depth);
    const subpath = match[5] || `.`;
    return VirtualFS.resolveVirtual(ppath.join(target, backstep, subpath));
  }
  getExtractHint(hints) {
    return this.baseFs.getExtractHint(hints);
  }
  getRealPath() {
    return this.baseFs.getRealPath();
  }
  realpathSync(p) {
    const match = p.match(VIRTUAL_REGEXP);
    if (!match)
      return this.baseFs.realpathSync(p);
    if (!match[5])
      return p;
    const realpath = this.baseFs.realpathSync(this.mapToBase(p));
    return VirtualFS.makeVirtualPath(match[1], match[3], realpath);
  }
  async realpathPromise(p) {
    const match = p.match(VIRTUAL_REGEXP);
    if (!match)
      return await this.baseFs.realpathPromise(p);
    if (!match[5])
      return p;
    const realpath = await this.baseFs.realpathPromise(this.mapToBase(p));
    return VirtualFS.makeVirtualPath(match[1], match[3], realpath);
  }
  mapToBase(p) {
    if (p === ``)
      return p;
    if (this.pathUtils.isAbsolute(p))
      return VirtualFS.resolveVirtual(p);
    const resolvedRoot = VirtualFS.resolveVirtual(this.baseFs.resolve(PortablePath.dot));
    const resolvedP = VirtualFS.resolveVirtual(this.baseFs.resolve(p));
    return ppath.relative(resolvedRoot, resolvedP) || PortablePath.dot;
  }
  mapFromBase(p) {
    return p;
  }
}

const ZIP_MASK = 4278190080;
const ZIP_MAGIC = 704643072;
const getArchivePart = (path, extension) => {
  let idx = path.indexOf(extension);
  if (idx <= 0)
    return null;
  let nextCharIdx = idx;
  while (idx >= 0) {
    nextCharIdx = idx + extension.length;
    if (path[nextCharIdx] === ppath.sep)
      break;
    if (path[idx - 1] === ppath.sep)
      return null;
    idx = path.indexOf(extension, nextCharIdx);
  }
  if (path.length > nextCharIdx && path[nextCharIdx] !== ppath.sep)
    return null;
  return path.slice(0, nextCharIdx);
};
class ZipOpenFS extends BasePortableFakeFS {
  constructor({ libzip, baseFs = new NodeFS(), filter = null, maxOpenFiles = Infinity, readOnlyArchives = false, useCache = true, maxAge = 5e3, fileExtensions = null }) {
    super();
    this.fdMap = /* @__PURE__ */ new Map();
    this.nextFd = 3;
    this.isZip = /* @__PURE__ */ new Set();
    this.notZip = /* @__PURE__ */ new Set();
    this.realPaths = /* @__PURE__ */ new Map();
    this.limitOpenFilesTimeout = null;
    this.libzipFactory = typeof libzip !== `function` ? () => libzip : libzip;
    this.baseFs = baseFs;
    this.zipInstances = useCache ? /* @__PURE__ */ new Map() : null;
    this.filter = filter;
    this.maxOpenFiles = maxOpenFiles;
    this.readOnlyArchives = readOnlyArchives;
    this.maxAge = maxAge;
    this.fileExtensions = fileExtensions;
  }
  static async openPromise(fn, opts) {
    const zipOpenFs = new ZipOpenFS(opts);
    try {
      return await fn(zipOpenFs);
    } finally {
      zipOpenFs.saveAndClose();
    }
  }
  get libzip() {
    if (typeof this.libzipInstance === `undefined`)
      this.libzipInstance = this.libzipFactory();
    return this.libzipInstance;
  }
  getExtractHint(hints) {
    return this.baseFs.getExtractHint(hints);
  }
  getRealPath() {
    return this.baseFs.getRealPath();
  }
  saveAndClose() {
    unwatchAllFiles(this);
    if (this.zipInstances) {
      for (const [path, { zipFs }] of this.zipInstances.entries()) {
        zipFs.saveAndClose();
        this.zipInstances.delete(path);
      }
    }
  }
  discardAndClose() {
    unwatchAllFiles(this);
    if (this.zipInstances) {
      for (const [path, { zipFs }] of this.zipInstances.entries()) {
        zipFs.discardAndClose();
        this.zipInstances.delete(path);
      }
    }
  }
  resolve(p) {
    return this.baseFs.resolve(p);
  }
  remapFd(zipFs, fd) {
    const remappedFd = this.nextFd++ | ZIP_MAGIC;
    this.fdMap.set(remappedFd, [zipFs, fd]);
    return remappedFd;
  }
  async openPromise(p, flags, mode) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.openPromise(p, flags, mode);
    }, async (zipFs, { subPath }) => {
      return this.remapFd(zipFs, await zipFs.openPromise(subPath, flags, mode));
    });
  }
  openSync(p, flags, mode) {
    return this.makeCallSync(p, () => {
      return this.baseFs.openSync(p, flags, mode);
    }, (zipFs, { subPath }) => {
      return this.remapFd(zipFs, zipFs.openSync(subPath, flags, mode));
    });
  }
  async opendirPromise(p, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.opendirPromise(p, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.opendirPromise(subPath, opts);
    }, {
      requireSubpath: false
    });
  }
  opendirSync(p, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.opendirSync(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.opendirSync(subPath, opts);
    }, {
      requireSubpath: false
    });
  }
  async readPromise(fd, buffer, offset, length, position) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return await this.baseFs.readPromise(fd, buffer, offset, length, position);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`read`);
    const [zipFs, realFd] = entry;
    return await zipFs.readPromise(realFd, buffer, offset, length, position);
  }
  readSync(fd, buffer, offset, length, position) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.readSync(fd, buffer, offset, length, position);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`readSync`);
    const [zipFs, realFd] = entry;
    return zipFs.readSync(realFd, buffer, offset, length, position);
  }
  async writePromise(fd, buffer, offset, length, position) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC) {
      if (typeof buffer === `string`) {
        return await this.baseFs.writePromise(fd, buffer, offset);
      } else {
        return await this.baseFs.writePromise(fd, buffer, offset, length, position);
      }
    }
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`write`);
    const [zipFs, realFd] = entry;
    if (typeof buffer === `string`) {
      return await zipFs.writePromise(realFd, buffer, offset);
    } else {
      return await zipFs.writePromise(realFd, buffer, offset, length, position);
    }
  }
  writeSync(fd, buffer, offset, length, position) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC) {
      if (typeof buffer === `string`) {
        return this.baseFs.writeSync(fd, buffer, offset);
      } else {
        return this.baseFs.writeSync(fd, buffer, offset, length, position);
      }
    }
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`writeSync`);
    const [zipFs, realFd] = entry;
    if (typeof buffer === `string`) {
      return zipFs.writeSync(realFd, buffer, offset);
    } else {
      return zipFs.writeSync(realFd, buffer, offset, length, position);
    }
  }
  async closePromise(fd) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return await this.baseFs.closePromise(fd);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`close`);
    this.fdMap.delete(fd);
    const [zipFs, realFd] = entry;
    return await zipFs.closePromise(realFd);
  }
  closeSync(fd) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.closeSync(fd);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`closeSync`);
    this.fdMap.delete(fd);
    const [zipFs, realFd] = entry;
    return zipFs.closeSync(realFd);
  }
  createReadStream(p, opts) {
    if (p === null)
      return this.baseFs.createReadStream(p, opts);
    return this.makeCallSync(p, () => {
      return this.baseFs.createReadStream(p, opts);
    }, (zipFs, { archivePath, subPath }) => {
      const stream = zipFs.createReadStream(subPath, opts);
      stream.path = npath.fromPortablePath(this.pathUtils.join(archivePath, subPath));
      return stream;
    });
  }
  createWriteStream(p, opts) {
    if (p === null)
      return this.baseFs.createWriteStream(p, opts);
    return this.makeCallSync(p, () => {
      return this.baseFs.createWriteStream(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.createWriteStream(subPath, opts);
    });
  }
  async realpathPromise(p) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.realpathPromise(p);
    }, async (zipFs, { archivePath, subPath }) => {
      let realArchivePath = this.realPaths.get(archivePath);
      if (typeof realArchivePath === `undefined`) {
        realArchivePath = await this.baseFs.realpathPromise(archivePath);
        this.realPaths.set(archivePath, realArchivePath);
      }
      return this.pathUtils.join(realArchivePath, this.pathUtils.relative(PortablePath.root, await zipFs.realpathPromise(subPath)));
    });
  }
  realpathSync(p) {
    return this.makeCallSync(p, () => {
      return this.baseFs.realpathSync(p);
    }, (zipFs, { archivePath, subPath }) => {
      let realArchivePath = this.realPaths.get(archivePath);
      if (typeof realArchivePath === `undefined`) {
        realArchivePath = this.baseFs.realpathSync(archivePath);
        this.realPaths.set(archivePath, realArchivePath);
      }
      return this.pathUtils.join(realArchivePath, this.pathUtils.relative(PortablePath.root, zipFs.realpathSync(subPath)));
    });
  }
  async existsPromise(p) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.existsPromise(p);
    }, async (zipFs, { subPath }) => {
      return await zipFs.existsPromise(subPath);
    });
  }
  existsSync(p) {
    return this.makeCallSync(p, () => {
      return this.baseFs.existsSync(p);
    }, (zipFs, { subPath }) => {
      return zipFs.existsSync(subPath);
    });
  }
  async accessPromise(p, mode) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.accessPromise(p, mode);
    }, async (zipFs, { subPath }) => {
      return await zipFs.accessPromise(subPath, mode);
    });
  }
  accessSync(p, mode) {
    return this.makeCallSync(p, () => {
      return this.baseFs.accessSync(p, mode);
    }, (zipFs, { subPath }) => {
      return zipFs.accessSync(subPath, mode);
    });
  }
  async statPromise(p, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.statPromise(p, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.statPromise(subPath, opts);
    });
  }
  statSync(p, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.statSync(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.statSync(subPath, opts);
    });
  }
  async fstatPromise(fd, opts) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.fstatPromise(fd, opts);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fstat`);
    const [zipFs, realFd] = entry;
    return zipFs.fstatPromise(realFd, opts);
  }
  fstatSync(fd, opts) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.fstatSync(fd, opts);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fstatSync`);
    const [zipFs, realFd] = entry;
    return zipFs.fstatSync(realFd, opts);
  }
  async lstatPromise(p, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.lstatPromise(p, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.lstatPromise(subPath, opts);
    });
  }
  lstatSync(p, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.lstatSync(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.lstatSync(subPath, opts);
    });
  }
  async fchmodPromise(fd, mask) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.fchmodPromise(fd, mask);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fchmod`);
    const [zipFs, realFd] = entry;
    return zipFs.fchmodPromise(realFd, mask);
  }
  fchmodSync(fd, mask) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.fchmodSync(fd, mask);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fchmodSync`);
    const [zipFs, realFd] = entry;
    return zipFs.fchmodSync(realFd, mask);
  }
  async chmodPromise(p, mask) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.chmodPromise(p, mask);
    }, async (zipFs, { subPath }) => {
      return await zipFs.chmodPromise(subPath, mask);
    });
  }
  chmodSync(p, mask) {
    return this.makeCallSync(p, () => {
      return this.baseFs.chmodSync(p, mask);
    }, (zipFs, { subPath }) => {
      return zipFs.chmodSync(subPath, mask);
    });
  }
  async fchownPromise(fd, uid, gid) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.fchownPromise(fd, uid, gid);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fchown`);
    const [zipFs, realFd] = entry;
    return zipFs.fchownPromise(realFd, uid, gid);
  }
  fchownSync(fd, uid, gid) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.fchownSync(fd, uid, gid);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`fchownSync`);
    const [zipFs, realFd] = entry;
    return zipFs.fchownSync(realFd, uid, gid);
  }
  async chownPromise(p, uid, gid) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.chownPromise(p, uid, gid);
    }, async (zipFs, { subPath }) => {
      return await zipFs.chownPromise(subPath, uid, gid);
    });
  }
  chownSync(p, uid, gid) {
    return this.makeCallSync(p, () => {
      return this.baseFs.chownSync(p, uid, gid);
    }, (zipFs, { subPath }) => {
      return zipFs.chownSync(subPath, uid, gid);
    });
  }
  async renamePromise(oldP, newP) {
    return await this.makeCallPromise(oldP, async () => {
      return await this.makeCallPromise(newP, async () => {
        return await this.baseFs.renamePromise(oldP, newP);
      }, async () => {
        throw Object.assign(new Error(`EEXDEV: cross-device link not permitted`), { code: `EEXDEV` });
      });
    }, async (zipFsO, { subPath: subPathO }) => {
      return await this.makeCallPromise(newP, async () => {
        throw Object.assign(new Error(`EEXDEV: cross-device link not permitted`), { code: `EEXDEV` });
      }, async (zipFsN, { subPath: subPathN }) => {
        if (zipFsO !== zipFsN) {
          throw Object.assign(new Error(`EEXDEV: cross-device link not permitted`), { code: `EEXDEV` });
        } else {
          return await zipFsO.renamePromise(subPathO, subPathN);
        }
      });
    });
  }
  renameSync(oldP, newP) {
    return this.makeCallSync(oldP, () => {
      return this.makeCallSync(newP, () => {
        return this.baseFs.renameSync(oldP, newP);
      }, () => {
        throw Object.assign(new Error(`EEXDEV: cross-device link not permitted`), { code: `EEXDEV` });
      });
    }, (zipFsO, { subPath: subPathO }) => {
      return this.makeCallSync(newP, () => {
        throw Object.assign(new Error(`EEXDEV: cross-device link not permitted`), { code: `EEXDEV` });
      }, (zipFsN, { subPath: subPathN }) => {
        if (zipFsO !== zipFsN) {
          throw Object.assign(new Error(`EEXDEV: cross-device link not permitted`), { code: `EEXDEV` });
        } else {
          return zipFsO.renameSync(subPathO, subPathN);
        }
      });
    });
  }
  async copyFilePromise(sourceP, destP, flags = 0) {
    const fallback = async (sourceFs, sourceP2, destFs, destP2) => {
      if ((flags & fs.constants.COPYFILE_FICLONE_FORCE) !== 0)
        throw Object.assign(new Error(`EXDEV: cross-device clone not permitted, copyfile '${sourceP2}' -> ${destP2}'`), { code: `EXDEV` });
      if (flags & fs.constants.COPYFILE_EXCL && await this.existsPromise(sourceP2))
        throw Object.assign(new Error(`EEXIST: file already exists, copyfile '${sourceP2}' -> '${destP2}'`), { code: `EEXIST` });
      let content;
      try {
        content = await sourceFs.readFilePromise(sourceP2);
      } catch (error) {
        throw Object.assign(new Error(`EINVAL: invalid argument, copyfile '${sourceP2}' -> '${destP2}'`), { code: `EINVAL` });
      }
      await destFs.writeFilePromise(destP2, content);
    };
    return await this.makeCallPromise(sourceP, async () => {
      return await this.makeCallPromise(destP, async () => {
        return await this.baseFs.copyFilePromise(sourceP, destP, flags);
      }, async (zipFsD, { subPath: subPathD }) => {
        return await fallback(this.baseFs, sourceP, zipFsD, subPathD);
      });
    }, async (zipFsS, { subPath: subPathS }) => {
      return await this.makeCallPromise(destP, async () => {
        return await fallback(zipFsS, subPathS, this.baseFs, destP);
      }, async (zipFsD, { subPath: subPathD }) => {
        if (zipFsS !== zipFsD) {
          return await fallback(zipFsS, subPathS, zipFsD, subPathD);
        } else {
          return await zipFsS.copyFilePromise(subPathS, subPathD, flags);
        }
      });
    });
  }
  copyFileSync(sourceP, destP, flags = 0) {
    const fallback = (sourceFs, sourceP2, destFs, destP2) => {
      if ((flags & fs.constants.COPYFILE_FICLONE_FORCE) !== 0)
        throw Object.assign(new Error(`EXDEV: cross-device clone not permitted, copyfile '${sourceP2}' -> ${destP2}'`), { code: `EXDEV` });
      if (flags & fs.constants.COPYFILE_EXCL && this.existsSync(sourceP2))
        throw Object.assign(new Error(`EEXIST: file already exists, copyfile '${sourceP2}' -> '${destP2}'`), { code: `EEXIST` });
      let content;
      try {
        content = sourceFs.readFileSync(sourceP2);
      } catch (error) {
        throw Object.assign(new Error(`EINVAL: invalid argument, copyfile '${sourceP2}' -> '${destP2}'`), { code: `EINVAL` });
      }
      destFs.writeFileSync(destP2, content);
    };
    return this.makeCallSync(sourceP, () => {
      return this.makeCallSync(destP, () => {
        return this.baseFs.copyFileSync(sourceP, destP, flags);
      }, (zipFsD, { subPath: subPathD }) => {
        return fallback(this.baseFs, sourceP, zipFsD, subPathD);
      });
    }, (zipFsS, { subPath: subPathS }) => {
      return this.makeCallSync(destP, () => {
        return fallback(zipFsS, subPathS, this.baseFs, destP);
      }, (zipFsD, { subPath: subPathD }) => {
        if (zipFsS !== zipFsD) {
          return fallback(zipFsS, subPathS, zipFsD, subPathD);
        } else {
          return zipFsS.copyFileSync(subPathS, subPathD, flags);
        }
      });
    });
  }
  async appendFilePromise(p, content, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.appendFilePromise(p, content, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.appendFilePromise(subPath, content, opts);
    });
  }
  appendFileSync(p, content, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.appendFileSync(p, content, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.appendFileSync(subPath, content, opts);
    });
  }
  async writeFilePromise(p, content, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.writeFilePromise(p, content, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.writeFilePromise(subPath, content, opts);
    });
  }
  writeFileSync(p, content, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.writeFileSync(p, content, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.writeFileSync(subPath, content, opts);
    });
  }
  async unlinkPromise(p) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.unlinkPromise(p);
    }, async (zipFs, { subPath }) => {
      return await zipFs.unlinkPromise(subPath);
    });
  }
  unlinkSync(p) {
    return this.makeCallSync(p, () => {
      return this.baseFs.unlinkSync(p);
    }, (zipFs, { subPath }) => {
      return zipFs.unlinkSync(subPath);
    });
  }
  async utimesPromise(p, atime, mtime) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.utimesPromise(p, atime, mtime);
    }, async (zipFs, { subPath }) => {
      return await zipFs.utimesPromise(subPath, atime, mtime);
    });
  }
  utimesSync(p, atime, mtime) {
    return this.makeCallSync(p, () => {
      return this.baseFs.utimesSync(p, atime, mtime);
    }, (zipFs, { subPath }) => {
      return zipFs.utimesSync(subPath, atime, mtime);
    });
  }
  async mkdirPromise(p, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.mkdirPromise(p, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.mkdirPromise(subPath, opts);
    });
  }
  mkdirSync(p, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.mkdirSync(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.mkdirSync(subPath, opts);
    });
  }
  async rmdirPromise(p, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.rmdirPromise(p, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.rmdirPromise(subPath, opts);
    });
  }
  rmdirSync(p, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.rmdirSync(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.rmdirSync(subPath, opts);
    });
  }
  async linkPromise(existingP, newP) {
    return await this.makeCallPromise(newP, async () => {
      return await this.baseFs.linkPromise(existingP, newP);
    }, async (zipFs, { subPath }) => {
      return await zipFs.linkPromise(existingP, subPath);
    });
  }
  linkSync(existingP, newP) {
    return this.makeCallSync(newP, () => {
      return this.baseFs.linkSync(existingP, newP);
    }, (zipFs, { subPath }) => {
      return zipFs.linkSync(existingP, subPath);
    });
  }
  async symlinkPromise(target, p, type) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.symlinkPromise(target, p, type);
    }, async (zipFs, { subPath }) => {
      return await zipFs.symlinkPromise(target, subPath);
    });
  }
  symlinkSync(target, p, type) {
    return this.makeCallSync(p, () => {
      return this.baseFs.symlinkSync(target, p, type);
    }, (zipFs, { subPath }) => {
      return zipFs.symlinkSync(target, subPath);
    });
  }
  async readFilePromise(p, encoding) {
    return this.makeCallPromise(p, async () => {
      switch (encoding) {
        case `utf8`:
          return await this.baseFs.readFilePromise(p, encoding);
        default:
          return await this.baseFs.readFilePromise(p, encoding);
      }
    }, async (zipFs, { subPath }) => {
      return await zipFs.readFilePromise(subPath, encoding);
    });
  }
  readFileSync(p, encoding) {
    return this.makeCallSync(p, () => {
      switch (encoding) {
        case `utf8`:
          return this.baseFs.readFileSync(p, encoding);
        default:
          return this.baseFs.readFileSync(p, encoding);
      }
    }, (zipFs, { subPath }) => {
      return zipFs.readFileSync(subPath, encoding);
    });
  }
  async readdirPromise(p, opts) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.readdirPromise(p, opts);
    }, async (zipFs, { subPath }) => {
      return await zipFs.readdirPromise(subPath, opts);
    }, {
      requireSubpath: false
    });
  }
  readdirSync(p, opts) {
    return this.makeCallSync(p, () => {
      return this.baseFs.readdirSync(p, opts);
    }, (zipFs, { subPath }) => {
      return zipFs.readdirSync(subPath, opts);
    }, {
      requireSubpath: false
    });
  }
  async readlinkPromise(p) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.readlinkPromise(p);
    }, async (zipFs, { subPath }) => {
      return await zipFs.readlinkPromise(subPath);
    });
  }
  readlinkSync(p) {
    return this.makeCallSync(p, () => {
      return this.baseFs.readlinkSync(p);
    }, (zipFs, { subPath }) => {
      return zipFs.readlinkSync(subPath);
    });
  }
  async truncatePromise(p, len) {
    return await this.makeCallPromise(p, async () => {
      return await this.baseFs.truncatePromise(p, len);
    }, async (zipFs, { subPath }) => {
      return await zipFs.truncatePromise(subPath, len);
    });
  }
  truncateSync(p, len) {
    return this.makeCallSync(p, () => {
      return this.baseFs.truncateSync(p, len);
    }, (zipFs, { subPath }) => {
      return zipFs.truncateSync(subPath, len);
    });
  }
  async ftruncatePromise(fd, len) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.ftruncatePromise(fd, len);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`ftruncate`);
    const [zipFs, realFd] = entry;
    return zipFs.ftruncatePromise(realFd, len);
  }
  ftruncateSync(fd, len) {
    if ((fd & ZIP_MASK) !== ZIP_MAGIC)
      return this.baseFs.ftruncateSync(fd, len);
    const entry = this.fdMap.get(fd);
    if (typeof entry === `undefined`)
      throw EBADF(`ftruncateSync`);
    const [zipFs, realFd] = entry;
    return zipFs.ftruncateSync(realFd, len);
  }
  watch(p, a, b) {
    return this.makeCallSync(p, () => {
      return this.baseFs.watch(
        p,
        a,
        b
      );
    }, (zipFs, { subPath }) => {
      return zipFs.watch(
        subPath,
        a,
        b
      );
    });
  }
  watchFile(p, a, b) {
    return this.makeCallSync(p, () => {
      return this.baseFs.watchFile(
        p,
        a,
        b
      );
    }, () => {
      return watchFile(this, p, a, b);
    });
  }
  unwatchFile(p, cb) {
    return this.makeCallSync(p, () => {
      return this.baseFs.unwatchFile(p, cb);
    }, () => {
      return unwatchFile(this, p, cb);
    });
  }
  async makeCallPromise(p, discard, accept, { requireSubpath = true } = {}) {
    if (typeof p !== `string`)
      return await discard();
    const normalizedP = this.resolve(p);
    const zipInfo = this.findZip(normalizedP);
    if (!zipInfo)
      return await discard();
    if (requireSubpath && zipInfo.subPath === `/`)
      return await discard();
    return await this.getZipPromise(zipInfo.archivePath, async (zipFs) => await accept(zipFs, zipInfo));
  }
  makeCallSync(p, discard, accept, { requireSubpath = true } = {}) {
    if (typeof p !== `string`)
      return discard();
    const normalizedP = this.resolve(p);
    const zipInfo = this.findZip(normalizedP);
    if (!zipInfo)
      return discard();
    if (requireSubpath && zipInfo.subPath === `/`)
      return discard();
    return this.getZipSync(zipInfo.archivePath, (zipFs) => accept(zipFs, zipInfo));
  }
  findZip(p) {
    if (this.filter && !this.filter.test(p))
      return null;
    let filePath = ``;
    while (true) {
      const pathPartWithArchive = p.substring(filePath.length);
      let archivePart;
      if (!this.fileExtensions) {
        archivePart = getArchivePart(pathPartWithArchive, `.zip`);
      } else {
        for (const ext of this.fileExtensions) {
          archivePart = getArchivePart(pathPartWithArchive, ext);
          if (archivePart) {
            break;
          }
        }
      }
      if (!archivePart)
        return null;
      filePath = this.pathUtils.join(filePath, archivePart);
      if (this.isZip.has(filePath) === false) {
        if (this.notZip.has(filePath))
          continue;
        try {
          if (!this.baseFs.lstatSync(filePath).isFile()) {
            this.notZip.add(filePath);
            continue;
          }
        } catch {
          return null;
        }
        this.isZip.add(filePath);
      }
      return {
        archivePath: filePath,
        subPath: this.pathUtils.join(PortablePath.root, p.substring(filePath.length))
      };
    }
  }
  limitOpenFiles(max) {
    if (this.zipInstances === null)
      return;
    const now = Date.now();
    let nextExpiresAt = now + this.maxAge;
    let closeCount = max === null ? 0 : this.zipInstances.size - max;
    for (const [path, { zipFs, expiresAt, refCount }] of this.zipInstances.entries()) {
      if (refCount !== 0 || zipFs.hasOpenFileHandles()) {
        continue;
      } else if (now >= expiresAt) {
        zipFs.saveAndClose();
        this.zipInstances.delete(path);
        closeCount -= 1;
        continue;
      } else if (max === null || closeCount <= 0) {
        nextExpiresAt = expiresAt;
        break;
      }
      zipFs.saveAndClose();
      this.zipInstances.delete(path);
      closeCount -= 1;
    }
    if (this.limitOpenFilesTimeout === null && (max === null && this.zipInstances.size > 0 || max !== null)) {
      this.limitOpenFilesTimeout = setTimeout(() => {
        this.limitOpenFilesTimeout = null;
        this.limitOpenFiles(null);
      }, nextExpiresAt - now).unref();
    }
  }
  async getZipPromise(p, accept) {
    const getZipOptions = async () => ({
      baseFs: this.baseFs,
      libzip: this.libzip,
      readOnly: this.readOnlyArchives,
      stats: await this.baseFs.statPromise(p)
    });
    if (this.zipInstances) {
      let cachedZipFs = this.zipInstances.get(p);
      if (!cachedZipFs) {
        const zipOptions = await getZipOptions();
        cachedZipFs = this.zipInstances.get(p);
        if (!cachedZipFs) {
          cachedZipFs = {
            zipFs: new ZipFS(p, zipOptions),
            expiresAt: 0,
            refCount: 0
          };
        }
      }
      this.zipInstances.delete(p);
      this.limitOpenFiles(this.maxOpenFiles - 1);
      this.zipInstances.set(p, cachedZipFs);
      cachedZipFs.expiresAt = Date.now() + this.maxAge;
      cachedZipFs.refCount += 1;
      try {
        return await accept(cachedZipFs.zipFs);
      } finally {
        cachedZipFs.refCount -= 1;
      }
    } else {
      const zipFs = new ZipFS(p, await getZipOptions());
      try {
        return await accept(zipFs);
      } finally {
        zipFs.saveAndClose();
      }
    }
  }
  getZipSync(p, accept) {
    const getZipOptions = () => ({
      baseFs: this.baseFs,
      libzip: this.libzip,
      readOnly: this.readOnlyArchives,
      stats: this.baseFs.statSync(p)
    });
    if (this.zipInstances) {
      let cachedZipFs = this.zipInstances.get(p);
      if (!cachedZipFs) {
        cachedZipFs = {
          zipFs: new ZipFS(p, getZipOptions()),
          expiresAt: 0,
          refCount: 0
        };
      }
      this.zipInstances.delete(p);
      this.limitOpenFiles(this.maxOpenFiles - 1);
      this.zipInstances.set(p, cachedZipFs);
      cachedZipFs.expiresAt = Date.now() + this.maxAge;
      return accept(cachedZipFs.zipFs);
    } else {
      const zipFs = new ZipFS(p, getZipOptions());
      try {
        return accept(zipFs);
      } finally {
        zipFs.saveAndClose();
      }
    }
  }
}

class NodePathFS extends ProxiedFS {
  constructor(baseFs) {
    super(npath);
    this.baseFs = baseFs;
  }
  mapFromBase(path) {
    return path;
  }
  mapToBase(path) {
    if (typeof path === `string`)
      return path;
    if (path instanceof url.URL)
      return url.fileURLToPath(path);
    if (Buffer.isBuffer(path)) {
      const str = path.toString();
      if (Buffer.byteLength(str) !== path.byteLength)
        throw new Error(`Non-utf8 buffers are not supported at the moment. Please upvote the following issue if you encounter this error: https://github.com/yarnpkg/berry/issues/4942`);
      return str;
    }
    throw new Error(`Unsupported path type: ${nodeUtils.inspect(path)}`);
  }
}

var _a, _b, _c, _d;
const kBaseFs = Symbol(`kBaseFs`);
const kFd = Symbol(`kFd`);
const kClosePromise = Symbol(`kClosePromise`);
const kCloseResolve = Symbol(`kCloseResolve`);
const kCloseReject = Symbol(`kCloseReject`);
const kRefs = Symbol(`kRefs`);
const kRef = Symbol(`kRef`);
const kUnref = Symbol(`kUnref`);
class FileHandle {
  constructor(fd, baseFs) {
    this[_a] = 1;
    this[_b] = void 0;
    this[_c] = void 0;
    this[_d] = void 0;
    this[kBaseFs] = baseFs;
    this[kFd] = fd;
  }
  get fd() {
    return this[kFd];
  }
  async appendFile(data, options) {
    var _a2;
    try {
      this[kRef](this.appendFile);
      const encoding = (_a2 = typeof options === `string` ? options : options == null ? void 0 : options.encoding) != null ? _a2 : void 0;
      return await this[kBaseFs].appendFilePromise(this.fd, data, encoding ? { encoding } : void 0);
    } finally {
      this[kUnref]();
    }
  }
  async chown(uid, gid) {
    try {
      this[kRef](this.chown);
      return await this[kBaseFs].fchownPromise(this.fd, uid, gid);
    } finally {
      this[kUnref]();
    }
  }
  async chmod(mode) {
    try {
      this[kRef](this.chmod);
      return await this[kBaseFs].fchmodPromise(this.fd, mode);
    } finally {
      this[kUnref]();
    }
  }
  createReadStream(options) {
    return this[kBaseFs].createReadStream(null, { ...options, fd: this.fd });
  }
  createWriteStream(options) {
    return this[kBaseFs].createWriteStream(null, { ...options, fd: this.fd });
  }
  datasync() {
    throw new Error(`Method not implemented.`);
  }
  sync() {
    throw new Error(`Method not implemented.`);
  }
  async read(bufferOrOptions, offset, length, position) {
    var _a2, _b2, _c2;
    try {
      this[kRef](this.read);
      let buffer;
      if (!Buffer.isBuffer(bufferOrOptions)) {
        bufferOrOptions != null ? bufferOrOptions : bufferOrOptions = {};
        buffer = (_a2 = bufferOrOptions.buffer) != null ? _a2 : Buffer.alloc(16384);
        offset = bufferOrOptions.offset || 0;
        length = (_b2 = bufferOrOptions.length) != null ? _b2 : buffer.byteLength;
        position = (_c2 = bufferOrOptions.position) != null ? _c2 : null;
      } else {
        buffer = bufferOrOptions;
      }
      offset != null ? offset : offset = 0;
      length != null ? length : length = 0;
      if (length === 0) {
        return {
          bytesRead: length,
          buffer
        };
      }
      const bytesRead = await this[kBaseFs].readPromise(this.fd, buffer, offset, length, position);
      return {
        bytesRead,
        buffer
      };
    } finally {
      this[kUnref]();
    }
  }
  async readFile(options) {
    var _a2;
    try {
      this[kRef](this.readFile);
      const encoding = (_a2 = typeof options === `string` ? options : options == null ? void 0 : options.encoding) != null ? _a2 : void 0;
      return await this[kBaseFs].readFilePromise(this.fd, encoding);
    } finally {
      this[kUnref]();
    }
  }
  readLines(options) {
    return readline.createInterface({
      input: this.createReadStream(options),
      crlfDelay: Infinity
    });
  }
  async stat(opts) {
    try {
      this[kRef](this.stat);
      return await this[kBaseFs].fstatPromise(this.fd, opts);
    } finally {
      this[kUnref]();
    }
  }
  async truncate(len) {
    try {
      this[kRef](this.truncate);
      return await this[kBaseFs].ftruncatePromise(this.fd, len);
    } finally {
      this[kUnref]();
    }
  }
  utimes(atime, mtime) {
    throw new Error(`Method not implemented.`);
  }
  async writeFile(data, options) {
    var _a2;
    try {
      this[kRef](this.writeFile);
      const encoding = (_a2 = typeof options === `string` ? options : options == null ? void 0 : options.encoding) != null ? _a2 : void 0;
      await this[kBaseFs].writeFilePromise(this.fd, data, encoding);
    } finally {
      this[kUnref]();
    }
  }
  async write(...args) {
    try {
      this[kRef](this.write);
      if (ArrayBuffer.isView(args[0])) {
        const [buffer, offset, length, position] = args;
        const bytesWritten = await this[kBaseFs].writePromise(this.fd, buffer, offset != null ? offset : void 0, length != null ? length : void 0, position != null ? position : void 0);
        return { bytesWritten, buffer };
      } else {
        const [data, position, encoding] = args;
        const bytesWritten = await this[kBaseFs].writePromise(this.fd, data, position, encoding);
        return { bytesWritten, buffer: data };
      }
    } finally {
      this[kUnref]();
    }
  }
  async writev(buffers, position) {
    try {
      this[kRef](this.writev);
      let bytesWritten = 0;
      if (typeof position !== `undefined`) {
        for (const buffer of buffers) {
          const writeResult = await this.write(buffer, void 0, void 0, position);
          bytesWritten += writeResult.bytesWritten;
          position += writeResult.bytesWritten;
        }
      } else {
        for (const buffer of buffers) {
          const writeResult = await this.write(buffer);
          bytesWritten += writeResult.bytesWritten;
        }
      }
      return {
        buffers,
        bytesWritten
      };
    } finally {
      this[kUnref]();
    }
  }
  readv(buffers, position) {
    throw new Error(`Method not implemented.`);
  }
  close() {
    if (this[kFd] === -1)
      return Promise.resolve();
    if (this[kClosePromise])
      return this[kClosePromise];
    this[kRefs]--;
    if (this[kRefs] === 0) {
      const fd = this[kFd];
      this[kFd] = -1;
      this[kClosePromise] = this[kBaseFs].closePromise(fd).finally(() => {
        this[kClosePromise] = void 0;
      });
    } else {
      this[kClosePromise] = new Promise((resolve, reject) => {
        this[kCloseResolve] = resolve;
        this[kCloseReject] = reject;
      }).finally(() => {
        this[kClosePromise] = void 0;
        this[kCloseReject] = void 0;
        this[kCloseResolve] = void 0;
      });
    }
    return this[kClosePromise];
  }
  [(_a = kRefs, _b = kClosePromise, _c = kCloseResolve, _d = kCloseReject, kRef)](caller) {
    if (this[kFd] === -1) {
      const err = new Error(`file closed`);
      err.code = `EBADF`;
      err.syscall = caller.name;
      throw err;
    }
    this[kRefs]++;
  }
  [kUnref]() {
    this[kRefs]--;
    if (this[kRefs] === 0) {
      const fd = this[kFd];
      this[kFd] = -1;
      this[kBaseFs].closePromise(fd).then(this[kCloseResolve], this[kCloseReject]);
    }
  }
}

const SYNC_IMPLEMENTATIONS = /* @__PURE__ */ new Set([
  `accessSync`,
  `appendFileSync`,
  `createReadStream`,
  `createWriteStream`,
  `chmodSync`,
  `fchmodSync`,
  `chownSync`,
  `fchownSync`,
  `closeSync`,
  `copyFileSync`,
  `linkSync`,
  `lstatSync`,
  `fstatSync`,
  `lutimesSync`,
  `mkdirSync`,
  `openSync`,
  `opendirSync`,
  `readlinkSync`,
  `readFileSync`,
  `readdirSync`,
  `readlinkSync`,
  `realpathSync`,
  `renameSync`,
  `rmdirSync`,
  `statSync`,
  `symlinkSync`,
  `truncateSync`,
  `ftruncateSync`,
  `unlinkSync`,
  `unwatchFile`,
  `utimesSync`,
  `watch`,
  `watchFile`,
  `writeFileSync`,
  `writeSync`
]);
const ASYNC_IMPLEMENTATIONS = /* @__PURE__ */ new Set([
  `accessPromise`,
  `appendFilePromise`,
  `fchmodPromise`,
  `chmodPromise`,
  `fchownPromise`,
  `chownPromise`,
  `closePromise`,
  `copyFilePromise`,
  `linkPromise`,
  `fstatPromise`,
  `lstatPromise`,
  `lutimesPromise`,
  `mkdirPromise`,
  `openPromise`,
  `opendirPromise`,
  `readdirPromise`,
  `realpathPromise`,
  `readFilePromise`,
  `readdirPromise`,
  `readlinkPromise`,
  `renamePromise`,
  `rmdirPromise`,
  `statPromise`,
  `symlinkPromise`,
  `truncatePromise`,
  `ftruncatePromise`,
  `unlinkPromise`,
  `utimesPromise`,
  `writeFilePromise`,
  `writeSync`
]);
function patchFs(patchedFs, fakeFs) {
  fakeFs = new NodePathFS(fakeFs);
  const setupFn = (target, name, replacement) => {
    const orig = target[name];
    target[name] = replacement;
    if (typeof (orig == null ? void 0 : orig[nodeUtils.promisify.custom]) !== `undefined`) {
      replacement[nodeUtils.promisify.custom] = orig[nodeUtils.promisify.custom];
    }
  };
  {
    setupFn(patchedFs, `exists`, (p, ...args) => {
      const hasCallback = typeof args[args.length - 1] === `function`;
      const callback = hasCallback ? args.pop() : () => {
      };
      process.nextTick(() => {
        fakeFs.existsPromise(p).then((exists) => {
          callback(exists);
        }, () => {
          callback(false);
        });
      });
    });
    setupFn(patchedFs, `read`, (...args) => {
      let [fd, buffer, offset, length, position, callback] = args;
      if (args.length <= 3) {
        let options = {};
        if (args.length < 3) {
          callback = args[1];
        } else {
          options = args[1];
          callback = args[2];
        }
        ({
          buffer = Buffer.alloc(16384),
          offset = 0,
          length = buffer.byteLength,
          position
        } = options);
      }
      if (offset == null)
        offset = 0;
      length |= 0;
      if (length === 0) {
        process.nextTick(() => {
          callback(null, 0, buffer);
        });
        return;
      }
      if (position == null)
        position = -1;
      process.nextTick(() => {
        fakeFs.readPromise(fd, buffer, offset, length, position).then((bytesRead) => {
          callback(null, bytesRead, buffer);
        }, (error) => {
          callback(error, 0, buffer);
        });
      });
    });
    for (const fnName of ASYNC_IMPLEMENTATIONS) {
      const origName = fnName.replace(/Promise$/, ``);
      if (typeof patchedFs[origName] === `undefined`)
        continue;
      const fakeImpl = fakeFs[fnName];
      if (typeof fakeImpl === `undefined`)
        continue;
      const wrapper = (...args) => {
        const hasCallback = typeof args[args.length - 1] === `function`;
        const callback = hasCallback ? args.pop() : () => {
        };
        process.nextTick(() => {
          fakeImpl.apply(fakeFs, args).then((result) => {
            callback(null, result);
          }, (error) => {
            callback(error);
          });
        });
      };
      setupFn(patchedFs, origName, wrapper);
    }
    patchedFs.realpath.native = patchedFs.realpath;
  }
  {
    setupFn(patchedFs, `existsSync`, (p) => {
      try {
        return fakeFs.existsSync(p);
      } catch (error) {
        return false;
      }
    });
    setupFn(patchedFs, `readSync`, (...args) => {
      let [fd, buffer, offset, length, position] = args;
      if (args.length <= 3) {
        const options = args[2] || {};
        ({ offset = 0, length = buffer.byteLength, position } = options);
      }
      if (offset == null)
        offset = 0;
      length |= 0;
      if (length === 0)
        return 0;
      if (position == null)
        position = -1;
      return fakeFs.readSync(fd, buffer, offset, length, position);
    });
    for (const fnName of SYNC_IMPLEMENTATIONS) {
      const origName = fnName;
      if (typeof patchedFs[origName] === `undefined`)
        continue;
      const fakeImpl = fakeFs[fnName];
      if (typeof fakeImpl === `undefined`)
        continue;
      setupFn(patchedFs, origName, fakeImpl.bind(fakeFs));
    }
    patchedFs.realpathSync.native = patchedFs.realpathSync;
  }
  {
    const origEmitWarning = process.emitWarning;
    process.emitWarning = () => {
    };
    let patchedFsPromises;
    try {
      patchedFsPromises = patchedFs.promises;
    } finally {
      process.emitWarning = origEmitWarning;
    }
    if (typeof patchedFsPromises !== `undefined`) {
      for (const fnName of ASYNC_IMPLEMENTATIONS) {
        const origName = fnName.replace(/Promise$/, ``);
        if (typeof patchedFsPromises[origName] === `undefined`)
          continue;
        const fakeImpl = fakeFs[fnName];
        if (typeof fakeImpl === `undefined`)
          continue;
        if (fnName === `open`)
          continue;
        setupFn(patchedFsPromises, origName, (pathLike, ...args) => {
          if (pathLike instanceof FileHandle) {
            return pathLike[origName].apply(pathLike, args);
          } else {
            return fakeImpl.call(fakeFs, pathLike, ...args);
          }
        });
      }
      setupFn(patchedFsPromises, `open`, async (...args) => {
        const fd = await fakeFs.openPromise(...args);
        return new FileHandle(fd, fakeFs);
      });
    }
  }
  {
    patchedFs.read[nodeUtils.promisify.custom] = async (fd, buffer, ...args) => {
      const res = fakeFs.readPromise(fd, buffer, ...args);
      return { bytesRead: await res, buffer };
    };
    patchedFs.write[nodeUtils.promisify.custom] = async (fd, buffer, ...args) => {
      const res = fakeFs.writePromise(fd, buffer, ...args);
      return { bytesWritten: await res, buffer };
    };
  }
}

var libzipSync = {exports: {}};

(function (module, exports) {
var frozenFs = Object.assign({}, fs__default.default);
var createModule = function() {
  var _scriptDir = void 0;
  if (typeof __filename !== "undefined")
    _scriptDir = _scriptDir || __filename;
  return function(createModule2) {
    createModule2 = createModule2 || {};
    var Module = typeof createModule2 !== "undefined" ? createModule2 : {};
    var readyPromiseResolve, readyPromiseReject;
    Module["ready"] = new Promise(function(resolve, reject) {
      readyPromiseResolve = resolve;
      readyPromiseReject = reject;
    });
    var moduleOverrides = {};
    var key;
    for (key in Module) {
      if (Module.hasOwnProperty(key)) {
        moduleOverrides[key] = Module[key];
      }
    }
    var scriptDirectory = "";
    function locateFile(path) {
      if (Module["locateFile"]) {
        return Module["locateFile"](path, scriptDirectory);
      }
      return scriptDirectory + path;
    }
    var read_, readBinary;
    var nodeFS;
    var nodePath;
    {
      {
        scriptDirectory = __dirname + "/";
      }
      read_ = function shell_read(filename, binary) {
        var ret = tryParseAsDataURI(filename);
        if (ret) {
          return binary ? ret : ret.toString();
        }
        if (!nodeFS)
          nodeFS = frozenFs;
        if (!nodePath)
          nodePath = path__default.default;
        filename = nodePath["normalize"](filename);
        return nodeFS["readFileSync"](filename, binary ? null : "utf8");
      };
      readBinary = function readBinary2(filename) {
        var ret = read_(filename, true);
        if (!ret.buffer) {
          ret = new Uint8Array(ret);
        }
        assert(ret.buffer);
        return ret;
      };
      if (process["argv"].length > 1) {
        process["argv"][1].replace(/\\/g, "/");
      }
      process["argv"].slice(2);
      Module["inspect"] = function() {
        return "[Emscripten Module object]";
      };
    }
    var out = Module["print"] || console.log.bind(console);
    var err = Module["printErr"] || console.warn.bind(console);
    for (key in moduleOverrides) {
      if (moduleOverrides.hasOwnProperty(key)) {
        Module[key] = moduleOverrides[key];
      }
    }
    moduleOverrides = null;
    if (Module["arguments"])
      ;
    if (Module["thisProgram"])
      ;
    if (Module["quit"])
      ;
    var STACK_ALIGN = 16;
    function alignMemory(size, factor) {
      if (!factor)
        factor = STACK_ALIGN;
      return Math.ceil(size / factor) * factor;
    }
    var wasmBinary;
    if (Module["wasmBinary"])
      wasmBinary = Module["wasmBinary"];
    Module["noExitRuntime"] || true;
    if (typeof WebAssembly !== "object") {
      abort("no native wasm support detected");
    }
    function getValue(ptr, type, noSafe) {
      type = type || "i8";
      if (type.charAt(type.length - 1) === "*")
        type = "i32";
      switch (type) {
        case "i1":
          return HEAP8[ptr >> 0];
        case "i8":
          return HEAP8[ptr >> 0];
        case "i16":
          return HEAP16[ptr >> 1];
        case "i32":
          return HEAP32[ptr >> 2];
        case "i64":
          return HEAP32[ptr >> 2];
        case "float":
          return HEAPF32[ptr >> 2];
        case "double":
          return HEAPF64[ptr >> 3];
        default:
          abort("invalid type for getValue: " + type);
      }
      return null;
    }
    var wasmMemory;
    var ABORT = false;
    function assert(condition, text) {
      if (!condition) {
        abort("Assertion failed: " + text);
      }
    }
    function getCFunc(ident) {
      var func = Module["_" + ident];
      assert(
        func,
        "Cannot call unknown function " + ident + ", make sure it is exported"
      );
      return func;
    }
    function ccall(ident, returnType, argTypes, args, opts) {
      var toC = {
        string: function(str) {
          var ret2 = 0;
          if (str !== null && str !== void 0 && str !== 0) {
            var len = (str.length << 2) + 1;
            ret2 = stackAlloc(len);
            stringToUTF8(str, ret2, len);
          }
          return ret2;
        },
        array: function(arr) {
          var ret2 = stackAlloc(arr.length);
          writeArrayToMemory(arr, ret2);
          return ret2;
        }
      };
      function convertReturnValue(ret2) {
        if (returnType === "string")
          return UTF8ToString(ret2);
        if (returnType === "boolean")
          return Boolean(ret2);
        return ret2;
      }
      var func = getCFunc(ident);
      var cArgs = [];
      var stack = 0;
      if (args) {
        for (var i = 0; i < args.length; i++) {
          var converter = toC[argTypes[i]];
          if (converter) {
            if (stack === 0)
              stack = stackSave();
            cArgs[i] = converter(args[i]);
          } else {
            cArgs[i] = args[i];
          }
        }
      }
      var ret = func.apply(null, cArgs);
      ret = convertReturnValue(ret);
      if (stack !== 0)
        stackRestore(stack);
      return ret;
    }
    function cwrap(ident, returnType, argTypes, opts) {
      argTypes = argTypes || [];
      var numericArgs = argTypes.every(function(type) {
        return type === "number";
      });
      var numericRet = returnType !== "string";
      if (numericRet && numericArgs && !opts) {
        return getCFunc(ident);
      }
      return function() {
        return ccall(ident, returnType, argTypes, arguments);
      };
    }
    var UTF8Decoder = typeof TextDecoder !== "undefined" ? new TextDecoder("utf8") : void 0;
    function UTF8ArrayToString(heap, idx, maxBytesToRead) {
      var endIdx = idx + maxBytesToRead;
      var endPtr = idx;
      while (heap[endPtr] && !(endPtr >= endIdx))
        ++endPtr;
      if (endPtr - idx > 16 && heap.subarray && UTF8Decoder) {
        return UTF8Decoder.decode(heap.subarray(idx, endPtr));
      } else {
        var str = "";
        while (idx < endPtr) {
          var u0 = heap[idx++];
          if (!(u0 & 128)) {
            str += String.fromCharCode(u0);
            continue;
          }
          var u1 = heap[idx++] & 63;
          if ((u0 & 224) == 192) {
            str += String.fromCharCode((u0 & 31) << 6 | u1);
            continue;
          }
          var u2 = heap[idx++] & 63;
          if ((u0 & 240) == 224) {
            u0 = (u0 & 15) << 12 | u1 << 6 | u2;
          } else {
            u0 = (u0 & 7) << 18 | u1 << 12 | u2 << 6 | heap[idx++] & 63;
          }
          if (u0 < 65536) {
            str += String.fromCharCode(u0);
          } else {
            var ch = u0 - 65536;
            str += String.fromCharCode(55296 | ch >> 10, 56320 | ch & 1023);
          }
        }
      }
      return str;
    }
    function UTF8ToString(ptr, maxBytesToRead) {
      return ptr ? UTF8ArrayToString(HEAPU8, ptr, maxBytesToRead) : "";
    }
    function stringToUTF8Array(str, heap, outIdx, maxBytesToWrite) {
      if (!(maxBytesToWrite > 0))
        return 0;
      var startIdx = outIdx;
      var endIdx = outIdx + maxBytesToWrite - 1;
      for (var i = 0; i < str.length; ++i) {
        var u = str.charCodeAt(i);
        if (u >= 55296 && u <= 57343) {
          var u1 = str.charCodeAt(++i);
          u = 65536 + ((u & 1023) << 10) | u1 & 1023;
        }
        if (u <= 127) {
          if (outIdx >= endIdx)
            break;
          heap[outIdx++] = u;
        } else if (u <= 2047) {
          if (outIdx + 1 >= endIdx)
            break;
          heap[outIdx++] = 192 | u >> 6;
          heap[outIdx++] = 128 | u & 63;
        } else if (u <= 65535) {
          if (outIdx + 2 >= endIdx)
            break;
          heap[outIdx++] = 224 | u >> 12;
          heap[outIdx++] = 128 | u >> 6 & 63;
          heap[outIdx++] = 128 | u & 63;
        } else {
          if (outIdx + 3 >= endIdx)
            break;
          heap[outIdx++] = 240 | u >> 18;
          heap[outIdx++] = 128 | u >> 12 & 63;
          heap[outIdx++] = 128 | u >> 6 & 63;
          heap[outIdx++] = 128 | u & 63;
        }
      }
      heap[outIdx] = 0;
      return outIdx - startIdx;
    }
    function stringToUTF8(str, outPtr, maxBytesToWrite) {
      return stringToUTF8Array(str, HEAPU8, outPtr, maxBytesToWrite);
    }
    function lengthBytesUTF8(str) {
      var len = 0;
      for (var i = 0; i < str.length; ++i) {
        var u = str.charCodeAt(i);
        if (u >= 55296 && u <= 57343)
          u = 65536 + ((u & 1023) << 10) | str.charCodeAt(++i) & 1023;
        if (u <= 127)
          ++len;
        else if (u <= 2047)
          len += 2;
        else if (u <= 65535)
          len += 3;
        else
          len += 4;
      }
      return len;
    }
    function allocateUTF8(str) {
      var size = lengthBytesUTF8(str) + 1;
      var ret = _malloc(size);
      if (ret)
        stringToUTF8Array(str, HEAP8, ret, size);
      return ret;
    }
    function writeArrayToMemory(array, buffer2) {
      HEAP8.set(array, buffer2);
    }
    function alignUp(x, multiple) {
      if (x % multiple > 0) {
        x += multiple - x % multiple;
      }
      return x;
    }
    var buffer, HEAP8, HEAPU8, HEAP16, HEAP32, HEAPF32, HEAPF64;
    function updateGlobalBufferAndViews(buf) {
      buffer = buf;
      Module["HEAP8"] = HEAP8 = new Int8Array(buf);
      Module["HEAP16"] = HEAP16 = new Int16Array(buf);
      Module["HEAP32"] = HEAP32 = new Int32Array(buf);
      Module["HEAPU8"] = HEAPU8 = new Uint8Array(buf);
      Module["HEAPU16"] = new Uint16Array(buf);
      Module["HEAPU32"] = new Uint32Array(buf);
      Module["HEAPF32"] = HEAPF32 = new Float32Array(buf);
      Module["HEAPF64"] = HEAPF64 = new Float64Array(buf);
    }
    Module["INITIAL_MEMORY"] || 16777216;
    var wasmTable;
    var __ATPRERUN__ = [];
    var __ATINIT__ = [];
    var __ATPOSTRUN__ = [];
    function preRun() {
      if (Module["preRun"]) {
        if (typeof Module["preRun"] == "function")
          Module["preRun"] = [Module["preRun"]];
        while (Module["preRun"].length) {
          addOnPreRun(Module["preRun"].shift());
        }
      }
      callRuntimeCallbacks(__ATPRERUN__);
    }
    function initRuntime() {
      if (!Module["noFSInit"] && !FS.init.initialized)
        FS.init();
      callRuntimeCallbacks(__ATINIT__);
    }
    function postRun() {
      if (Module["postRun"]) {
        if (typeof Module["postRun"] == "function")
          Module["postRun"] = [Module["postRun"]];
        while (Module["postRun"].length) {
          addOnPostRun(Module["postRun"].shift());
        }
      }
      callRuntimeCallbacks(__ATPOSTRUN__);
    }
    function addOnPreRun(cb) {
      __ATPRERUN__.unshift(cb);
    }
    function addOnInit(cb) {
      __ATINIT__.unshift(cb);
    }
    function addOnPostRun(cb) {
      __ATPOSTRUN__.unshift(cb);
    }
    var runDependencies = 0;
    var dependenciesFulfilled = null;
    function addRunDependency(id) {
      runDependencies++;
      if (Module["monitorRunDependencies"]) {
        Module["monitorRunDependencies"](runDependencies);
      }
    }
    function removeRunDependency(id) {
      runDependencies--;
      if (Module["monitorRunDependencies"]) {
        Module["monitorRunDependencies"](runDependencies);
      }
      if (runDependencies == 0) {
        if (dependenciesFulfilled) {
          var callback = dependenciesFulfilled;
          dependenciesFulfilled = null;
          callback();
        }
      }
    }
    Module["preloadedImages"] = {};
    Module["preloadedAudios"] = {};
    function abort(what) {
      if (Module["onAbort"]) {
        Module["onAbort"](what);
      }
      what += "";
      err(what);
      ABORT = true;
      what = "abort(" + what + "). Build with -s ASSERTIONS=1 for more info.";
      var e = new WebAssembly.RuntimeError(what);
      readyPromiseReject(e);
      throw e;
    }
    var dataURIPrefix = "data:application/octet-stream;base64,";
    function isDataURI(filename) {
      return filename.startsWith(dataURIPrefix);
    }
    var wasmBinaryFile = "data:application/octet-stream;base64, ";
    if (!isDataURI(wasmBinaryFile)) {
      wasmBinaryFile = locateFile(wasmBinaryFile);
    }
    function getBinary(file) {
      try {
        if (file == wasmBinaryFile && wasmBinary) {
          return new Uint8Array(wasmBinary);
        }
        var binary = tryParseAsDataURI(file);
        if (binary) {
          return binary;
        }
        if (readBinary) {
          return readBinary(file);
        } else {
          throw "sync fetching of the wasm failed: you can preload it to Module['wasmBinary'] manually, or emcc.py will do that for you when generating HTML (but not JS)";
        }
      } catch (err2) {
        abort(err2);
      }
    }
    function instantiateSync(file, info) {
      var instance;
      var module2;
      var binary;
      try {
        binary = getBinary(file);
        module2 = new WebAssembly.Module(binary);
        instance = new WebAssembly.Instance(module2, info);
      } catch (e) {
        var str = e.toString();
        err("failed to compile wasm module: " + str);
        if (str.includes("imported Memory") || str.includes("memory import")) {
          err(
            "Memory size incompatibility issues may be due to changing INITIAL_MEMORY at runtime to something too large. Use ALLOW_MEMORY_GROWTH to allow any size memory (and also make sure not to set INITIAL_MEMORY at runtime to something smaller than it was at compile time)."
          );
        }
        throw e;
      }
      return [instance, module2];
    }
    function createWasm() {
      var info = { a: asmLibraryArg };
      function receiveInstance(instance, module2) {
        var exports3 = instance.exports;
        Module["asm"] = exports3;
        wasmMemory = Module["asm"]["u"];
        updateGlobalBufferAndViews(wasmMemory.buffer);
        wasmTable = Module["asm"]["pa"];
        addOnInit(Module["asm"]["v"]);
        removeRunDependency();
      }
      addRunDependency();
      if (Module["instantiateWasm"]) {
        try {
          var exports2 = Module["instantiateWasm"](info, receiveInstance);
          return exports2;
        } catch (e) {
          err("Module.instantiateWasm callback failed with error: " + e);
          return false;
        }
      }
      var result = instantiateSync(wasmBinaryFile, info);
      receiveInstance(result[0]);
      return Module["asm"];
    }
    var tempDouble;
    var tempI64;
    function callRuntimeCallbacks(callbacks) {
      while (callbacks.length > 0) {
        var callback = callbacks.shift();
        if (typeof callback == "function") {
          callback(Module);
          continue;
        }
        var func = callback.func;
        if (typeof func === "number") {
          if (callback.arg === void 0) {
            wasmTable.get(func)();
          } else {
            wasmTable.get(func)(callback.arg);
          }
        } else {
          func(callback.arg === void 0 ? null : callback.arg);
        }
      }
    }
    function _gmtime_r(time, tmPtr) {
      var date = new Date(HEAP32[time >> 2] * 1e3);
      HEAP32[tmPtr >> 2] = date.getUTCSeconds();
      HEAP32[tmPtr + 4 >> 2] = date.getUTCMinutes();
      HEAP32[tmPtr + 8 >> 2] = date.getUTCHours();
      HEAP32[tmPtr + 12 >> 2] = date.getUTCDate();
      HEAP32[tmPtr + 16 >> 2] = date.getUTCMonth();
      HEAP32[tmPtr + 20 >> 2] = date.getUTCFullYear() - 1900;
      HEAP32[tmPtr + 24 >> 2] = date.getUTCDay();
      HEAP32[tmPtr + 36 >> 2] = 0;
      HEAP32[tmPtr + 32 >> 2] = 0;
      var start = Date.UTC(date.getUTCFullYear(), 0, 1, 0, 0, 0, 0);
      var yday = (date.getTime() - start) / (1e3 * 60 * 60 * 24) | 0;
      HEAP32[tmPtr + 28 >> 2] = yday;
      if (!_gmtime_r.GMTString)
        _gmtime_r.GMTString = allocateUTF8("GMT");
      HEAP32[tmPtr + 40 >> 2] = _gmtime_r.GMTString;
      return tmPtr;
    }
    function ___gmtime_r(a0, a1) {
      return _gmtime_r(a0, a1);
    }
    var PATH = {
      splitPath: function(filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },
      normalizeArray: function(parts, allowAboveRoot) {
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === ".") {
            parts.splice(i, 1);
          } else if (last === "..") {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        if (allowAboveRoot) {
          for (; up; up--) {
            parts.unshift("..");
          }
        }
        return parts;
      },
      normalize: function(path) {
        var isAbsolute = path.charAt(0) === "/", trailingSlash = path.substr(-1) === "/";
        path = PATH.normalizeArray(
          path.split("/").filter(function(p) {
            return !!p;
          }),
          !isAbsolute
        ).join("/");
        if (!path && !isAbsolute) {
          path = ".";
        }
        if (path && trailingSlash) {
          path += "/";
        }
        return (isAbsolute ? "/" : "") + path;
      },
      dirname: function(path) {
        var result = PATH.splitPath(path), root = result[0], dir = result[1];
        if (!root && !dir) {
          return ".";
        }
        if (dir) {
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },
      basename: function(path) {
        if (path === "/")
          return "/";
        path = PATH.normalize(path);
        path = path.replace(/\/$/, "");
        var lastSlash = path.lastIndexOf("/");
        if (lastSlash === -1)
          return path;
        return path.substr(lastSlash + 1);
      },
      extname: function(path) {
        return PATH.splitPath(path)[3];
      },
      join: function() {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join("/"));
      },
      join2: function(l, r) {
        return PATH.normalize(l + "/" + r);
      }
    };
    function getRandomDevice() {
      {
        try {
          var crypto_module = require("crypto");
          return function() {
            return crypto_module["randomBytes"](1)[0];
          };
        } catch (e) {
        }
      }
      return function() {
        abort("randomDevice");
      };
    }
    var PATH_FS = {
      resolve: function() {
        var resolvedPath = "", resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = i >= 0 ? arguments[i] : FS.cwd();
          if (typeof path !== "string") {
            throw new TypeError("Arguments to path.resolve must be strings");
          } else if (!path) {
            return "";
          }
          resolvedPath = path + "/" + resolvedPath;
          resolvedAbsolute = path.charAt(0) === "/";
        }
        resolvedPath = PATH.normalizeArray(
          resolvedPath.split("/").filter(function(p) {
            return !!p;
          }),
          !resolvedAbsolute
        ).join("/");
        return (resolvedAbsolute ? "/" : "") + resolvedPath || ".";
      },
      relative: function(from, to) {
        from = PATH_FS.resolve(from).substr(1);
        to = PATH_FS.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== "")
              break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== "")
              break;
          }
          if (start > end)
            return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split("/"));
        var toParts = trim(to.split("/"));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push("..");
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join("/");
      }
    };
    var TTY = {
      ttys: [],
      init: function() {
      },
      shutdown: function() {
      },
      register: function(dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },
      stream_ops: {
        open: function(stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(43);
          }
          stream.tty = tty;
          stream.seekable = false;
        },
        close: function(stream) {
          stream.tty.ops.flush(stream.tty);
        },
        flush: function(stream) {
          stream.tty.ops.flush(stream.tty);
        },
        read: function(stream, buffer2, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(60);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(29);
            }
            if (result === void 0 && bytesRead === 0) {
              throw new FS.ErrnoError(6);
            }
            if (result === null || result === void 0)
              break;
            bytesRead++;
            buffer2[offset + i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },
        write: function(stream, buffer2, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(60);
          }
          try {
            for (var i = 0; i < length; i++) {
              stream.tty.ops.put_char(stream.tty, buffer2[offset + i]);
            }
          } catch (e) {
            throw new FS.ErrnoError(29);
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }
      },
      default_tty_ops: {
        get_char: function(tty) {
          if (!tty.input.length) {
            var result = null;
            {
              var BUFSIZE = 256;
              var buf = Buffer.alloc ? Buffer.alloc(BUFSIZE) : new Buffer(BUFSIZE);
              var bytesRead = 0;
              try {
                bytesRead = nodeFS.readSync(
                  process.stdin.fd,
                  buf,
                  0,
                  BUFSIZE,
                  null
                );
              } catch (e) {
                if (e.toString().includes("EOF"))
                  bytesRead = 0;
                else
                  throw e;
              }
              if (bytesRead > 0) {
                result = buf.slice(0, bytesRead).toString("utf-8");
              } else {
                result = null;
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },
        put_char: function(tty, val) {
          if (val === null || val === 10) {
            out(UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0)
              tty.output.push(val);
          }
        },
        flush: function(tty) {
          if (tty.output && tty.output.length > 0) {
            out(UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }
      },
      default_tty1_ops: {
        put_char: function(tty, val) {
          if (val === null || val === 10) {
            err(UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0)
              tty.output.push(val);
          }
        },
        flush: function(tty) {
          if (tty.output && tty.output.length > 0) {
            err(UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }
      }
    };
    function mmapAlloc(size) {
      var alignedSize = alignMemory(size, 65536);
      var ptr = _malloc(alignedSize);
      while (size < alignedSize)
        HEAP8[ptr + size++] = 0;
      return ptr;
    }
    var MEMFS = {
      ops_table: null,
      mount: function(mount) {
        return MEMFS.createNode(null, "/", 16384 | 511, 0);
      },
      createNode: function(parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          throw new FS.ErrnoError(63);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: { llseek: MEMFS.stream_ops.llseek }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap,
                msync: MEMFS.stream_ops.msync
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0;
          node.contents = null;
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        if (parent) {
          parent.contents[name] = node;
          parent.timestamp = node.timestamp;
        }
        return node;
      },
      getFileDataAsTypedArray: function(node) {
        if (!node.contents)
          return new Uint8Array(0);
        if (node.contents.subarray)
          return node.contents.subarray(0, node.usedBytes);
        return new Uint8Array(node.contents);
      },
      expandFileStorage: function(node, newCapacity) {
        var prevCapacity = node.contents ? node.contents.length : 0;
        if (prevCapacity >= newCapacity)
          return;
        var CAPACITY_DOUBLING_MAX = 1024 * 1024;
        newCapacity = Math.max(
          newCapacity,
          prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2 : 1.125) >>> 0
        );
        if (prevCapacity != 0)
          newCapacity = Math.max(newCapacity, 256);
        var oldContents = node.contents;
        node.contents = new Uint8Array(newCapacity);
        if (node.usedBytes > 0)
          node.contents.set(oldContents.subarray(0, node.usedBytes), 0);
      },
      resizeFileStorage: function(node, newSize) {
        if (node.usedBytes == newSize)
          return;
        if (newSize == 0) {
          node.contents = null;
          node.usedBytes = 0;
        } else {
          var oldContents = node.contents;
          node.contents = new Uint8Array(newSize);
          if (oldContents) {
            node.contents.set(
              oldContents.subarray(0, Math.min(newSize, node.usedBytes))
            );
          }
          node.usedBytes = newSize;
        }
      },
      node_ops: {
        getattr: function(node) {
          var attr = {};
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },
        setattr: function(node, attr) {
          if (attr.mode !== void 0) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== void 0) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== void 0) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },
        lookup: function(parent, name) {
          throw FS.genericErrors[44];
        },
        mknod: function(parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },
        rename: function(old_node, new_dir, new_name) {
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(55);
              }
            }
          }
          delete old_node.parent.contents[old_node.name];
          old_node.parent.timestamp = Date.now();
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          new_dir.timestamp = old_node.parent.timestamp;
          old_node.parent = new_dir;
        },
        unlink: function(parent, name) {
          delete parent.contents[name];
          parent.timestamp = Date.now();
        },
        rmdir: function(parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(55);
          }
          delete parent.contents[name];
          parent.timestamp = Date.now();
        },
        readdir: function(node) {
          var entries = [".", ".."];
          for (var key2 in node.contents) {
            if (!node.contents.hasOwnProperty(key2)) {
              continue;
            }
            entries.push(key2);
          }
          return entries;
        },
        symlink: function(parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 | 40960, 0);
          node.link = oldpath;
          return node;
        },
        readlink: function(node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(28);
          }
          return node.link;
        }
      },
      stream_ops: {
        read: function(stream, buffer2, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes)
            return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          if (size > 8 && contents.subarray) {
            buffer2.set(contents.subarray(position, position + size), offset);
          } else {
            for (var i = 0; i < size; i++)
              buffer2[offset + i] = contents[position + i];
          }
          return size;
        },
        write: function(stream, buffer2, offset, length, position, canOwn) {
          if (buffer2.buffer === HEAP8.buffer) {
            canOwn = false;
          }
          if (!length)
            return 0;
          var node = stream.node;
          node.timestamp = Date.now();
          if (buffer2.subarray && (!node.contents || node.contents.subarray)) {
            if (canOwn) {
              node.contents = buffer2.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) {
              node.contents = buffer2.slice(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) {
              node.contents.set(
                buffer2.subarray(offset, offset + length),
                position
              );
              return length;
            }
          }
          MEMFS.expandFileStorage(node, position + length);
          if (node.contents.subarray && buffer2.subarray) {
            node.contents.set(
              buffer2.subarray(offset, offset + length),
              position
            );
          } else {
            for (var i = 0; i < length; i++) {
              node.contents[position + i] = buffer2[offset + i];
            }
          }
          node.usedBytes = Math.max(node.usedBytes, position + length);
          return length;
        },
        llseek: function(stream, offset, whence) {
          var position = offset;
          if (whence === 1) {
            position += stream.position;
          } else if (whence === 2) {
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(28);
          }
          return position;
        },
        allocate: function(stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(
            stream.node.usedBytes,
            offset + length
          );
        },
        mmap: function(stream, address, length, position, prot, flags) {
          if (address !== 0) {
            throw new FS.ErrnoError(28);
          }
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(43);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          if (!(flags & 2) && contents.buffer === buffer) {
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            if (position > 0 || position + length < contents.length) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(
                  contents,
                  position,
                  position + length
                );
              }
            }
            allocated = true;
            ptr = mmapAlloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(48);
            }
            HEAP8.set(contents, ptr);
          }
          return { ptr, allocated };
        },
        msync: function(stream, buffer2, offset, length, mmapFlags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(43);
          }
          if (mmapFlags & 2) {
            return 0;
          }
          MEMFS.stream_ops.write(
            stream,
            buffer2,
            0,
            length,
            offset,
            false
          );
          return 0;
        }
      }
    };
    var ERRNO_CODES = {
      EPERM: 63,
      ENOENT: 44,
      ESRCH: 71,
      EINTR: 27,
      EIO: 29,
      ENXIO: 60,
      E2BIG: 1,
      ENOEXEC: 45,
      EBADF: 8,
      ECHILD: 12,
      EAGAIN: 6,
      EWOULDBLOCK: 6,
      ENOMEM: 48,
      EACCES: 2,
      EFAULT: 21,
      ENOTBLK: 105,
      EBUSY: 10,
      EEXIST: 20,
      EXDEV: 75,
      ENODEV: 43,
      ENOTDIR: 54,
      EISDIR: 31,
      EINVAL: 28,
      ENFILE: 41,
      EMFILE: 33,
      ENOTTY: 59,
      ETXTBSY: 74,
      EFBIG: 22,
      ENOSPC: 51,
      ESPIPE: 70,
      EROFS: 69,
      EMLINK: 34,
      EPIPE: 64,
      EDOM: 18,
      ERANGE: 68,
      ENOMSG: 49,
      EIDRM: 24,
      ECHRNG: 106,
      EL2NSYNC: 156,
      EL3HLT: 107,
      EL3RST: 108,
      ELNRNG: 109,
      EUNATCH: 110,
      ENOCSI: 111,
      EL2HLT: 112,
      EDEADLK: 16,
      ENOLCK: 46,
      EBADE: 113,
      EBADR: 114,
      EXFULL: 115,
      ENOANO: 104,
      EBADRQC: 103,
      EBADSLT: 102,
      EDEADLOCK: 16,
      EBFONT: 101,
      ENOSTR: 100,
      ENODATA: 116,
      ETIME: 117,
      ENOSR: 118,
      ENONET: 119,
      ENOPKG: 120,
      EREMOTE: 121,
      ENOLINK: 47,
      EADV: 122,
      ESRMNT: 123,
      ECOMM: 124,
      EPROTO: 65,
      EMULTIHOP: 36,
      EDOTDOT: 125,
      EBADMSG: 9,
      ENOTUNIQ: 126,
      EBADFD: 127,
      EREMCHG: 128,
      ELIBACC: 129,
      ELIBBAD: 130,
      ELIBSCN: 131,
      ELIBMAX: 132,
      ELIBEXEC: 133,
      ENOSYS: 52,
      ENOTEMPTY: 55,
      ENAMETOOLONG: 37,
      ELOOP: 32,
      EOPNOTSUPP: 138,
      EPFNOSUPPORT: 139,
      ECONNRESET: 15,
      ENOBUFS: 42,
      EAFNOSUPPORT: 5,
      EPROTOTYPE: 67,
      ENOTSOCK: 57,
      ENOPROTOOPT: 50,
      ESHUTDOWN: 140,
      ECONNREFUSED: 14,
      EADDRINUSE: 3,
      ECONNABORTED: 13,
      ENETUNREACH: 40,
      ENETDOWN: 38,
      ETIMEDOUT: 73,
      EHOSTDOWN: 142,
      EHOSTUNREACH: 23,
      EINPROGRESS: 26,
      EALREADY: 7,
      EDESTADDRREQ: 17,
      EMSGSIZE: 35,
      EPROTONOSUPPORT: 66,
      ESOCKTNOSUPPORT: 137,
      EADDRNOTAVAIL: 4,
      ENETRESET: 39,
      EISCONN: 30,
      ENOTCONN: 53,
      ETOOMANYREFS: 141,
      EUSERS: 136,
      EDQUOT: 19,
      ESTALE: 72,
      ENOTSUP: 138,
      ENOMEDIUM: 148,
      EILSEQ: 25,
      EOVERFLOW: 61,
      ECANCELED: 11,
      ENOTRECOVERABLE: 56,
      EOWNERDEAD: 62,
      ESTRPIPE: 135
    };
    var NODEFS = {
      isWindows: false,
      staticInit: function() {
        NODEFS.isWindows = !!process.platform.match(/^win/);
        var flags = { fs: fs.constants };
        if (flags["fs"]) {
          flags = flags["fs"];
        }
        NODEFS.flagsForNodeMap = {
          1024: flags["O_APPEND"],
          64: flags["O_CREAT"],
          128: flags["O_EXCL"],
          256: flags["O_NOCTTY"],
          0: flags["O_RDONLY"],
          2: flags["O_RDWR"],
          4096: flags["O_SYNC"],
          512: flags["O_TRUNC"],
          1: flags["O_WRONLY"]
        };
      },
      bufferFrom: function(arrayBuffer) {
        return Buffer["alloc"] ? Buffer.from(arrayBuffer) : new Buffer(arrayBuffer);
      },
      convertNodeCode: function(e) {
        var code = e.code;
        return ERRNO_CODES[code];
      },
      mount: function(mount) {
        return NODEFS.createNode(null, "/", NODEFS.getMode(mount.opts.root), 0);
      },
      createNode: function(parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(28);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },
      getMode: function(path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            stat.mode = stat.mode | (stat.mode & 292) >> 2;
          }
        } catch (e) {
          if (!e.code)
            throw e;
          throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
        }
        return stat.mode;
      },
      realPath: function(node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },
      flagsForNode: function(flags) {
        flags &= ~2097152;
        flags &= ~2048;
        flags &= ~32768;
        flags &= ~524288;
        var newFlags = 0;
        for (var k in NODEFS.flagsForNodeMap) {
          if (flags & k) {
            newFlags |= NODEFS.flagsForNodeMap[k];
            flags ^= k;
          }
        }
        if (!flags) {
          return newFlags;
        } else {
          throw new FS.ErrnoError(28);
        }
      },
      node_ops: {
        getattr: function(node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size + stat.blksize - 1) / stat.blksize | 0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },
        setattr: function(node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== void 0) {
              fs.chmodSync(path, attr.mode);
              node.mode = attr.mode;
            }
            if (attr.timestamp !== void 0) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== void 0) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        lookup: function(parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },
        mknod: function(parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, "", { mode: node.mode });
            }
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
          return node;
        },
        rename: function(oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
          oldNode.name = newName;
        },
        unlink: function(parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        rmdir: function(parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        readdir: function(node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        symlink: function(parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        readlink: function(node) {
          var path = NODEFS.realPath(node);
          try {
            path = fs.readlinkSync(path);
            path = NODEJS_PATH.relative(
              NODEJS_PATH.resolve(node.mount.opts.root),
              path
            );
            return path;
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        }
      },
      stream_ops: {
        open: function(stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsForNode(stream.flags));
            }
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        close: function(stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        read: function(stream, buffer2, offset, length, position) {
          if (length === 0)
            return 0;
          try {
            return fs.readSync(
              stream.nfd,
              NODEFS.bufferFrom(buffer2.buffer),
              offset,
              length,
              position
            );
          } catch (e) {
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        write: function(stream, buffer2, offset, length, position) {
          try {
            return fs.writeSync(
              stream.nfd,
              NODEFS.bufferFrom(buffer2.buffer),
              offset,
              length,
              position
            );
          } catch (e) {
            throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
          }
        },
        llseek: function(stream, offset, whence) {
          var position = offset;
          if (whence === 1) {
            position += stream.position;
          } else if (whence === 2) {
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
              }
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(28);
          }
          return position;
        },
        mmap: function(stream, address, length, position, prot, flags) {
          if (address !== 0) {
            throw new FS.ErrnoError(28);
          }
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(43);
          }
          var ptr = mmapAlloc(length);
          NODEFS.stream_ops.read(stream, HEAP8, ptr, length, position);
          return { ptr, allocated: true };
        },
        msync: function(stream, buffer2, offset, length, mmapFlags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(43);
          }
          if (mmapFlags & 2) {
            return 0;
          }
          NODEFS.stream_ops.write(
            stream,
            buffer2,
            0,
            length,
            offset,
            false
          );
          return 0;
        }
      }
    };
    var NODERAWFS = {
      lookupPath: function(path) {
        return { path, node: { mode: NODEFS.getMode(path) } };
      },
      createStandardStreams: function() {
        FS.streams[0] = {
          fd: 0,
          nfd: 0,
          position: 0,
          path: "",
          flags: 0,
          tty: true,
          seekable: false
        };
        for (var i = 1; i < 3; i++) {
          FS.streams[i] = {
            fd: i,
            nfd: i,
            position: 0,
            path: "",
            flags: 577,
            tty: true,
            seekable: false
          };
        }
      },
      cwd: function() {
        return process.cwd();
      },
      chdir: function() {
        process.chdir.apply(void 0, arguments);
      },
      mknod: function(path, mode) {
        if (FS.isDir(path)) {
          fs.mkdirSync(path, mode);
        } else {
          fs.writeFileSync(path, "", { mode });
        }
      },
      mkdir: function() {
        fs.mkdirSync.apply(void 0, arguments);
      },
      symlink: function() {
        fs.symlinkSync.apply(void 0, arguments);
      },
      rename: function() {
        fs.renameSync.apply(void 0, arguments);
      },
      rmdir: function() {
        fs.rmdirSync.apply(void 0, arguments);
      },
      readdir: function() {
        fs.readdirSync.apply(void 0, arguments);
      },
      unlink: function() {
        fs.unlinkSync.apply(void 0, arguments);
      },
      readlink: function() {
        return fs.readlinkSync.apply(void 0, arguments);
      },
      stat: function() {
        return fs.statSync.apply(void 0, arguments);
      },
      lstat: function() {
        return fs.lstatSync.apply(void 0, arguments);
      },
      chmod: function() {
        fs.chmodSync.apply(void 0, arguments);
      },
      fchmod: function() {
        fs.fchmodSync.apply(void 0, arguments);
      },
      chown: function() {
        fs.chownSync.apply(void 0, arguments);
      },
      fchown: function() {
        fs.fchownSync.apply(void 0, arguments);
      },
      truncate: function() {
        fs.truncateSync.apply(void 0, arguments);
      },
      ftruncate: function(fd, len) {
        if (len < 0) {
          throw new FS.ErrnoError(28);
        }
        fs.ftruncateSync.apply(void 0, arguments);
      },
      utime: function() {
        fs.utimesSync.apply(void 0, arguments);
      },
      open: function(path, flags, mode, suggestFD) {
        if (typeof flags === "string") {
          flags = VFS.modeStringToFlags(flags);
        }
        var nfd = fs.openSync(path, NODEFS.flagsForNode(flags), mode);
        var fd = suggestFD != null ? suggestFD : FS.nextfd(nfd);
        var stream = {
          fd,
          nfd,
          position: 0,
          path,
          flags,
          seekable: true
        };
        FS.streams[fd] = stream;
        return stream;
      },
      close: function(stream) {
        if (!stream.stream_ops) {
          fs.closeSync(stream.nfd);
        }
        FS.closeStream(stream.fd);
      },
      llseek: function(stream, offset, whence) {
        if (stream.stream_ops) {
          return VFS.llseek(stream, offset, whence);
        }
        var position = offset;
        if (whence === 1) {
          position += stream.position;
        } else if (whence === 2) {
          position += fs.fstatSync(stream.nfd).size;
        } else if (whence !== 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        stream.position = position;
        return position;
      },
      read: function(stream, buffer2, offset, length, position) {
        if (stream.stream_ops) {
          return VFS.read(stream, buffer2, offset, length, position);
        }
        var seeking = typeof position !== "undefined";
        if (!seeking && stream.seekable)
          position = stream.position;
        var bytesRead = fs.readSync(
          stream.nfd,
          NODEFS.bufferFrom(buffer2.buffer),
          offset,
          length,
          position
        );
        if (!seeking)
          stream.position += bytesRead;
        return bytesRead;
      },
      write: function(stream, buffer2, offset, length, position) {
        if (stream.stream_ops) {
          return VFS.write(stream, buffer2, offset, length, position);
        }
        if (stream.flags & +"1024") {
          FS.llseek(stream, 0, +"2");
        }
        var seeking = typeof position !== "undefined";
        if (!seeking && stream.seekable)
          position = stream.position;
        var bytesWritten = fs.writeSync(
          stream.nfd,
          NODEFS.bufferFrom(buffer2.buffer),
          offset,
          length,
          position
        );
        if (!seeking)
          stream.position += bytesWritten;
        return bytesWritten;
      },
      allocate: function() {
        throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
      },
      mmap: function(stream, address, length, position, prot, flags) {
        if (stream.stream_ops) {
          return VFS.mmap(stream, address, length, position, prot, flags);
        }
        if (address !== 0) {
          throw new FS.ErrnoError(28);
        }
        var ptr = mmapAlloc(length);
        FS.read(stream, HEAP8, ptr, length, position);
        return { ptr, allocated: true };
      },
      msync: function(stream, buffer2, offset, length, mmapFlags) {
        if (stream.stream_ops) {
          return VFS.msync(stream, buffer2, offset, length, mmapFlags);
        }
        if (mmapFlags & 2) {
          return 0;
        }
        FS.write(stream, buffer2, 0, length, offset);
        return 0;
      },
      munmap: function() {
        return 0;
      },
      ioctl: function() {
        throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
      }
    };
    var FS = {
      root: null,
      mounts: [],
      devices: {},
      streams: [],
      nextInode: 1,
      nameTable: null,
      currentPath: "/",
      initialized: false,
      ignorePermissions: true,
      trackingDelegate: {},
      tracking: { openFlags: { READ: 1, WRITE: 2 } },
      ErrnoError: null,
      genericErrors: {},
      filesystems: null,
      syncFSRequests: 0,
      lookupPath: function(path, opts) {
        path = PATH_FS.resolve(FS.cwd(), path);
        opts = opts || {};
        if (!path)
          return { path: "", node: null };
        var defaults = { follow_mount: true, recurse_count: 0 };
        for (var key2 in defaults) {
          if (opts[key2] === void 0) {
            opts[key2] = defaults[key2];
          }
        }
        if (opts.recurse_count > 8) {
          throw new FS.ErrnoError(32);
        }
        var parts = PATH.normalizeArray(
          path.split("/").filter(function(p) {
            return !!p;
          }),
          false
        );
        var current = FS.root;
        var current_path = "/";
        for (var i = 0; i < parts.length; i++) {
          var islast = i === parts.length - 1;
          if (islast && opts.parent) {
            break;
          }
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
          if (FS.isMountpoint(current)) {
            if (!islast || islast && opts.follow_mount) {
              current = current.mounted.root;
            }
          }
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH_FS.resolve(PATH.dirname(current_path), link);
              var lookup = FS.lookupPath(current_path, {
                recurse_count: opts.recurse_count
              });
              current = lookup.node;
              if (count++ > 40) {
                throw new FS.ErrnoError(32);
              }
            }
          }
        }
        return { path: current_path, node: current };
      },
      getPath: function(node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path)
              return mount;
            return mount[mount.length - 1] !== "/" ? mount + "/" + path : mount + path;
          }
          path = path ? node.name + "/" + path : node.name;
          node = node.parent;
        }
      },
      hashName: function(parentid, name) {
        var hash = 0;
        for (var i = 0; i < name.length; i++) {
          hash = (hash << 5) - hash + name.charCodeAt(i) | 0;
        }
        return (parentid + hash >>> 0) % FS.nameTable.length;
      },
      hashAddNode: function(node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },
      hashRemoveNode: function(node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },
      lookupNode: function(parent, name) {
        var errCode = FS.mayLookup(parent);
        if (errCode) {
          throw new FS.ErrnoError(errCode, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        return FS.lookup(parent, name);
      },
      createNode: function(parent, name, mode, rdev) {
        var node = new FS.FSNode(parent, name, mode, rdev);
        FS.hashAddNode(node);
        return node;
      },
      destroyNode: function(node) {
        FS.hashRemoveNode(node);
      },
      isRoot: function(node) {
        return node === node.parent;
      },
      isMountpoint: function(node) {
        return !!node.mounted;
      },
      isFile: function(mode) {
        return (mode & 61440) === 32768;
      },
      isDir: function(mode) {
        return (mode & 61440) === 16384;
      },
      isLink: function(mode) {
        return (mode & 61440) === 40960;
      },
      isChrdev: function(mode) {
        return (mode & 61440) === 8192;
      },
      isBlkdev: function(mode) {
        return (mode & 61440) === 24576;
      },
      isFIFO: function(mode) {
        return (mode & 61440) === 4096;
      },
      isSocket: function(mode) {
        return (mode & 49152) === 49152;
      },
      flagModes: { r: 0, "r+": 2, w: 577, "w+": 578, a: 1089, "a+": 1090 },
      modeStringToFlags: function(str) {
        var flags = FS.flagModes[str];
        if (typeof flags === "undefined") {
          throw new Error("Unknown file open mode: " + str);
        }
        return flags;
      },
      flagsToPermissionString: function(flag) {
        var perms = ["r", "w", "rw"][flag & 3];
        if (flag & 512) {
          perms += "w";
        }
        return perms;
      },
      nodePermissions: function(node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        if (perms.includes("r") && !(node.mode & 292)) {
          return 2;
        } else if (perms.includes("w") && !(node.mode & 146)) {
          return 2;
        } else if (perms.includes("x") && !(node.mode & 73)) {
          return 2;
        }
        return 0;
      },
      mayLookup: function(dir) {
        var errCode = FS.nodePermissions(dir, "x");
        if (errCode)
          return errCode;
        if (!dir.node_ops.lookup)
          return 2;
        return 0;
      },
      mayCreate: function(dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return 20;
        } catch (e) {
        }
        return FS.nodePermissions(dir, "wx");
      },
      mayDelete: function(dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var errCode = FS.nodePermissions(dir, "wx");
        if (errCode) {
          return errCode;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return 54;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return 10;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return 31;
          }
        }
        return 0;
      },
      mayOpen: function(node, flags) {
        if (!node) {
          return 44;
        }
        if (FS.isLink(node.mode)) {
          return 32;
        } else if (FS.isDir(node.mode)) {
          if (FS.flagsToPermissionString(flags) !== "r" || flags & 512) {
            return 31;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },
      MAX_OPEN_FDS: 4096,
      nextfd: function(fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(33);
      },
      getStream: function(fd) {
        return FS.streams[fd];
      },
      createStream: function(stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function() {
          };
          FS.FSStream.prototype = {
            object: {
              get: function() {
                return this.node;
              },
              set: function(val) {
                this.node = val;
              }
            },
            isRead: {
              get: function() {
                return (this.flags & 2097155) !== 1;
              }
            },
            isWrite: {
              get: function() {
                return (this.flags & 2097155) !== 0;
              }
            },
            isAppend: {
              get: function() {
                return this.flags & 1024;
              }
            }
          };
        }
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },
      closeStream: function(fd) {
        FS.streams[fd] = null;
      },
      chrdev_stream_ops: {
        open: function(stream) {
          var device = FS.getDevice(stream.node.rdev);
          stream.stream_ops = device.stream_ops;
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },
        llseek: function() {
          throw new FS.ErrnoError(70);
        }
      },
      major: function(dev) {
        return dev >> 8;
      },
      minor: function(dev) {
        return dev & 255;
      },
      makedev: function(ma, mi) {
        return ma << 8 | mi;
      },
      registerDevice: function(dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },
      getDevice: function(dev) {
        return FS.devices[dev];
      },
      getMounts: function(mount) {
        var mounts = [];
        var check = [mount];
        while (check.length) {
          var m = check.pop();
          mounts.push(m);
          check.push.apply(check, m.mounts);
        }
        return mounts;
      },
      syncfs: function(populate, callback) {
        if (typeof populate === "function") {
          callback = populate;
          populate = false;
        }
        FS.syncFSRequests++;
        if (FS.syncFSRequests > 1) {
          err(
            "warning: " + FS.syncFSRequests + " FS.syncfs operations in flight at once, probably just doing extra work"
          );
        }
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
        function doCallback(errCode) {
          FS.syncFSRequests--;
          return callback(errCode);
        }
        function done(errCode) {
          if (errCode) {
            if (!done.errored) {
              done.errored = true;
              return doCallback(errCode);
            }
            return;
          }
          if (++completed >= mounts.length) {
            doCallback(null);
          }
        }
        mounts.forEach(function(mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },
      mount: function(type, opts, mountpoint) {
        var root = mountpoint === "/";
        var pseudo = !mountpoint;
        var node;
        if (root && FS.root) {
          throw new FS.ErrnoError(10);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
          mountpoint = lookup.path;
          node = lookup.node;
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(10);
          }
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(54);
          }
        }
        var mount = {
          type,
          opts,
          mountpoint,
          mounts: []
        };
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          node.mounted = mount;
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
        return mountRoot;
      },
      unmount: function(mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(28);
        }
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
        Object.keys(FS.nameTable).forEach(function(hash) {
          var current = FS.nameTable[hash];
          while (current) {
            var next = current.name_next;
            if (mounts.includes(current.mount)) {
              FS.destroyNode(current);
            }
            current = next;
          }
        });
        node.mounted = null;
        var idx = node.mount.mounts.indexOf(mount);
        node.mount.mounts.splice(idx, 1);
      },
      lookup: function(parent, name) {
        return parent.node_ops.lookup(parent, name);
      },
      mknod: function(path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === "." || name === "..") {
          throw new FS.ErrnoError(28);
        }
        var errCode = FS.mayCreate(parent, name);
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(63);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },
      create: function(path, mode) {
        mode = mode !== void 0 ? mode : 438;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },
      mkdir: function(path, mode) {
        mode = mode !== void 0 ? mode : 511;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },
      mkdirTree: function(path, mode) {
        var dirs = path.split("/");
        var d = "";
        for (var i = 0; i < dirs.length; ++i) {
          if (!dirs[i])
            continue;
          d += "/" + dirs[i];
          try {
            FS.mkdir(d, mode);
          } catch (e) {
            if (e.errno != 20)
              throw e;
          }
        }
      },
      mkdev: function(path, mode, dev) {
        if (typeof dev === "undefined") {
          dev = mode;
          mode = 438;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },
      symlink: function(oldpath, newpath) {
        if (!PATH_FS.resolve(oldpath)) {
          throw new FS.ErrnoError(44);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(44);
        }
        var newname = PATH.basename(newpath);
        var errCode = FS.mayCreate(parent, newname);
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(63);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },
      rename: function(old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        var lookup, old_dir, new_dir;
        lookup = FS.lookupPath(old_path, { parent: true });
        old_dir = lookup.node;
        lookup = FS.lookupPath(new_path, { parent: true });
        new_dir = lookup.node;
        if (!old_dir || !new_dir)
          throw new FS.ErrnoError(44);
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(75);
        }
        var old_node = FS.lookupNode(old_dir, old_name);
        var relative = PATH_FS.relative(old_path, new_dirname);
        if (relative.charAt(0) !== ".") {
          throw new FS.ErrnoError(28);
        }
        relative = PATH_FS.relative(new_path, old_dirname);
        if (relative.charAt(0) !== ".") {
          throw new FS.ErrnoError(55);
        }
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
        }
        if (old_node === new_node) {
          return;
        }
        var isdir = FS.isDir(old_node.mode);
        var errCode = FS.mayDelete(old_dir, old_name, isdir);
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        errCode = new_node ? FS.mayDelete(new_dir, new_name, isdir) : FS.mayCreate(new_dir, new_name);
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(63);
        }
        if (FS.isMountpoint(old_node) || new_node && FS.isMountpoint(new_node)) {
          throw new FS.ErrnoError(10);
        }
        if (new_dir !== old_dir) {
          errCode = FS.nodePermissions(old_dir, "w");
          if (errCode) {
            throw new FS.ErrnoError(errCode);
          }
        }
        try {
          if (FS.trackingDelegate["willMovePath"]) {
            FS.trackingDelegate["willMovePath"](old_path, new_path);
          }
        } catch (e) {
          err(
            "FS.trackingDelegate['willMovePath']('" + old_path + "', '" + new_path + "') threw an exception: " + e.message
          );
        }
        FS.hashRemoveNode(old_node);
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate["onMovePath"])
            FS.trackingDelegate["onMovePath"](old_path, new_path);
        } catch (e) {
          err(
            "FS.trackingDelegate['onMovePath']('" + old_path + "', '" + new_path + "') threw an exception: " + e.message
          );
        }
      },
      rmdir: function(path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var errCode = FS.mayDelete(parent, name, true);
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(63);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(10);
        }
        try {
          if (FS.trackingDelegate["willDeletePath"]) {
            FS.trackingDelegate["willDeletePath"](path);
          }
        } catch (e) {
          err(
            "FS.trackingDelegate['willDeletePath']('" + path + "') threw an exception: " + e.message
          );
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate["onDeletePath"])
            FS.trackingDelegate["onDeletePath"](path);
        } catch (e) {
          err(
            "FS.trackingDelegate['onDeletePath']('" + path + "') threw an exception: " + e.message
          );
        }
      },
      readdir: function(path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(54);
        }
        return node.node_ops.readdir(node);
      },
      unlink: function(path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var errCode = FS.mayDelete(parent, name, false);
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(63);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(10);
        }
        try {
          if (FS.trackingDelegate["willDeletePath"]) {
            FS.trackingDelegate["willDeletePath"](path);
          }
        } catch (e) {
          err(
            "FS.trackingDelegate['willDeletePath']('" + path + "') threw an exception: " + e.message
          );
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate["onDeletePath"])
            FS.trackingDelegate["onDeletePath"](path);
        } catch (e) {
          err(
            "FS.trackingDelegate['onDeletePath']('" + path + "') threw an exception: " + e.message
          );
        }
      },
      readlink: function(path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(44);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(28);
        }
        return PATH_FS.resolve(
          FS.getPath(link.parent),
          link.node_ops.readlink(link)
        );
      },
      stat: function(path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(44);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(63);
        }
        return node.node_ops.getattr(node);
      },
      lstat: function(path) {
        return FS.stat(path, true);
      },
      chmod: function(path, mode, dontFollow) {
        var node;
        if (typeof path === "string") {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(63);
        }
        node.node_ops.setattr(node, {
          mode: mode & 4095 | node.mode & ~4095,
          timestamp: Date.now()
        });
      },
      lchmod: function(path, mode) {
        FS.chmod(path, mode, true);
      },
      fchmod: function(fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(8);
        }
        FS.chmod(stream.node, mode);
      },
      chown: function(path, uid, gid, dontFollow) {
        var node;
        if (typeof path === "string") {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(63);
        }
        node.node_ops.setattr(node, { timestamp: Date.now() });
      },
      lchown: function(path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },
      fchown: function(fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(8);
        }
        FS.chown(stream.node, uid, gid);
      },
      truncate: function(path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(28);
        }
        var node;
        if (typeof path === "string") {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(63);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(31);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(28);
        }
        var errCode = FS.nodePermissions(node, "w");
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        node.node_ops.setattr(node, { size: len, timestamp: Date.now() });
      },
      ftruncate: function(fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(8);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(28);
        }
        FS.truncate(stream.node, len);
      },
      utime: function(path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, { timestamp: Math.max(atime, mtime) });
      },
      open: function(path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(44);
        }
        flags = typeof flags === "string" ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === "undefined" ? 438 : mode;
        if (flags & 64) {
          mode = mode & 4095 | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === "object") {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, { follow: !(flags & 131072) });
            node = lookup.node;
          } catch (e) {
          }
        }
        var created = false;
        if (flags & 64) {
          if (node) {
            if (flags & 128) {
              throw new FS.ErrnoError(20);
            }
          } else {
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(44);
        }
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        if (flags & 65536 && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(54);
        }
        if (!created) {
          var errCode = FS.mayOpen(node, flags);
          if (errCode) {
            throw new FS.ErrnoError(errCode);
          }
        }
        if (flags & 512) {
          FS.truncate(node, 0);
        }
        flags &= ~(128 | 512 | 131072);
        var stream = FS.createStream(
          {
            node,
            path: FS.getPath(node),
            flags,
            seekable: true,
            position: 0,
            stream_ops: node.stream_ops,
            ungotten: [],
            error: false
          },
          fd_start,
          fd_end
        );
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module["logReadFiles"] && !(flags & 1)) {
          if (!FS.readFiles)
            FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            err("FS.trackingDelegate error on read file: " + path);
          }
        }
        try {
          if (FS.trackingDelegate["onOpenFile"]) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate["onOpenFile"](path, trackingFlags);
          }
        } catch (e) {
          err(
            "FS.trackingDelegate['onOpenFile']('" + path + "', flags) threw an exception: " + e.message
          );
        }
        return stream;
      },
      close: function(stream) {
        if (FS.isClosed(stream)) {
          throw new FS.ErrnoError(8);
        }
        if (stream.getdents)
          stream.getdents = null;
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
        stream.fd = null;
      },
      isClosed: function(stream) {
        return stream.fd === null;
      },
      llseek: function(stream, offset, whence) {
        if (FS.isClosed(stream)) {
          throw new FS.ErrnoError(8);
        }
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(70);
        }
        if (whence != 0 && whence != 1 && whence != 2) {
          throw new FS.ErrnoError(28);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },
      read: function(stream, buffer2, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(28);
        }
        if (FS.isClosed(stream)) {
          throw new FS.ErrnoError(8);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(8);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(31);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(28);
        }
        var seeking = typeof position !== "undefined";
        if (!seeking) {
          position = stream.position;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(70);
        }
        var bytesRead = stream.stream_ops.read(
          stream,
          buffer2,
          offset,
          length,
          position
        );
        if (!seeking)
          stream.position += bytesRead;
        return bytesRead;
      },
      write: function(stream, buffer2, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(28);
        }
        if (FS.isClosed(stream)) {
          throw new FS.ErrnoError(8);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(8);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(31);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(28);
        }
        if (stream.seekable && stream.flags & 1024) {
          FS.llseek(stream, 0, 2);
        }
        var seeking = typeof position !== "undefined";
        if (!seeking) {
          position = stream.position;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(70);
        }
        var bytesWritten = stream.stream_ops.write(
          stream,
          buffer2,
          offset,
          length,
          position,
          canOwn
        );
        if (!seeking)
          stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate["onWriteToFile"])
            FS.trackingDelegate["onWriteToFile"](stream.path);
        } catch (e) {
          err(
            "FS.trackingDelegate['onWriteToFile']('" + stream.path + "') threw an exception: " + e.message
          );
        }
        return bytesWritten;
      },
      allocate: function(stream, offset, length) {
        if (FS.isClosed(stream)) {
          throw new FS.ErrnoError(8);
        }
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(28);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(8);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(43);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(138);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },
      mmap: function(stream, address, length, position, prot, flags) {
        if ((prot & 2) !== 0 && (flags & 2) === 0 && (stream.flags & 2097155) !== 2) {
          throw new FS.ErrnoError(2);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(2);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(43);
        }
        return stream.stream_ops.mmap(
          stream,
          address,
          length,
          position,
          prot,
          flags
        );
      },
      msync: function(stream, buffer2, offset, length, mmapFlags) {
        if (!stream || !stream.stream_ops.msync) {
          return 0;
        }
        return stream.stream_ops.msync(
          stream,
          buffer2,
          offset,
          length,
          mmapFlags
        );
      },
      munmap: function(stream) {
        return 0;
      },
      ioctl: function(stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(59);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },
      readFile: function(path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 0;
        opts.encoding = opts.encoding || "binary";
        if (opts.encoding !== "utf8" && opts.encoding !== "binary") {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === "utf8") {
          ret = UTF8ArrayToString(buf, 0);
        } else if (opts.encoding === "binary") {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },
      writeFile: function(path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 577;
        var stream = FS.open(path, opts.flags, opts.mode);
        if (typeof data === "string") {
          var buf = new Uint8Array(lengthBytesUTF8(data) + 1);
          var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
          FS.write(stream, buf, 0, actualNumBytes, void 0, opts.canOwn);
        } else if (ArrayBuffer.isView(data)) {
          FS.write(stream, data, 0, data.byteLength, void 0, opts.canOwn);
        } else {
          throw new Error("Unsupported data type");
        }
        FS.close(stream);
      },
      cwd: function() {
        return FS.currentPath;
      },
      chdir: function(path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (lookup.node === null) {
          throw new FS.ErrnoError(44);
        }
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(54);
        }
        var errCode = FS.nodePermissions(lookup.node, "x");
        if (errCode) {
          throw new FS.ErrnoError(errCode);
        }
        FS.currentPath = lookup.path;
      },
      createDefaultDirectories: function() {
        FS.mkdir("/tmp");
        FS.mkdir("/home");
        FS.mkdir("/home/web_user");
      },
      createDefaultDevices: function() {
        FS.mkdir("/dev");
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() {
            return 0;
          },
          write: function(stream, buffer2, offset, length, pos) {
            return length;
          }
        });
        FS.mkdev("/dev/null", FS.makedev(1, 3));
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev("/dev/tty", FS.makedev(5, 0));
        FS.mkdev("/dev/tty1", FS.makedev(6, 0));
        var random_device = getRandomDevice();
        FS.createDevice("/dev", "random", random_device);
        FS.createDevice("/dev", "urandom", random_device);
        FS.mkdir("/dev/shm");
        FS.mkdir("/dev/shm/tmp");
      },
      createSpecialDirectories: function() {
        FS.mkdir("/proc");
        var proc_self = FS.mkdir("/proc/self");
        FS.mkdir("/proc/self/fd");
        FS.mount(
          {
            mount: function() {
              var node = FS.createNode(proc_self, "fd", 16384 | 511, 73);
              node.node_ops = {
                lookup: function(parent, name) {
                  var fd = +name;
                  var stream = FS.getStream(fd);
                  if (!stream)
                    throw new FS.ErrnoError(8);
                  var ret = {
                    parent: null,
                    mount: { mountpoint: "fake" },
                    node_ops: {
                      readlink: function() {
                        return stream.path;
                      }
                    }
                  };
                  ret.parent = ret;
                  return ret;
                }
              };
              return node;
            }
          },
          {},
          "/proc/self/fd"
        );
      },
      createStandardStreams: function() {
        if (Module["stdin"]) {
          FS.createDevice("/dev", "stdin", Module["stdin"]);
        } else {
          FS.symlink("/dev/tty", "/dev/stdin");
        }
        if (Module["stdout"]) {
          FS.createDevice("/dev", "stdout", null, Module["stdout"]);
        } else {
          FS.symlink("/dev/tty", "/dev/stdout");
        }
        if (Module["stderr"]) {
          FS.createDevice("/dev", "stderr", null, Module["stderr"]);
        } else {
          FS.symlink("/dev/tty1", "/dev/stderr");
        }
        FS.open("/dev/stdin", 0);
        FS.open("/dev/stdout", 1);
        FS.open("/dev/stderr", 1);
      },
      ensureErrnoError: function() {
        if (FS.ErrnoError)
          return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          this.node = node;
          this.setErrno = function(errno2) {
            this.errno = errno2;
          };
          this.setErrno(errno);
          this.message = "FS error";
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        [44].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = "<generic error, no stack>";
        });
      },
      staticInit: function() {
        FS.ensureErrnoError();
        FS.nameTable = new Array(4096);
        FS.mount(MEMFS, {}, "/");
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
        FS.createSpecialDirectories();
        FS.filesystems = { MEMFS, NODEFS };
      },
      init: function(input, output, error) {
        FS.init.initialized = true;
        FS.ensureErrnoError();
        Module["stdin"] = input || Module["stdin"];
        Module["stdout"] = output || Module["stdout"];
        Module["stderr"] = error || Module["stderr"];
        FS.createStandardStreams();
      },
      quit: function() {
        FS.init.initialized = false;
        var fflush = Module["_fflush"];
        if (fflush)
          fflush(0);
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },
      getMode: function(canRead, canWrite) {
        var mode = 0;
        if (canRead)
          mode |= 292 | 73;
        if (canWrite)
          mode |= 146;
        return mode;
      },
      findObject: function(path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          return null;
        }
      },
      analyzePath: function(path, dontResolveLastLink) {
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false,
          exists: false,
          error: 0,
          name: null,
          path: null,
          object: null,
          parentExists: false,
          parentPath: null,
          parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === "/";
        } catch (e) {
          ret.error = e.errno;
        }
        return ret;
      },
      createPath: function(parent, path, canRead, canWrite) {
        parent = typeof parent === "string" ? parent : FS.getPath(parent);
        var parts = path.split("/").reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part)
            continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
          }
          parent = current;
        }
        return current;
      },
      createFile: function(parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(
          typeof parent === "string" ? parent : FS.getPath(parent),
          name
        );
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },
      createDataFile: function(parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(
          typeof parent === "string" ? parent : FS.getPath(parent),
          name
        ) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === "string") {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i)
              arr[i] = data.charCodeAt(i);
            data = arr;
          }
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 577);
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },
      createDevice: function(parent, name, input, output) {
        var path = PATH.join2(
          typeof parent === "string" ? parent : FS.getPath(parent),
          name
        );
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major)
          FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer2, offset, length, pos) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(29);
              }
              if (result === void 0 && bytesRead === 0) {
                throw new FS.ErrnoError(6);
              }
              if (result === null || result === void 0)
                break;
              bytesRead++;
              buffer2[offset + i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer2, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer2[offset + i]);
              } catch (e) {
                throw new FS.ErrnoError(29);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },
      forceLoadFile: function(obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents)
          return true;
        if (read_) {
          try {
            obj.contents = intArrayFromString(read_(obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            throw new FS.ErrnoError(29);
          }
        } else {
          throw new Error("Cannot load without read() or XMLHttpRequest.");
        }
      },
      createLazyFile: function(parent, name, url, canRead, canWrite) {
        var properties; {
          var properties = { isDevice: false, url };
        }
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        Object.defineProperties(node, {
          usedBytes: {
            get: function() {
              return this.contents.length;
            }
          }
        });
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key2) {
          var fn = node.stream_ops[key2];
          stream_ops[key2] = function forceLoadLazyFile() {
            FS.forceLoadFile(node);
            return fn.apply(null, arguments);
          };
        });
        stream_ops.read = function stream_ops_read(stream, buffer2, offset, length, position) {
          FS.forceLoadFile(node);
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          if (contents.slice) {
            for (var i = 0; i < size; i++) {
              buffer2[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) {
              buffer2[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },
      createPreloadedFile: function(parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) {
        Browser.init();
        var fullname = name ? PATH_FS.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray2) {
            if (preFinish)
              preFinish();
            if (!dontCreateFile) {
              FS.createDataFile(
                parent,
                name,
                byteArray2,
                canRead,
                canWrite,
                canOwn
              );
            }
            if (onload)
              onload();
            removeRunDependency();
          }
          var handled = false;
          Module["preloadPlugins"].forEach(function(plugin) {
            if (handled)
              return;
            if (plugin["canHandle"](fullname)) {
              plugin["handle"](byteArray, fullname, finish, function() {
                if (onerror)
                  onerror();
                removeRunDependency();
              });
              handled = true;
            }
          });
          if (!handled)
            finish(byteArray);
        }
        addRunDependency();
        if (typeof url == "string") {
          Browser.asyncLoad(
            url,
            function(byteArray) {
              processData(byteArray);
            },
            onerror
          );
        } else {
          processData(url);
        }
      },
      indexedDB: function() {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },
      DB_NAME: function() {
        return "EM_FS_" + window.location.pathname;
      },
      DB_VERSION: 20,
      DB_STORE_NAME: "FILE_DATA",
      saveFilesToDB: function(paths, onload, onerror) {
        onload = onload || function() {
        };
        onerror = onerror || function() {
        };
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          out("creating db");
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], "readwrite");
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0)
              onload();
            else
              onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(
              FS.analyzePath(path).object.contents,
              path
            );
            putRequest.onsuccess = function putRequest_onsuccess() {
              ok++;
              if (ok + fail == total)
                finish();
            };
            putRequest.onerror = function putRequest_onerror() {
              fail++;
              if (ok + fail == total)
                finish();
            };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },
      loadFilesFromDB: function(paths, onload, onerror) {
        onload = onload || function() {
        };
        onerror = onerror || function() {
        };
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror;
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], "readonly");
          } catch (e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0)
              onload();
            else
              onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(
                PATH.dirname(path),
                PATH.basename(path),
                getRequest.result,
                true,
                true,
                true
              );
              ok++;
              if (ok + fail == total)
                finish();
            };
            getRequest.onerror = function getRequest_onerror() {
              fail++;
              if (ok + fail == total)
                finish();
            };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }
    };
    var SYSCALLS = {
      mappings: {},
      DEFAULT_POLLMASK: 5,
      umask: 511,
      calculateAt: function(dirfd, path, allowEmpty) {
        if (path[0] === "/") {
          return path;
        }
        var dir;
        if (dirfd === -100) {
          dir = FS.cwd();
        } else {
          var dirstream = FS.getStream(dirfd);
          if (!dirstream)
            throw new FS.ErrnoError(8);
          dir = dirstream.path;
        }
        if (path.length == 0) {
          if (!allowEmpty) {
            throw new FS.ErrnoError(44);
          }
          return dir;
        }
        return PATH.join2(dir, path);
      },
      doStat: function(func, path, buf) {
        try {
          var stat = func(path);
        } catch (e) {
          if (e && e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
            return -54;
          }
          throw e;
        }
        HEAP32[buf >> 2] = stat.dev;
        HEAP32[buf + 4 >> 2] = 0;
        HEAP32[buf + 8 >> 2] = stat.ino;
        HEAP32[buf + 12 >> 2] = stat.mode;
        HEAP32[buf + 16 >> 2] = stat.nlink;
        HEAP32[buf + 20 >> 2] = stat.uid;
        HEAP32[buf + 24 >> 2] = stat.gid;
        HEAP32[buf + 28 >> 2] = stat.rdev;
        HEAP32[buf + 32 >> 2] = 0;
        tempI64 = [
          stat.size >>> 0,
          (tempDouble = stat.size, +Math.abs(tempDouble) >= 1 ? tempDouble > 0 ? (Math.min(+Math.floor(tempDouble / 4294967296), 4294967295) | 0) >>> 0 : ~~+Math.ceil(
            (tempDouble - +(~~tempDouble >>> 0)) / 4294967296
          ) >>> 0 : 0)
        ], HEAP32[buf + 40 >> 2] = tempI64[0], HEAP32[buf + 44 >> 2] = tempI64[1];
        HEAP32[buf + 48 >> 2] = 4096;
        HEAP32[buf + 52 >> 2] = stat.blocks;
        HEAP32[buf + 56 >> 2] = stat.atime.getTime() / 1e3 | 0;
        HEAP32[buf + 60 >> 2] = 0;
        HEAP32[buf + 64 >> 2] = stat.mtime.getTime() / 1e3 | 0;
        HEAP32[buf + 68 >> 2] = 0;
        HEAP32[buf + 72 >> 2] = stat.ctime.getTime() / 1e3 | 0;
        HEAP32[buf + 76 >> 2] = 0;
        tempI64 = [
          stat.ino >>> 0,
          (tempDouble = stat.ino, +Math.abs(tempDouble) >= 1 ? tempDouble > 0 ? (Math.min(+Math.floor(tempDouble / 4294967296), 4294967295) | 0) >>> 0 : ~~+Math.ceil(
            (tempDouble - +(~~tempDouble >>> 0)) / 4294967296
          ) >>> 0 : 0)
        ], HEAP32[buf + 80 >> 2] = tempI64[0], HEAP32[buf + 84 >> 2] = tempI64[1];
        return 0;
      },
      doMsync: function(addr, stream, len, flags, offset) {
        var buffer2 = HEAPU8.slice(addr, addr + len);
        FS.msync(stream, buffer2, offset, len, flags);
      },
      doMkdir: function(path, mode) {
        path = PATH.normalize(path);
        if (path[path.length - 1] === "/")
          path = path.substr(0, path.length - 1);
        FS.mkdir(path, mode, 0);
        return 0;
      },
      doMknod: function(path, mode, dev) {
        switch (mode & 61440) {
          case 32768:
          case 8192:
          case 24576:
          case 4096:
          case 49152:
            break;
          default:
            return -28;
        }
        FS.mknod(path, mode, dev);
        return 0;
      },
      doReadlink: function(path, buf, bufsize) {
        if (bufsize <= 0)
          return -28;
        var ret = FS.readlink(path);
        var len = Math.min(bufsize, lengthBytesUTF8(ret));
        var endChar = HEAP8[buf + len];
        stringToUTF8(ret, buf, bufsize + 1);
        HEAP8[buf + len] = endChar;
        return len;
      },
      doAccess: function(path, amode) {
        if (amode & ~7) {
          return -28;
        }
        var node;
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
        if (!node) {
          return -44;
        }
        var perms = "";
        if (amode & 4)
          perms += "r";
        if (amode & 2)
          perms += "w";
        if (amode & 1)
          perms += "x";
        if (perms && FS.nodePermissions(node, perms)) {
          return -2;
        }
        return 0;
      },
      doDup: function(path, flags, suggestFD) {
        var suggest = FS.getStream(suggestFD);
        if (suggest)
          FS.close(suggest);
        return FS.open(path, flags, 0, suggestFD, suggestFD).fd;
      },
      doReadv: function(stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[iov + i * 8 >> 2];
          var len = HEAP32[iov + (i * 8 + 4) >> 2];
          var curr = FS.read(stream, HEAP8, ptr, len, offset);
          if (curr < 0)
            return -1;
          ret += curr;
          if (curr < len)
            break;
        }
        return ret;
      },
      doWritev: function(stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[iov + i * 8 >> 2];
          var len = HEAP32[iov + (i * 8 + 4) >> 2];
          var curr = FS.write(stream, HEAP8, ptr, len, offset);
          if (curr < 0)
            return -1;
          ret += curr;
        }
        return ret;
      },
      varargs: void 0,
      get: function() {
        SYSCALLS.varargs += 4;
        var ret = HEAP32[SYSCALLS.varargs - 4 >> 2];
        return ret;
      },
      getStr: function(ptr) {
        var ret = UTF8ToString(ptr);
        return ret;
      },
      getStreamFromFD: function(fd) {
        var stream = FS.getStream(fd);
        if (!stream)
          throw new FS.ErrnoError(8);
        return stream;
      },
      get64: function(low, high) {
        return low;
      }
    };
    function ___sys_chmod(path, mode) {
      try {
        path = SYSCALLS.getStr(path);
        FS.chmod(path, mode);
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function setErrNo(value) {
      HEAP32[___errno_location() >> 2] = value;
      return value;
    }
    function ___sys_fcntl64(fd, cmd, varargs) {
      SYSCALLS.varargs = varargs;
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        switch (cmd) {
          case 0: {
            var arg = SYSCALLS.get();
            if (arg < 0) {
              return -28;
            }
            var newStream;
            newStream = FS.open(stream.path, stream.flags, 0, arg);
            return newStream.fd;
          }
          case 1:
          case 2:
            return 0;
          case 3:
            return stream.flags;
          case 4: {
            var arg = SYSCALLS.get();
            stream.flags |= arg;
            return 0;
          }
          case 12: {
            var arg = SYSCALLS.get();
            var offset = 0;
            HEAP16[arg + offset >> 1] = 2;
            return 0;
          }
          case 13:
          case 14:
            return 0;
          case 16:
          case 8:
            return -28;
          case 9:
            setErrNo(28);
            return -1;
          default: {
            return -28;
          }
        }
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_fstat64(fd, buf) {
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        return SYSCALLS.doStat(FS.stat, stream.path, buf);
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_ioctl(fd, op, varargs) {
      SYSCALLS.varargs = varargs;
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        switch (op) {
          case 21509:
          case 21505: {
            if (!stream.tty)
              return -59;
            return 0;
          }
          case 21510:
          case 21511:
          case 21512:
          case 21506:
          case 21507:
          case 21508: {
            if (!stream.tty)
              return -59;
            return 0;
          }
          case 21519: {
            if (!stream.tty)
              return -59;
            var argp = SYSCALLS.get();
            HEAP32[argp >> 2] = 0;
            return 0;
          }
          case 21520: {
            if (!stream.tty)
              return -59;
            return -28;
          }
          case 21531: {
            var argp = SYSCALLS.get();
            return FS.ioctl(stream, op, argp);
          }
          case 21523: {
            if (!stream.tty)
              return -59;
            return 0;
          }
          case 21524: {
            if (!stream.tty)
              return -59;
            return 0;
          }
          default:
            abort("bad ioctl syscall " + op);
        }
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_open(path, flags, varargs) {
      SYSCALLS.varargs = varargs;
      try {
        var pathname = SYSCALLS.getStr(path);
        var mode = varargs ? SYSCALLS.get() : 0;
        var stream = FS.open(pathname, flags, mode);
        return stream.fd;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_rename(old_path, new_path) {
      try {
        old_path = SYSCALLS.getStr(old_path);
        new_path = SYSCALLS.getStr(new_path);
        FS.rename(old_path, new_path);
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_rmdir(path) {
      try {
        path = SYSCALLS.getStr(path);
        FS.rmdir(path);
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_stat64(path, buf) {
      try {
        path = SYSCALLS.getStr(path);
        return SYSCALLS.doStat(FS.stat, path, buf);
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function ___sys_unlink(path) {
      try {
        path = SYSCALLS.getStr(path);
        FS.unlink(path);
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return -e.errno;
      }
    }
    function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.copyWithin(dest, src, src + num);
    }
    function emscripten_realloc_buffer(size) {
      try {
        wasmMemory.grow(size - buffer.byteLength + 65535 >>> 16);
        updateGlobalBufferAndViews(wasmMemory.buffer);
        return 1;
      } catch (e) {
      }
    }
    function _emscripten_resize_heap(requestedSize) {
      var oldSize = HEAPU8.length;
      requestedSize = requestedSize >>> 0;
      var maxHeapSize = 2147483648;
      if (requestedSize > maxHeapSize) {
        return false;
      }
      for (var cutDown = 1; cutDown <= 4; cutDown *= 2) {
        var overGrownHeapSize = oldSize * (1 + 0.2 / cutDown);
        overGrownHeapSize = Math.min(
          overGrownHeapSize,
          requestedSize + 100663296
        );
        var newSize = Math.min(
          maxHeapSize,
          alignUp(Math.max(requestedSize, overGrownHeapSize), 65536)
        );
        var replacement = emscripten_realloc_buffer(newSize);
        if (replacement) {
          return true;
        }
      }
      return false;
    }
    function _fd_close(fd) {
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        FS.close(stream);
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return e.errno;
      }
    }
    function _fd_fdstat_get(fd, pbuf) {
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        var type = stream.tty ? 2 : FS.isDir(stream.mode) ? 3 : FS.isLink(stream.mode) ? 7 : 4;
        HEAP8[pbuf >> 0] = type;
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return e.errno;
      }
    }
    function _fd_read(fd, iov, iovcnt, pnum) {
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        var num = SYSCALLS.doReadv(stream, iov, iovcnt);
        HEAP32[pnum >> 2] = num;
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return e.errno;
      }
    }
    function _fd_seek(fd, offset_low, offset_high, whence, newOffset) {
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        var HIGH_OFFSET = 4294967296;
        var offset = offset_high * HIGH_OFFSET + (offset_low >>> 0);
        var DOUBLE_LIMIT = 9007199254740992;
        if (offset <= -DOUBLE_LIMIT || offset >= DOUBLE_LIMIT) {
          return -61;
        }
        FS.llseek(stream, offset, whence);
        tempI64 = [
          stream.position >>> 0,
          (tempDouble = stream.position, +Math.abs(tempDouble) >= 1 ? tempDouble > 0 ? (Math.min(+Math.floor(tempDouble / 4294967296), 4294967295) | 0) >>> 0 : ~~+Math.ceil(
            (tempDouble - +(~~tempDouble >>> 0)) / 4294967296
          ) >>> 0 : 0)
        ], HEAP32[newOffset >> 2] = tempI64[0], HEAP32[newOffset + 4 >> 2] = tempI64[1];
        if (stream.getdents && offset === 0 && whence === 0)
          stream.getdents = null;
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return e.errno;
      }
    }
    function _fd_write(fd, iov, iovcnt, pnum) {
      try {
        var stream = SYSCALLS.getStreamFromFD(fd);
        var num = SYSCALLS.doWritev(stream, iov, iovcnt);
        HEAP32[pnum >> 2] = num;
        return 0;
      } catch (e) {
        if (typeof FS === "undefined" || !(e instanceof FS.ErrnoError))
          abort(e);
        return e.errno;
      }
    }
    function _setTempRet0(val) {
    }
    function _time(ptr) {
      var ret = Date.now() / 1e3 | 0;
      if (ptr) {
        HEAP32[ptr >> 2] = ret;
      }
      return ret;
    }
    function _tzset() {
      if (_tzset.called)
        return;
      _tzset.called = true;
      var currentYear = new Date().getFullYear();
      var winter = new Date(currentYear, 0, 1);
      var summer = new Date(currentYear, 6, 1);
      var winterOffset = winter.getTimezoneOffset();
      var summerOffset = summer.getTimezoneOffset();
      var stdTimezoneOffset = Math.max(winterOffset, summerOffset);
      HEAP32[__get_timezone() >> 2] = stdTimezoneOffset * 60;
      HEAP32[__get_daylight() >> 2] = Number(winterOffset != summerOffset);
      function extractZone(date) {
        var match = date.toTimeString().match(/\(([A-Za-z ]+)\)$/);
        return match ? match[1] : "GMT";
      }
      var winterName = extractZone(winter);
      var summerName = extractZone(summer);
      var winterNamePtr = allocateUTF8(winterName);
      var summerNamePtr = allocateUTF8(summerName);
      if (summerOffset < winterOffset) {
        HEAP32[__get_tzname() >> 2] = winterNamePtr;
        HEAP32[__get_tzname() + 4 >> 2] = summerNamePtr;
      } else {
        HEAP32[__get_tzname() >> 2] = summerNamePtr;
        HEAP32[__get_tzname() + 4 >> 2] = winterNamePtr;
      }
    }
    function _timegm(tmPtr) {
      _tzset();
      var time = Date.UTC(
        HEAP32[tmPtr + 20 >> 2] + 1900,
        HEAP32[tmPtr + 16 >> 2],
        HEAP32[tmPtr + 12 >> 2],
        HEAP32[tmPtr + 8 >> 2],
        HEAP32[tmPtr + 4 >> 2],
        HEAP32[tmPtr >> 2],
        0
      );
      var date = new Date(time);
      HEAP32[tmPtr + 24 >> 2] = date.getUTCDay();
      var start = Date.UTC(date.getUTCFullYear(), 0, 1, 0, 0, 0, 0);
      var yday = (date.getTime() - start) / (1e3 * 60 * 60 * 24) | 0;
      HEAP32[tmPtr + 28 >> 2] = yday;
      return date.getTime() / 1e3 | 0;
    }
    var FSNode = function(parent, name, mode, rdev) {
      if (!parent) {
        parent = this;
      }
      this.parent = parent;
      this.mount = parent.mount;
      this.mounted = null;
      this.id = FS.nextInode++;
      this.name = name;
      this.mode = mode;
      this.node_ops = {};
      this.stream_ops = {};
      this.rdev = rdev;
    };
    var readMode = 292 | 73;
    var writeMode = 146;
    Object.defineProperties(FSNode.prototype, {
      read: {
        get: function() {
          return (this.mode & readMode) === readMode;
        },
        set: function(val) {
          val ? this.mode |= readMode : this.mode &= ~readMode;
        }
      },
      write: {
        get: function() {
          return (this.mode & writeMode) === writeMode;
        },
        set: function(val) {
          val ? this.mode |= writeMode : this.mode &= ~writeMode;
        }
      },
      isFolder: {
        get: function() {
          return FS.isDir(this.mode);
        }
      },
      isDevice: {
        get: function() {
          return FS.isChrdev(this.mode);
        }
      }
    });
    FS.FSNode = FSNode;
    FS.staticInit();
    {
      var fs = frozenFs;
      var NODEJS_PATH = path__default.default;
      NODEFS.staticInit();
    }
    {
      var _wrapNodeError = function(func) {
        return function() {
          try {
            return func.apply(this, arguments);
          } catch (e) {
            if (!e.code)
              throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        };
      };
      var VFS = Object.assign({}, FS);
      for (var _key in NODERAWFS)
        FS[_key] = _wrapNodeError(NODERAWFS[_key]);
    }
    function intArrayFromString(stringy, dontAddNull, length) {
      var len = length > 0 ? length : lengthBytesUTF8(stringy) + 1;
      var u8array = new Array(len);
      var numBytesWritten = stringToUTF8Array(
        stringy,
        u8array,
        0,
        u8array.length
      );
      if (dontAddNull)
        u8array.length = numBytesWritten;
      return u8array;
    }
    function intArrayFromBase64(s) {
      {
        var buf;
        try {
          buf = Buffer.from(s, "base64");
        } catch (_) {
          buf = new Buffer(s, "base64");
        }
        return new Uint8Array(
          buf["buffer"],
          buf["byteOffset"],
          buf["byteLength"]
        );
      }
    }
    function tryParseAsDataURI(filename) {
      if (!isDataURI(filename)) {
        return;
      }
      return intArrayFromBase64(filename.slice(dataURIPrefix.length));
    }
    var asmLibraryArg = {
      s: ___gmtime_r,
      p: ___sys_chmod,
      e: ___sys_fcntl64,
      k: ___sys_fstat64,
      o: ___sys_ioctl,
      q: ___sys_open,
      i: ___sys_rename,
      r: ___sys_rmdir,
      c: ___sys_stat64,
      h: ___sys_unlink,
      l: _emscripten_memcpy_big,
      m: _emscripten_resize_heap,
      f: _fd_close,
      j: _fd_fdstat_get,
      g: _fd_read,
      n: _fd_seek,
      d: _fd_write,
      a: _setTempRet0,
      b: _time,
      t: _timegm
    };
    var asm = createWasm();
    Module["___wasm_call_ctors"] = asm["v"];
    Module["_zip_ext_count_symlinks"] = asm["w"];
    Module["_zip_file_get_external_attributes"] = asm["x"];
    Module["_zipstruct_stat"] = asm["y"];
    Module["_zipstruct_statS"] = asm["z"];
    Module["_zipstruct_stat_name"] = asm["A"];
    Module["_zipstruct_stat_index"] = asm["B"];
    Module["_zipstruct_stat_size"] = asm["C"];
    Module["_zipstruct_stat_mtime"] = asm["D"];
    Module["_zipstruct_stat_crc"] = asm["E"];
    Module["_zipstruct_error"] = asm["F"];
    Module["_zipstruct_errorS"] = asm["G"];
    Module["_zipstruct_error_code_zip"] = asm["H"];
    Module["_zipstruct_stat_comp_size"] = asm["I"];
    Module["_zipstruct_stat_comp_method"] = asm["J"];
    Module["_zip_close"] = asm["K"];
    Module["_zip_delete"] = asm["L"];
    Module["_zip_dir_add"] = asm["M"];
    Module["_zip_discard"] = asm["N"];
    Module["_zip_error_init_with_code"] = asm["O"];
    Module["_zip_get_error"] = asm["P"];
    Module["_zip_file_get_error"] = asm["Q"];
    Module["_zip_error_strerror"] = asm["R"];
    Module["_zip_fclose"] = asm["S"];
    Module["_zip_file_add"] = asm["T"];
    Module["_free"] = asm["U"];
    var _malloc = Module["_malloc"] = asm["V"];
    var ___errno_location = Module["___errno_location"] = asm["W"];
    Module["_zip_source_error"] = asm["X"];
    Module["_zip_source_seek"] = asm["Y"];
    Module["_zip_file_set_external_attributes"] = asm["Z"];
    Module["_zip_file_set_mtime"] = asm["_"];
    Module["_zip_fopen"] = asm["$"];
    Module["_zip_fopen_index"] = asm["aa"];
    Module["_zip_fread"] = asm["ba"];
    Module["_zip_get_name"] = asm["ca"];
    Module["_zip_get_num_entries"] = asm["da"];
    Module["_zip_source_read"] = asm["ea"];
    Module["_zip_name_locate"] = asm["fa"];
    Module["_zip_open"] = asm["ga"];
    Module["_zip_open_from_source"] = asm["ha"];
    Module["_zip_set_file_compression"] = asm["ia"];
    Module["_zip_source_buffer"] = asm["ja"];
    Module["_zip_source_buffer_create"] = asm["ka"];
    Module["_zip_source_close"] = asm["la"];
    Module["_zip_source_free"] = asm["ma"];
    Module["_zip_source_keep"] = asm["na"];
    Module["_zip_source_open"] = asm["oa"];
    Module["_zip_source_set_mtime"] = asm["qa"];
    Module["_zip_source_tell"] = asm["ra"];
    Module["_zip_stat"] = asm["sa"];
    Module["_zip_stat_index"] = asm["ta"];
    var __get_tzname = Module["__get_tzname"] = asm["ua"];
    var __get_daylight = Module["__get_daylight"] = asm["va"];
    var __get_timezone = Module["__get_timezone"] = asm["wa"];
    var stackSave = Module["stackSave"] = asm["xa"];
    var stackRestore = Module["stackRestore"] = asm["ya"];
    var stackAlloc = Module["stackAlloc"] = asm["za"];
    Module["cwrap"] = cwrap;
    Module["getValue"] = getValue;
    var calledRun;
    dependenciesFulfilled = function runCaller() {
      if (!calledRun)
        run();
      if (!calledRun)
        dependenciesFulfilled = runCaller;
    };
    function run(args) {
      if (runDependencies > 0) {
        return;
      }
      preRun();
      if (runDependencies > 0) {
        return;
      }
      function doRun() {
        if (calledRun)
          return;
        calledRun = true;
        Module["calledRun"] = true;
        if (ABORT)
          return;
        initRuntime();
        readyPromiseResolve(Module);
        if (Module["onRuntimeInitialized"])
          Module["onRuntimeInitialized"]();
        postRun();
      }
      if (Module["setStatus"]) {
        Module["setStatus"]("Running...");
        setTimeout(function() {
          setTimeout(function() {
            Module["setStatus"]("");
          }, 1);
          doRun();
        }, 1);
      } else {
        doRun();
      }
    }
    Module["run"] = run;
    if (Module["preInit"]) {
      if (typeof Module["preInit"] == "function")
        Module["preInit"] = [Module["preInit"]];
      while (Module["preInit"].length > 0) {
        Module["preInit"].pop()();
      }
    }
    run();
    return createModule2;
  };
}();
module.exports = createModule;
}(libzipSync));

const createModule = libzipSync.exports;

const number64 = [
  `number`,
  `number`
];
var Errors = /* @__PURE__ */ ((Errors2) => {
  Errors2[Errors2["ZIP_ER_OK"] = 0] = "ZIP_ER_OK";
  Errors2[Errors2["ZIP_ER_MULTIDISK"] = 1] = "ZIP_ER_MULTIDISK";
  Errors2[Errors2["ZIP_ER_RENAME"] = 2] = "ZIP_ER_RENAME";
  Errors2[Errors2["ZIP_ER_CLOSE"] = 3] = "ZIP_ER_CLOSE";
  Errors2[Errors2["ZIP_ER_SEEK"] = 4] = "ZIP_ER_SEEK";
  Errors2[Errors2["ZIP_ER_READ"] = 5] = "ZIP_ER_READ";
  Errors2[Errors2["ZIP_ER_WRITE"] = 6] = "ZIP_ER_WRITE";
  Errors2[Errors2["ZIP_ER_CRC"] = 7] = "ZIP_ER_CRC";
  Errors2[Errors2["ZIP_ER_ZIPCLOSED"] = 8] = "ZIP_ER_ZIPCLOSED";
  Errors2[Errors2["ZIP_ER_NOENT"] = 9] = "ZIP_ER_NOENT";
  Errors2[Errors2["ZIP_ER_EXISTS"] = 10] = "ZIP_ER_EXISTS";
  Errors2[Errors2["ZIP_ER_OPEN"] = 11] = "ZIP_ER_OPEN";
  Errors2[Errors2["ZIP_ER_TMPOPEN"] = 12] = "ZIP_ER_TMPOPEN";
  Errors2[Errors2["ZIP_ER_ZLIB"] = 13] = "ZIP_ER_ZLIB";
  Errors2[Errors2["ZIP_ER_MEMORY"] = 14] = "ZIP_ER_MEMORY";
  Errors2[Errors2["ZIP_ER_CHANGED"] = 15] = "ZIP_ER_CHANGED";
  Errors2[Errors2["ZIP_ER_COMPNOTSUPP"] = 16] = "ZIP_ER_COMPNOTSUPP";
  Errors2[Errors2["ZIP_ER_EOF"] = 17] = "ZIP_ER_EOF";
  Errors2[Errors2["ZIP_ER_INVAL"] = 18] = "ZIP_ER_INVAL";
  Errors2[Errors2["ZIP_ER_NOZIP"] = 19] = "ZIP_ER_NOZIP";
  Errors2[Errors2["ZIP_ER_INTERNAL"] = 20] = "ZIP_ER_INTERNAL";
  Errors2[Errors2["ZIP_ER_INCONS"] = 21] = "ZIP_ER_INCONS";
  Errors2[Errors2["ZIP_ER_REMOVE"] = 22] = "ZIP_ER_REMOVE";
  Errors2[Errors2["ZIP_ER_DELETED"] = 23] = "ZIP_ER_DELETED";
  Errors2[Errors2["ZIP_ER_ENCRNOTSUPP"] = 24] = "ZIP_ER_ENCRNOTSUPP";
  Errors2[Errors2["ZIP_ER_RDONLY"] = 25] = "ZIP_ER_RDONLY";
  Errors2[Errors2["ZIP_ER_NOPASSWD"] = 26] = "ZIP_ER_NOPASSWD";
  Errors2[Errors2["ZIP_ER_WRONGPASSWD"] = 27] = "ZIP_ER_WRONGPASSWD";
  Errors2[Errors2["ZIP_ER_OPNOTSUPP"] = 28] = "ZIP_ER_OPNOTSUPP";
  Errors2[Errors2["ZIP_ER_INUSE"] = 29] = "ZIP_ER_INUSE";
  Errors2[Errors2["ZIP_ER_TELL"] = 30] = "ZIP_ER_TELL";
  Errors2[Errors2["ZIP_ER_COMPRESSED_DATA"] = 31] = "ZIP_ER_COMPRESSED_DATA";
  return Errors2;
})(Errors || {});
const makeInterface = (libzip) => ({
  get HEAP8() {
    return libzip.HEAP8;
  },
  get HEAPU8() {
    return libzip.HEAPU8;
  },
  errors: Errors,
  SEEK_SET: 0,
  SEEK_CUR: 1,
  SEEK_END: 2,
  ZIP_CHECKCONS: 4,
  ZIP_CREATE: 1,
  ZIP_EXCL: 2,
  ZIP_TRUNCATE: 8,
  ZIP_RDONLY: 16,
  ZIP_FL_OVERWRITE: 8192,
  ZIP_FL_COMPRESSED: 4,
  ZIP_OPSYS_DOS: 0,
  ZIP_OPSYS_AMIGA: 1,
  ZIP_OPSYS_OPENVMS: 2,
  ZIP_OPSYS_UNIX: 3,
  ZIP_OPSYS_VM_CMS: 4,
  ZIP_OPSYS_ATARI_ST: 5,
  ZIP_OPSYS_OS_2: 6,
  ZIP_OPSYS_MACINTOSH: 7,
  ZIP_OPSYS_Z_SYSTEM: 8,
  ZIP_OPSYS_CPM: 9,
  ZIP_OPSYS_WINDOWS_NTFS: 10,
  ZIP_OPSYS_MVS: 11,
  ZIP_OPSYS_VSE: 12,
  ZIP_OPSYS_ACORN_RISC: 13,
  ZIP_OPSYS_VFAT: 14,
  ZIP_OPSYS_ALTERNATE_MVS: 15,
  ZIP_OPSYS_BEOS: 16,
  ZIP_OPSYS_TANDEM: 17,
  ZIP_OPSYS_OS_400: 18,
  ZIP_OPSYS_OS_X: 19,
  ZIP_CM_DEFAULT: -1,
  ZIP_CM_STORE: 0,
  ZIP_CM_DEFLATE: 8,
  uint08S: libzip._malloc(1),
  uint16S: libzip._malloc(2),
  uint32S: libzip._malloc(4),
  uint64S: libzip._malloc(8),
  malloc: libzip._malloc,
  free: libzip._free,
  getValue: libzip.getValue,
  open: libzip.cwrap(`zip_open`, `number`, [`string`, `number`, `number`]),
  openFromSource: libzip.cwrap(`zip_open_from_source`, `number`, [`number`, `number`, `number`]),
  close: libzip.cwrap(`zip_close`, `number`, [`number`]),
  discard: libzip.cwrap(`zip_discard`, null, [`number`]),
  getError: libzip.cwrap(`zip_get_error`, `number`, [`number`]),
  getName: libzip.cwrap(`zip_get_name`, `string`, [`number`, `number`, `number`]),
  getNumEntries: libzip.cwrap(`zip_get_num_entries`, `number`, [`number`, `number`]),
  delete: libzip.cwrap(`zip_delete`, `number`, [`number`, `number`]),
  stat: libzip.cwrap(`zip_stat`, `number`, [`number`, `string`, `number`, `number`]),
  statIndex: libzip.cwrap(`zip_stat_index`, `number`, [`number`, ...number64, `number`, `number`]),
  fopen: libzip.cwrap(`zip_fopen`, `number`, [`number`, `string`, `number`]),
  fopenIndex: libzip.cwrap(`zip_fopen_index`, `number`, [`number`, ...number64, `number`]),
  fread: libzip.cwrap(`zip_fread`, `number`, [`number`, `number`, `number`, `number`]),
  fclose: libzip.cwrap(`zip_fclose`, `number`, [`number`]),
  dir: {
    add: libzip.cwrap(`zip_dir_add`, `number`, [`number`, `string`])
  },
  file: {
    add: libzip.cwrap(`zip_file_add`, `number`, [`number`, `string`, `number`, `number`]),
    getError: libzip.cwrap(`zip_file_get_error`, `number`, [`number`]),
    getExternalAttributes: libzip.cwrap(`zip_file_get_external_attributes`, `number`, [`number`, ...number64, `number`, `number`, `number`]),
    setExternalAttributes: libzip.cwrap(`zip_file_set_external_attributes`, `number`, [`number`, ...number64, `number`, `number`, `number`]),
    setMtime: libzip.cwrap(`zip_file_set_mtime`, `number`, [`number`, ...number64, `number`, `number`]),
    setCompression: libzip.cwrap(`zip_set_file_compression`, `number`, [`number`, ...number64, `number`, `number`])
  },
  ext: {
    countSymlinks: libzip.cwrap(`zip_ext_count_symlinks`, `number`, [`number`])
  },
  error: {
    initWithCode: libzip.cwrap(`zip_error_init_with_code`, null, [`number`, `number`]),
    strerror: libzip.cwrap(`zip_error_strerror`, `string`, [`number`])
  },
  name: {
    locate: libzip.cwrap(`zip_name_locate`, `number`, [`number`, `string`, `number`])
  },
  source: {
    fromUnattachedBuffer: libzip.cwrap(`zip_source_buffer_create`, `number`, [`number`, ...number64, `number`, `number`]),
    fromBuffer: libzip.cwrap(`zip_source_buffer`, `number`, [`number`, `number`, ...number64, `number`]),
    free: libzip.cwrap(`zip_source_free`, null, [`number`]),
    keep: libzip.cwrap(`zip_source_keep`, null, [`number`]),
    open: libzip.cwrap(`zip_source_open`, `number`, [`number`]),
    close: libzip.cwrap(`zip_source_close`, `number`, [`number`]),
    seek: libzip.cwrap(`zip_source_seek`, `number`, [`number`, ...number64, `number`]),
    tell: libzip.cwrap(`zip_source_tell`, `number`, [`number`]),
    read: libzip.cwrap(`zip_source_read`, `number`, [`number`, `number`, `number`]),
    error: libzip.cwrap(`zip_source_error`, `number`, [`number`]),
    setMtime: libzip.cwrap(`zip_source_set_mtime`, `number`, [`number`, `number`])
  },
  struct: {
    stat: libzip.cwrap(`zipstruct_stat`, `number`, []),
    statS: libzip.cwrap(`zipstruct_statS`, `number`, []),
    statName: libzip.cwrap(`zipstruct_stat_name`, `string`, [`number`]),
    statIndex: libzip.cwrap(`zipstruct_stat_index`, `number`, [`number`]),
    statSize: libzip.cwrap(`zipstruct_stat_size`, `number`, [`number`]),
    statCompSize: libzip.cwrap(`zipstruct_stat_comp_size`, `number`, [`number`]),
    statCompMethod: libzip.cwrap(`zipstruct_stat_comp_method`, `number`, [`number`]),
    statMtime: libzip.cwrap(`zipstruct_stat_mtime`, `number`, [`number`]),
    statCrc: libzip.cwrap(`zipstruct_stat_crc`, `number`, [`number`]),
    error: libzip.cwrap(`zipstruct_error`, `number`, []),
    errorS: libzip.cwrap(`zipstruct_errorS`, `number`, []),
    errorCodeZip: libzip.cwrap(`zipstruct_error_code_zip`, `number`, [`number`])
  }
});

let mod = null;
function getLibzipSync() {
  if (mod === null)
    mod = makeInterface(createModule());
  return mod;
}

var ErrorCode = /* @__PURE__ */ ((ErrorCode2) => {
  ErrorCode2["API_ERROR"] = `API_ERROR`;
  ErrorCode2["BUILTIN_NODE_RESOLUTION_FAILED"] = `BUILTIN_NODE_RESOLUTION_FAILED`;
  ErrorCode2["EXPORTS_RESOLUTION_FAILED"] = `EXPORTS_RESOLUTION_FAILED`;
  ErrorCode2["MISSING_DEPENDENCY"] = `MISSING_DEPENDENCY`;
  ErrorCode2["MISSING_PEER_DEPENDENCY"] = `MISSING_PEER_DEPENDENCY`;
  ErrorCode2["QUALIFIED_PATH_RESOLUTION_FAILED"] = `QUALIFIED_PATH_RESOLUTION_FAILED`;
  ErrorCode2["INTERNAL"] = `INTERNAL`;
  ErrorCode2["UNDECLARED_DEPENDENCY"] = `UNDECLARED_DEPENDENCY`;
  ErrorCode2["UNSUPPORTED"] = `UNSUPPORTED`;
  return ErrorCode2;
})(ErrorCode || {});
const MODULE_NOT_FOUND_ERRORS = /* @__PURE__ */ new Set([
  "BUILTIN_NODE_RESOLUTION_FAILED" /* BUILTIN_NODE_RESOLUTION_FAILED */,
  "MISSING_DEPENDENCY" /* MISSING_DEPENDENCY */,
  "MISSING_PEER_DEPENDENCY" /* MISSING_PEER_DEPENDENCY */,
  "QUALIFIED_PATH_RESOLUTION_FAILED" /* QUALIFIED_PATH_RESOLUTION_FAILED */,
  "UNDECLARED_DEPENDENCY" /* UNDECLARED_DEPENDENCY */
]);
function makeError(pnpCode, message, data = {}, code) {
  code != null ? code : code = MODULE_NOT_FOUND_ERRORS.has(pnpCode) ? `MODULE_NOT_FOUND` : pnpCode;
  const propertySpec = {
    configurable: true,
    writable: true,
    enumerable: false
  };
  return Object.defineProperties(new Error(message), {
    code: {
      ...propertySpec,
      value: code
    },
    pnpCode: {
      ...propertySpec,
      value: pnpCode
    },
    data: {
      ...propertySpec,
      value: data
    }
  });
}
function getIssuerModule(parent) {
  let issuer = parent;
  while (issuer && (issuer.id === `[eval]` || issuer.id === `<repl>` || !issuer.filename))
    issuer = issuer.parent;
  return issuer || null;
}
function getPathForDisplay(p) {
  return npath.normalize(npath.fromPortablePath(p));
}

const [major, minor] = process.versions.node.split(`.`).map((value) => parseInt(value, 10));
const WATCH_MODE_MESSAGE_USES_ARRAYS = major > 19 || major === 19 && minor >= 2 || major === 18 && minor >= 13;

const builtinModules = new Set(require$$0.Module.builtinModules || Object.keys(process.binding(`natives`)));
const isBuiltinModule = (request) => request.startsWith(`node:`) || builtinModules.has(request);
function readPackageScope(checkPath) {
  const rootSeparatorIndex = checkPath.indexOf(npath.sep);
  let separatorIndex;
  do {
    separatorIndex = checkPath.lastIndexOf(npath.sep);
    checkPath = checkPath.slice(0, separatorIndex);
    if (checkPath.endsWith(`${npath.sep}node_modules`))
      return false;
    const pjson = readPackage(checkPath + npath.sep);
    if (pjson) {
      return {
        data: pjson,
        path: checkPath
      };
    }
  } while (separatorIndex > rootSeparatorIndex);
  return false;
}
function readPackage(requestPath) {
  const jsonPath = npath.resolve(requestPath, `package.json`);
  if (!fs__default.default.existsSync(jsonPath))
    return null;
  return JSON.parse(fs__default.default.readFileSync(jsonPath, `utf8`));
}
function ERR_REQUIRE_ESM(filename, parentPath = null) {
  const basename = parentPath && path__default.default.basename(filename) === path__default.default.basename(parentPath) ? filename : path__default.default.basename(filename);
  const msg = `require() of ES Module ${filename}${parentPath ? ` from ${parentPath}` : ``} not supported.
Instead change the require of ${basename} in ${parentPath} to a dynamic import() which is available in all CommonJS modules.`;
  const err = new Error(msg);
  err.code = `ERR_REQUIRE_ESM`;
  return err;
}
function reportRequiredFilesToWatchMode(files) {
  if (process.env.WATCH_REPORT_DEPENDENCIES && process.send) {
    files = files.map((filename) => npath.fromPortablePath(VirtualFS.resolveVirtual(npath.toPortablePath(filename))));
    if (WATCH_MODE_MESSAGE_USES_ARRAYS) {
      process.send({ "watch:require": files });
    } else {
      for (const filename of files) {
        process.send({ "watch:require": filename });
      }
    }
  }
}

function applyPatch(pnpapi, opts) {
  const defaultCache = {};
  let enableNativeHooks = true;
  process.versions.pnp = String(pnpapi.VERSIONS.std);
  const moduleExports = require$$0__default.default;
  moduleExports.findPnpApi = (lookupSource) => {
    const lookupPath = lookupSource instanceof url.URL ? url.fileURLToPath(lookupSource) : lookupSource;
    const apiPath = opts.manager.findApiPathFor(lookupPath);
    if (apiPath === null)
      return null;
    const apiEntry = opts.manager.getApiEntry(apiPath, true);
    return apiEntry.instance.findPackageLocator(lookupPath) ? apiEntry.instance : null;
  };
  function getRequireStack(parent) {
    const requireStack = [];
    for (let cursor = parent; cursor; cursor = cursor.parent)
      requireStack.push(cursor.filename || cursor.id);
    return requireStack;
  }
  const originalModuleLoad = require$$0.Module._load;
  require$$0.Module._load = function(request, parent, isMain) {
    if (!enableNativeHooks)
      return originalModuleLoad.call(require$$0.Module, request, parent, isMain);
    if (isBuiltinModule(request)) {
      try {
        enableNativeHooks = false;
        return originalModuleLoad.call(require$$0.Module, request, parent, isMain);
      } finally {
        enableNativeHooks = true;
      }
    }
    const parentApiPath = opts.manager.getApiPathFromParent(parent);
    const parentApi = parentApiPath !== null ? opts.manager.getApiEntry(parentApiPath, true).instance : null;
    if (parentApi === null)
      return originalModuleLoad(request, parent, isMain);
    if (request === `pnpapi`)
      return parentApi;
    const modulePath = require$$0.Module._resolveFilename(request, parent, isMain);
    const isOwnedByRuntime = parentApi !== null ? parentApi.findPackageLocator(modulePath) !== null : false;
    const moduleApiPath = isOwnedByRuntime ? parentApiPath : opts.manager.findApiPathFor(npath.dirname(modulePath));
    const entry = moduleApiPath !== null ? opts.manager.getApiEntry(moduleApiPath) : { instance: null, cache: defaultCache };
    const cacheEntry = entry.cache[modulePath];
    if (cacheEntry) {
      if (cacheEntry.loaded === false && cacheEntry.isLoading !== true) {
        try {
          cacheEntry.isLoading = true;
          if (isMain) {
            process.mainModule = cacheEntry;
            cacheEntry.id = `.`;
          }
          cacheEntry.load(modulePath);
        } finally {
          cacheEntry.isLoading = false;
        }
      }
      return cacheEntry.exports;
    }
    const module = new require$$0.Module(modulePath, parent != null ? parent : void 0);
    module.pnpApiPath = moduleApiPath;
    reportRequiredFilesToWatchMode([modulePath]);
    entry.cache[modulePath] = module;
    if (isMain) {
      process.mainModule = module;
      module.id = `.`;
    }
    let hasThrown = true;
    try {
      module.isLoading = true;
      module.load(modulePath);
      hasThrown = false;
    } finally {
      module.isLoading = false;
      if (hasThrown) {
        delete require$$0.Module._cache[modulePath];
      }
    }
    return module.exports;
  };
  function getIssuerSpecsFromPaths(paths) {
    return paths.map((path) => ({
      apiPath: opts.manager.findApiPathFor(path),
      path,
      module: null
    }));
  }
  function getIssuerSpecsFromModule(module) {
    var _a;
    if (module && module.id !== `<repl>` && module.id !== `internal/preload` && !module.parent && !module.filename && module.paths.length > 0) {
      return [{
        apiPath: opts.manager.findApiPathFor(module.paths[0]),
        path: module.paths[0],
        module
      }];
    }
    const issuer = getIssuerModule(module);
    if (issuer !== null) {
      const path = npath.dirname(issuer.filename);
      const apiPath = opts.manager.getApiPathFromParent(issuer);
      return [{ apiPath, path, module }];
    } else {
      const path = process.cwd();
      const apiPath = (_a = opts.manager.findApiPathFor(npath.join(path, `[file]`))) != null ? _a : opts.manager.getApiPathFromParent(null);
      return [{ apiPath, path, module }];
    }
  }
  function makeFakeParent(path) {
    const fakeParent = new require$$0.Module(``);
    const fakeFilePath = npath.join(path, `[file]`);
    fakeParent.paths = require$$0.Module._nodeModulePaths(fakeFilePath);
    return fakeParent;
  }
  const pathRegExp = /^(?![a-zA-Z]:[\\/]|\\\\|\.{0,2}(?:\/|$))((?:@[^/]+\/)?[^/]+)\/*(.*|)$/;
  const originalModuleResolveFilename = require$$0.Module._resolveFilename;
  require$$0.Module._resolveFilename = function(request, parent, isMain, options) {
    if (isBuiltinModule(request))
      return request;
    if (!enableNativeHooks)
      return originalModuleResolveFilename.call(require$$0.Module, request, parent, isMain, options);
    if (options && options.plugnplay === false) {
      const { plugnplay, ...rest } = options;
      const forwardedOptions = Object.keys(rest).length > 0 ? rest : void 0;
      try {
        enableNativeHooks = false;
        return originalModuleResolveFilename.call(require$$0.Module, request, parent, isMain, forwardedOptions);
      } finally {
        enableNativeHooks = true;
      }
    }
    if (options) {
      const optionNames = new Set(Object.keys(options));
      optionNames.delete(`paths`);
      optionNames.delete(`plugnplay`);
      if (optionNames.size > 0) {
        throw makeError(
          ErrorCode.UNSUPPORTED,
          `Some options passed to require() aren't supported by PnP yet (${Array.from(optionNames).join(`, `)})`
        );
      }
    }
    const issuerSpecs = options && options.paths ? getIssuerSpecsFromPaths(options.paths) : getIssuerSpecsFromModule(parent);
    if (request.match(pathRegExp) === null) {
      const parentDirectory = (parent == null ? void 0 : parent.filename) != null ? npath.dirname(parent.filename) : null;
      const absoluteRequest = npath.isAbsolute(request) ? request : parentDirectory !== null ? npath.resolve(parentDirectory, request) : null;
      if (absoluteRequest !== null) {
        const apiPath = parentDirectory === npath.dirname(absoluteRequest) && (parent == null ? void 0 : parent.pnpApiPath) ? parent.pnpApiPath : opts.manager.findApiPathFor(absoluteRequest);
        if (apiPath !== null) {
          issuerSpecs.unshift({
            apiPath,
            path: parentDirectory,
            module: null
          });
        }
      }
    }
    let firstError;
    for (const { apiPath, path, module } of issuerSpecs) {
      let resolution;
      const issuerApi = apiPath !== null ? opts.manager.getApiEntry(apiPath, true).instance : null;
      try {
        if (issuerApi !== null) {
          resolution = issuerApi.resolveRequest(request, path !== null ? `${path}/` : null);
        } else {
          if (path === null)
            throw new Error(`Assertion failed: Expected the path to be set`);
          resolution = originalModuleResolveFilename.call(require$$0.Module, request, module || makeFakeParent(path), isMain);
        }
      } catch (error) {
        firstError = firstError || error;
        continue;
      }
      if (resolution !== null) {
        return resolution;
      }
    }
    const requireStack = getRequireStack(parent);
    Object.defineProperty(firstError, `requireStack`, {
      configurable: true,
      writable: true,
      enumerable: false,
      value: requireStack
    });
    if (requireStack.length > 0)
      firstError.message += `
Require stack:
- ${requireStack.join(`
- `)}`;
    if (typeof firstError.pnpCode === `string`)
      Error.captureStackTrace(firstError);
    throw firstError;
  };
  const originalFindPath = require$$0.Module._findPath;
  require$$0.Module._findPath = function(request, paths, isMain) {
    if (request === `pnpapi`)
      return false;
    if (!enableNativeHooks)
      return originalFindPath.call(require$$0.Module, request, paths, isMain);
    const isAbsolute = npath.isAbsolute(request);
    if (isAbsolute)
      paths = [``];
    else if (!paths || paths.length === 0)
      return false;
    for (const path of paths) {
      let resolution;
      try {
        const pnpApiPath = opts.manager.findApiPathFor(isAbsolute ? request : path);
        if (pnpApiPath !== null) {
          const api = opts.manager.getApiEntry(pnpApiPath, true).instance;
          resolution = api.resolveRequest(request, path) || false;
        } else {
          resolution = originalFindPath.call(require$$0.Module, request, [path], isMain);
        }
      } catch (error) {
        continue;
      }
      if (resolution) {
        return resolution;
      }
    }
    return false;
  };
  const originalExtensionJSFunction = require$$0.Module._extensions[`.js`];
  require$$0.Module._extensions[`.js`] = function(module, filename) {
    var _a, _b;
    if (filename.endsWith(`.js`)) {
      const pkg = readPackageScope(filename);
      if (pkg && ((_a = pkg.data) == null ? void 0 : _a.type) === `module`) {
        const err = ERR_REQUIRE_ESM(filename, (_b = module.parent) == null ? void 0 : _b.filename);
        Error.captureStackTrace(err);
        throw err;
      }
    }
    originalExtensionJSFunction.call(this, module, filename);
  };
  const originalDlopen = process.dlopen;
  process.dlopen = function(...args) {
    const [module, filename, ...rest] = args;
    return originalDlopen.call(
      this,
      module,
      npath.fromPortablePath(VirtualFS.resolveVirtual(npath.toPortablePath(filename))),
      ...rest
    );
  };
  const originalEmit = process.emit;
  process.emit = function(name, data, ...args) {
    if (name === `warning` && typeof data === `object` && data.name === `ExperimentalWarning` && (data.message.includes(`--experimental-loader`) || data.message.includes(`Custom ESM Loaders is an experimental feature`)))
      return false;
    return originalEmit.apply(process, arguments);
  };
  patchFs(fs__default.default, new PosixFS(opts.fakeFs));
}

function hydrateRuntimeState(data, { basePath }) {
  const portablePath = npath.toPortablePath(basePath);
  const absolutePortablePath = ppath.resolve(portablePath);
  const ignorePattern = data.ignorePatternData !== null ? new RegExp(data.ignorePatternData) : null;
  const packageLocatorsByLocations = /* @__PURE__ */ new Map();
  const packageRegistry = new Map(data.packageRegistryData.map(([packageName, packageStoreData]) => {
    return [packageName, new Map(packageStoreData.map(([packageReference, packageInformationData]) => {
      var _a;
      if (packageName === null !== (packageReference === null))
        throw new Error(`Assertion failed: The name and reference should be null, or neither should`);
      const discardFromLookup = (_a = packageInformationData.discardFromLookup) != null ? _a : false;
      const packageLocator = { name: packageName, reference: packageReference };
      const entry = packageLocatorsByLocations.get(packageInformationData.packageLocation);
      if (!entry) {
        packageLocatorsByLocations.set(packageInformationData.packageLocation, { locator: packageLocator, discardFromLookup });
      } else {
        entry.discardFromLookup = entry.discardFromLookup && discardFromLookup;
        if (!discardFromLookup) {
          entry.locator = packageLocator;
        }
      }
      let resolvedPackageLocation = null;
      return [packageReference, {
        packageDependencies: new Map(packageInformationData.packageDependencies),
        packagePeers: new Set(packageInformationData.packagePeers),
        linkType: packageInformationData.linkType,
        discardFromLookup,
        get packageLocation() {
          return resolvedPackageLocation || (resolvedPackageLocation = ppath.join(absolutePortablePath, packageInformationData.packageLocation));
        }
      }];
    }))];
  }));
  const fallbackExclusionList = new Map(data.fallbackExclusionList.map(([packageName, packageReferences]) => {
    return [packageName, new Set(packageReferences)];
  }));
  const fallbackPool = new Map(data.fallbackPool);
  const dependencyTreeRoots = data.dependencyTreeRoots;
  const enableTopLevelFallback = data.enableTopLevelFallback;
  return {
    basePath: portablePath,
    dependencyTreeRoots,
    enableTopLevelFallback,
    fallbackExclusionList,
    fallbackPool,
    ignorePattern,
    packageLocatorsByLocations,
    packageRegistry
  };
}

const ArrayIsArray = Array.isArray;
const JSONStringify = JSON.stringify;
const ObjectGetOwnPropertyNames = Object.getOwnPropertyNames;
const ObjectPrototypeHasOwnProperty = (obj, prop) => Object.prototype.hasOwnProperty.call(obj, prop);
const RegExpPrototypeExec = (obj, string) => RegExp.prototype.exec.call(obj, string);
const RegExpPrototypeSymbolReplace = (obj, ...rest) => RegExp.prototype[Symbol.replace].apply(obj, rest);
const StringPrototypeEndsWith = (str, ...rest) => String.prototype.endsWith.apply(str, rest);
const StringPrototypeIncludes = (str, ...rest) => String.prototype.includes.apply(str, rest);
const StringPrototypeLastIndexOf = (str, ...rest) => String.prototype.lastIndexOf.apply(str, rest);
const StringPrototypeIndexOf = (str, ...rest) => String.prototype.indexOf.apply(str, rest);
const StringPrototypeReplace = (str, ...rest) => String.prototype.replace.apply(str, rest);
const StringPrototypeSlice = (str, ...rest) => String.prototype.slice.apply(str, rest);
const StringPrototypeStartsWith = (str, ...rest) => String.prototype.startsWith.apply(str, rest);
const SafeMap = Map;
const JSONParse = JSON.parse;

function createErrorType(code, messageCreator, errorType) {
  return class extends errorType {
    constructor(...args) {
      super(messageCreator(...args));
      this.code = code;
      this.name = `${errorType.name} [${code}]`;
    }
  };
}
const ERR_PACKAGE_IMPORT_NOT_DEFINED = createErrorType(
  `ERR_PACKAGE_IMPORT_NOT_DEFINED`,
  (specifier, packagePath, base) => {
    return `Package import specifier "${specifier}" is not defined${packagePath ? ` in package ${packagePath}package.json` : ``} imported from ${base}`;
  },
  TypeError
);
const ERR_INVALID_MODULE_SPECIFIER = createErrorType(
  `ERR_INVALID_MODULE_SPECIFIER`,
  (request, reason, base = void 0) => {
    return `Invalid module "${request}" ${reason}${base ? ` imported from ${base}` : ``}`;
  },
  TypeError
);
const ERR_INVALID_PACKAGE_TARGET = createErrorType(
  `ERR_INVALID_PACKAGE_TARGET`,
  (pkgPath, key, target, isImport = false, base = void 0) => {
    const relError = typeof target === `string` && !isImport && target.length && !StringPrototypeStartsWith(target, `./`);
    if (key === `.`) {
      assert__default.default(isImport === false);
      return `Invalid "exports" main target ${JSONStringify(target)} defined in the package config ${pkgPath}package.json${base ? ` imported from ${base}` : ``}${relError ? `; targets must start with "./"` : ``}`;
    }
    return `Invalid "${isImport ? `imports` : `exports`}" target ${JSONStringify(
      target
    )} defined for '${key}' in the package config ${pkgPath}package.json${base ? ` imported from ${base}` : ``}${relError ? `; targets must start with "./"` : ``}`;
  },
  Error
);
const ERR_INVALID_PACKAGE_CONFIG = createErrorType(
  `ERR_INVALID_PACKAGE_CONFIG`,
  (path, base, message) => {
    return `Invalid package config ${path}${base ? ` while importing ${base}` : ``}${message ? `. ${message}` : ``}`;
  },
  Error
);
const ERR_PACKAGE_PATH_NOT_EXPORTED = createErrorType(
  "ERR_PACKAGE_PATH_NOT_EXPORTED",
  (pkgPath, subpath, base = void 0) => {
    if (subpath === ".")
      return `No "exports" main defined in ${pkgPath}package.json${base ? ` imported from ${base}` : ""}`;
    return `Package subpath '${subpath}' is not defined by "exports" in ${pkgPath}package.json${base ? ` imported from ${base}` : ""}`;
  },
  Error
);

function filterOwnProperties(source, keys) {
  const filtered = /* @__PURE__ */ Object.create(null);
  for (let i = 0; i < keys.length; i++) {
    const key = keys[i];
    if (ObjectPrototypeHasOwnProperty(source, key)) {
      filtered[key] = source[key];
    }
  }
  return filtered;
}

const packageJSONCache = new SafeMap();
function getPackageConfig(path, specifier, base, readFileSyncFn) {
  const existing = packageJSONCache.get(path);
  if (existing !== void 0) {
    return existing;
  }
  const source = readFileSyncFn(path);
  if (source === void 0) {
    const packageConfig2 = {
      pjsonPath: path,
      exists: false,
      main: void 0,
      name: void 0,
      type: "none",
      exports: void 0,
      imports: void 0
    };
    packageJSONCache.set(path, packageConfig2);
    return packageConfig2;
  }
  let packageJSON;
  try {
    packageJSON = JSONParse(source);
  } catch (error) {
    throw new ERR_INVALID_PACKAGE_CONFIG(
      path,
      (base ? `"${specifier}" from ` : "") + url.fileURLToPath(base || specifier),
      error.message
    );
  }
  let { imports, main, name, type } = filterOwnProperties(packageJSON, [
    "imports",
    "main",
    "name",
    "type"
  ]);
  const exports = ObjectPrototypeHasOwnProperty(packageJSON, "exports") ? packageJSON.exports : void 0;
  if (typeof imports !== "object" || imports === null) {
    imports = void 0;
  }
  if (typeof main !== "string") {
    main = void 0;
  }
  if (typeof name !== "string") {
    name = void 0;
  }
  if (type !== "module" && type !== "commonjs") {
    type = "none";
  }
  const packageConfig = {
    pjsonPath: path,
    exists: true,
    main,
    name,
    type,
    exports,
    imports
  };
  packageJSONCache.set(path, packageConfig);
  return packageConfig;
}
function getPackageScopeConfig(resolved, readFileSyncFn) {
  let packageJSONUrl = new URL("./package.json", resolved);
  while (true) {
    const packageJSONPath2 = packageJSONUrl.pathname;
    if (StringPrototypeEndsWith(packageJSONPath2, "node_modules/package.json")) {
      break;
    }
    const packageConfig2 = getPackageConfig(
      url.fileURLToPath(packageJSONUrl),
      resolved,
      void 0,
      readFileSyncFn
    );
    if (packageConfig2.exists) {
      return packageConfig2;
    }
    const lastPackageJSONUrl = packageJSONUrl;
    packageJSONUrl = new URL("../package.json", packageJSONUrl);
    if (packageJSONUrl.pathname === lastPackageJSONUrl.pathname) {
      break;
    }
  }
  const packageJSONPath = url.fileURLToPath(packageJSONUrl);
  const packageConfig = {
    pjsonPath: packageJSONPath,
    exists: false,
    main: void 0,
    name: void 0,
    type: "none",
    exports: void 0,
    imports: void 0
  };
  packageJSONCache.set(packageJSONPath, packageConfig);
  return packageConfig;
}

/**
  @license
  Copyright Node.js contributors. All rights reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
*/
function throwImportNotDefined(specifier, packageJSONUrl, base) {
  throw new ERR_PACKAGE_IMPORT_NOT_DEFINED(
    specifier,
    packageJSONUrl && url.fileURLToPath(new URL(".", packageJSONUrl)),
    url.fileURLToPath(base)
  );
}
function throwInvalidSubpath(subpath, packageJSONUrl, internal, base) {
  const reason = `request is not a valid subpath for the "${internal ? "imports" : "exports"}" resolution of ${url.fileURLToPath(packageJSONUrl)}`;
  throw new ERR_INVALID_MODULE_SPECIFIER(
    subpath,
    reason,
    base && url.fileURLToPath(base)
  );
}
function throwInvalidPackageTarget(subpath, target, packageJSONUrl, internal, base) {
  if (typeof target === "object" && target !== null) {
    target = JSONStringify(target, null, "");
  } else {
    target = `${target}`;
  }
  throw new ERR_INVALID_PACKAGE_TARGET(
    url.fileURLToPath(new URL(".", packageJSONUrl)),
    subpath,
    target,
    internal,
    base && url.fileURLToPath(base)
  );
}
const invalidSegmentRegEx = /(^|\\|\/)((\.|%2e)(\.|%2e)?|(n|%6e|%4e)(o|%6f|%4f)(d|%64|%44)(e|%65|%45)(_|%5f)(m|%6d|%4d)(o|%6f|%4f)(d|%64|%44)(u|%75|%55)(l|%6c|%4c)(e|%65|%45)(s|%73|%53))(\\|\/|$)/i;
const patternRegEx = /\*/g;
function resolvePackageTargetString(target, subpath, match, packageJSONUrl, base, pattern, internal, conditions) {
  if (subpath !== "" && !pattern && target[target.length - 1] !== "/")
    throwInvalidPackageTarget(match, target, packageJSONUrl, internal, base);
  if (!StringPrototypeStartsWith(target, "./")) {
    if (internal && !StringPrototypeStartsWith(target, "../") && !StringPrototypeStartsWith(target, "/")) {
      let isURL = false;
      try {
        new URL(target);
        isURL = true;
      } catch {
      }
      if (!isURL) {
        const exportTarget = pattern ? RegExpPrototypeSymbolReplace(patternRegEx, target, () => subpath) : target + subpath;
        return exportTarget;
      }
    }
    throwInvalidPackageTarget(match, target, packageJSONUrl, internal, base);
  }
  if (RegExpPrototypeExec(
    invalidSegmentRegEx,
    StringPrototypeSlice(target, 2)
  ) !== null)
    throwInvalidPackageTarget(match, target, packageJSONUrl, internal, base);
  const resolved = new URL(target, packageJSONUrl);
  const resolvedPath = resolved.pathname;
  const packagePath = new URL(".", packageJSONUrl).pathname;
  if (!StringPrototypeStartsWith(resolvedPath, packagePath))
    throwInvalidPackageTarget(match, target, packageJSONUrl, internal, base);
  if (subpath === "")
    return resolved;
  if (RegExpPrototypeExec(invalidSegmentRegEx, subpath) !== null) {
    const request = pattern ? StringPrototypeReplace(match, "*", () => subpath) : match + subpath;
    throwInvalidSubpath(request, packageJSONUrl, internal, base);
  }
  if (pattern) {
    return new URL(
      RegExpPrototypeSymbolReplace(patternRegEx, resolved.href, () => subpath)
    );
  }
  return new URL(subpath, resolved);
}
function isArrayIndex(key) {
  const keyNum = +key;
  if (`${keyNum}` !== key)
    return false;
  return keyNum >= 0 && keyNum < 4294967295;
}
function resolvePackageTarget(packageJSONUrl, target, subpath, packageSubpath, base, pattern, internal, conditions) {
  if (typeof target === "string") {
    return resolvePackageTargetString(
      target,
      subpath,
      packageSubpath,
      packageJSONUrl,
      base,
      pattern,
      internal);
  } else if (ArrayIsArray(target)) {
    if (target.length === 0) {
      return null;
    }
    let lastException;
    for (let i = 0; i < target.length; i++) {
      const targetItem = target[i];
      let resolveResult;
      try {
        resolveResult = resolvePackageTarget(
          packageJSONUrl,
          targetItem,
          subpath,
          packageSubpath,
          base,
          pattern,
          internal,
          conditions
        );
      } catch (e) {
        lastException = e;
        if (e.code === "ERR_INVALID_PACKAGE_TARGET") {
          continue;
        }
        throw e;
      }
      if (resolveResult === void 0) {
        continue;
      }
      if (resolveResult === null) {
        lastException = null;
        continue;
      }
      return resolveResult;
    }
    if (lastException === void 0 || lastException === null)
      return lastException;
    throw lastException;
  } else if (typeof target === "object" && target !== null) {
    const keys = ObjectGetOwnPropertyNames(target);
    for (let i = 0; i < keys.length; i++) {
      const key = keys[i];
      if (isArrayIndex(key)) {
        throw new ERR_INVALID_PACKAGE_CONFIG(
          url.fileURLToPath(packageJSONUrl),
          base,
          '"exports" cannot contain numeric property keys.'
        );
      }
    }
    for (let i = 0; i < keys.length; i++) {
      const key = keys[i];
      if (key === "default" || conditions.has(key)) {
        const conditionalTarget = target[key];
        const resolveResult = resolvePackageTarget(
          packageJSONUrl,
          conditionalTarget,
          subpath,
          packageSubpath,
          base,
          pattern,
          internal,
          conditions
        );
        if (resolveResult === void 0)
          continue;
        return resolveResult;
      }
    }
    return void 0;
  } else if (target === null) {
    return null;
  }
  throwInvalidPackageTarget(
    packageSubpath,
    target,
    packageJSONUrl,
    internal,
    base
  );
}
function patternKeyCompare(a, b) {
  const aPatternIndex = StringPrototypeIndexOf(a, "*");
  const bPatternIndex = StringPrototypeIndexOf(b, "*");
  const baseLenA = aPatternIndex === -1 ? a.length : aPatternIndex + 1;
  const baseLenB = bPatternIndex === -1 ? b.length : bPatternIndex + 1;
  if (baseLenA > baseLenB)
    return -1;
  if (baseLenB > baseLenA)
    return 1;
  if (aPatternIndex === -1)
    return 1;
  if (bPatternIndex === -1)
    return -1;
  if (a.length > b.length)
    return -1;
  if (b.length > a.length)
    return 1;
  return 0;
}
function isConditionalExportsMainSugar(exports, packageJSONUrl, base) {
  if (typeof exports === "string" || ArrayIsArray(exports))
    return true;
  if (typeof exports !== "object" || exports === null)
    return false;
  const keys = ObjectGetOwnPropertyNames(exports);
  let isConditionalSugar = false;
  let i = 0;
  for (let j = 0; j < keys.length; j++) {
    const key = keys[j];
    const curIsConditionalSugar = key === "" || key[0] !== ".";
    if (i++ === 0) {
      isConditionalSugar = curIsConditionalSugar;
    } else if (isConditionalSugar !== curIsConditionalSugar) {
      throw new ERR_INVALID_PACKAGE_CONFIG(
        url.fileURLToPath(packageJSONUrl),
        base,
        `"exports" cannot contain some keys starting with '.' and some not. The exports object must either be an object of package subpath keys or an object of main entry condition name keys only.`
      );
    }
  }
  return isConditionalSugar;
}
function throwExportsNotFound(subpath, packageJSONUrl, base) {
  throw new ERR_PACKAGE_PATH_NOT_EXPORTED(
    url.fileURLToPath(new URL(".", packageJSONUrl)),
    subpath,
    base && url.fileURLToPath(base)
  );
}
const emittedPackageWarnings = /* @__PURE__ */ new Set();
function emitTrailingSlashPatternDeprecation(match, pjsonUrl, base) {
  const pjsonPath = url.fileURLToPath(pjsonUrl);
  if (emittedPackageWarnings.has(pjsonPath + "|" + match))
    return;
  emittedPackageWarnings.add(pjsonPath + "|" + match);
  process.emitWarning(
    `Use of deprecated trailing slash pattern mapping "${match}" in the "exports" field module resolution of the package at ${pjsonPath}${base ? ` imported from ${url.fileURLToPath(base)}` : ""}. Mapping specifiers ending in "/" is no longer supported.`,
    "DeprecationWarning",
    "DEP0155"
  );
}
function packageExportsResolve({
  packageJSONUrl,
  packageSubpath,
  exports,
  base,
  conditions
}) {
  if (isConditionalExportsMainSugar(exports, packageJSONUrl, base))
    exports = { ".": exports };
  if (ObjectPrototypeHasOwnProperty(exports, packageSubpath) && !StringPrototypeIncludes(packageSubpath, "*") && !StringPrototypeEndsWith(packageSubpath, "/")) {
    const target = exports[packageSubpath];
    const resolveResult = resolvePackageTarget(
      packageJSONUrl,
      target,
      "",
      packageSubpath,
      base,
      false,
      false,
      conditions
    );
    if (resolveResult == null) {
      throwExportsNotFound(packageSubpath, packageJSONUrl, base);
    }
    return resolveResult;
  }
  let bestMatch = "";
  let bestMatchSubpath;
  const keys = ObjectGetOwnPropertyNames(exports);
  for (let i = 0; i < keys.length; i++) {
    const key = keys[i];
    const patternIndex = StringPrototypeIndexOf(key, "*");
    if (patternIndex !== -1 && StringPrototypeStartsWith(
      packageSubpath,
      StringPrototypeSlice(key, 0, patternIndex)
    )) {
      if (StringPrototypeEndsWith(packageSubpath, "/"))
        emitTrailingSlashPatternDeprecation(
          packageSubpath,
          packageJSONUrl,
          base
        );
      const patternTrailer = StringPrototypeSlice(key, patternIndex + 1);
      if (packageSubpath.length >= key.length && StringPrototypeEndsWith(packageSubpath, patternTrailer) && patternKeyCompare(bestMatch, key) === 1 && StringPrototypeLastIndexOf(key, "*") === patternIndex) {
        bestMatch = key;
        bestMatchSubpath = StringPrototypeSlice(
          packageSubpath,
          patternIndex,
          packageSubpath.length - patternTrailer.length
        );
      }
    }
  }
  if (bestMatch) {
    const target = exports[bestMatch];
    const resolveResult = resolvePackageTarget(
      packageJSONUrl,
      target,
      bestMatchSubpath,
      bestMatch,
      base,
      true,
      false,
      conditions
    );
    if (resolveResult == null) {
      throwExportsNotFound(packageSubpath, packageJSONUrl, base);
    }
    return resolveResult;
  }
  throwExportsNotFound(packageSubpath, packageJSONUrl, base);
}
function packageImportsResolve({ name, base, conditions, readFileSyncFn }) {
  if (name === "#" || StringPrototypeStartsWith(name, "#/") || StringPrototypeEndsWith(name, "/")) {
    const reason = "is not a valid internal imports specifier name";
    throw new ERR_INVALID_MODULE_SPECIFIER(name, reason, url.fileURLToPath(base));
  }
  let packageJSONUrl;
  const packageConfig = getPackageScopeConfig(base, readFileSyncFn);
  if (packageConfig.exists) {
    packageJSONUrl = url.pathToFileURL(packageConfig.pjsonPath);
    const imports = packageConfig.imports;
    if (imports) {
      if (ObjectPrototypeHasOwnProperty(imports, name) && !StringPrototypeIncludes(name, "*")) {
        const resolveResult = resolvePackageTarget(
          packageJSONUrl,
          imports[name],
          "",
          name,
          base,
          false,
          true,
          conditions
        );
        if (resolveResult != null) {
          return resolveResult;
        }
      } else {
        let bestMatch = "";
        let bestMatchSubpath;
        const keys = ObjectGetOwnPropertyNames(imports);
        for (let i = 0; i < keys.length; i++) {
          const key = keys[i];
          const patternIndex = StringPrototypeIndexOf(key, "*");
          if (patternIndex !== -1 && StringPrototypeStartsWith(
            name,
            StringPrototypeSlice(key, 0, patternIndex)
          )) {
            const patternTrailer = StringPrototypeSlice(key, patternIndex + 1);
            if (name.length >= key.length && StringPrototypeEndsWith(name, patternTrailer) && patternKeyCompare(bestMatch, key) === 1 && StringPrototypeLastIndexOf(key, "*") === patternIndex) {
              bestMatch = key;
              bestMatchSubpath = StringPrototypeSlice(
                name,
                patternIndex,
                name.length - patternTrailer.length
              );
            }
          }
        }
        if (bestMatch) {
          const target = imports[bestMatch];
          const resolveResult = resolvePackageTarget(
            packageJSONUrl,
            target,
            bestMatchSubpath,
            bestMatch,
            base,
            true,
            true,
            conditions
          );
          if (resolveResult != null) {
            return resolveResult;
          }
        }
      }
    }
  }
  throwImportNotDefined(name, packageJSONUrl, base);
}

const flagSymbol = Symbol('arg flag');

class ArgError extends Error {
	constructor(msg, code) {
		super(msg);
		this.name = 'ArgError';
		this.code = code;

		Object.setPrototypeOf(this, ArgError.prototype);
	}
}

function arg(
	opts,
	{
		argv = process.argv.slice(2),
		permissive = false,
		stopAtPositional = false
	} = {}
) {
	if (!opts) {
		throw new ArgError(
			'argument specification object is required',
			'ARG_CONFIG_NO_SPEC'
		);
	}

	const result = { _: [] };

	const aliases = {};
	const handlers = {};

	for (const key of Object.keys(opts)) {
		if (!key) {
			throw new ArgError(
				'argument key cannot be an empty string',
				'ARG_CONFIG_EMPTY_KEY'
			);
		}

		if (key[0] !== '-') {
			throw new ArgError(
				`argument key must start with '-' but found: '${key}'`,
				'ARG_CONFIG_NONOPT_KEY'
			);
		}

		if (key.length === 1) {
			throw new ArgError(
				`argument key must have a name; singular '-' keys are not allowed: ${key}`,
				'ARG_CONFIG_NONAME_KEY'
			);
		}

		if (typeof opts[key] === 'string') {
			aliases[key] = opts[key];
			continue;
		}

		let type = opts[key];
		let isFlag = false;

		if (
			Array.isArray(type) &&
			type.length === 1 &&
			typeof type[0] === 'function'
		) {
			const [fn] = type;
			type = (value, name, prev = []) => {
				prev.push(fn(value, name, prev[prev.length - 1]));
				return prev;
			};
			isFlag = fn === Boolean || fn[flagSymbol] === true;
		} else if (typeof type === 'function') {
			isFlag = type === Boolean || type[flagSymbol] === true;
		} else {
			throw new ArgError(
				`type missing or not a function or valid array type: ${key}`,
				'ARG_CONFIG_VAD_TYPE'
			);
		}

		if (key[1] !== '-' && key.length > 2) {
			throw new ArgError(
				`short argument keys (with a single hyphen) must have only one character: ${key}`,
				'ARG_CONFIG_SHORTOPT_TOOLONG'
			);
		}

		handlers[key] = [type, isFlag];
	}

	for (let i = 0, len = argv.length; i < len; i++) {
		const wholeArg = argv[i];

		if (stopAtPositional && result._.length > 0) {
			result._ = result._.concat(argv.slice(i));
			break;
		}

		if (wholeArg === '--') {
			result._ = result._.concat(argv.slice(i + 1));
			break;
		}

		if (wholeArg.length > 1 && wholeArg[0] === '-') {
			/* eslint-disable operator-linebreak */
			const separatedArguments =
				wholeArg[1] === '-' || wholeArg.length === 2
					? [wholeArg]
					: wholeArg
							.slice(1)
							.split('')
							.map((a) => `-${a}`);
			/* eslint-enable operator-linebreak */

			for (let j = 0; j < separatedArguments.length; j++) {
				const arg = separatedArguments[j];
				const [originalArgName, argStr] =
					arg[1] === '-' ? arg.split(/=(.*)/, 2) : [arg, undefined];

				let argName = originalArgName;
				while (argName in aliases) {
					argName = aliases[argName];
				}

				if (!(argName in handlers)) {
					if (permissive) {
						result._.push(arg);
						continue;
					} else {
						throw new ArgError(
							`unknown or unexpected option: ${originalArgName}`,
							'ARG_UNKNOWN_OPTION'
						);
					}
				}

				const [type, isFlag] = handlers[argName];

				if (!isFlag && j + 1 < separatedArguments.length) {
					throw new ArgError(
						`option requires argument (but was followed by another short argument): ${originalArgName}`,
						'ARG_MISSING_REQUIRED_SHORTARG'
					);
				}

				if (isFlag) {
					result[argName] = type(true, argName, result[argName]);
				} else if (argStr === undefined) {
					if (
						argv.length < i + 2 ||
						(argv[i + 1].length > 1 &&
							argv[i + 1][0] === '-' &&
							!(
								argv[i + 1].match(/^-?\d*(\.(?=\d))?\d*$/) &&
								(type === Number ||
									// eslint-disable-next-line no-undef
									(typeof BigInt !== 'undefined' && type === BigInt))
							))
					) {
						const extended =
							originalArgName === argName ? '' : ` (alias for ${argName})`;
						throw new ArgError(
							`option requires argument: ${originalArgName}${extended}`,
							'ARG_MISSING_REQUIRED_LONGARG'
						);
					}

					result[argName] = type(argv[i + 1], argName, result[argName]);
					++i;
				} else {
					result[argName] = type(argStr, argName, result[argName]);
				}
			}
		} else {
			result._.push(wholeArg);
		}
	}

	return result;
}

arg.flag = (fn) => {
	fn[flagSymbol] = true;
	return fn;
};

// Utility types
arg.COUNT = arg.flag((v, name, existingCount) => (existingCount || 0) + 1);

// Expose error class
arg.ArgError = ArgError;

var arg_1 = arg;

/**
  @license
  The MIT License (MIT)

  Copyright (c) 2014 Blake Embrey (hello@blakeembrey.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
*/
function getOptionValue(opt) {
  parseOptions();
  return options[opt];
}
let options;
function parseOptions() {
  if (!options) {
    options = {
      "--conditions": [],
      ...parseArgv(getNodeOptionsEnvArgv()),
      ...parseArgv(process.execArgv)
    };
  }
}
function parseArgv(argv) {
  return arg_1(
    {
      "--conditions": [String],
      "-C": "--conditions"
    },
    {
      argv,
      permissive: true
    }
  );
}
function getNodeOptionsEnvArgv() {
  const errors = [];
  const envArgv = ParseNodeOptionsEnvVar(process.env.NODE_OPTIONS || "", errors);
  if (errors.length !== 0) ;
  return envArgv;
}
function ParseNodeOptionsEnvVar(node_options, errors) {
  const env_argv = [];
  let is_in_string = false;
  let will_start_new_arg = true;
  for (let index = 0; index < node_options.length; ++index) {
    let c = node_options[index];
    if (c === "\\" && is_in_string) {
      if (index + 1 === node_options.length) {
        errors.push("invalid value for NODE_OPTIONS (invalid escape)\n");
        return env_argv;
      } else {
        c = node_options[++index];
      }
    } else if (c === " " && !is_in_string) {
      will_start_new_arg = true;
      continue;
    } else if (c === '"') {
      is_in_string = !is_in_string;
      continue;
    }
    if (will_start_new_arg) {
      env_argv.push(c);
      will_start_new_arg = false;
    } else {
      env_argv[env_argv.length - 1] += c;
    }
  }
  if (is_in_string) {
    errors.push("invalid value for NODE_OPTIONS (unterminated string)\n");
  }
  return env_argv;
}

function makeApi(runtimeState, opts) {
  const alwaysWarnOnFallback = Number(process.env.PNP_ALWAYS_WARN_ON_FALLBACK) > 0;
  const debugLevel = Number(process.env.PNP_DEBUG_LEVEL);
  const pathRegExp = /^(?![a-zA-Z]:[\\/]|\\\\|\.{0,2}(?:\/|$))((?:node:)?(?:@[^/]+\/)?[^/]+)\/*(.*|)$/;
  const isStrictRegExp = /^(\/|\.{1,2}(\/|$))/;
  const isDirRegExp = /\/$/;
  const isRelativeRegexp = /^\.{0,2}\//;
  const topLevelLocator = { name: null, reference: null };
  const fallbackLocators = [];
  const emittedWarnings = /* @__PURE__ */ new Set();
  if (runtimeState.enableTopLevelFallback === true)
    fallbackLocators.push(topLevelLocator);
  if (opts.compatibilityMode !== false) {
    for (const name of [`react-scripts`, `gatsby`]) {
      const packageStore = runtimeState.packageRegistry.get(name);
      if (packageStore) {
        for (const reference of packageStore.keys()) {
          if (reference === null) {
            throw new Error(`Assertion failed: This reference shouldn't be null`);
          } else {
            fallbackLocators.push({ name, reference });
          }
        }
      }
    }
  }
  const {
    ignorePattern,
    packageRegistry,
    packageLocatorsByLocations
  } = runtimeState;
  function makeLogEntry(name, args) {
    return {
      fn: name,
      args,
      error: null,
      result: null
    };
  }
  function trace(entry) {
    var _a, _b, _c, _d, _e, _f;
    const colors = (_c = (_b = (_a = process.stderr) == null ? void 0 : _a.hasColors) == null ? void 0 : _b.call(_a)) != null ? _c : process.stdout.isTTY;
    const c = (n, str) => `\x1B[${n}m${str}\x1B[0m`;
    const error = entry.error;
    if (error)
      console.error(c(`31;1`, `\u2716 ${(_d = entry.error) == null ? void 0 : _d.message.replace(/\n.*/s, ``)}`));
    else
      console.error(c(`33;1`, `\u203C Resolution`));
    if (entry.args.length > 0)
      console.error();
    for (const arg of entry.args)
      console.error(`  ${c(`37;1`, `In \u2190`)} ${nodeUtils.inspect(arg, { colors, compact: true })}`);
    if (entry.result) {
      console.error();
      console.error(`  ${c(`37;1`, `Out \u2192`)} ${nodeUtils.inspect(entry.result, { colors, compact: true })}`);
    }
    const stack = (_f = (_e = new Error().stack.match(/(?<=^ +)at.*/gm)) == null ? void 0 : _e.slice(2)) != null ? _f : [];
    if (stack.length > 0) {
      console.error();
      for (const line of stack) {
        console.error(`  ${c(`38;5;244`, line)}`);
      }
    }
    console.error();
  }
  function maybeLog(name, fn) {
    if (opts.allowDebug === false)
      return fn;
    if (Number.isFinite(debugLevel)) {
      if (debugLevel >= 2) {
        return (...args) => {
          const logEntry = makeLogEntry(name, args);
          try {
            return logEntry.result = fn(...args);
          } catch (error) {
            throw logEntry.error = error;
          } finally {
            trace(logEntry);
          }
        };
      } else if (debugLevel >= 1) {
        return (...args) => {
          try {
            return fn(...args);
          } catch (error) {
            const logEntry = makeLogEntry(name, args);
            logEntry.error = error;
            trace(logEntry);
            throw error;
          }
        };
      }
    }
    return fn;
  }
  function getPackageInformationSafe(packageLocator) {
    const packageInformation = getPackageInformation(packageLocator);
    if (!packageInformation) {
      throw makeError(
        ErrorCode.INTERNAL,
        `Couldn't find a matching entry in the dependency tree for the specified parent (this is probably an internal error)`
      );
    }
    return packageInformation;
  }
  function isDependencyTreeRoot(packageLocator) {
    if (packageLocator.name === null)
      return true;
    for (const dependencyTreeRoot of runtimeState.dependencyTreeRoots)
      if (dependencyTreeRoot.name === packageLocator.name && dependencyTreeRoot.reference === packageLocator.reference)
        return true;
    return false;
  }
  const defaultExportsConditions = /* @__PURE__ */ new Set([
    `node`,
    `require`,
    ...getOptionValue(`--conditions`)
  ]);
  function applyNodeExportsResolution(unqualifiedPath, conditions = defaultExportsConditions, issuer) {
    const locator = findPackageLocator(ppath.join(unqualifiedPath, `internal.js`), {
      resolveIgnored: true,
      includeDiscardFromLookup: true
    });
    if (locator === null) {
      throw makeError(
        ErrorCode.INTERNAL,
        `The locator that owns the "${unqualifiedPath}" path can't be found inside the dependency tree (this is probably an internal error)`
      );
    }
    const { packageLocation } = getPackageInformationSafe(locator);
    const manifestPath = ppath.join(packageLocation, Filename.manifest);
    if (!opts.fakeFs.existsSync(manifestPath))
      return null;
    const pkgJson = JSON.parse(opts.fakeFs.readFileSync(manifestPath, `utf8`));
    if (pkgJson.exports == null)
      return null;
    let subpath = ppath.contains(packageLocation, unqualifiedPath);
    if (subpath === null) {
      throw makeError(
        ErrorCode.INTERNAL,
        `unqualifiedPath doesn't contain the packageLocation (this is probably an internal error)`
      );
    }
    if (subpath !== `.` && !isRelativeRegexp.test(subpath))
      subpath = `./${subpath}`;
    try {
      const resolvedExport = packageExportsResolve({
        packageJSONUrl: url.pathToFileURL(npath.fromPortablePath(manifestPath)),
        packageSubpath: subpath,
        exports: pkgJson.exports,
        base: issuer ? url.pathToFileURL(npath.fromPortablePath(issuer)) : null,
        conditions
      });
      return npath.toPortablePath(url.fileURLToPath(resolvedExport));
    } catch (error) {
      throw makeError(
        ErrorCode.EXPORTS_RESOLUTION_FAILED,
        error.message,
        { unqualifiedPath: getPathForDisplay(unqualifiedPath), locator, pkgJson, subpath: getPathForDisplay(subpath), conditions },
        error.code
      );
    }
  }
  function applyNodeExtensionResolution(unqualifiedPath, candidates, { extensions }) {
    let stat;
    try {
      candidates.push(unqualifiedPath);
      stat = opts.fakeFs.statSync(unqualifiedPath);
    } catch (error) {
    }
    if (stat && !stat.isDirectory())
      return opts.fakeFs.realpathSync(unqualifiedPath);
    if (stat && stat.isDirectory()) {
      let pkgJson;
      try {
        pkgJson = JSON.parse(opts.fakeFs.readFileSync(ppath.join(unqualifiedPath, Filename.manifest), `utf8`));
      } catch (error) {
      }
      let nextUnqualifiedPath;
      if (pkgJson && pkgJson.main)
        nextUnqualifiedPath = ppath.resolve(unqualifiedPath, pkgJson.main);
      if (nextUnqualifiedPath && nextUnqualifiedPath !== unqualifiedPath) {
        const resolution = applyNodeExtensionResolution(nextUnqualifiedPath, candidates, { extensions });
        if (resolution !== null) {
          return resolution;
        }
      }
    }
    for (let i = 0, length = extensions.length; i < length; i++) {
      const candidateFile = `${unqualifiedPath}${extensions[i]}`;
      candidates.push(candidateFile);
      if (opts.fakeFs.existsSync(candidateFile)) {
        return candidateFile;
      }
    }
    if (stat && stat.isDirectory()) {
      for (let i = 0, length = extensions.length; i < length; i++) {
        const candidateFile = ppath.format({ dir: unqualifiedPath, name: `index`, ext: extensions[i] });
        candidates.push(candidateFile);
        if (opts.fakeFs.existsSync(candidateFile)) {
          return candidateFile;
        }
      }
    }
    return null;
  }
  function makeFakeModule(path) {
    const fakeModule = new require$$0.Module(path, null);
    fakeModule.filename = path;
    fakeModule.paths = require$$0.Module._nodeModulePaths(path);
    return fakeModule;
  }
  function callNativeResolution(request, issuer) {
    if (issuer.endsWith(`/`))
      issuer = ppath.join(issuer, `internal.js`);
    return require$$0.Module._resolveFilename(npath.fromPortablePath(request), makeFakeModule(npath.fromPortablePath(issuer)), false, { plugnplay: false });
  }
  function isPathIgnored(path) {
    if (ignorePattern === null)
      return false;
    const subPath = ppath.contains(runtimeState.basePath, path);
    if (subPath === null)
      return false;
    if (ignorePattern.test(subPath.replace(/\/$/, ``))) {
      return true;
    } else {
      return false;
    }
  }
  const VERSIONS = { std: 3, resolveVirtual: 1, getAllLocators: 1 };
  const topLevel = topLevelLocator;
  function getPackageInformation({ name, reference }) {
    const packageInformationStore = packageRegistry.get(name);
    if (!packageInformationStore)
      return null;
    const packageInformation = packageInformationStore.get(reference);
    if (!packageInformation)
      return null;
    return packageInformation;
  }
  function findPackageDependents({ name, reference }) {
    const dependents = [];
    for (const [dependentName, packageInformationStore] of packageRegistry) {
      if (dependentName === null)
        continue;
      for (const [dependentReference, packageInformation] of packageInformationStore) {
        if (dependentReference === null)
          continue;
        const dependencyReference = packageInformation.packageDependencies.get(name);
        if (dependencyReference !== reference)
          continue;
        if (dependentName === name && dependentReference === reference)
          continue;
        dependents.push({
          name: dependentName,
          reference: dependentReference
        });
      }
    }
    return dependents;
  }
  function findBrokenPeerDependencies(dependency, initialPackage) {
    const brokenPackages = /* @__PURE__ */ new Map();
    const alreadyVisited = /* @__PURE__ */ new Set();
    const traversal = (currentPackage) => {
      const identifier = JSON.stringify(currentPackage.name);
      if (alreadyVisited.has(identifier))
        return;
      alreadyVisited.add(identifier);
      const dependents = findPackageDependents(currentPackage);
      for (const dependent of dependents) {
        const dependentInformation = getPackageInformationSafe(dependent);
        if (dependentInformation.packagePeers.has(dependency)) {
          traversal(dependent);
        } else {
          let brokenSet = brokenPackages.get(dependent.name);
          if (typeof brokenSet === `undefined`)
            brokenPackages.set(dependent.name, brokenSet = /* @__PURE__ */ new Set());
          brokenSet.add(dependent.reference);
        }
      }
    };
    traversal(initialPackage);
    const brokenList = [];
    for (const name of [...brokenPackages.keys()].sort())
      for (const reference of [...brokenPackages.get(name)].sort())
        brokenList.push({ name, reference });
    return brokenList;
  }
  function findPackageLocator(location, { resolveIgnored = false, includeDiscardFromLookup = false } = {}) {
    if (isPathIgnored(location) && !resolveIgnored)
      return null;
    let relativeLocation = ppath.relative(runtimeState.basePath, location);
    if (!relativeLocation.match(isStrictRegExp))
      relativeLocation = `./${relativeLocation}`;
    if (!relativeLocation.endsWith(`/`))
      relativeLocation = `${relativeLocation}/`;
    do {
      const entry = packageLocatorsByLocations.get(relativeLocation);
      if (typeof entry === `undefined` || entry.discardFromLookup && !includeDiscardFromLookup) {
        relativeLocation = relativeLocation.substring(0, relativeLocation.lastIndexOf(`/`, relativeLocation.length - 2) + 1);
        continue;
      }
      return entry.locator;
    } while (relativeLocation !== ``);
    return null;
  }
  function tryReadFile(filePath) {
    try {
      return opts.fakeFs.readFileSync(npath.toPortablePath(filePath), `utf8`);
    } catch (err) {
      if (err.code === `ENOENT`)
        return void 0;
      throw err;
    }
  }
  function resolveToUnqualified(request, issuer, { considerBuiltins = true } = {}) {
    if (request.startsWith(`#`))
      throw new Error(`resolveToUnqualified can not handle private import mappings`);
    if (request === `pnpapi`)
      return npath.toPortablePath(opts.pnpapiResolution);
    if (considerBuiltins && isBuiltinModule(request))
      return null;
    const requestForDisplay = getPathForDisplay(request);
    const issuerForDisplay = issuer && getPathForDisplay(issuer);
    if (issuer && isPathIgnored(issuer)) {
      if (!ppath.isAbsolute(request) || findPackageLocator(request) === null) {
        const result = callNativeResolution(request, issuer);
        if (result === false) {
          throw makeError(
            ErrorCode.BUILTIN_NODE_RESOLUTION_FAILED,
            `The builtin node resolution algorithm was unable to resolve the requested module (it didn't go through the pnp resolver because the issuer was explicitely ignored by the regexp)

Require request: "${requestForDisplay}"
Required by: ${issuerForDisplay}
`,
            { request: requestForDisplay, issuer: issuerForDisplay }
          );
        }
        return npath.toPortablePath(result);
      }
    }
    let unqualifiedPath;
    const dependencyNameMatch = request.match(pathRegExp);
    if (!dependencyNameMatch) {
      if (ppath.isAbsolute(request)) {
        unqualifiedPath = ppath.normalize(request);
      } else {
        if (!issuer) {
          throw makeError(
            ErrorCode.API_ERROR,
            `The resolveToUnqualified function must be called with a valid issuer when the path isn't a builtin nor absolute`,
            { request: requestForDisplay, issuer: issuerForDisplay }
          );
        }
        const absoluteIssuer = ppath.resolve(issuer);
        if (issuer.match(isDirRegExp)) {
          unqualifiedPath = ppath.normalize(ppath.join(absoluteIssuer, request));
        } else {
          unqualifiedPath = ppath.normalize(ppath.join(ppath.dirname(absoluteIssuer), request));
        }
      }
    } else {
      if (!issuer) {
        throw makeError(
          ErrorCode.API_ERROR,
          `The resolveToUnqualified function must be called with a valid issuer when the path isn't a builtin nor absolute`,
          { request: requestForDisplay, issuer: issuerForDisplay }
        );
      }
      const [, dependencyName, subPath] = dependencyNameMatch;
      const issuerLocator = findPackageLocator(issuer);
      if (!issuerLocator) {
        const result = callNativeResolution(request, issuer);
        if (result === false) {
          throw makeError(
            ErrorCode.BUILTIN_NODE_RESOLUTION_FAILED,
            `The builtin node resolution algorithm was unable to resolve the requested module (it didn't go through the pnp resolver because the issuer doesn't seem to be part of the Yarn-managed dependency tree).

Require path: "${requestForDisplay}"
Required by: ${issuerForDisplay}
`,
            { request: requestForDisplay, issuer: issuerForDisplay }
          );
        }
        return npath.toPortablePath(result);
      }
      const issuerInformation = getPackageInformationSafe(issuerLocator);
      let dependencyReference = issuerInformation.packageDependencies.get(dependencyName);
      let fallbackReference = null;
      if (dependencyReference == null) {
        if (issuerLocator.name !== null) {
          const exclusionEntry = runtimeState.fallbackExclusionList.get(issuerLocator.name);
          const canUseFallbacks = !exclusionEntry || !exclusionEntry.has(issuerLocator.reference);
          if (canUseFallbacks) {
            for (let t = 0, T = fallbackLocators.length; t < T; ++t) {
              const fallbackInformation = getPackageInformationSafe(fallbackLocators[t]);
              const reference = fallbackInformation.packageDependencies.get(dependencyName);
              if (reference == null)
                continue;
              if (alwaysWarnOnFallback)
                fallbackReference = reference;
              else
                dependencyReference = reference;
              break;
            }
            if (runtimeState.enableTopLevelFallback) {
              if (dependencyReference == null && fallbackReference === null) {
                const reference = runtimeState.fallbackPool.get(dependencyName);
                if (reference != null) {
                  fallbackReference = reference;
                }
              }
            }
          }
        }
      }
      let error = null;
      if (dependencyReference === null) {
        if (isDependencyTreeRoot(issuerLocator)) {
          error = makeError(
            ErrorCode.MISSING_PEER_DEPENDENCY,
            `Your application tried to access ${dependencyName} (a peer dependency); this isn't allowed as there is no ancestor to satisfy the requirement. Use a devDependency if needed.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerForDisplay}
`,
            { request: requestForDisplay, issuer: issuerForDisplay, dependencyName }
          );
        } else {
          const brokenAncestors = findBrokenPeerDependencies(dependencyName, issuerLocator);
          if (brokenAncestors.every((ancestor) => isDependencyTreeRoot(ancestor))) {
            error = makeError(
              ErrorCode.MISSING_PEER_DEPENDENCY,
              `${issuerLocator.name} tried to access ${dependencyName} (a peer dependency) but it isn't provided by your application; this makes the require call ambiguous and unsound.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerLocator.name}@${issuerLocator.reference} (via ${issuerForDisplay})
${brokenAncestors.map((ancestorLocator) => `Ancestor breaking the chain: ${ancestorLocator.name}@${ancestorLocator.reference}
`).join(``)}
`,
              { request: requestForDisplay, issuer: issuerForDisplay, issuerLocator: Object.assign({}, issuerLocator), dependencyName, brokenAncestors }
            );
          } else {
            error = makeError(
              ErrorCode.MISSING_PEER_DEPENDENCY,
              `${issuerLocator.name} tried to access ${dependencyName} (a peer dependency) but it isn't provided by its ancestors; this makes the require call ambiguous and unsound.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerLocator.name}@${issuerLocator.reference} (via ${issuerForDisplay})

${brokenAncestors.map((ancestorLocator) => `Ancestor breaking the chain: ${ancestorLocator.name}@${ancestorLocator.reference}
`).join(``)}
`,
              { request: requestForDisplay, issuer: issuerForDisplay, issuerLocator: Object.assign({}, issuerLocator), dependencyName, brokenAncestors }
            );
          }
        }
      } else if (dependencyReference === void 0) {
        if (!considerBuiltins && isBuiltinModule(request)) {
          if (isDependencyTreeRoot(issuerLocator)) {
            error = makeError(
              ErrorCode.UNDECLARED_DEPENDENCY,
              `Your application tried to access ${dependencyName}. While this module is usually interpreted as a Node builtin, your resolver is running inside a non-Node resolution context where such builtins are ignored. Since ${dependencyName} isn't otherwise declared in your dependencies, this makes the require call ambiguous and unsound.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerForDisplay}
`,
              { request: requestForDisplay, issuer: issuerForDisplay, dependencyName }
            );
          } else {
            error = makeError(
              ErrorCode.UNDECLARED_DEPENDENCY,
              `${issuerLocator.name} tried to access ${dependencyName}. While this module is usually interpreted as a Node builtin, your resolver is running inside a non-Node resolution context where such builtins are ignored. Since ${dependencyName} isn't otherwise declared in ${issuerLocator.name}'s dependencies, this makes the require call ambiguous and unsound.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerForDisplay}
`,
              { request: requestForDisplay, issuer: issuerForDisplay, issuerLocator: Object.assign({}, issuerLocator), dependencyName }
            );
          }
        } else {
          if (isDependencyTreeRoot(issuerLocator)) {
            error = makeError(
              ErrorCode.UNDECLARED_DEPENDENCY,
              `Your application tried to access ${dependencyName}, but it isn't declared in your dependencies; this makes the require call ambiguous and unsound.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerForDisplay}
`,
              { request: requestForDisplay, issuer: issuerForDisplay, dependencyName }
            );
          } else {
            error = makeError(
              ErrorCode.UNDECLARED_DEPENDENCY,
              `${issuerLocator.name} tried to access ${dependencyName}, but it isn't declared in its dependencies; this makes the require call ambiguous and unsound.

Required package: ${dependencyName}${dependencyName !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerLocator.name}@${issuerLocator.reference} (via ${issuerForDisplay})
`,
              { request: requestForDisplay, issuer: issuerForDisplay, issuerLocator: Object.assign({}, issuerLocator), dependencyName }
            );
          }
        }
      }
      if (dependencyReference == null) {
        if (fallbackReference === null || error === null)
          throw error || new Error(`Assertion failed: Expected an error to have been set`);
        dependencyReference = fallbackReference;
        const message = error.message.replace(/\n.*/g, ``);
        error.message = message;
        if (!emittedWarnings.has(message) && debugLevel !== 0) {
          emittedWarnings.add(message);
          process.emitWarning(error);
        }
      }
      const dependencyLocator = Array.isArray(dependencyReference) ? { name: dependencyReference[0], reference: dependencyReference[1] } : { name: dependencyName, reference: dependencyReference };
      const dependencyInformation = getPackageInformationSafe(dependencyLocator);
      if (!dependencyInformation.packageLocation) {
        throw makeError(
          ErrorCode.MISSING_DEPENDENCY,
          `A dependency seems valid but didn't get installed for some reason. This might be caused by a partial install, such as dev vs prod.

Required package: ${dependencyLocator.name}@${dependencyLocator.reference}${dependencyLocator.name !== requestForDisplay ? ` (via "${requestForDisplay}")` : ``}
Required by: ${issuerLocator.name}@${issuerLocator.reference} (via ${issuerForDisplay})
`,
          { request: requestForDisplay, issuer: issuerForDisplay, dependencyLocator: Object.assign({}, dependencyLocator) }
        );
      }
      const dependencyLocation = dependencyInformation.packageLocation;
      if (subPath) {
        unqualifiedPath = ppath.join(dependencyLocation, subPath);
      } else {
        unqualifiedPath = dependencyLocation;
      }
    }
    return ppath.normalize(unqualifiedPath);
  }
  function resolveUnqualifiedExport(request, unqualifiedPath, conditions = defaultExportsConditions, issuer) {
    if (isStrictRegExp.test(request))
      return unqualifiedPath;
    const unqualifiedExportPath = applyNodeExportsResolution(unqualifiedPath, conditions, issuer);
    if (unqualifiedExportPath) {
      return ppath.normalize(unqualifiedExportPath);
    } else {
      return unqualifiedPath;
    }
  }
  function resolveUnqualified(unqualifiedPath, { extensions = Object.keys(require$$0.Module._extensions) } = {}) {
    var _a, _b;
    const candidates = [];
    const qualifiedPath = applyNodeExtensionResolution(unqualifiedPath, candidates, { extensions });
    if (qualifiedPath) {
      return ppath.normalize(qualifiedPath);
    } else {
      reportRequiredFilesToWatchMode(candidates.map((candidate) => npath.fromPortablePath(candidate)));
      const unqualifiedPathForDisplay = getPathForDisplay(unqualifiedPath);
      const containingPackage = findPackageLocator(unqualifiedPath);
      if (containingPackage) {
        const { packageLocation } = getPackageInformationSafe(containingPackage);
        let exists = true;
        try {
          opts.fakeFs.accessSync(packageLocation);
        } catch (err) {
          if ((err == null ? void 0 : err.code) === `ENOENT`) {
            exists = false;
          } else {
            const readableError = ((_b = (_a = err == null ? void 0 : err.message) != null ? _a : err) != null ? _b : `empty exception thrown`).replace(/^[A-Z]/, ($0) => $0.toLowerCase());
            throw makeError(ErrorCode.QUALIFIED_PATH_RESOLUTION_FAILED, `Required package exists but could not be accessed (${readableError}).

Missing package: ${containingPackage.name}@${containingPackage.reference}
Expected package location: ${getPathForDisplay(packageLocation)}
`, { unqualifiedPath: unqualifiedPathForDisplay, extensions });
          }
        }
        if (!exists) {
          const errorMessage = packageLocation.includes(`/unplugged/`) ? `Required unplugged package missing from disk. This may happen when switching branches without running installs (unplugged packages must be fully materialized on disk to work).` : `Required package missing from disk. If you keep your packages inside your repository then restarting the Node process may be enough. Otherwise, try to run an install first.`;
          throw makeError(
            ErrorCode.QUALIFIED_PATH_RESOLUTION_FAILED,
            `${errorMessage}

Missing package: ${containingPackage.name}@${containingPackage.reference}
Expected package location: ${getPathForDisplay(packageLocation)}
`,
            { unqualifiedPath: unqualifiedPathForDisplay, extensions }
          );
        }
      }
      throw makeError(
        ErrorCode.QUALIFIED_PATH_RESOLUTION_FAILED,
        `Qualified path resolution failed: we looked for the following paths, but none could be accessed.

Source path: ${unqualifiedPathForDisplay}
${candidates.map((candidate) => `Not found: ${getPathForDisplay(candidate)}
`).join(``)}`,
        { unqualifiedPath: unqualifiedPathForDisplay, extensions }
      );
    }
  }
  function resolvePrivateRequest(request, issuer, opts2) {
    var _a;
    if (!issuer)
      throw new Error(`Assertion failed: An issuer is required to resolve private import mappings`);
    const resolved = packageImportsResolve({
      name: request,
      base: url.pathToFileURL(npath.fromPortablePath(issuer)),
      conditions: (_a = opts2.conditions) != null ? _a : defaultExportsConditions,
      readFileSyncFn: tryReadFile
    });
    if (resolved instanceof url.URL) {
      return resolveUnqualified(npath.toPortablePath(url.fileURLToPath(resolved)), { extensions: opts2.extensions });
    } else {
      if (resolved.startsWith(`#`))
        throw new Error(`Mapping from one private import to another isn't allowed`);
      return resolveRequest(resolved, issuer, opts2);
    }
  }
  function resolveRequest(request, issuer, opts2 = {}) {
    try {
      if (request.startsWith(`#`))
        return resolvePrivateRequest(request, issuer, opts2);
      const { considerBuiltins, extensions, conditions } = opts2;
      const unqualifiedPath = resolveToUnqualified(request, issuer, { considerBuiltins });
      if (request === `pnpapi`)
        return unqualifiedPath;
      if (unqualifiedPath === null)
        return null;
      const isIssuerIgnored = () => issuer !== null ? isPathIgnored(issuer) : false;
      const remappedPath = (!considerBuiltins || !isBuiltinModule(request)) && !isIssuerIgnored() ? resolveUnqualifiedExport(request, unqualifiedPath, conditions, issuer) : unqualifiedPath;
      return resolveUnqualified(remappedPath, { extensions });
    } catch (error) {
      if (Object.prototype.hasOwnProperty.call(error, `pnpCode`))
        Object.assign(error.data, { request: getPathForDisplay(request), issuer: issuer && getPathForDisplay(issuer) });
      throw error;
    }
  }
  function resolveVirtual(request) {
    const normalized = ppath.normalize(request);
    const resolved = VirtualFS.resolveVirtual(normalized);
    return resolved !== normalized ? resolved : null;
  }
  return {
    VERSIONS,
    topLevel,
    getLocator: (name, referencish) => {
      if (Array.isArray(referencish)) {
        return { name: referencish[0], reference: referencish[1] };
      } else {
        return { name, reference: referencish };
      }
    },
    getDependencyTreeRoots: () => {
      return [...runtimeState.dependencyTreeRoots];
    },
    getAllLocators() {
      const locators = [];
      for (const [name, entry] of packageRegistry)
        for (const reference of entry.keys())
          if (name !== null && reference !== null)
            locators.push({ name, reference });
      return locators;
    },
    getPackageInformation: (locator) => {
      const info = getPackageInformation(locator);
      if (info === null)
        return null;
      const packageLocation = npath.fromPortablePath(info.packageLocation);
      const nativeInfo = { ...info, packageLocation };
      return nativeInfo;
    },
    findPackageLocator: (path) => {
      return findPackageLocator(npath.toPortablePath(path));
    },
    resolveToUnqualified: maybeLog(`resolveToUnqualified`, (request, issuer, opts2) => {
      const portableIssuer = issuer !== null ? npath.toPortablePath(issuer) : null;
      const resolution = resolveToUnqualified(npath.toPortablePath(request), portableIssuer, opts2);
      if (resolution === null)
        return null;
      return npath.fromPortablePath(resolution);
    }),
    resolveUnqualified: maybeLog(`resolveUnqualified`, (unqualifiedPath, opts2) => {
      return npath.fromPortablePath(resolveUnqualified(npath.toPortablePath(unqualifiedPath), opts2));
    }),
    resolveRequest: maybeLog(`resolveRequest`, (request, issuer, opts2) => {
      const portableIssuer = issuer !== null ? npath.toPortablePath(issuer) : null;
      const resolution = resolveRequest(npath.toPortablePath(request), portableIssuer, opts2);
      if (resolution === null)
        return null;
      return npath.fromPortablePath(resolution);
    }),
    resolveVirtual: maybeLog(`resolveVirtual`, (path) => {
      const result = resolveVirtual(npath.toPortablePath(path));
      if (result !== null) {
        return npath.fromPortablePath(result);
      } else {
        return null;
      }
    })
  };
}

function makeManager(pnpapi, opts) {
  const initialApiPath = npath.toPortablePath(pnpapi.resolveToUnqualified(`pnpapi`, null));
  const initialApiStats = opts.fakeFs.statSync(npath.toPortablePath(initialApiPath));
  const apiMetadata = /* @__PURE__ */ new Map([
    [initialApiPath, {
      cache: require$$0.Module._cache,
      instance: pnpapi,
      stats: initialApiStats,
      lastRefreshCheck: Date.now()
    }]
  ]);
  function loadApiInstance(pnpApiPath) {
    const nativePath = npath.fromPortablePath(pnpApiPath);
    const module = new require$$0.Module(nativePath, null);
    module.load(nativePath);
    return module.exports;
  }
  function refreshApiEntry(pnpApiPath, apiEntry) {
    const timeNow = Date.now();
    if (timeNow - apiEntry.lastRefreshCheck < 500)
      return;
    apiEntry.lastRefreshCheck = timeNow;
    const stats = opts.fakeFs.statSync(pnpApiPath);
    if (stats.mtime > apiEntry.stats.mtime) {
      process.emitWarning(`[Warning] The runtime detected new informations in a PnP file; reloading the API instance (${npath.fromPortablePath(pnpApiPath)})`);
      apiEntry.stats = stats;
      apiEntry.instance = loadApiInstance(pnpApiPath);
    }
  }
  function getApiEntry(pnpApiPath, refresh = false) {
    let apiEntry = apiMetadata.get(pnpApiPath);
    if (typeof apiEntry !== `undefined`) {
      if (refresh) {
        refreshApiEntry(pnpApiPath, apiEntry);
      }
    } else {
      apiMetadata.set(pnpApiPath, apiEntry = {
        cache: {},
        instance: loadApiInstance(pnpApiPath),
        stats: opts.fakeFs.statSync(pnpApiPath),
        lastRefreshCheck: Date.now()
      });
    }
    return apiEntry;
  }
  const findApiPathCache = /* @__PURE__ */ new Map();
  function addToCacheAndReturn(start, end, target) {
    if (target !== null)
      target = VirtualFS.resolveVirtual(target);
    let curr;
    let next = start;
    do {
      curr = next;
      findApiPathCache.set(curr, target);
      next = ppath.dirname(curr);
    } while (curr !== end);
    return target;
  }
  function findApiPathFor(modulePath) {
    let bestCandidate = null;
    for (const [apiPath, apiEntry] of apiMetadata) {
      const locator = apiEntry.instance.findPackageLocator(modulePath);
      if (!locator)
        continue;
      if (apiMetadata.size === 1)
        return apiPath;
      const packageInformation = apiEntry.instance.getPackageInformation(locator);
      if (!packageInformation)
        throw new Error(`Assertion failed: Couldn't get package information for '${modulePath}'`);
      if (!bestCandidate)
        bestCandidate = { packageLocation: packageInformation.packageLocation, apiPaths: [] };
      if (packageInformation.packageLocation === bestCandidate.packageLocation) {
        bestCandidate.apiPaths.push(apiPath);
      } else if (packageInformation.packageLocation.length > bestCandidate.packageLocation.length) {
        bestCandidate = { packageLocation: packageInformation.packageLocation, apiPaths: [apiPath] };
      }
    }
    if (bestCandidate) {
      if (bestCandidate.apiPaths.length === 1)
        return bestCandidate.apiPaths[0];
      const controlSegment = bestCandidate.apiPaths.map((apiPath) => `  ${npath.fromPortablePath(apiPath)}`).join(`
`);
      throw new Error(`Unable to locate pnpapi, the module '${modulePath}' is controlled by multiple pnpapi instances.
This is usually caused by using the global cache (enableGlobalCache: true)

Controlled by:
${controlSegment}
`);
    }
    const start = ppath.resolve(npath.toPortablePath(modulePath));
    let curr;
    let next = start;
    do {
      curr = next;
      const cached = findApiPathCache.get(curr);
      if (cached !== void 0)
        return addToCacheAndReturn(start, curr, cached);
      const cjsCandidate = ppath.join(curr, Filename.pnpCjs);
      if (opts.fakeFs.existsSync(cjsCandidate) && opts.fakeFs.statSync(cjsCandidate).isFile())
        return addToCacheAndReturn(start, curr, cjsCandidate);
      const legacyCjsCandidate = ppath.join(curr, Filename.pnpJs);
      if (opts.fakeFs.existsSync(legacyCjsCandidate) && opts.fakeFs.statSync(legacyCjsCandidate).isFile())
        return addToCacheAndReturn(start, curr, legacyCjsCandidate);
      next = ppath.dirname(curr);
    } while (curr !== PortablePath.root);
    return addToCacheAndReturn(start, curr, null);
  }
  function getApiPathFromParent(parent) {
    if (parent == null)
      return initialApiPath;
    if (typeof parent.pnpApiPath === `undefined`) {
      if (parent.filename !== null) {
        return parent.pnpApiPath = findApiPathFor(parent.filename);
      } else {
        return initialApiPath;
      }
    }
    if (parent.pnpApiPath !== null)
      return parent.pnpApiPath;
    return null;
  }
  return {
    getApiPathFromParent,
    findApiPathFor,
    getApiEntry
  };
}

const localFs = { ...fs__default.default };
const nodeFs = new NodeFS(localFs);
const defaultRuntimeState = $$SETUP_STATE(hydrateRuntimeState);
const defaultPnpapiResolution = __filename;
const defaultFsLayer = new VirtualFS({
  baseFs: new ZipOpenFS({
    baseFs: nodeFs,
    libzip: () => getLibzipSync(),
    maxOpenFiles: 80,
    readOnlyArchives: true
  })
});
class DynamicFS extends ProxiedFS {
  constructor() {
    super(ppath);
    this.baseFs = defaultFsLayer;
  }
  mapToBase(p) {
    return p;
  }
  mapFromBase(p) {
    return p;
  }
}
const dynamicFsLayer = new DynamicFS();
let manager;
const defaultApi = Object.assign(makeApi(defaultRuntimeState, {
  fakeFs: dynamicFsLayer,
  pnpapiResolution: defaultPnpapiResolution
}), {
  makeApi: ({
    basePath = void 0,
    fakeFs = dynamicFsLayer,
    pnpapiResolution = defaultPnpapiResolution,
    ...rest
  }) => {
    const apiRuntimeState = typeof basePath !== `undefined` ? $$SETUP_STATE(hydrateRuntimeState, basePath) : defaultRuntimeState;
    return makeApi(apiRuntimeState, {
      fakeFs,
      pnpapiResolution,
      ...rest
    });
  },
  setup: (api) => {
    applyPatch(api || defaultApi, {
      fakeFs: defaultFsLayer,
      manager
    });
    dynamicFsLayer.baseFs = new NodeFS(fs__default.default);
  }
});
manager = makeManager(defaultApi, {
  fakeFs: dynamicFsLayer
});
if (module.parent && module.parent.id === `internal/preload`) {
  defaultApi.setup();
  if (module.filename) {
    delete require$$0__default.default._cache[module.filename];
  }
}
if (process.mainModule === module) {
  const reportError = (code, message, data) => {
    process.stdout.write(`${JSON.stringify([{ code, message, data }, null])}
`);
  };
  const reportSuccess = (resolution) => {
    process.stdout.write(`${JSON.stringify([null, resolution])}
`);
  };
  const processResolution = (request, issuer) => {
    try {
      reportSuccess(defaultApi.resolveRequest(request, issuer));
    } catch (error) {
      reportError(error.code, error.message, error.data);
    }
  };
  const processRequest = (data) => {
    try {
      const [request, issuer] = JSON.parse(data);
      processResolution(request, issuer);
    } catch (error) {
      reportError(`INVALID_JSON`, error.message, error.data);
    }
  };
  if (process.argv.length > 2) {
    if (process.argv.length !== 4) {
      process.stderr.write(`Usage: ${process.argv[0]} ${process.argv[1]} <request> <issuer>
`);
      process.exitCode = 64;
    } else {
      processResolution(process.argv[2], process.argv[3]);
    }
  } else {
    let buffer = ``;
    const decoder = new StringDecoder__default.default.StringDecoder();
    process.stdin.on(`data`, (chunk) => {
      buffer += decoder.write(chunk);
      do {
        const index = buffer.indexOf(`
`);
        if (index === -1)
          break;
        const line = buffer.slice(0, index);
        buffer = buffer.slice(index + 1);
        processRequest(line);
      } while (true);
    });
  }
}

module.exports = defaultApi;
